@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,8 @@
       subroutine advecforc(nope,voldl,ithermal,xl,nelemload,nelemadvec,
      &  nload,lakon,xload,istep,time,ttime,dtime,sideload,vold,mi,
      &  xloadold,reltime,nmethod,tnl,iinc,iponoel,inoel,ielprop,prop,
-     &  ielmat,shcon,nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon)
+     &  ielmat,shcon,nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon,
+     &  ipobody,ibody,xbody)
 !
 !     calculates the stiffness of an advective element.
 !     An advective element consists of a face with a forced convection
@@ -30,23 +31,21 @@
       character*8 lakonl,lakon(*) 
       character*20 sideload(*),sideloadl
 !
-      integer nope,i,ithermal(2),j,nelemload(2,*),nelemadvec,nload,id,
+      integer nope,i,ithermal(*),j,nelemload(2,*),nelemadvec,nload,id,
      &  nelem,ig,mint2d,iflag,istep,jltyp,nfield,mi(*),nmethod,k,iinc,
-     &  node,nopes,iponoel(*),inoel(2,*),ielprop(*),ielmat(*),ipkon(*),
-     &  nshcon(*),nrhcon(*),ntmat_,kon(*),ncocon(2,*)
+     &  node,nopes,iponoel(*),inoel(2,*),ielprop(*),ielmat(mi(3),*),
+     &  ipkon(*),
+     &  nshcon(*),nrhcon(*),ntmat_,kon(*),ncocon(2,*),ipobody(2,*),
+     &  ibody(3,*)
 !
       real*8 tl2(9),voldl(0:mi(2),20),xl(3,9),sinktemp,xi,et,weight,
      &  xl2(3,8),xsj2(3),shp2(7,9),coords(3),xs2(3,7),dxsj2,areaj,
      &  temp,xload(2,*),timeend(2),time,ttime,dtime,field,reltime,
      &  vold(0:mi(2),*),xloadold(2,*),tnl(9),tnlref,prop(*),
-     &  shcon(0:3,ntmat_,*),rhcon(0:1,ntmat_,*),cocon(0:6,ntmat_,*)
+     &  shcon(0:3,ntmat_,*),rhcon(0:1,ntmat_,*),cocon(0:6,ntmat_,*),
+     &  xbody(7,*),heatnod,heatfac
 !
-      intent(in) nope,voldl,ithermal,xl,nelemload,nelemadvec,
-     &  nload,lakon,istep,time,ttime,dtime,sideload,vold,mi,
-     &  xloadold,reltime,nmethod,iinc,iponoel,inoel,ielprop,prop,
-     &  ielmat,shcon,nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon
 !
-      intent(inout) xload,tnl
 !
       include "gauss.f"
 !
@@ -54,6 +53,8 @@
 !
       timeend(1)=time
       timeend(2)=ttime+time
+!
+      heatfac=0.d0
 !
 !     number of nodes in the advective face
 !
@@ -197,12 +198,13 @@ c            jltyp=jltyp+10
      &           iinc,timeend,nelem,i,coords,jltyp,field,nfield,
      &           sideloadl,node,areaj,vold,mi,
      &           ipkon,kon,lakon,iponoel,inoel,ielprop,prop,ielmat,
-     &           shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon)
-            if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
-     &           (xload(1,id)-xloadold(1,id))*reltime
+     &           shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon,
+     &           ipobody,xbody,ibody,heatnod,heatfac)
+c            if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
+c     &           (xload(1,id)-xloadold(1,id))*reltime
          endif
 !
-         tnlref=xload(1,id)*(sinktemp-temp)*areaj
+         tnlref=(xload(1,id)*(sinktemp-temp)+heatfac)*areaj
          do j=1,nopes
             tnl(j)=tnl(j)-tnlref*shp2(4,j)
          enddo
