@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -51,11 +51,9 @@
       real*8 elas(21),c(3,3,3,3),rho,xi(-1:1,-1:1),et(-1:1,-1:1),
      &       wavspd,d1,distmin,a
 !
-      intent(in) rho,iorth
 !
-      intent(inout) elas,wavspd
 !     
-      write(*,*)'++cMT: calculating max. speed in ANISOTROPIC...'
+c      write(*,*)'++cMT: calculating max. speed in ANISOTROPIC...'
 !     
 !--------IF IORTHTROPIC-----------------------------     
       if(iorth.eq.1)then
