@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,11 +20,13 @@
      &  ialset,nset,ielmat,matname,nmat,ielorien,orname,norien,
      &  thicke,ipkon,iponor,xnor,ixfree,
      &  offset,lakon,irstrt,istep,istat,n,iline,ipol,inl,ipoinp,inp,
-     &  ipoinpc,mi,nelcon)
+     &  ipoinpc,mi,nelcon,ier)
 !
 !     reading the input deck: *BEAM SECTION
 !
       implicit none
+!
+      logical nodalthickness
 !
       character*1 inpc(*)
       character*4 section
@@ -34,23 +36,24 @@
       character*132 textpart(16)
 !
       integer istartset(*),iendset(*),ialset(*),mi(*),ielmat(mi(3),*),
-     &  ipoinpc(0:*),
+     &  ipoinpc(0:*),numnod,
      &  ielorien(mi(3),*),ipkon(*),iline,ipol,inl,ipoinp(2,*),
      &  inp(3,*),nset,nmat,norien,istep,istat,n,key,i,j,k,l,imaterial,
      &  iorientation,ipos,m,iponor(2,*),ixfree,
-     &  indexx,indexe,irstrt,nelcon(2,*)
+     &  indexx,indexe,irstrt(*),nelcon(2,*),ier
 !
       real*8 thicke(mi(3),*),thickness1,thickness2,p(3),xnor(*),
-     &  offset(2,*),
-     &  offset1,offset2,dd
+     &  offset(2,*),offset1,offset2,dd
 !
-      if((istep.gt.0).and.(irstrt.ge.0)) then
+      if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) 
      &       '*ERROR reading *BEAM SECTION: *BEAM SECTION should'
          write(*,*) '  be placed before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
+      nodalthickness=.false.
       offset1=0.d0
       offset2=0.d0
       orientation='                    
@@ -65,9 +68,11 @@
             orientation=textpart(i)(13:92)
          elseif(textpart(i)(1:6).eq.'ELSET=') then
             elset=textpart(i)(7:86)
-            elset(21:21)=' '
+            elset(81:81)=' '
             ipos=index(elset,' ')
             elset(ipos:ipos)='E'
+         elseif(textpart(i)(1:14).eq.'NODALTHICKNESS') then
+            nodalthickness=.true.
          elseif(textpart(i)(1:8).eq.'SECTION=') then
             if(textpart(i)(9:12).eq.'CIRC') then
                section='CIRC'
@@ -76,16 +81,23 @@
             else
                write(*,*) 
      &           '*ERROR reading *BEAM SECTION: unknown section'
-               call exit(201)
+               ier=1
+               return
             endif
          elseif(textpart(i)(1:8).eq.'OFFSET1=') then
             read(textpart(i)(9:28),'(f20.0)',iostat=istat) offset1
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*BEAM SECTION%",ier)
+               return
+            endif
          elseif(textpart(i)(1:8).eq.'OFFSET2=') then
             read(textpart(i)(9:28),'(f20.0)',iostat=istat) offset2
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*BEAM SECTION%",ier)
+               return
+            endif
          else
             write(*,*) 
      &       '*WARNING reading *BEAM SECTION: parameter not recognized:'
@@ -100,7 +112,8 @@
 !
       if(section.eq.'    ') then
          write(*,*) '*ERROR reading *BEAM SECTION: no section defined'
-         call exit(201)
+         ier=1
+         return
       endif
 !
 !     check for the existence of the set,the material and orientation
@@ -112,20 +125,20 @@
          write(*,*) '*ERROR reading *BEAM SECTION: nonexistent material'
          write(*,*) '  '
          call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
-         call exit(201)
+     &        "*BEAM SECTION%",ier)
+         return
       endif
       imaterial=i
 !
       if(orientation.eq.'                    
      &                                 ') then
          iorientation=0
-      elseif(nelcon(1,i).eq.2) then
-         write(*,*) '*INFO reading *SOLID SECTION: an orientation'
-         write(*,*) '      is for isotropic materials irrelevant'
-         call inputinfo(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         iorientation=0
+c      elseif(nelcon(1,i).eq.2) then
+c         write(*,*) '*INFO reading *BEAM SECTION: an orientation'
+c         write(*,*) '      is for isotropic materials irrelevant'
+c         call inputinfo(inpc,ipoinpc,iline,
+c     &"*BEAM SECTION%")
+c         iorientation=0
       else
          do i=1,norien
             if(orname(i).eq.orientation) exit
@@ -135,8 +148,8 @@
      &         '*ERROR reading *BEAM SECTION: nonexistent orientation'
             write(*,*) '  '
             call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
-            call exit(201)
+     &           "*BEAM SECTION%",ier)
+            return
          endif
          iorientation=i
       endif
@@ -150,8 +163,8 @@
      &      elset(1:ipos)
          write(*,*) '  has not yet been defined. '
          call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
-         call exit(201)
+     &        "*BEAM SECTION%",ier)
+         return
       endif
 !
 !     assigning the elements of the set the appropriate material,
@@ -165,7 +178,8 @@
                write(*,*) '       only be used for beam elements.'
                write(*,*) '       Element ',ialset(j),' is not a beam el
      &ement.'
-               call exit(201)
+               ier=1
+               return
             endif
             ielmat(1,ialset(j))=imaterial
             ielorien(1,ialset(j))=iorientation
@@ -187,7 +201,8 @@
                   write(*,*) '       only be used for beam elements.'
                   write(*,*) '       Element ',k,' is not a beam element
      &.'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                ielmat(1,k)=imaterial
                ielorien(1,k)=iorientation
@@ -207,29 +222,43 @@
 !
 !     assigning a thickness to the elements
 !
-      read(textpart(1)(1:20),'(f20.0)',iostat=istat) thickness1
-      if(istat.gt.0) then
-         write(*,*) 
-     &   '*ERROR reading *BEAM SECTION: first beam thickness is lacking'
-         call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
-      endif
-      if(n.gt.1) then
-         read(textpart(2)(1:20),'(f20.0)',iostat=istat) thickness2
+      if(.not.nodalthickness) then
+         read(textpart(1)(1:20),'(f20.0)',iostat=istat) thickness1
          if(istat.gt.0) then
             write(*,*) 
-     &        '*ERROR reading *BEAM SECTION: ',
-     &        'second beam thickness is lacking'
+     &   '*ERROR reading *BEAM SECTION: first beam thickness is lacking'
             call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+     &        "*BEAM SECTION%",ier)
+            return
+         endif
+         if(n.gt.1) then
+            read(textpart(2)(1:20),'(f20.0)',iostat=istat) thickness2
+            if(istat.gt.0) then
+               write(*,*) 
+     &              '*ERROR reading *BEAM SECTION: ',
+     &              'second beam thickness is lacking'
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*BEAM SECTION%",ier)
+               return
+            endif
+         else
+            thickness2=thickness1
          endif
       else
+!
+!        for those elements for which nodal thickness is activated
+!        the thickness is set to -1.d0
+!     
+         thickness1=-1.d0
          thickness2=thickness1
       endif
+!
       do j=istartset(i),iendset(i)
          if(ialset(j).gt.0) then
             indexe=ipkon(ialset(j))
-            do l=1,8
+            read(lakon(ialset(j))(3:3),'(i1)') numnod
+            numnod=numnod+1
+            do l=1,numnod
                thicke(1,indexe+l)=thickness1
                thicke(2,indexe+l)=thickness2
             enddo
@@ -239,7 +268,9 @@
                k=k-ialset(j)
                if(k.ge.ialset(j-1)) exit
                indexe=ipkon(k)
-               do l=1,8
+               read(lakon(k)(3:3),'(i1)') numnod
+               numnod=numnod+1
+               do l=1,numnod
                   thicke(1,indexe+l)=thickness1
                   thicke(2,indexe+l)=thickness2
                enddo
@@ -255,20 +286,30 @@
 !
       indexx=-1
       read(textpart(1)(1:20),'(f20.0)',iostat=istat) p(1)
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*BEAM SECTION%",ier)
+         return
+      endif
       read(textpart(2)(1:20),'(f20.0)',iostat=istat) p(2)
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*BEAM SECTION%",ier)
+         return
+      endif
       read(textpart(3)(1:20),'(f20.0)',iostat=istat) p(3)
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*BEAM SECTION%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*BEAM SECTION%",ier)
+         return
+      endif
       dd=dsqrt(p(1)*p(1)+p(2)*p(2)+p(3)*p(3))
       if(dd.lt.1.d-10) then
          write(*,*) 
      &       '*ERROR reading *BEAM SECTION: normal in direction 1'
          write(*,*) '       has zero size'
-         call exit(201)
+         ier=1
+         return
       endif
       do j=1,3
          p(j)=p(j)/dd
@@ -276,7 +317,9 @@
       do j=istartset(i),iendset(i)
          if(ialset(j).gt.0) then
             indexe=ipkon(ialset(j))
-            do l=1,8
+            read(lakon(ialset(j))(3:3),'(i1)') numnod
+            numnod=numnod+1
+            do l=1,numnod
                if(indexx.eq.-1) then
                   indexx=ixfree
                   do m=1,3
@@ -292,7 +335,9 @@
                k=k-ialset(j)
                if(k.ge.ialset(j-1)) exit
                indexe=ipkon(k)
-               do l=1,8
+               read(lakon(k)(3:3),'(i1)') numnod
+               numnod=numnod+1
+               do l=1,numnod
                if(indexx.eq.-1) then
                   indexx=ixfree
                   do m=1,3
