@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -26,7 +26,7 @@
 !
       implicit none
 !
-      logical fixed
+      logical fixed,rottracoupling
 !
       character*1 type,typeboun(*)
       character*20 labmpc(*),label
@@ -35,8 +35,8 @@
      &  iamboun(*),iamplitude,nam,ipompc(*),nodempc(3,*),nmpc,nmpc_,
      &  mpcfree,inotr(2,*),ntrans,ikboun(*),ilboun(*),ikmpc(*),
      &  ilmpc(*),itr,idof,newnode,number,id,idofnew,idnew,nk,nk_,
-     &  mpcfreenew,nmethod,iperturb,ii,nodetrue,mi(*),three,kflag,
-     &  iy(3),inumber
+     &  mpcfreenew,nmethod,iperturb(*),ii,nodetrue,mi(*),three,kflag,
+     &  iy(3),inumber,irotnode(11),irotdof(11)
 !
       real*8 xboun(*),val,coefmpc(*),trab(7,*),a(3,3),co(3,*),
      &  vold(0:mi(2),*),dx(3)
@@ -48,6 +48,36 @@
       else
          itr=inotr(1,node)
       endif
+!
+!     checking for boundary conditions on rotational dofs of
+!     distributing couplings 
+!
+      rottracoupling=.false.
+      if((ie.ge.4).and.(ie.le.6)) then
+!
+!        rotational dof
+!
+         do ii=is,ie
+            irotnode(ii)=node
+            if(ii.gt.3) then
+c               idof=8*(node-1)+ii+1
+               idof=8*(node-1)+ii
+               call nident(ikmpc,idof,nmpc,id)
+               if(id.gt.0) then
+                  if(ikmpc(id).eq.idof) then
+                     if(labmpc(ilmpc(id))(1:14).eq.'ROTTRACOUPLING')then
+                        rottracoupling=.true.
+                        irotnode(ii)=
+     &                     nodempc(1,nodempc(3,ipompc(ilmpc(id))))
+                        irotdof(ii)=
+     &                     nodempc(2,nodempc(3,ipompc(ilmpc(id))))
+                        itr=0
+                     endif
+                  endif
+               endif
+            endif
+         enddo
+      endif
 !
       loop: do ii=is,ie
 !
@@ -59,23 +89,34 @@
 !
 !        no transformation applies: simple SPC
 !
-            if(ii.le.3) then
-               i=ii
-            elseif(ii.eq.4) then
-               i=5
-            elseif(ii.eq.5) then
-               i=6
-            elseif(ii.eq.6) then
-               i=7
-            elseif(ii.eq.8) then
-               i=4
-            elseif(ii.eq.11) then
-               i=0
+            if(rottracoupling) then
+               node=irotnode(ii)
+               if(ii.gt.3) then
+                  i=irotdof(ii)
+               else
+                  i=ii
+               endif
             else
-               write(*,*) '*ERROR in bounadd: unknown DOF: ',
-     &              ii
-               call exit(201)
+c               if(ii.le.3) then
+               if(ii.le.6) then
+                  i=ii
+c               elseif(ii.eq.4) then
+c                  i=5
+c               elseif(ii.eq.5) then
+c                  i=6
+c               elseif(ii.eq.6) then
+c                  i=7
+               elseif(ii.eq.8) then
+                  i=4
+               elseif(ii.eq.11) then
+                  i=0
+               else
+                  write(*,*) '*ERROR in bounadd: unknown DOF: ',
+     &                 ii
+                  call exit(201)
+               endif
             endif
+!
             if((fixed).and.(i.lt.5)) then
                val=vold(i,nodetrue)
             elseif(fixed) then
@@ -88,11 +129,8 @@
             if(id.gt.0) then
                if(ikboun(id).eq.idof) then
                   j=ilboun(id)
-c
                   if(typeboun(j).ne.type) cycle loop
-c
                   xboun(j)=val
-c                  typeboun(j)=type
                   if(nam.gt.0) iamboun(j)=iamplitude
                   cycle loop
                endif
@@ -102,7 +140,7 @@ c                  typeboun(j)=type
                write(*,*) '*ERROR in bounadd: increase nboun_'
                call exit(201)
             endif
-            if((nmethod.eq.4).and.(iperturb.le.1)) then
+            if((nmethod.eq.4).and.(iperturb(1).le.1)) then
                write(*,*) '*ERROR in bounadd: in a modal dynamic step'
                write(*,*) '       new SPCs are not allowed'
                call exit(201)
@@ -121,22 +159,21 @@ c                  typeboun(j)=type
             enddo
             ikboun(id+1)=idof
             ilboun(id+1)=nboun
-c         enddo loop
-      else
+         else
 !
 !        transformation applies: SPC is MPC in global carthesian
 !        coordinates
 !
-         call transformatrix(trab(1,itr),co(1,node),a)
-c         do ii=is,ie
-            if(ii.le.3) then
+            call transformatrix(trab(1,itr),co(1,node),a)
+c            if(ii.le.3) then
+            if(ii.le.6) then
                i=ii
-            elseif(ii.eq.4) then
-               i=5
-            elseif(ii.eq.5) then
-               i=6
-            elseif(ii.eq.6) then
-               i=7
+c            elseif(ii.eq.4) then
+c               i=5
+c            elseif(ii.eq.5) then
+c               i=6
+c            elseif(ii.eq.6) then
+c               i=7
             elseif(ii.eq.8) then
                i=4
             elseif(ii.eq.11) then
@@ -173,7 +210,7 @@ c                     typeboun(j)=type
 !
 !              new node is generated for the inhomogeneous MPC term
 !
-               if((nmethod.eq.4).and.(iperturb.le.1)) then
+               if((nmethod.eq.4).and.(iperturb(1).le.1)) then
                   write(*,*)'*ERROR in bounadd: in a modal dynamic step'
                   write(*,*) '       new SPCs are not allowed'
                   call exit(201)
@@ -260,7 +297,7 @@ c               if(dabs(a(number,i)).lt.1.d-5) cycle
                coefmpc(mpcfree)=a(number,i)
                mpcfree=nodempc(3,mpcfree)
                if(mpcfree.eq.0) then
-                  write(*,*) '*ERROR in bounadd: increase nmpc_'
+                  write(*,*) '*ERROR in bounadd: increase memmpc_'
                   call exit(201)
                endif
             enddo
