@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2007 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
       real*8 function calc_residual_tee(pt1,Tt1,xflow1,xflow2,
-     &pt2,Tt2,A1,A2,zeta_fac,kappa,R,ider,iflag)
+     &pt2,Tt2,A1,A2,zeta_fac,kappa,R,ider,iflag,zeta)
 !
 !     Function for calculating the residual of both branches of a tee
 !
@@ -128,8 +128,18 @@
      &      dsqrt(Tt1),kappa,R) 
 !
 !     Velocity ratio
-      w2w1=w2/w1
-      w1w2=w1/w2
+c      w2w1=w2/w1
+c      w1w2=w1/w2
+      if(w2.eq.0.d0)then
+         w1w2=1d30
+      else
+         w1w2=w1/w2
+      endif
+      if(w1.eq.0.d0)then
+         w2w1=1d30
+      else
+         w2w1=w2/w1
+      endif
 !
 !     Zeta calculation
       zeta=1.d0+0.3d0*W2W1**2
@@ -162,17 +172,17 @@
          pspt2 = (Ts2/Tt2)**(kappa/(kappa-1))
          call machpi(M2,pspt2,kappa,R)
       
-         write(1,80)'Inlet: Tt1= ',Tt1,
-     &              ', pt1= ',pt1,', M1= ',M1
-     
-         write(1,77)'mass flow = ',xflow2,', kappa = ',kappa,
-     &              ', zeta= ',zeta
-
-         write(1,80)'Outlet: Tt2= ',Tt2,
-     &              ', pt2= ',pt2,', M2= ',M2
-     
- 80   format(3x,a,f10.6,a,f10.2,a,f10.6)
- 77   format(3x,a,f10.6,a,f10.2,a,f10.6)
+!         write(1,80)'Inlet: Tt1= ',Tt1,
+!     &              ', pt1= ',pt1,', M1= ',M1
+!     
+!         write(1,77)'mass flow = ',xflow2,', kappa = ',kappa,
+!     &              ', zeta= ',zeta
+!
+!         write(1,80)'Outlet: Tt2= ',Tt2,
+!     &              ', pt2= ',pt2,', M2= ',M2
+!     
+! 80   format(3x,a,f10.6,a,f10.2,a,f10.6)
+! 77   format(3x,a,f10.6,a,f10.2,a,f10.6)
       endif
 !
       calc_residual_tee=f
