@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
       subroutine calcenergy(ipkon,lakon,kon,co,ener,mi,ne,
-     &     thicke,ielmat,energyini,energy,ielprop,prop)
+     &     thicke,ielmat,energy,ielprop,prop,nea,neb)
 !
 !     calculates the energy in a *DYNAMIC calculation
 !
@@ -27,13 +27,15 @@
 !
       integer ipkon(*),nelem,kon(*),mi(*),nope,indexe,i,j,k,
      &  konl(20),mint3d,jj,iflag,ne,ki,kl,ilayer,nlayer,kk,
-     &  nopes,ielmat(mi(3),*),mint2d,null,ielprop(*)
+     &  nopes,ielmat(mi(3),*),mint2d,null,ielprop(*),nea,neb
 !
       real*8 ener(mi(1),*),enerinttot,enerint,co(3,*),prop(*),
      &  xl(3,20),xi,et,ze,xsj,shp(4,20),weight,enerkintot,enerkin,
-     &  a,gs(8,4),dlayer(4),tlayer(4),thickness,energyini(*),
+     &  a,gs(8,4),dlayer(4),tlayer(4),thickness,
      &  thicke(mi(3),*),xlayer(mi(3),4),shp2(7,8),xs2(3,7),xsj2(3),
      &  xl2(3,8),energy(*),enerelctot,enervictot,enerelc,enervic
+!
+!
 !
       include "gauss.f"
 !
@@ -43,9 +45,9 @@
       enerinttot=0.d0
       enerkintot=0.d0
       enerelctot=0.d0
-      enervictot=energyini(4)
+      enervictot=0.d0
 !
-      do nelem=1,ne
+      do nelem=nea,neb
          if(ipkon(nelem).lt.0) cycle
 !
          lakonl=lakon(nelem)
@@ -81,34 +83,66 @@
                   nlayer=nlayer+1
                endif
             enddo
-            mint2d=4
-            nopes=8
+!
+            if(lakonl(4:4).eq.'2') then
+               mint2d=4
+               nopes=8
 !     
 !     determining the layer thickness and global thickness
 !     at the shell integration points
 !     
-            iflag=1
-            indexe=ipkon(nelem)
-            do kk=1,mint2d
-               xi=gauss3d2(1,kk)
-               et=gauss3d2(2,kk)
-               call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-               tlayer(kk)=0.d0
-               do i=1,nlayer
-                  thickness=0.d0
-                  do j=1,nopes
-                     thickness=thickness+thicke(i,indexe+j)*shp2(4,j)
+               iflag=1
+               indexe=ipkon(nelem)
+               do kk=1,mint2d
+                  xi=gauss3d2(1,kk)
+                  et=gauss3d2(2,kk)
+                  call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                  tlayer(kk)=0.d0
+                  do i=1,nlayer
+                     thickness=0.d0
+                     do j=1,nopes
+                        thickness=thickness+thicke(i,indexe+j)*shp2(4,j)
+                     enddo
+                     tlayer(kk)=tlayer(kk)+thickness
+                     xlayer(i,kk)=thickness
                   enddo
-                  tlayer(kk)=tlayer(kk)+thickness
-                  xlayer(i,kk)=thickness
                enddo
-            enddo
-            iflag=2
+               iflag=2
 !     
-            ilayer=0
-            do i=1,4
-               dlayer(i)=0.d0
-            enddo
+               ilayer=0
+               do i=1,4
+                  dlayer(i)=0.d0
+               enddo
+            elseif(lakonl(4:5).eq.'15') then
+               mint2d=3
+               nopes=6
+!     
+!     determining the layer thickness and global thickness
+!     at the shell integration points
+!     
+               iflag=1
+               indexe=ipkon(nelem)
+               do kk=1,mint2d
+                  xi=gauss3d10(1,kk)
+                  et=gauss3d10(2,kk)
+                  call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                  tlayer(kk)=0.d0
+                  do i=1,nlayer
+                     thickness=0.d0
+                     do j=1,nopes
+                        thickness=thickness+thicke(i,indexe+j)*shp2(4,j)
+                     enddo
+                     tlayer(kk)=tlayer(kk)+thickness
+                     xlayer(i,kk)=thickness
+                  enddo
+               enddo
+               iflag=2
+!     
+               ilayer=0
+               do i=1,3
+                  dlayer(i)=0.d0
+               enddo
+            endif
 !     
          endif
 !     
@@ -147,7 +181,11 @@
          elseif(lakonl(4:4).eq.'4') then
             mint3d=1
          elseif(lakonl(4:5).eq.'15') then
-            mint3d=9
+            if(lakonl(7:8).eq.'LC') then
+               mint3d=6*nlayer
+            else
+               mint3d=9
+            endif
          elseif(lakonl(4:5).eq.'6') then
             mint3d=2
          elseif(lakonl(1:2).eq.'ES') then
@@ -170,10 +208,10 @@
                weight=weight3d1(jj)
             elseif(lakonl(4:7).eq.'20RB') then
                if((lakonl(8:8).eq.'R').or.(lakonl(8:8).eq.'C')) then
-                  xi=gauss3d13(1,kk)
-                  et=gauss3d13(2,kk)
-                  ze=gauss3d13(3,kk)
-                  weight=weight3d13(kk)
+                  xi=gauss3d13(1,jj)
+                  et=gauss3d13(2,jj)
+                  ze=gauss3d13(3,jj)
+                  weight=weight3d13(jj)
                else
                   call beamintscheme(lakonl,mint3d,ielprop(nelem),prop,
      &                 kk,xi,et,ze,weight)
@@ -226,10 +264,35 @@
                ze=gauss3d4(3,jj)
                weight=weight3d4(jj)
             elseif(lakonl(4:5).eq.'15') then
-               xi=gauss3d8(1,jj)
-               et=gauss3d8(2,jj)
-               ze=gauss3d8(3,jj)
-               weight=weight3d8(jj)
+               if(lakonl(7:8).ne.'LC') then
+                  xi=gauss3d8(1,jj)
+                  et=gauss3d8(2,jj)
+                  ze=gauss3d8(3,jj)
+                  weight=weight3d8(jj)
+               else
+                  kl=mod(jj,6)
+                  if(kl.eq.0) kl=6
+!     
+                  xi=gauss3d10(1,kl)
+                  et=gauss3d10(2,kl)
+                  ze=gauss3d10(3,kl)
+                  weight=weight3d10(kl)
+!     
+                  ki=mod(jj,3)
+                  if(ki.eq.0) ki=3
+!     
+                  if(kl.eq.1) then
+                     ilayer=ilayer+1
+                     if(ilayer.gt.1) then
+                        do i=1,3
+                           dlayer(i)=dlayer(i)+xlayer(ilayer-1,i)
+                        enddo
+                     endif
+                  endif
+                  ze=2.d0*(dlayer(ki)+(ze+1.d0)/2.d0*xlayer(ilayer,ki))/
+     &                 tlayer(ki)-1.d0
+                  weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+               endif
             else
                xi=gauss3d7(1,jj)
                et=gauss3d7(2,jj)
