@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -26,13 +26,12 @@
       integer nef,i,ncocon(2,*),imat,ntmat_,mi(*),
      &   ielmat(mi(3),*)
 !
-      real*8 t1l,vel(nef,0:5),cond,cocon(0:6,ntmat_,*),hcel(*)
+      real*8 t1l,vel(nef,0:7),cocon(0:6,ntmat_,*),hcel(*)
 !     
       do i=1,nef
          t1l=vel(i,0)
          imat=ielmat(1,i)
-         call materialdata_cond(imat,ntmat_,t1l,cocon,ncocon,cond)
-         hcel(i)=cond
+         call materialdata_cond(imat,ntmat_,t1l,cocon,ncocon,hcel(i))
       enddo
 !            
       return
