@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine calcinitialflux(area,vfa,xxn,
+      subroutine calcinitialflux(area,vfa,xxna,
      &  ipnei,nef,neifa,lakonf,flux)
 !
 !     correction of v due to the balance of mass
@@ -28,26 +28,16 @@
 !
       integer i,j,indexf,ipnei(*),ifa,nef,neifa(*),numfaces
 !
-      real*8 area(*),vfa(0:5,*),xxn(3,*),flux(*)
+      real*8 area(*),vfa(0:7,*),xxna(3,*),flux(*)
 !
       do i=1,nef
-         indexf=ipnei(i)
-         if(lakonf(i)(4:4).eq.'8') then
-            numfaces=6
-         elseif(lakonf(i)(4:4).eq.'6') then
-            numfaces=5
-         else
-            numfaces=4
-         endif
-         do j=1,numfaces
-            indexf=indexf+1
+         do indexf=ipnei(i)+1,ipnei(i+1)
             ifa=neifa(indexf)
-            flux(indexf)=area(ifa)*vfa(5,ifa)*
-     &               (vfa(1,ifa)*xxn(1,indexf)+
-     &                vfa(2,ifa)*xxn(2,indexf)+
-     &                vfa(3,ifa)*xxn(3,indexf))
+            flux(indexf)=vfa(5,ifa)*
+     &               (vfa(1,ifa)*xxna(1,indexf)+
+     &                vfa(2,ifa)*xxna(2,indexf)+
+     &                vfa(3,ifa)*xxna(3,indexf))
          enddo
-c         write(*,*) 'correctvfa mass check ',i,totflux
       enddo
 !  
       return
