@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -24,20 +24,17 @@
 !
       implicit none
 !
-      integer nef,i,nrhcon(*),imat,ithermal,ntmat_,mi(*),
+      integer nef,i,nrhcon(*),imat,ithermal(*),ntmat_,mi(*),
      &  ielmat(mi(3),*)
 !
-      real*8 t1l,vel(nef,0:5),rho,rhcon(0:1,ntmat_,*)
+      real*8 t1l,vel(nef,0:7),rhcon(0:1,ntmat_,*)
 !     
       do i=1,nef
          t1l=vel(i,0)
          imat=ielmat(1,i)
-         call materialdata_rho(rhcon,nrhcon,imat,rho,t1l,ntmat_,
+         call materialdata_rho(rhcon,nrhcon,imat,vel(i,5),t1l,ntmat_,
      &            ithermal)
-         vel(i,5)=rho
-c         write(*,*) 'calcrhoel rho',i,rho
       enddo
-c      write(*,*)
 !            
       return
       end
