@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,30 +16,27 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine calcrhoelcomp(nef,vel,shcon,ielmat,ntmat_,
-     &  mi)
+      subroutine calcrhoelcomp(nef,vel,shcon,ielmatf,ntmat_,
+     &  mi,nefa,nefb)
 !
 !     calculation of rho in the element centers (compressible
 !     fluids)
 !
       implicit none
 !
-      integer nef,i,imat,ntmat_,mi(*),
-     &  ielmat(mi(3),*)
+      integer nef,i,imat,ntmat_,mi(*),ielmatf(mi(3),*),nefa,nefb
+!
+      real*8 t1l,vel(nef,0:7),shcon(0:3,ntmat_,*)
+!
 !
-      real*8 t1l,vel(nef,0:5),shcon(0:3,ntmat_,*)
 !     
-c$omp parallel default(none)
-c$omp& shared(nef,vel,ielmat,shcon)
-c$omp& private(i,t1l,imat)
-c$omp do
-      do i=1,nef
+      do i=nefa,nefb
          t1l=vel(i,0)
-         imat=ielmat(1,i)
+         imat=ielmatf(1,i)
          vel(i,5)=vel(i,4)/(shcon(3,1,imat)*t1l)
+c         write(*,*) 'calcrhoelcomp ',i,t1l,vel(i,4),vel(i,5),
+c     &         shcon(3,1,imat)
       enddo
-c$omp end do
-c$omp end parallel
 !            
       return
       end
