@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -23,15 +23,11 @@
 !
       implicit none
 !
-      integer nef,i,nshcon(*),imat,ithermal,ntmat_,mi(*),
+      integer nef,i,nshcon(*),imat,ithermal(*),ntmat_,mi(*),
      &  ielmat(mi(3),*)
 !
-      real*8 t1l,vel(nef,0:5),dvi,shcon(0:3,ntmat_,*),umel(*)
+      real*8 t1l,vel(nef,0:7),dvi,shcon(0:3,ntmat_,*),umel(*)
 !     
-c$omp parallel default(none)
-c$omp& shared(nef,vel,ielmat,shcon,nshcon,ntmat_,ithermal,umel)
-c$omp& private(i,t1l,imat,dvi)
-c$omp do
       do i=1,nef
          t1l=vel(i,0)
          imat=ielmat(1,i)
@@ -39,8 +35,6 @@ c$omp do
      &            ithermal)
          umel(i)=dvi
       enddo
-c$omp end do
-c$omp end parallel
 !            
       return
       end
