@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -33,7 +33,7 @@
 
 void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
    ITG *mpcfree, ITG *nodeboun, ITG *ndirboun, ITG*nboun, ITG*ikmpc,
-   ITG *ilmpc, ITG *ikboun, ITG *ilboun, ITG *mpcend, ITG *mpcmult,
+   ITG *ilmpc, ITG *ikboun, ITG *ilboun, ITG *mpcend,
    char *labmpc, ITG *nk, ITG *memmpc_, ITG *icascade, ITG *maxlenmpc,
    ITG *callfrommain, ITG *iperturb, ITG *ithermal){
 
@@ -92,7 +92,7 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     nodempc=*nodempcp;
     coefmpc=*coefmpcp;
     
-    /*    for(i=0;i<*nmpc;i++){
+    /*      for(i=0;i<*nmpc;i++){
 	j=i+1;
 	FORTRAN(writempc,(ipompc,nodempc,coefmpc,labmpc,&j));
 	}*/
@@ -110,10 +110,10 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	if(id>0){
 	    if(ikboun[id-1]==ikmpc[i]){
 		if(strcmp1(&labmpc[20*i],"FLUID")!=0){
-		    printf("*ERROR in cascade: the DOF corresponding to \n node %" ITGFORMAT " in direction %" ITGFORMAT " is detected on the \n dependent side of a MPC and a SPC\n",
+		    printf("*ERROR in cascade: the DOF corresponding to \n node %" ITGFORMAT " in direction %" ITGFORMAT " is detected on the \n dependent side of a MPC and a SPC\n\n",
 			   (ikmpc[i])/8+1,ikmpc[i]-8*((ikmpc[i])/8));
 		}else{
-		    printf("*ERROR in cascade: the DOF corresponding to \n face %" ITGFORMAT " in direction %" ITGFORMAT " is detected on the \n dependent side of a MPC and a SPC\n",
+		    printf("*ERROR in cascade: the DOF corresponding to \n face %" ITGFORMAT " in direction %" ITGFORMAT " is detected on the \n dependent side of a MPC and a SPC\n\n",
 			   (-ikmpc[i])/8+1,-ikmpc[i]-8*((-ikmpc[i])/8));
 		}
 		FORTRAN(stop,());
@@ -138,7 +138,7 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	   (strcmp1(&labmpc[20*i],"SUBCYCLIC")==0)||
 	   (strcmp1(&labmpc[20*i],"PRETENSION")==0)||
 	   (strcmp1(&labmpc[20*i],"THERMALPRET")==0)||
-	   (strcmp1(&labmpc[20*i],"CONTACT")==0)||
+//	   (strcmp1(&labmpc[20*i],"CONTACT")==0)||
 	   (strcmp1(&labmpc[20*i],"FLUID")==0)||
            (*iperturb<2)) jmpc[i]=0;
 
@@ -147,6 +147,7 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	else if((strcmp1(&labmpc[20*i],"RIGID")==0) ||
 	   (strcmp1(&labmpc[20*i],"KNOT")==0) ||
 	   (strcmp1(&labmpc[20*i],"PLANE")==0) ||
+	   (strcmp1(&labmpc[20*i],"BEAM")==0) ||
 	   (strcmp1(&labmpc[20*i],"STRAIGHT")==0)) jmpc[i]=1;
 
         /* user mpc */
@@ -255,11 +256,11 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 		    /* check for zero coefficients on the dependent side */
 		    
 		    	    index1=ipompc[i];
-			if(fabs(coefmpc[index1-1])<1.e-10){
+			    if(fabs(coefmpc[index1-1])<1.e-10){
 				printf("*ERROR in cascade: zero coefficient on the\n");
 				printf("       dependent side of an equation\n");
 				printf("       dependent node: %" ITGFORMAT "",nodempc[3*index1-3]);
-				printf("       direction: %" ITGFORMAT "",nodempc[3*index1-2]);
+				printf("       direction: %" ITGFORMAT "\n\n",nodempc[3*index1-2]);
 				FORTRAN(stop,());
 			    }
 		    
@@ -346,12 +347,15 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	    index1=ipompc[i];
 	    index1old=0;
 	    do {
+//		printf("index1=%d\n",index1);
+//		printf("coefmpc=%e\n",coefmpc[index1-1]);
+
 		if(fabs(coefmpc[index1-1])<1.e-10){
 		    if(index1old==0){
 			printf("*ERROR in cascade: zero coefficient on the\n");
 			printf("       dependent side of an equation\n");
 			printf("       dependent node: %" ITGFORMAT "",nodempc[3*index1-3]);
-			printf("       direction: %" ITGFORMAT "",nodempc[3*index1-2]);
+			printf("       direction: %" ITGFORMAT "\n\n",nodempc[3*index1-2]);
 			FORTRAN(stop,());
 		    }
 		    else{
@@ -436,7 +440,7 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 		    if((irownl==1)||(icolnl==1)){
 			*icascade=2;
 			InpMtx_free(mtxA);
-			printf("*ERROR in cascade: linear and nonlinear MPCs depend on each other");
+			printf("*ERROR in cascade: linear and nonlinear MPCs depend on each other\n\n");
 			FORTRAN(stop,());
 		    }
 		}
@@ -710,7 +714,6 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
        the reallocation*/
 
     *mpcend=0;
-    *mpcmult=0;
     *maxlenmpc=0;
     for(i=0;i<*nmpc;i++){
 	index=ipompc[i];
@@ -719,7 +722,6 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	while(1){
 	    index=nodempc[3*index-1];
 	    if(index==0){
-		*mpcmult+=nterm*(nterm-1);
 		*maxlenmpc=max(*maxlenmpc,nterm);
 		break;
 	    }
@@ -733,7 +735,7 @@ void cascade(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     *nodempcp=nodempc;
     *coefmpcp=coefmpc;
     
-/*           for(i=0;i<*nmpc;i++){
+    /*          for(i=0;i<*nmpc;i++){
 	j=i+1;
 	FORTRAN(writempc,(ipompc,nodempc,coefmpc,labmpc,&j));
 	}*/
