@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -54,20 +54,18 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	  char *jobnamec,ITG *mortar,ITG *nmat,ITG *ielprop,double *prop,
 	  ITG *ialeatoric,ITG *kscale,
 	  double *energy, double *allwk, double *energyref, double *emax,
-	  double *enres, double *enetoll, double *energyini, double *allwkini,
+	  double *r_abs, double *enetoll, double *energyini, double *allwkini,
 	  double *temax, double *sizemaxinc, ITG* ne0, ITG* neini,
 	  double *dampwk, double *dampwkini, double *energystartstep) {
 
     ITG i0,ir,ip,ic,il,ig,ia,iest,iest1=0,iest2=0,iconvergence,idivergence,
 	ngraph=1,k,*ipneigh=NULL,*neigh=NULL,*inum=NULL,id,istart,iend,inew,
-        i,j,mt=mi[1]+1,iexceed,
-        iforceincsize=0;
+        i,j,mt=mi[1]+1,iexceed,iforceincsize=0,kscalemax,itf2f;
 
     double df,dc,db,dd,ran,can,rap,ea,cae,ral,da,*vr=NULL,*vi=NULL,*stnr=NULL,
 	*stni=NULL,*vmax=NULL,*stnmax=NULL,*cs=NULL,c1[2],c2[2],reftime,
         *fn=NULL,*eenmax=NULL,*fnr=NULL,*fni=NULL,*qfx=NULL,*cdn=NULL,
-        *cdnr=NULL,*cdni=NULL,
-        tmp, maxdecay=0.0, r_hat;
+        *cdnr=NULL,*cdni=NULL,tmp, maxdecay=0.0, r_rel,cetol;
 
     /* reset ialeatoric to zero */
 
@@ -84,8 +82,8 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
     i0=ctrl[0];ir=ctrl[1];ip=ctrl[2];ic=ctrl[3];il=ctrl[4];ig=ctrl[5];
     ia=ctrl[7];df=ctrl[10];dc=ctrl[11];db=ctrl[12];da=ctrl[13];dd=ctrl[16];
-    ran=ctrl[18];can=ctrl[19];rap=ctrl[22];
-    ea=ctrl[23];cae=ctrl[24];ral=ctrl[25];
+    ran=ctrl[18];can=ctrl[19];rap=ctrl[22];ea=ctrl[23];cae=ctrl[24];
+    ral=ctrl[25];cetol=ctrl[39];kscalemax=ctrl[55];itf2f=ctrl[56];
 
     /* for face-to-face penalty contact: increase the number of iterations
        in two subsequent increments in order to increase the increment size */
@@ -134,7 +132,15 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     /* mechanical */
 
     if(*ithermal<2){
-	if((*iit>1)&&(ram[0]<=c1[0]*qam[0])&&(*iflagact==0)&&
+//         number of iterations exceeding 1
+	if((*iit>1)&&
+//         force residual criterion satisfied (0.5 %)
+           (ram[0]<=c1[0]*qam[0])&&
+//         no significant change in contact elements
+           (*iflagact==0)&&
+//         cetol criterion satisfied if *visco
+           ((*nmethod!=-1)||(qa[3]<=cetol))&&
+//         solution change criterion satisfied (1 %)
 	   ((cam[0]<=c2[0]*uam[0])||
 	    (((ram[0]*cam[0]<c2[0]*uam[0]*ram2[0])||(ram[0]<=ral*qam[0])||
 	      (qa[0]<=ea*qam[0]))&&(*ntg==0))||
@@ -147,7 +153,9 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	if((ram[1]<=c1[1]*qam[1])&&
            (cam[2]<*deltmx)&&
 	   ((cam[1]<=c2[1]*uam[1])||
-	    (((ram[1]*cam[1]<c2[1]*uam[1]*ram2[1])||(ram[1]<=ral*qam[1])||
+//   change 25.11.2017
+//	    (((ram[1]*cam[1]<c2[1]*uam[1]*ram2[1])||(ram[1]<=ral*qam[1])||
+	    (((ram[1]*cam[1]<c2[1]*uam[1]*ram2[1])||((ram[1]<=ral*qam[1])&&(*iit>1))||
 	      (qa[1]<=ea*qam[1]))&&(*ntg==0))||
 	    (cam[1]<1.e-8)))iconvergence=1;
     }
@@ -156,6 +164,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
     if(*ithermal==3){
 	if(((*iit>1)&&(ram[0]<=c1[0]*qam[0])&&
+            ((*nmethod!=-1)||(qa[3]<=cetol))&&
 	    ((cam[0]<=c2[0]*uam[0])||
 	     (((ram[0]*cam[0]<c2[0]*uam[0]*ram2[0])||(ram[0]<=ral*qam[0])||
 	       (qa[0]<=ea*qam[0]))&&(*ntg==0))||
@@ -186,9 +195,9 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
          Main variables and meaning
          
-         r_hat     : modified energy conservation criterion
+         r_rel     : modified energy conservation criterion
          emax      : maximum value of the energy over the time history
-         enres     : energy residual (Eint + K + Econt - Wext - Wdamp -Eref)
+         r     : energy residual (Eint + K + Econt - Wext - Wdamp -Eref)
          enetoll   : energy conservation tolerance
          tmp       : auxiliary temporary variable
          maxdecay  : \hat(r)^{max}(\theta) -> value of the decay boundary 
@@ -205,13 +214,14 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	*emax=max(*emax,fabs(energy[1]));
 	*emax=max(*emax,fabs(*allwk));
 	
-	// energy residual (only calculated in the absence of contact 
+	// energy residual (only calculated in the absence of contact); 
+        // if <=0: loss of energy 
 
-	*enres=energy[0]+energy[1]+energy[2]+energy[3]-*energyref-*allwk-*dampwk;
+	*r_abs=energy[0]+energy[1]+energy[2]+energy[3]-*energyref-*allwk-*dampwk;
 	
 	// Absolute tolerance check (when the error is really small --> beginning of simulation)
 
-	if(fabs(*enres)>=*enetoll/4) {
+	if(fabs(*r_abs)>=*enetoll/4) {
 
 	    // Normal strategy: Relative error
 	    
@@ -221,14 +231,17 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    
 	    /* modified r_hat criterion */
 
-	    r_hat=*enres/(*emax);
-	    if(r_hat<=-maxdecay) {
-		idivergence=1;
+	    r_rel=*r_abs/(*emax);
+	    if(r_rel<=-maxdecay) {
+	      // deactivated on 22.03.2019
+	      // energy/work may not be calculated correctly for forced BC
+	      // (example: shaft)
+	      		idivergence=1;
 	    }else{
 		
 		/* Check if the residual is too close to the boundary */
 
-		if(r_hat*0.9<=-0.90*maxdecay) {
+		if(r_rel<=-0.9*maxdecay) {
 		    *istab=0; // keep the increment size
 		}
 	    }
@@ -242,7 +255,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     if((*nmethod==4)&&(*ithermal<2)&&(*uncoupled==0)&&(iconvergence==1)&&((*ne!=*ne0)||(*neini!=*ne0))){
 
 	/* store temporarly the value of emax: in case of forced divergence 
-	   emax has to be resetted. */
+	   emax has to be reset. */
 
 	tmp=*emax;
 	
@@ -259,7 +272,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
 	FORTRAN(checkimpacts,(ne,neini,temax,sizemaxinc,energyref,
 			      tmin,tper,&idivergence,
-			      &iforceincsize,istab,dtheta,enres,energy,energyini,
+			      &iforceincsize,istab,dtheta,r_abs,energy,energyini,
 			      allwk,allwkini,dampwk,dampwkini,emax,mortar,
 			      &maxdecay,enetoll));
 
@@ -274,8 +287,8 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	   The user is aware of it via the output string. */
 
 	if((*ne==*ne0)&&(*neini>*ne0)&&(idivergence==0)){
-	    *enres=fabs(energy[0]+energy[1]+energy[2]+energy[3]-*energyref-*allwk-*dampwk);
-	    tmp=1.3*(2.0* *enres/(*emax))/(1.0+sqrt(*theta));
+	    *r_abs=fabs(energy[0]+energy[1]+energy[2]+energy[3]-*energyref-*allwk-*dampwk);
+	    tmp=1.3*(2.0*(*r_abs)/(*emax))/(1.0+sqrt(*theta));
 	    *enetoll=max(*enetoll,tmp);
 	    printf("\n Adaption of the max-decay boundary, enetoll = %f \n",*enetoll);
 	}
@@ -287,14 +300,14 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
 	if((iconvergence==1)&&(*ne<=*neini)){
 	    tmp=energy[0]+energy[1]+energy[2]+energy[3]-*energyref-*allwk-*dampwk;
-	    if(tmp>*enres){
-		*enres=tmp;
+	    if(tmp>*r_abs){
+		*r_abs=tmp;
 		printf("\n Adaption of the energy residual in persistent contact, \n");
 		printf("   an increasing trend has been detected.\n");
 	    }
 	}
     } else {
-	sizemaxinc=tmax;
+	*sizemaxinc=*tmax;
     }
 //    MPADD end
 // # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -303,7 +316,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
     if((iconvergence==1)&&(idivergence==0)){
 
-	FORTRAN(writesummary,(istep,iinc,icutb,iit,ttime,time,dtime));
+	FORTRAN(writesta,(istep,iinc,icutb,iit,ttime,time,dtime));
 	if(*uncoupled){
 	    if(*ithermal==2){
 	        *iitterm=*iit;
@@ -369,7 +382,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                         ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
                         &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
                         ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,
-                        cdn,mortar,cdnr,cdni,nmat);
+                        cdn,mortar,cdnr,cdni,nmat,ielprop,prop);
 
 		    FORTRAN(stop,());
 		}
@@ -401,15 +414,24 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	/* check whether new increment size exceeds maximum increment
            size allowed (by the user) */
 
-//	if((*dtheta>*tmax)&&(*idrct==0)){
-//	    *dtheta=*tmax;
 	if((*dtheta>*sizemaxinc)&&(*idrct==0)){
 	    *dtheta=*sizemaxinc;
 	    *dthetaref=*dtheta;
-	    printf(" the increment size exceeds thetamax and is decreased to %e\n\n",*dtheta**tper);
+	    printf(" the increment size exceeds the maximum allowed and is decreased to %e\n\n",*dtheta**tper);
 	}
 
-        /* if itp=1 the increment just finished ends at a time point */
+        /* check whether new time point exceeds end of step */
+
+	if(*dtheta>=1.-*theta){
+	    if(*dtheta>1.-*theta){iexceed=1;}else{iexceed=0;}
+	    *dtheta=1.-*theta;
+	    *dthetaref=*dtheta;
+	    if(iexceed==1)
+		printf(" the increment size exceeds the remainder of the step and is decreased to %e\n\n",*dtheta**tper);
+	}
+
+        /* check whether the end of the new increment exceeds a time point;
+           if itp=1 the increment just finished ends at a time point */
 
 	if((*itpamp>0)&&(*idrct==0)){
 	    if(*itp==1){
@@ -431,6 +453,14 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		inew=id+1;
 	    }
 
+            /* if the end of the new increment is less than a time
+               point by less than 1.e-6 (theta-value) dtheta is
+               enlarged up to this time point */
+
+	    if((*inext==inew)&&(inew<=iend)){
+		if(amta[2*inew-2]-reftime<1.e-6**tper){inew++;}
+	    }
+
             /* inew: smallest time point exceeding time+dtheta*tper
                inext: smallest time point exceeding time */
 
@@ -445,16 +475,6 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		printf(" the increment size exceeds a time point and is decreased to %e\n\n",*dtheta**tper);
 	    }else{*itp=0;}
 	}
-
-        /* check whether new time point exceeds end of step */
-
-	if(*dtheta>=1.-*theta){
-	    if(*dtheta>1.-*theta){iexceed=1;}else{iexceed=0;}
-	    *dtheta=1.-*theta;
-	    *dthetaref=*dtheta;
-	    if(iexceed==1)
-		printf(" the increment size exceeds the remainder of the step and is decreased to %e\n\n",*dtheta**tper);
-	}
     }
     else{
 
@@ -462,11 +482,11 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
 	/* check for the amount of iterations */
 	
-      if(((*iit>ic)&&(*mortar==0))||((*mortar>1)&&(*iit>200))){
+	if(((*iit>ic)&&(*mortar==0))||((*mortar>1)&&(*iit>200))){
 	    printf("\n *ERROR: too many iterations needed\n");
 	    printf(" best solution and residuals are in the frd file\n\n");
 
-	    FORTRAN(writesummarydiv,(istep,iinc,icutb,iit,ttime,time,dtime));
+	    FORTRAN(writestadiv,(istep,iinc,icutb,iit,ttime,time,dtime));
 
 	    NNEW(fn,double,mt**nk);
 	    NNEW(inum,ITG,*nk);for(k=0;k<*nk;k++) inum[k]=1;
@@ -477,28 +497,27 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
 	    (*ttime)+=(*time);
 	    frd(co,nk,kon,ipkon,lakon,ne,vold,stn,inum,nmethod,
-			kode,filab,een,t1act,fn,ttime,epn,ielmat,matname,enern,
-                        xstaten,nstate_,istep,iinc,ithermal,qfn,mode,noddiam,
-                        trab,inotr,ntrans,orab,ielorien,norien,description,
-                        ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
-                        &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
-		        ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-                        mortar,cdnr,cdni,nmat);
+		kode,filab,een,t1act,fn,ttime,epn,ielmat,matname,enern,
+		xstaten,nstate_,istep,iinc,ithermal,qfn,mode,noddiam,
+		trab,inotr,ntrans,orab,ielorien,norien,description,
+		ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
+		&ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
+		ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
+		mortar,cdnr,cdni,nmat,ielprop,prop);
 
 	    FORTRAN(stop,());
 	}	
 	
 	/* check for diverging residuals */
-	
-//	if((*iit>=i0)||(fabs(ram[0])>1.e20)||(fabs(cam[0])>1.e20)||
-//	               (fabs(ram[1])>1.e20)||(fabs(cam[1])>1.e20)||
-//	               (cam[2]>*deltmx)||(qa[2]>0.)){
-//	if((*iit>=i0)||(fabs(ram[0])>1.e20)||(fabs(cam[0])>1.e20)||
-//	               (fabs(ram[1])>1.e20)||(fabs(cam[1])>1.e20)||
-//	               (cam[2]>*deltmx)||(idivergence==1)){
+
+        /* if the user has not defined deltmx on the *HEAT
+	 TRANSFER card it is set to a large value (1.e30,
+	 cf. CalculiX.c); therefore, a comparison of cam[2]
+	 with deltmx only makes sense for cam[2]<1.e30 */
+      
 	if((*iit>=i0)||(fabs(ram[0])>1.e20)||(fabs(cam[0])>1.e20)||
 	               (fabs(ram[1])>1.e20)||(fabs(cam[1])>1.e20)||
-	               (cam[2]>*deltmx)||(idivergence==1)||
+	   ((cam[2]<1.e30)&&(cam[2]>*deltmx))||(idivergence==1)||
 	               (iforceincsize==1)){
 	    if((*ithermal!=2)&&(*mortar!=1)){
 		if((ram1[0]>ram2[0])&&(ram[0]>ram2[0])&&(ram[0]>c1[0]*qam[0]))
@@ -526,7 +545,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    
                 /* rate of number of contact elements is increasing */
 
-		if(((ITG)ram[6]*(ITG)ram1[6]<0)&&((ITG)ram1[6]*(ITG)ram2[6]<0)){
+		if(((ITG)ram[5]*(ITG)ram1[5]<0)&&((ITG)ram1[5]*(ITG)ram2[5]<0)){
 		    
 		    if(((ram[4]>0.98*ram1[4])&&(ram[4]<1.02*ram1[4]))&&
 		       ((ram[4]>0.98*ram2[4])&&(ram[4]<1.02*ram2[4]))){
@@ -537,6 +556,11 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		}
 	    }
 
+            /* check whether in a viscous step the allowable increase in viscous
+               strain has been exceeded */
+
+	    if((idivergence==0)&&((*nmethod==-1)&&(qa[3]>cetol))) idivergence=2;
+
 
             /* for thermal calculations the maximum temperature change
                is checked as well */
@@ -544,8 +568,15 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
             if(*ithermal>1){
 	        if((ram1[1]>ram2[1])&&(ram[1]>ram2[1])&&(ram[1]>c1[1]*qam[1]))
 		    idivergence=1;
-		if(cam[2]>*deltmx) idivergence=2;
+
+		/* if the user has not defined deltmx on the *HEAT
+                   TRANSFER card it is set to a large value (1.e30,
+                   cf. CalculiX.c); therefore, a comparison of cam[2]
+                   with deltmx only makes sense for cam[2]<1.e30 */
+
+		if((cam[2]<1.e30)&&(cam[2]>*deltmx)) idivergence=2;
 	    }
+
 	    if(idivergence>0){
 		if(*idrct==1){
 		    if((*mortar<=1)||((*mortar>1)&&(*iit>200))) {
@@ -556,7 +587,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 			printf(" automatic incrementation; program stops\n");
 			printf(" best solution and residuals are in the frd file\n\n");
 			
-			FORTRAN(writesummarydiv,(istep,iinc,icutb,iit,ttime,time,
+			FORTRAN(writestadiv,(istep,iinc,icutb,iit,ttime,time,
 						 dtime));
 			
 			NNEW(fn,double,mt**nk);
@@ -574,7 +605,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 			    ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
 			    &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
 			    ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-			    mortar,cdnr,cdni,nmat);
+			    mortar,cdnr,cdni,nmat,ielprop,prop);
 			
 			FORTRAN(stop,());
 		    }
@@ -595,16 +626,19 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                                 *dtheta=*sizemaxinc;                    // MPADD
                               }                                         // MPADD
                             }                                           // MPADD
-//			    if((*mortar!=1)||(*icutb!=0)) *dtheta=*dtheta*df;
 			}else{
-			    *dtheta=*dtheta**deltmx/cam[2]*da;
+			    if(*nmethod==-1){
+				*dtheta=*dtheta*cetol/qa[3]*da;
+			    }else{
+				*dtheta=*dtheta**deltmx/cam[2]*da;
+			    }
 			}
 		    }
 		    *dthetaref=*dtheta;
 		    printf(" divergence; the increment size is decreased to %e\n",*dtheta**tper);
 		    printf(" the increment is reattempted\n\n");
 
-		    FORTRAN(writesummarydiv,(istep,iinc,icutb,iit,ttime,time,
+		    FORTRAN(writestadiv,(istep,iinc,icutb,iit,ttime,time,
 					     dtime));
 
 		    *istab=0;
@@ -633,15 +667,15 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                         ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
                         &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
                         ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-                        mortar,cdnr,cdni,nmat);
+			mortar,cdnr,cdni,nmat,ielprop,prop);
 
 			FORTRAN(stop,());
 		    }
 		    *icntrl=1;
 		    (*icutb)++;
 		    if(*mortar==1){
-			*kscale=100;
-			printf("\n reducing the constant stiffnesses by a factor of 100 \n\n");
+			*kscale=kscalemax;
+			printf("\n reducing the constant stiffnesses by a factor of %d \n\n",*kscale);
 		    }
 
                     /* check whether too many cutbacks */
@@ -664,7 +698,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                         ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
                         &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
                         ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-                        mortar,cdnr,cdni,nmat);
+			mortar,cdnr,cdni,nmat,ielprop,prop);
 
 			FORTRAN(stop,());
 		    }
@@ -687,7 +721,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
 	/* check for too slow convergence */
 	
-	if(*iit>=ir){
+	if((*iit>=ir)||((*mortar==1)&&(*iit==itf2f))){
 	    if(*ithermal!=2){
 		iest1=(ITG)ceil(*iit+log(ran*qam[0]/(ram[0]))/
 				log(ram[0]/(ram1[0])));
@@ -701,18 +735,27 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    printf(" estimated number of iterations till convergence = %" ITGFORMAT "\n",
 		   iest);
 	    }
-	    if((((iest>ic)||(*iit==ic))&&(*mortar!=1))||((*mortar==1)&&(*iit==60))){
+	    if((((iest>ic)||(*iit==ic))&&(*mortar!=1))||((*mortar==1)&&(*iit==itf2f))){
 		
 		if(*idrct!=1){
 		    if((*mortar!=1)||(*icutb!=0)) *dtheta=*dtheta*dc;
 		    *dthetaref=*dtheta;
-		    printf(" too slow convergence; the increment size is decreased to %e\n",*dtheta**tper);
-		    printf(" the increment is reattempted\n\n");
+		    if((*mortar==1)&&(*iit==itf2f)){
+			printf( "maximum number of iterations for face-to-face contact reached\n");
+			printf(" the increment size is decreased to %e\n",*dtheta**tper);
+			printf(" the increment is reattempted\n\n");
+		    }else{
+			printf(" too slow convergence; the increment size is decreased to %e\n",*dtheta**tper);
+			printf(" the increment is reattempted\n\n");
+		    }
 
-		    FORTRAN(writesummarydiv,(istep,iinc,icutb,iit,ttime,
+		    FORTRAN(writestadiv,(istep,iinc,icutb,iit,ttime,
 					     time,dtime));
-
 		    *istab=0;
+		    if(*itp==1){
+		      *itp=0;
+		      (*inext)--;
+		    }
 
                     /* check whether new increment size is smaller than minimum */
 
@@ -734,13 +777,18 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                         ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
                         &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
                         ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-                        mortar,cdnr,cdni,nmat);
+			mortar,cdnr,cdni,nmat,ielprop,prop);
 
 			FORTRAN(stop,());
 		    }
 		    *icntrl=1;
 		    (*icutb)++;
-		    if(*mortar==1) *kscale=100;
+		    if(*mortar==1){
+			*kscale=kscalemax;
+			printf("\n reducing the constant stiffnesses by a factor of %d \n\n",*kscale);
+		    }
+//		    if(*mortar==1) *kscale=100;
+
 		    if(*icutb>ia){
 			printf("\n *ERROR: too many cutbacks\n");
 			printf(" best solution and residuals are in the frd file\n\n");
@@ -759,7 +807,7 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
                         ipneigh,neigh,mi,sti,vr,vi,stnr,stni,vmax,stnmax,
                         &ngraph,veold,ener,ne,cs,set,nset,istartset,iendset,
                         ialset,eenmax,fnr,fni,emn,thicke,jobnamec,output,qfx,cdn,
-                        mortar,cdnr,cdni,nmat);
+			mortar,cdnr,cdni,nmat,ielprop,prop);
 
 			FORTRAN(stop,());
 		    }
@@ -769,6 +817,11 @@ void checkconvergence(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		      }
 		      *ithermal=3;
 		    }
+
+                    /* default value for qa[2] */
+
+		    qa[2]=-1.;
+
 		    *iflagact=0;
 		    return;
 		}
