@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
       character*87 filab(*)
 !
       integer imddof(*),nmddof,nrset,istartset(*),iendset(*),mi(*),
-     &  ialset(*),nactdof(0:mi(2),*),node,ithermal,j,k,l,
+     &  ialset(*),nactdof(0:mi(2),*),node,ithermal(*),j,k,l,
      &  ikmpc(*),ilmpc(*),ipompc(*),nodempc(3,*),nmpc,
      &  imdnode(*),nmdnode,imdmpc(*),nmdmpc,nprint,ipos,
      &  imdboun(*),nmdboun,ikboun(*),nboun,indexe1,indexe,islav,
@@ -79,7 +79,7 @@
      &             2,3,6,5,8,15,11,14,
      &             3,1,4,6,9,13,12,15/
 !
-      data nlabel /47/
+      data nlabel /48/
 !
 !     if 1d/2d elements are part of the mesh, no node selection
 !     is performed (because of the renumbering due to the
@@ -99,7 +99,8 @@
          endif
       enddo
 !
-!     storing the nodes for which *NODE FILE or *EL FILE was selected
+!     storing the nodes, dofs, spcs and mpcs for which *NODE FILE 
+!     or *EL FILE was selected
 !
       do i=1,nlabel
 !
@@ -152,7 +153,7 @@
                if(ialset(j).gt.0) then
                   node=ialset(j)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                   nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -172,7 +173,7 @@
                      node=node-ialset(j)
                      if(node.ge.ialset(j-1)) exit
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,
      &                      ipompc,nodempc,nmpc,imdnode,nmdnode,imddof,
@@ -193,7 +194,8 @@
          endif
       enddo
 !
-!     storing the nodes for which *NODE PRINT was selected
+!     storing the nodes, dofs, spcs and mpcs for which *NODE PRINT 
+!     was selected
 !
       do i=1,nprint
          if((prlab(i)(1:4).eq.'U   ').or.
@@ -219,7 +221,7 @@
                if(ialset(j).gt.0) then
                   node=ialset(j)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                   nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -239,7 +241,7 @@
                      node=node-ialset(j)
                      if(node.ge.ialset(j-1)) exit
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,
      &                      ipompc,nodempc,nmpc,imdnode,nmdnode,imddof,
@@ -259,7 +261,8 @@
          endif
       enddo
 !
-!     check whether all contact slave and master nodes were selected
+!     check whether all contact slave and master nodes (and corresponding
+!     dofs, spcs and mpcs were selected
 !
       do i=1,ntie
 !     
@@ -348,7 +351,7 @@
                do l=1,nnodelem
                   node=nodef(l)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -371,7 +374,15 @@
 !           a nodal slave surface
 !
             if(slavset(ipos:ipos).eq.'S') then
+!
+!              'S' means node-to-face contact, it does not mean
+!              that the slave surface is node-based;
+!              start with assuming a nodal surface (default),
+!              if not present, check for a facial surface
+!
                nodeslavsurf=.true.
+            else
+               nodeslavsurf=.false.
             endif
 !
 !           determining the slave surface 
@@ -399,7 +410,7 @@
                   if(ialset(j).gt.0) then
                      node=ialset(j)
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -414,7 +425,7 @@
                         if(k.ge.ialset(j-1)) exit
                         node=k
                         call addimd(imdnode,nmdnode,node)
-                        if(ithermal.ne.2) then
+                        if(ithermal(1).ne.2) then
                            do k=1,3
                               call addimdnodedof(node,k,ikmpc,ilmpc,
      &                          ipompc,nodempc,nmpc,imdnode,nmdnode,
@@ -493,7 +504,7 @@
                   do l=1,nnodelem
                      node=nodef(l)
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -508,18 +519,18 @@
          endif
       enddo
 !
-!     adding nodes belonging to nonlinear MPC's (why only dependent nodes?)
+!     adding nodes, dofs, spcs and mpcs belonging to nonlinear MPC's 
+!     (why only dependent nodes?)
 !      
       do i=1,nmpc
          if((labmpc(i)(1:20).ne.'                    ').and.
-     &          (labmpc(i)(1:7).ne.'CONTACT').and.
      &          (labmpc(i)(1:6).ne.'CYCLIC').and.
      &          (labmpc(i)(1:9).ne.'SUBCYCLIC')) then
             indexe1=ipompc(i)
             if(indexe1.eq.0) cycle
             node=nodempc(1,indexe1)
             call addimd(imdnode,nmdnode,node)
-            if(ithermal.ne.2) then
+            if(ithermal(1).ne.2) then
                do k=1,3
                   call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                 nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -535,14 +546,6 @@
             endif
          endif
       enddo
-!
-!     subtracting 1 to comply with the C-convention
-!
-c      do j=1,nmddof
-c         imddof(j)=imddof(j)-1
-c      enddo
-!
-c      write (*,*) 'nmddof, nmdnode',nmddof,nmdnode
 !
       return
       end
