@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
      &  ipoinp,inp,ntie,mcs,lprev,ithermal,rcscg,rcs0cg,zcscg,
      &  zcs0cg,nrcg,nzcg,jcs,kontri,straight,ne,ipkon,kon,
      &  lakon,lcs,ifacetet,inodface,ipoinpc,maxsectors,
-     &  trab,ntrans,ntrans_,jobnamec,vold,cfd,mi,iaxial)
+     &  trab,ntrans,ntrans_,jobnamec,vold,nef,mi,iaxial,ier)
 !
 !     reading the input deck: *CYCLIC SYMMETRY MODEL
 !
@@ -48,6 +48,10 @@
 !     cs(15,mcs): cos(angle); angle = 2*pi/cs(1,mcs)
 !     cs(16,mcs): sin(angle)
 !     cs(17,mcs): number of tie constraint
+!
+!     notice that in this routine ics, zcs and rcs start for 1 for
+!     each *cyclic symmetry model card (look at the pointer in
+!     the cyclicsymmetrymodels call in calinput.f)
 !
       implicit none
 !
@@ -65,16 +69,16 @@
      &  nset,istep,istat,n,key,i,j,k,nk,nmpc,nmpc_,mpcfree,ics(*),
      &  nr(*),nz(*),jdep,jindep,l,noded,ikmpc(*),ilmpc(*),lcs(*),
      &  kflag,node,ncsnodes,ncs_,iline,ipol,inl,ipoinp(2,*),nneigh,
-     &  inp(3,*),itie,iset,ipos,mcs,lprev,ntie,ithermal,ncounter,
+     &  inp(3,*),itie,iset,ipos,mcs,lprev,ntie,ithermal(*),ncounter,
      &  nrcg(*),nzcg(*),jcs(*),kontri(3,*),ne,ipkon(*),kon(*),nodei,
      &  ifacetet(*),inodface(*),ipoinpc(0:*),maxsectors,id,jfaces,
-     &  noden(2),ntrans,ntrans_,cfd,mi(*),ifaceq(8,6),ifacet(6,4),
-     &  ifacew1(4,5),ifacew2(8,5),idof
+     &  noden(2),ntrans,ntrans_,nef,mi(*),ifaceq(8,6),ifacet(6,4),
+     &  ifacew1(4,5),ifacew2(8,5),idof,ier
 !
       real*8 tolloc,co(3,*),coefmpc(*),rcs(*),zcs(*),rcs0(*),zcs0(*),
      &  csab(7),xn,yn,zn,dd,xap,yap,zap,tietol(3,*),cs(17,*),xsectors,
      &  gsectors,x3,y3,z3,phi,rcscg(*),rcs0cg(*),zcscg(*),zcs0cg(*),
-     &  straight(9,*),x1,y1,z1,x2,y2,z2,zp,rp,dist,trab(7,*),
+     &  straight(9,*),x1,y1,z1,x2,y2,z2,zp,rp,dist,trab(7,*),rpd,zpd,
      &  vold(0:mi(2),*),calculated_angle,user_angle
 !
 !     nodes per face for hex elements
@@ -113,7 +117,8 @@
          write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
          write(*,*) '       *CYCLIC SYMMETRY MODEL should'
          write(*,*) '       be placed before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       check=.true.
@@ -125,22 +130,34 @@
       do i=2,n
          if(textpart(i)(1:2).eq.'N=') then
             read(textpart(i)(3:22),'(f20.0)',iostat=istat) xsectors
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*CYCLIC SYMMETRY MODEL%",ier)
+               return
+            endif
          elseif(textpart(i)(1:8).eq.'CHECK=NO') then
             check=.false.
          elseif(textpart(i)(1:7).eq.'NGRAPH=') then
             read(textpart(i)(8:27),'(f20.0)',iostat=istat) gsectors
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*CYCLIC SYMMETRY MODEL%",ier)
+               return
+            endif
          elseif(textpart(i)(1:4).eq.'TIE=') then
             read(textpart(i)(5:84),'(a80)',iostat=istat) tie
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*CYCLIC SYMMETRY MODEL%",ier)
+               return
+            endif
          elseif(textpart(i)(1:6).eq.'ELSET=') then
             read(textpart(i)(7:86),'(a80)',iostat=istat) elset
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*CYCLIC SYMMETRY MODEL%",ier)
+               return
+            endif
             elset(81:81)=' '
             ipos=index(elset,' ')
             elset(ipos:ipos)='E'
@@ -154,36 +171,11 @@
      &"*CYCLIC SYMMETRY MODEL%")
          endif
       enddo
-!
-      if(xsectors.le.0) then
-         write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
-         write(*,*) '       the required parameter N'
-         write(*,*) '       is lacking on the *CYCLIC SYMMETRY MODEL'
-         write(*,*) '       keyword card or has a value <=0'
-         call exit(201)
-      endif
-      if(gsectors.lt.1) then
-         write(*,*) '*WARNING reading *CYCLIC SYMMETRY MODEL:'
-         write(*,*) '         cannot plot less than'
-         write(*,*) '         one sector: one sector will be plotted'
-         gsectors=1
-      endif
-      if(gsectors.gt.xsectors) then
-         write(*,*) '*WARNING reading *CYCLIC SYMMETRY MODEL:'
-         write(*,*) '         cannot plot more than'
-         write(*,*) '         ',xsectors,'sectors;',
-     &           xsectors,' sectors will'
-         write(*,*) '       be plotted'
-         gsectors=xsectors
-      endif
-!
-      maxsectors=max(maxsectors,int(xsectors+0.5d0))
-c      iaxial=max(iaxial,maxsectors)
 !
       mcs=mcs+1
-      cs(2,mcs)=-0.5
-      cs(3,mcs)=-0.5
-      cs(14,mcs)=lprev+0.5
+      cs(2,mcs)=-0.5d0
+      cs(3,mcs)=-0.5d0
+      cs(14,mcs)=lprev+0.5d0
 !
 !     determining the tie constraint
 !
@@ -206,7 +198,7 @@ c      iaxial=max(iaxial,maxsectors)
                read(textpart(j)(1:20),'(f20.0)',iostat=istat) 
      &              cs(5+j,mcs)
             enddo
-            cs(17,mcs)=itie+0.5
+            cs(17,mcs)=itie+0.5d0
 !
 !           counting the number of faces on the master side (should be the
 !           same as on the slave side)
@@ -219,25 +211,38 @@ c      iaxial=max(iaxial,maxsectors)
 !           max 8 nodes per face
 !
             cs(4,mcs)=(iendset(j)-istartset(j)+1)*8+0.5d0
+!     
+            call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &           ipoinp,inp,ipoinpc)
+!
             return
          elseif((tieset(1,i)(1:80).eq.tie).and.
      &      (tieset(1,i)(81:81).eq.'Z')) then
             itie=i
 !
 !           fluid periodicity (rotational)
+!
+            if(xsectors.le.0) then
+               write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
+               write(*,*) '       the required parameter N'
+               write(*,*) 
+     &          '       is lacking on the *CYCLIC SYMMETRY MODEL'
+               write(*,*) '       keyword card or has a value <=0'
+               ier=1
+               return
+            endif
 !
             call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &           ipoinp,inp,ipoinpc)
 !
-!           translation vector from the slave surface to the
-!           master surface
+!           cyclic symmetry axis
 !
             do j=1,6
                read(textpart(j)(1:20),'(f20.0)',iostat=istat) 
      &              cs(5+j,mcs)
             enddo
             cs(1,mcs)=xsectors
-            cs(17,mcs)=itie+0.5
+            cs(17,mcs)=itie+0.5d0
 !
 !           counting the number of faces on the master side (should be the
 !           same as on the slave side)
@@ -250,14 +255,24 @@ c      iaxial=max(iaxial,maxsectors)
 !           max 8 nodes per face
 !
             cs(4,mcs)=(iendset(j)-istartset(j)+1)*8+0.5d0
+!     
+            call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &           ipoinp,inp,ipoinpc)
+!
             return
          elseif((tieset(1,i)(1:80).eq.tie).and.
      &      (tieset(1,i)(81:81).ne.'C').and.
-     &      (tieset(1,i)(81:81).ne.'T')) then
+     &      (tieset(1,i)(81:81).ne.'T').and.
+     &      (tieset(1,i)(81:81).ne.'M').and.
+     &      (tieset(1,i)(81:81).ne.'S').and.
+     &      (tieset(1,i)(81:81).ne.'D')) then
             itie=i
             exit
          elseif((tieset(1,i)(81:81).ne.'C').and.
-     &      (tieset(1,i)(81:81).ne.'T')) then
+     &          (tieset(1,i)(81:81).ne.'T').and.
+     &          (tieset(1,i)(81:81).ne.'M').and.
+     &          (tieset(1,i)(81:81).ne.'S').and.
+     &          (tieset(1,i)(81:81).ne.'D')) then
             ntiecyc=ntiecyc+1
             itiecyc=i
          endif
@@ -270,13 +285,39 @@ c      iaxial=max(iaxial,maxsectors)
      &                 '*ERROR reading *CYCLIC SYMMETRY MODEL:'
             write(*,*) '       tie constraint is nonexistent'
             call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+     &           "*CYCLIC SYMMETRY MODEL%",ier)
+            return
          endif
       endif
+!
+      if(xsectors.le.0) then
+         write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
+         write(*,*) '       the required parameter N'
+         write(*,*) '       is lacking on the *CYCLIC SYMMETRY MODEL'
+         write(*,*) '       keyword card or has a value <=0'
+         ier=1
+         return
+      endif
+      if(gsectors.lt.1) then
+         write(*,*) '*WARNING reading *CYCLIC SYMMETRY MODEL:'
+         write(*,*) '         cannot plot less than'
+         write(*,*) '         one sector: one sector will be plotted'
+         gsectors=1
+      endif
+      if(gsectors.gt.xsectors) then
+         write(*,*) '*WARNING reading *CYCLIC SYMMETRY MODEL:'
+         write(*,*) '         cannot plot more than'
+         write(*,*) '         ',xsectors,'sectors;',
+     &           xsectors,' sectors will'
+         write(*,*) '       be plotted'
+         gsectors=xsectors
+      endif
+!
+      maxsectors=max(maxsectors,int(xsectors+0.5d0))
 !
       cs(1,mcs)=xsectors
-      cs(5,mcs)=gsectors+0.5
-      cs(17,mcs)=itie+0.5
+      cs(5,mcs)=gsectors+0.5d0
+      cs(17,mcs)=itie+0.5d0
       depset=tieset(2,itie)
       indepset=tieset(3,itie)
       tolloc=tietol(1,itie)
@@ -301,10 +342,11 @@ c      iaxial=max(iaxial,maxsectors)
             write(*,*) '       element set does not'
             write(*,*) '       exist; '
             call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+     &           "*CYCLIC SYMMETRY MODEL%",ier)
+            return
          endif
       endif
-      cs(13,mcs)=iset+0.5
+      cs(13,mcs)=iset+0.5d0
 !
       call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &     ipoinp,inp,ipoinpc)
@@ -313,21 +355,26 @@ c      iaxial=max(iaxial,maxsectors)
          write(*,*)'*ERROR reading *CYCLIC SYMMETRY MODEL:'
          write(*,*) '      definition of the cyclic'
          write(*,*) '      symmetry model is not complete'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       ntrans=ntrans+1
       if(ntrans.gt.ntrans_) then
          write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
          write(*,*) '       increase ntrans_'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       do i=1,6
          read(textpart(i)(1:20),'(f20.0)',iostat=istat) csab(i)
          trab(i,ntrans)=csab(i)
-         if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*CYCLIC SYMMETRY MODEL%")
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*CYCLIC SYMMETRY MODEL%",ier)
+            return
+         endif
       enddo
 !
 !     cyclic coordinate system
@@ -357,7 +404,8 @@ c      iaxial=max(iaxial,maxsectors)
             write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
             write(*,*) '       surface ',depset
             write(*,*) '       has not yet been defined.' 
-            call exit(201)
+            ier=1
+            return
          endif
       endif
       jdep=i
@@ -378,7 +426,8 @@ c      iaxial=max(iaxial,maxsectors)
             write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
             write(*,*) '       surface ',indepset
             write(*,*) '       has not yet been defined.' 
-            call exit(201)
+            ier=1
+            return
          endif
       endif
       jindep=i
@@ -453,10 +502,10 @@ c      iaxial=max(iaxial,maxsectors)
                   elseif(lakon(nelems)(4:5).eq.'15') then
                      node=kon(indexe+ifacew2(m,jfaces))
                   endif
-                  call nident(ics(lprev+1),node,l,id)
+                  call nident(ics,node,l,id)
                   exist=.FALSE.
                   if(id.gt.0) then
-                     if(ics(lprev+id).eq.node) then
+                     if(ics(id).eq.node) then
                         exist=.TRUE.
                      endif
                   endif
@@ -465,9 +514,10 @@ c      iaxial=max(iaxial,maxsectors)
                   if(lprev+l.gt.ncs_) then
                      write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
                      write(*,*) '       increase ncs_'
-                     call exit(201)
+                     ier=1
+                     return
                   endif
-                  do k=lprev+l,lprev+id+2,-1
+                  do k=l,id+2,-1
                      ics(k)=ics(k-1)
                      zcs(k)=zcs(k-1)
                      rcs(k)=rcs(k-1)
@@ -477,17 +527,18 @@ c      iaxial=max(iaxial,maxsectors)
                   yap=co(2,node)-csab(2)
                   zap=co(3,node)-csab(3)
 !     
-                  ics(lprev+id+1)=node
-                  zcs(lprev+id+1)=xap*xn+yap*yn+zap*zn
-                  rcs(lprev+id+1)=dsqrt((xap-zcs(lprev+id+1)*xn)**2+
-     &                                  (yap-zcs(lprev+id+1)*yn)**2+
-     &                                  (zap-zcs(lprev+id+1)*zn)**2)
+                  ics(id+1)=node
+                  zcs(id+1)=xap*xn+yap*yn+zap*zn
+                  rcs(id+1)=dsqrt((xap-zcs(id+1)*xn)**2+
+     &                            (yap-zcs(id+1)*yn)**2+
+     &                            (zap-zcs(id+1)*zn)**2)
                else
                   l=l+1
                   if(lprev+l.gt.ncs_) then
                      write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
                      write(*,*) '       increase ncs_'
-                     call exit(201)
+                     ier=1
+                     return
                   endif
                   node =ialset(j)
 !     
@@ -511,7 +562,8 @@ c      iaxial=max(iaxial,maxsectors)
                if(l.gt.ncs_) then
                   write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL:'
                   write(*,*) '       increase ncs_'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                node=k
 !
@@ -592,6 +644,7 @@ c      iaxial=max(iaxial,maxsectors)
 !
       calcangle=.false.
       nodesonaxis=.false.
+      phi=0.d0
 !
       nneigh=1
       loop1: do i=istartset(jdep),iendset(jdep)
@@ -655,23 +708,24 @@ c      iaxial=max(iaxial,maxsectors)
                   endif
                   noded=ialset(i)
                endif
-c            enddo
 !
                xap=co(1,noded)-csab(1)
                yap=co(2,noded)-csab(2)
                zap=co(3,noded)-csab(3)
 !     
-               zp=xap*xn+yap*yn+zap*zn
-               rp=dsqrt((xap-zp*xn)**2+(yap-zp*yn)**2+(zap-zp*zn)**2)
+               zpd=xap*xn+yap*yn+zap*zn
+               rpd=dsqrt((xap-zpd*xn)**2+(yap-zpd*yn)**2+
+     &                   (zap-zpd*zn)**2)
 !     
-               if((.not.calcangle).and.(rp.gt.1.d-10)) then
-                  x2=(xap-zp*xn)/rp
-                  y2=(yap-zp*yn)/rp
-                  z2=(zap-zp*zn)/rp
+               if((.not.calcangle).and.(rpd.gt.1.d-10)) then
+                  x2=(xap-zpd*xn)/rpd
+                  y2=(yap-zpd*yn)/rpd
+                  z2=(zap-zpd*zn)/rpd
                endif
 !
-               call near2d(rcs0,zcs0,rcs,zcs,nr,nz,rp,zp,ncsnodes,node,
-     &            nneigh)
+               call near2d(rcs0,zcs0,rcs,zcs,nr,nz,rpd,zpd,ncsnodes,
+     &              noden,nneigh)
+               node=noden(1)
 !
                nodei=ics(node)
                if(nodei.lt.0) cycle
@@ -688,7 +742,12 @@ c            enddo
                zp=xap*xn+yap*yn+zap*zn
                rp=dsqrt((xap-zp*xn)**2+(yap-zp*yn)**2+(zap-zp*zn)**2)
 !     
-               if((.not.calcangle).and.(rp.gt.1.d-10)) then
+!              in order for the angle to be correct the axial position
+!              of the dependent and independent node must be the same
+!              (important for non-coincident meshes)
+!
+               if((.not.calcangle).and.(rp.gt.1.d-10).and.
+     &            (dabs(zp-zpd).lt.1.d-10)) then
                   x3=(xap-zp*xn)/rp
                   y3=(yap-zp*yn)/rp
                   z3=(zap-zp*zn)/rp
@@ -712,7 +771,8 @@ c            enddo
      &                       user_angle*57.29577951d0
                         write(*,*)'       angle based on the geometry:',
      &                       calculated_angle*57.29577951d0
-                        call exit(201)
+                        ier=1
+                        return
                      endif
                   else
                      write(*,*) '*INFO in cyclicsymmetrymodels: angle'
@@ -737,17 +797,19 @@ c            enddo
                yap=co(2,noded)-csab(2)
                zap=co(3,noded)-csab(3)
 !     
-               zp=xap*xn+yap*yn+zap*zn
-               rp=dsqrt((xap-zp*xn)**2+(yap-zp*yn)**2+(zap-zp*zn)**2)
+               zpd=xap*xn+yap*yn+zap*zn
+               rpd=dsqrt((xap-zpd*xn)**2+(yap-zpd*yn)**2+
+     &                   (zap-zpd*zn)**2)
 !     
-               if((.not.calcangle).and.(rp.gt.1.d-10)) then
-                  x2=(xap-zp*xn)/rp
-                  y2=(yap-zp*yn)/rp
-                  z2=(zap-zp*zn)/rp
+               if((.not.calcangle).and.(rpd.gt.1.d-10)) then
+                  x2=(xap-zpd*xn)/rpd
+                  y2=(yap-zpd*yn)/rpd
+                  z2=(zap-zpd*zn)/rpd
                endif
 !     
-               call near2d(rcs0,zcs0,rcs,zcs,nr,nz,rp,zp,ncsnodes,node,
-     &              nneigh)
+               call near2d(rcs0,zcs0,rcs,zcs,nr,nz,rpd,zpd,ncsnodes,
+     &              noden,nneigh)
+               node=noden(1)
 !     
                nodei=ics(node)
                if(nodei.lt.0) cycle
@@ -764,7 +826,12 @@ c            enddo
                zp=xap*xn+yap*yn+zap*zn
                rp=dsqrt((xap-zp*xn)**2+(yap-zp*yn)**2+(zap-zp*zn)**2)
 !     
-               if((.not.calcangle).and.(rp.gt.1.d-10)) then
+!              in order for the angle to be correct the axial position
+!              of the dependent and independent node must be the same
+!              (important for non-coincident meshes)
+!
+               if((.not.calcangle).and.(rp.gt.1.d-10).and.
+     &            (dabs(zp-zpd).lt.1.d-10)) then
                   x3=(xap-zp*xn)/rp
                   y3=(yap-zp*yn)/rp
                   z3=(zap-zp*zn)/rp
@@ -788,7 +855,8 @@ c            enddo
      &                    user_angle*57.29577951d0
                         write(*,*) '       angle based on the geometry:'
      &                       ,calculated_angle*57.29577951d0
-                        call exit(201)
+                        ier=1
+                        return
                      endif
                   endif
                   calcangle=.true.
@@ -798,6 +866,16 @@ c            enddo
          endif
 !
       enddo loop1
+!
+      if(phi.eq.0.d0) then
+         write(*,*) '*ERROR reading *CYCLIC SYMMETRY MODEL'
+         write(*,*) '       sector angle cannot be determined:'
+         write(*,*) '       there exists no dependent node'
+         write(*,*) '       with the same axial position as'
+         write(*,*) '       an independent node'
+         ier=1
+         return
+      endif
 !
 !     allocating a node of the depset to each node of the indepset 
 !
@@ -816,6 +894,27 @@ c            enddo
       write(*,*) 
      &     '      read WarnNodeMissCyclicSymmetry.nam inp'
       write(*,*)
+!     
+!     generating the thermal MPC's; the generated MPC's are for nodal
+!     diameter 0. BETTER: based on ithermal(2), cf. gen3dfrom2d.f
+!     
+!     about next info write statement:
+!     in tempload cyclic symmetry is enforced for field t1, but not
+!     for field t0. This may lead to stresses if t1 is not cyclic
+!     symmetric. If there is a *initial conditions,type=temperature
+!     card in the input deck but no *temperature card t1 is copied from
+!     t0 before ensuring the cyclic symmetry for t1. So also in this
+!     case a non-cyclic symmetric field t0 can lead to stresses.
+!
+      if(ithermal(1).eq.1) then
+         write(*,*) '*INFO reading *CYCLIC SYMMETRY MODEL'
+         write(*,*) '      cyclic symmetry equations are generated'
+         write(*,*) '      for the temperature; if the initial'
+         write(*,*) '      temperatures are not cyclic symmetric'
+         write(*,*) '      and/or the applied temperature is not'
+         write(*,*) '      cyclic symmetric this may lead to'
+         write(*,*) '      additional stresses'
+      endif
 !
       loop2: do i=istartset(jdep),iendset(jdep)
          if(ialset(i).gt.0) then
@@ -878,7 +977,6 @@ c            enddo
                   endif
                   noded=ialset(i)
                endif
-c            enddo
 !
 !           check whether cyclic MPC's have already been
 !           generated (e.g. for nodes belonging to several
@@ -898,8 +996,8 @@ c            enddo
      &         mcs,triangulation,csab,xn,yn,zn,phi,noded,
      &         ncsnodes,rcscg,rcs0cg,zcscg,zcs0cg,nrcg,
      &         nzcg,jcs,lcs,kontri,straight,ne,ipkon,kon,lakon,
-     &         ifacetet,inodface,ncounter,jobnamec,vold,cfd,mi,
-     &         indepset)
+     &         ifacetet,inodface,ncounter,jobnamec,vold,nef,mi,
+     &         indepset,ithermal)
             enddo
 !
          else
@@ -927,8 +1025,8 @@ c            enddo
      &           mcs,triangulation,csab,xn,yn,zn,phi,noded,
      &           ncsnodes,rcscg,rcs0cg,zcscg,zcs0cg,nrcg,
      &           nzcg,jcs,lcs,kontri,straight,ne,ipkon,kon,lakon,
-     &           ifacetet,inodface,ncounter,jobnamec,vold,cfd,mi,
-     &           indepset)
+     &           ifacetet,inodface,ncounter,jobnamec,vold,nef,mi,
+     &           indepset,ithermal)
             enddo
          endif
 !
@@ -940,9 +1038,12 @@ c            enddo
          write(*,*) '*WARNING reading *CYCLIC SYMMETRY MODEL:'
          write(*,*) '        for at least one dependent'
          write(*,*) '        node in a cyclic symmetry definition no '
-         write(*,*) '        independent counterpart was found'
+         write(*,*) '        independent counterpart was found.'
+         write(*,*) '        Failed nodes are stored in file '
+         write(*,*) '        WarnNodeMissCyclicSymmetry.nam'
 c     next line was commented on 19/04/2012
-c         call exit(201)
+c         ier=1
+c         return
       endif
 !
 !     sorting ics
@@ -950,7 +1051,7 @@ c         call exit(201)
 !
       kflag=1
       call isortii(ics,nr,ncsnodes,kflag)
-      cs(4,mcs)=ncsnodes+0.5
+      cs(4,mcs)=ncsnodes+0.5d0
       lprev=lprev+ncsnodes
 !
 !     check orientation of (xn,yn,zn) (important for copying of base
