@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -26,19 +26,15 @@
 !
       implicit none
 !
-      integer nterms,i,j,iy,kflag,n
+      integer nterms,i,j,n
 !
       real*8 ratio(20),pneigh(3,*),pnode(3),a,xi,et,xig,etg,p(3),
      &  dummy,ze,zeg,omg,omh,omr,opg,oph,opr,dx(3),al,
      &  tpgphpr,tmgphpr,tmgmhpr,tpgmhpr,tpgphmr,tmgphmr,tmgmhmr,tpgmhmr,
      &  omgopg,omhoph,omropr,omgmopg,omhmoph,omrmopr
 !
-      intent(in) xig,etg,zeg,pneigh,pnode,
-     &          nterms
 !
-      intent(inout) ratio,a,p
 !
-      kflag=1
       n=3
 !
       if(nterms.eq.4) then
@@ -48,19 +44,14 @@
          dx(1)=xi
          dx(2)=et
          dx(3)=ze
-         call dsort(dx,iy,n,kflag)
+         call insertsortd(dx,n)
+c         call dsort(dx,iy,n,kflag)
          if(dx(3).gt.1.d-30) then
             al=dx(3)/(xi+et+ze)
             xi=al*xi
             et=al*et
             ze=al*ze
          endif
-c         if(xi+et+ze.gt.1.d0) then
-c            dummy=2.d0*(1.d0-xi-et-ze)/3.d0
-c            xi=dummy+xi
-c            et=dummy+et
-c            ze=dummy+ze
-c         endif
 !
 !        shape functions
 !
@@ -118,7 +109,8 @@ c         endif
          dx(1)=xi
          dx(2)=et
          dx(3)=ze
-         call dsort(dx,iy,n,kflag)
+         call insertsortd(dx,n)
+c         call dsort(dx,iy,n,kflag)
          if(dx(3).gt.1.d-30) then
             al=dx(3)/(xi+et+ze)
             xi=al*xi
@@ -159,21 +151,21 @@ c         endif
 !
 !     shape functions
 !
-         ratio(1)=-0.5*a*(1.0-ze)*(2.0*xi+2.0*et+ze)
-         ratio(2)=0.5*xi*(1.0-ze)*(2.0*xi-2.0-ze)
-         ratio(3)=0.5*et*(1.0-ze)*(2.0*et-2.0-ze)
-         ratio(4)=-0.5*a*(1.0+ze)*(2.0*xi+2.0*et-ze)
-         ratio(5)=0.5*xi*(1.0+ze)*(2.0*xi-2.0+ze)
-         ratio(6)=0.5*et*(1.0+ze)*(2.0*et-2.0+ze)
-         ratio(7)=2.0*xi*a*(1.0-ze)
-         ratio(8)=2.0*xi*et*(1.0-ze)
-         ratio(9)=2.0*et*a*(1.0-ze)
-         ratio(10)=2.0*xi*a*(1.0+ze)
-         ratio(11)=2.0*xi*et*(1.0+ze) 
-         ratio(12)=2.0*et*a*(1.0+ze)
-         ratio(13)= a*(1.0-ze*ze)
-         ratio(14)=xi*(1.0-ze*ze)
-         ratio(15)=et*(1.0-ze*ze)
+         ratio(1)=-0.5d0*a*(1.d0-ze)*(2.d0*xi+2.d0*et+ze)
+         ratio(2)=0.5d0*xi*(1.d0-ze)*(2.d0*xi-2.d0-ze)
+         ratio(3)=0.5d0*et*(1.d0-ze)*(2.d0*et-2.d0-ze)
+         ratio(4)=-0.5d0*a*(1.d0+ze)*(2.d0*xi+2.d0*et-ze)
+         ratio(5)=0.5d0*xi*(1.d0+ze)*(2.d0*xi-2.d0+ze)
+         ratio(6)=0.5d0*et*(1.d0+ze)*(2.d0*et-2.d0+ze)
+         ratio(7)=2.d0*xi*a*(1.d0-ze)
+         ratio(8)=2.d0*xi*et*(1.d0-ze)
+         ratio(9)=2.d0*et*a*(1.d0-ze)
+         ratio(10)=2.d0*xi*a*(1.d0+ze)
+         ratio(11)=2.d0*xi*et*(1.d0+ze) 
+         ratio(12)=2.d0*et*a*(1.d0+ze)
+         ratio(13)= a*(1.d0-ze*ze)
+         ratio(14)=xi*(1.d0-ze*ze)
+         ratio(15)=et*(1.d0-ze*ze)
       elseif(nterms.eq.20) then
          xi=xig
          et=etg
