@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                   */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -95,15 +95,15 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
               ITG *cyclicsymmetry,double *xnoels,ITG *ielas,ITG *ielprop,
               double *prop){
     
-  char lakonl[9]="        \0";
+  char lakonl[9]="        \0",jobnamef[396]="";
 
-  ITG i,j,k,l,init,*itg=NULL,ntg=0,maxlenmpc,icascade=0,loop,
+  ITG i,j,k,l,init,*itg=NULL,ntg=0,maxlenmpc,icascade=0,loop,kchdep=0,
       konl[20],imat,nope,kodem,indexe,j1,jdof,kmin,kmax,
-      id,newstep=0,idiscon,*ipiv=NULL,info,nrhs=1,kode,iener=0,
+      id,newstep=0,idiscon,*ipiv=NULL,info,nrhs=1,kode,
       *ikactcont=NULL,*ilactcont=NULL,*ikactcont1=NULL,nactcont1=0,
       i1,icutb=0,iconvergence=0,idivergence=0,mt=mi[1]+1,
       nactcont1_=100,*ikactmech=NULL,iabsload=0,im,nasym=0,mortar=0,
-      ialeatoric=0;
+      ialeatoric=0,*iponoel=NULL,*inoel=NULL,*nshcon=NULL;
 
   long long i2;
 
@@ -116,9 +116,14 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
       *bjincp=NULL,sump,h14,*dbjp=NULL,senergy=0.0,*xforcdiff=NULL,
       df,i0,ic,ia,dbjmaxOLD1,dbjmaxOLD2,*xloaddiff=NULL,*dbcont=NULL,
       zl=0.0,*xbodydiff=NULL,*t1diff=NULL,*xboundiff=NULL,*bdiff=NULL,
-      *pslavsurf=NULL,*pmastsurf=NULL,*clearini=NULL,venergy=0.0;
+      *pslavsurf=NULL,*pmastsurf=NULL,*clearini=NULL,venergy=0.0,
+      *shcon=NULL,alea;
 
   ikactcont=*ikactcontp;ikactmech=*ikactmechp;
+  
+  for(k=0;k<3;k++){
+      strcpy1(&jobnamef[k*132],&jobnamec[k*132],132);
+  }
 
   if(*inonlinmpc==1) iabsload=2;
 
@@ -142,7 +147,7 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 
   /* load the convergence constants from ctrl*/
 
-  i0=ctrl[0];ic=ctrl[3];ia=ctrl[7];df=ctrl[10];
+  i0=ctrl[0];ic=ctrl[3];ia=ctrl[7];df=ctrl[10];alea=ctrl[54];
 
   /* set the convergence parameters*/
 
@@ -159,13 +164,13 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 	  ialset,itietri,lakon,ipkon,kon,koncont,ne,cg,
 	  straight,nkon,co,vold,ielmat,cs,elcon,istep,
 	  iinc,iit,ncmat_,ntmat_,ne0,
-	  vini,nmethod,nmpc,mpcfree,memmpc_,
-	  &ipompc,&labmpc,&ikmpc,&ilmpc,&fmpc,&nodempc,&coefmpc,
+	  vini,nmethod,
           iperturb,ikboun,nboun,mi,imastop,nslavnode,islavnode,islavsurf,
           itiefac,areaslav,iponoels,inoels,springarea,tietol,reltime,
 	  imastnode,nmastnode,xmastnor,filab,mcs,ics,&nasym,
           xnoels,&mortar,pslavsurf,pmastsurf,clearini,theta,
-	  xstateini,xstate,nstate_,&icutb,&ialeatoric);
+	  xstateini,xstate,nstate_,&icutb,&ialeatoric,jobnamef,
+          &alea);
 
   NNEW(ikactcont1,ITG,nactcont1_);
 
@@ -188,9 +193,10 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
       FORTRAN(springforc_n2f,(xl,konl,voldl,&imat,elcon,nelcon,elas,
 			  fnl,ncmat_,ntmat_,&nope,lakonl,
 			  &t1l,&kodem,elconloc,plicon,nplicon,npmat_,
-			  &senergy,&iener,cstr,mi,
+			  &senergy,nener,cstr,mi,
                           &springarea[2*(konl[nope]-1)],nmethod,ne0,
-			  nstate_,xstateini,xstate,reltime,ielas,&venergy));
+			  nstate_,xstateini,xstate,reltime,ielas,
+			  &venergy,ielorien,orab,norien,&i));
 
       storecontactdof(&nope,nactdof,&mt,konl,&ikactcont1,&nactcont1,
 		      &nactcont1_,bcont,fnl,ikmpc,nmpc,ilmpc,ipompc,nodempc, 
@@ -322,12 +328,13 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 		nelemload,sideload,mi,
 		xforcdiff,xloaddiff,xbodydiff,t1diff,xboundiff,&iabsload,
 		iprescribedboundary,ntrans,trab,inotr,veold,nactdof,bcont,
-                fn));
+                fn,ipobody,iponoel,inoel,ipkon,kon,lakon,ielprop,prop,ielmat,
+                shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon));
 	      
       /* calculating the instantaneous loading vector */
       
       if(iabsload!=2){
-	  FORTRAN(rhs,(co,nk,kon,ipkon,lakon,ne,
+	  rhsmain(co,nk,kon,ipkon,lakon,ne,
 		   ipompc,nodempc,coefmpc,nmpc,nodeforc,ndirforc,xforcdiff,
 		   nforc,nelemload,sideload,xloaddiff,nload,xbodydiff,
 		   ipobody,nbody,cgr,b,nactdof,&neq[1],nmethod,
@@ -337,9 +344,10 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 		   nplicon,plkcon,nplkcon,
 		   npmat_,ttime,time,istep,iinc,dtime,physcon,ibody,
 		   xbodyold,reltime,veold,matname,mi,ikactmech,nactmech,
-                   ielprop,prop));
+                   ielprop,prop,sti,xstateini,xstate,nstate_,ntrans,inotr,
+                   trab);
       }else{
-	  FORTRAN(rhs,(co,nk,kon,ipkon,lakon,ne,
+	  rhsmain(co,nk,kon,ipkon,lakon,ne,
 		   ipompc,nodempc,coefmpc,nmpc,nodeforc,ndirforc,xforcact,
 		   nforc,nelemload,sideload,xloadact,nload,xbodyact,
 		   ipobody,nbody,cgr,b,nactdof,&neq[1],nmethod,
@@ -349,7 +357,8 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 		   nplicon,plkcon,nplkcon,
 		   npmat_,ttime,time,istep,iinc,dtime,physcon,ibody,
 		   xbodyold,reltime,veold,matname,mi,ikactmech,nactmech,
-                   ielprop,prop));
+                   ielprop,prop,sti,xstateini,xstate,nstate_,ntrans,inotr,
+                   trab);
       }
       
       /* correction for nonzero SPC's */
@@ -360,7 +369,7 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 		  xbounact,iamboun,nboun,nodeboun,ndirboun,ad,au,adb,
 		  aub,icol,irow,neq,nzs,&sigma,b,isolver,
 		  &alpham,&betam,nzl,&init,bact,bmin,jq,amname,bv,
-  	          bprev,bdiff,nactmech,&iabsload,iprev);
+  	          bprev,bdiff,nactmech,&iabsload,iprev,reltime);
       }
 
       /* correcting aamech */
@@ -681,7 +690,8 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 			 nmpc,ikboun,ilboun,nboun,xbounact,aux,iaux,
 			 &maxlenmpc,ikmpc,ilmpc,&icascade,
 			 kon,ipkon,lakon,ne,reltime,&newstep,xboun,fmpc,
-			 iit,&idiscon,ncont,trab,ntrans,ithermal,mi));
+			 iit,&idiscon,ncont,trab,ntrans,ithermal,mi,
+			 &kchdep));
       }
       
       /* calculating displacements/temperatures */
@@ -713,13 +723,13 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 	      ialset,itietri,lakon,ipkon,kon,koncont,ne,cg,
 	      straight,nkon,co,vold,ielmat,cs,elcon,istep,
 	      iinc,iit,ncmat_,ntmat_,ne0,
-	      vini,nmethod,nmpc,mpcfree,memmpc_,
-	      &ipompc,&labmpc,&ikmpc,&ilmpc,&fmpc,&nodempc,&coefmpc,
+	      vini,nmethod,
 	      iperturb,ikboun,nboun,mi,imastop,nslavnode,islavnode,islavsurf,
               itiefac,areaslav,iponoels,inoels,springarea,tietol,reltime,
 	      imastnode,nmastnode,xmastnor,filab,mcs,ics,&nasym,
               xnoels,&mortar,pslavsurf,pmastsurf,clearini,theta,
-	      xstateini,xstate,nstate_,&icutb,&ialeatoric);
+	      xstateini,xstate,nstate_,&icutb,&ialeatoric,jobnamef,
+              &alea);
 
       for(i=*ne0;i<*ne;i++){
 	indexe=ipkon[i];
@@ -740,14 +750,16 @@ void dynacont(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon, ITG *ne,
 	FORTRAN(springforc_n2f,(xl,konl,voldl,&imat,elcon,nelcon,elas,
 			    fnl,ncmat_,ntmat_,&nope,lakonl,
 			    &t1l,&kodem,elconloc,plicon,nplicon,npmat_,
-			    &senergy,&iener,cstr,mi,
+			    &senergy,nener,cstr,mi,
                             &springarea[2*(konl[nope]-1)],nmethod,ne0,
-			    nstate_,xstateini,xstate,reltime,ielas,&venergy));
+			    nstate_,xstateini,xstate,reltime,ielas,
+			    &venergy,ielorien,orab,norien,&i));
 	
 	FORTRAN(springstiff_n2f,(xl,elas,konl,voldl,s,&imat,elcon,nelcon,
 	        ncmat_,ntmat_,&nope,lakonl,&t1l,&kode,elconloc,
 		plicon,nplicon,npmat_,iperturb,&springarea[2*(konl[nope]-1)],
-		nmethod,mi,ne0,nstate_,xstateini,xstate,reltime,&nasym));
+		nmethod,mi,ne0,nstate_,xstateini,xstate,reltime,&nasym,
+		ielorien,orab,norien,&i));
 
 	dfdbj(bcont,&dbcont,&neq[1],&nope,konl,nactdof,
 	      s,z,ikmpc,ilmpc,ipompc,nodempc,nmpc,coefmpc,
