@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,8 @@
 !
       subroutine dynamics(inpc,textpart,nmethod,iperturb,tinc,tper,
      &  tmin,tmax,idrct,alpha,iexpl,isolver,istep,istat,n,iline,
-     &  ipol,inl,ipoinp,inp,ithermal,ipoinpc,cfd,ctrl,tincf,nener)
+     &  ipol,inl,ipoinp,inp,ithermal,ipoinpc,nef,ctrl,tincf,nener,
+     &  ier)
 !
 !     reading the input deck: *DYNAMIC
 !
@@ -28,6 +29,7 @@
 !             4: sgi solver
 !             5: TAUCS
 !             7: pardiso
+!             8: pastix
 !
 !      iexpl==0:  structure:implicit, fluid:incompressible
 !      iexpl==2:  structure:explicit, fluid:incompressible
@@ -38,16 +40,17 @@
       character*20 solver
       character*132 textpart(16)
 !
-      integer nmethod,istep,istat,n,key,i,iperturb,idrct,iexpl,
-     &  isolver,iline,ipol,inl,ipoinp(2,*),inp(3,*),ithermal,
-     &  ipoinpc(0:*),cfd,nener
+      integer nmethod,istep,istat,n,key,i,iperturb(*),idrct,iexpl,
+     &  isolver,iline,ipol,inl,ipoinp(2,*),inp(3,*),ithermal(*),
+     &  ipoinpc(0:*),nef,nener,ier
 !
-      real*8 tinc,tper,tmin,tmax,alpha,ctrl(*),tincf
+      real*8 tinc,tper,tmin,tmax,alpha(*),ctrl(*),tincf
 !
       if(istep.lt.1) then
          write(*,*) '*ERROR reading *DYNAMIC: *DYNAMIC can only'
          write(*,*) '  be used within a STEP'
-         call exit(201)
+         ier=1
+         return
       endif
 !
 !     default is implicit
@@ -56,18 +59,19 @@
 !
 !     no heat transfer analysis
 !
-      if(ithermal.gt.1) then
-         ithermal=1
+      if(ithermal(1).gt.1) then
+         ithermal(1)=1
       endif
 !
 !     only nonlinear analysis allowed for this procedure
 !
-      if(iperturb.lt.2) iperturb=2
+      if(iperturb(1).lt.2) iperturb(1)=2
 !
 !     default values
 !
       idrct=0
-      alpha=-0.05d0
+      alpha(1)=-0.05d0
+      alpha(2)=0.5d0
       tmin=0.d0
       tmax=0.d0
       tincf=1.d-2
@@ -87,21 +91,26 @@
          solver(1:5)='TAUCS'
       elseif(isolver.eq.7) then
          solver(1:7)='PARDISO'
+      elseif(isolver.eq.8) then
+         solver(1:6)='PASTIX'
       endif
 !
       do i=2,n
          if(textpart(i)(1:6).eq.'ALPHA=') then
-            read(textpart(i)(7:26),'(f20.0)',iostat=istat) alpha
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
-            if(alpha.lt.-1.d0/3.d0) then
+            read(textpart(i)(7:26),'(f20.0)',iostat=istat) alpha(1)
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*DYNAMIC%",ier)
+               return
+            endif
+            if(alpha(1).lt.-1.d0/3.d0) then
                write(*,*) '*WARNING reading *DYNAMIC: alpha is smaller'
                write(*,*) '  than -1/3 and is reset to -1/3'
-               alpha=-1.d0/3.d0
-            elseif(alpha.gt.0.d0) then
+               alpha(1)=-1.d0/3.d0
+            elseif(alpha(1).gt.0.d0) then
                write(*,*) '*WARNING reading *DYNAMIC: alpha is greater'
                write(*,*) '  than 0 and is reset to 0'
-               alpha=0.d0
+               alpha(1)=0.d0
             endif
          elseif(textpart(i)(1:8).eq.'EXPLICIT') then
             iexpl=2
@@ -110,6 +119,8 @@
             idrct=1
          elseif(textpart(i)(1:7).eq.'SOLVER=') then
             read(textpart(i)(8:27),'(a20)') solver
+         elseif(textpart(i)(1:18).eq.'RELATIVETOABSOLUTE') then
+            alpha(2)=1.5
          else
             write(*,*) 
      &        '*WARNING reading *DYNAMIC: parameter not recognized:'
@@ -132,6 +143,8 @@
          isolver=5
       elseif(solver(1:7).eq.'PARDISO') then
          isolver=7
+      elseif(solver(1:6).eq.'PASTIX') then
+         isolver=8
       else
          write(*,*) '*WARNING reading *DYNAMIC: unknown solver;'
          write(*,*) '         the default solver is used'
@@ -140,7 +153,7 @@
       call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &     ipoinp,inp,ipoinpc)
       if((istat.lt.0).or.(key.eq.1)) then
-         if((iperturb.ge.2).or.(cfd.eq.1)) then
+         if((iperturb(1).ge.2).or.(nef.gt.0)) then
             write(*,*)'*WARNING reading *DYNAMIC: a nonlinear analysis i
      &s requested'
             write(*,*) '         but no time increment nor step is speci
@@ -157,20 +170,35 @@
       endif
 !
       read(textpart(1)(1:20),'(f20.0)',iostat=istat) tinc
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*DYNAMIC%",ier)
+         return
+      endif
       read(textpart(2)(1:20),'(f20.0)',iostat=istat) tper
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*DYNAMIC%",ier)
+         return
+      endif
       read(textpart(3)(1:20),'(f20.0)',iostat=istat) tmin
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*DYNAMIC%",ier)
+         return
+      endif
       read(textpart(4)(1:20),'(f20.0)',iostat=istat) tmax
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*DYNAMIC%",ier)
+         return
+      endif
       read(textpart(4)(1:20),'(f20.0)',iostat=istat) tincf
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*DYNAMIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*DYNAMIC%",ier)
+         return
+      endif
 !
       if(tinc.le.0.d0) then
          write(*,*)'*ERROR reading *DYNAMIC: initial increment size is n
@@ -183,7 +211,7 @@
          write(*,*)'*ERROR reading *DYNAMIC: initial increment size exce
      &eds step size'
       endif
-      if((cfd.eq.1).and.(tincf.le.0.d0)) then
+      if((nef.gt.0).and.(tincf.le.0.d0)) then
          write(*,*) '*WARNING reading *DYNAMIC: initial CFD increment si
      &ze is zero or negative; the default of 0.01 is taken'
          tincf=1.d-2
@@ -196,6 +224,18 @@
          if(dabs(tmax).lt.1.d-10) then
             tmax=1.d+30
          endif
+         if(dabs(tmax).gt.tper) then
+            tmax=tper
+         endif
+         if(tinc.gt.dabs(tmax)) then
+            write(*,*) '*WARNING reading *DYNAMIC:'
+            write(*,*) '         the initial increment ',tinc
+            write(*,*) '         exceeds the maximum increment ',
+     &          tmax
+            write(*,*) '         the initial increment is reduced'
+            write(*,*) '         to the maximum value'
+            tinc=dabs(tmax)
+         endif
       endif
 !
 !     10 cutbacks allowed for dynamics (because of contact)
