@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -27,7 +27,8 @@
      &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,veold,springarea,
      &  nstate_,xstateini,xstate,ne0,ipkon,thicke,
      &  integerglob,doubleglob,tieset,istartset,iendset,ialset,ntie,
-     &  nasym,pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,kscale)
+     &  nasym,pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,kscale,
+     &  smscalel,mscalmethod)
 !
 !     computation of the element matrix and rhs for the element with
 !     the topology in konl
@@ -51,8 +52,8 @@
       character*81 tieset(3,*)
 !
       integer konl(26),ifaceq(8,6),nelemload(2,*),nbody,nelem,
-     &  mi(*),iloc,jfaces,igauss,mortar,kon(*),ielprop(*),null,
-     &  mattyp,ithermal,iperturb(*),nload,idist,i,j,k,l,i1,i2,j1,
+     &  mi(*),jfaces,igauss,mortar,kon(*),ielprop(*),null,
+     &  mattyp,ithermal(*),iperturb(*),nload,idist,i,j,k,l,i1,i2,j1,
      &  nmethod,k1,l1,ii,jj,ii1,jj1,id,ipointer,ig,m1,m2,m3,m4,kk,
      &  nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),six,
      &  ielorien(mi(3),*),ilayer,nlayer,ki,kl,ipkon(*),indexe,
@@ -62,7 +63,8 @@
      &  layer,kspt,jltyp,iflag,iperm(60),m,ipompc(*),nodempc(3,*),
      &  nmpc,ikmpc(*),ilmpc(*),iscale,nstate_,ne0,iselect(6),kscale,
      &  istartset(*),iendset(*),ialset(*),ntie,integerglob(*),nasym,
-     &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,nopered
+     &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,nopered,
+     &  mscalmethod
 !
       real*8 co(3,*),xl(3,26),shp(4,26),xs2(3,7),veold(0:mi(2),*),
      &  s(60,60),w(3,3),p1(3),p2(3),bodyf(3),bodyfx(3),ff(60),
@@ -82,22 +84,10 @@
      &  plicon(0:2*npmat_,ntmat_,*),plkcon(0:2*npmat_,ntmat_,*),
      &  xstiff(27,mi(1),*),plconloc(802),dtime,ttime,time,tvar(2),
      &  sax(60,60),ffax(60),gs(8,4),a,stress(6),stre(3,3),
-     &  pslavsurf(3,*),pmastsurf(6,*)
+     &  pslavsurf(3,*),pmastsurf(6,*),xmass,xsjmass,shpmass(4,26),
+     &  shpjmass(4,26),smscalel,smfactor
 !
-      intent(in) co,kon,lakonl,p1,p2,omx,bodyfx,nbody,
-     &  nelem,elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,
-     &  ielmat,ielorien,norien,orab,ntmat_,
-     &  t0,t1,ithermal,vold,iperturb,nelemload,
-     &  sideload,nload,idist,sti,stx,iexpl,plicon,
-     &  nplicon,plkcon,nplkcon,xstiff,npmat_,dtime,
-     &  matname,mi,ncmat_,mass,stiffness,buckling,rhsi,intscheme,
-     &  ttime,time,istep,iinc,coriolis,xloadold,reltime,
-     &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,veold,
-     &  nstate_,xstateini,ne0,ipkon,thicke,
-     &  integerglob,doubleglob,tieset,istartset,iendset,ialset,ntie,
-     &  nasym,pslavsurf,pmastsurf,mortar,clearini,ielprop,prop
 !
-      intent(inout) s,sm,ff,xload,nmethod,springarea,xstate
 !
       include "gauss.f"
 !
@@ -169,6 +159,8 @@ c     Bernhardi end
          else
             nope=ichar(lakonl(8:8))-47
          endif
+      elseif(lakonl(1:4).eq.'MASS') then
+         nope=1
       endif
 !
 !     material and orientation
@@ -178,7 +170,7 @@ c     Bernhardi end
          imat=ielmat(1,nelem)
          amat=matname(imat)
          if(norien.gt.0) then
-            iorien=ielorien(1,nelem)
+            iorien=max(0,ielorien(1,nelem))
          else
             iorien=0
          endif
@@ -188,7 +180,7 @@ c     Bernhardi end
          else
             ihyper=0
          endif
-      else
+      elseif(lakonl(4:5).eq.'20') then
 !
 !        composite materials
 !
@@ -226,6 +218,45 @@ c     Bernhardi end
          do i=1,4
             dlayer(i)=0.d0
          enddo
+!
+      elseif(lakonl(4:5).eq.'15') then
+!
+!        composite materials
+!
+!        determining the number of layers
+!
+         nlayer=0
+         do k=1,mi(3)
+            if(ielmat(k,nelem).ne.0) then
+               nlayer=nlayer+1
+            endif
+         enddo
+         mint2d=3
+!
+!        determining the layer thickness and global thickness
+!        at the shell integration points
+!
+         iflag=1
+         do kk=1,mint2d
+            xi=gauss3d10(1,kk)
+            et=gauss3d10(2,kk)
+            call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+            tlayer(kk)=0.d0
+            do i=1,nlayer
+               thickness=0.d0
+               do j=1,6
+                  thickness=thickness+thicke(i,indexe+j)*shp2(4,j)
+               enddo
+               tlayer(kk)=tlayer(kk)+thickness
+               xlayer(i,kk)=thickness
+            enddo
+         enddo
+         iflag=3
+!
+         ilayer=0
+         do i=1,3
+            dlayer(i)=0.d0
+         enddo
 !     
       endif
 !
@@ -265,7 +296,11 @@ c     Bernhardi end
             mint2d=1
             mint3d=1
          elseif(lakonl(4:5).eq.'15') then
-            mint3d=9
+            if(lakonl(7:8).eq.'LC') then
+               mint3d=6*nlayer
+            else
+               mint3d=9
+            endif
          elseif(lakonl(4:4).eq.'6') then
             mint3d=2
          else
@@ -349,7 +384,8 @@ c     Bernhardi end
         enddo
       enddo
 !
-!     calculating the stiffness matrix for the contact spring elements
+!     calculating the stiffness matrix for the (contact) spring elements
+!     and the mass matrix for the mass elements
 !
       if(mint3d.eq.0) then
 !
@@ -367,37 +403,88 @@ c     Bernhardi end
          endif
 !
          kode=nelcon(1,imat)
-         if(lakonl(7:7).eq.'A') then
-            t0l=0.d0
-            t1l=0.d0
-            if(ithermal.eq.1) then
-               t0l=(t0(konl(1))+t0(konl(2)))/2.d0
-               t1l=(t1(konl(1))+t1(konl(2)))/2.d0
-            elseif(ithermal.ge.2) then
-               t0l=(t0(konl(1))+t0(konl(2)))/2.d0
-               t1l=(vold(0,konl(1))+vold(0,konl(2)))/2.d0
+         if(lakonl(1:2).eq.'ES') then
+            if(lakonl(7:7).ne.'C') then
+               t0l=0.d0
+               t1l=0.d0
+               if(ithermal(1).eq.1) then
+                  t0l=(t0(konl(1))+t0(konl(2)))/2.d0
+                  t1l=(t1(konl(1))+t1(konl(2)))/2.d0
+               elseif(ithermal(1).ge.2) then
+                  t0l=(t0(konl(1))+t0(konl(2)))/2.d0
+                  t1l=(vold(0,konl(1))+vold(0,konl(2)))/2.d0
+               endif
             endif
-         else
+!     
+            if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'1').or.
+     &         (lakonl(7:7).eq.'2').or.(mortar.eq.0)) then
+               call springstiff_n2f(xl,elas,konl,voldl,s,imat,elcon,
+     &              nelcon,ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,
+     &              plicon,nplicon,npmat_,iperturb,
+     &              springarea(1,konl(nope+1)),nmethod,mi,ne0,nstate_,
+     &              xstateini,xstate,reltime,nasym,ielorien,orab,norien,
+     &              nelem)
+            elseif(mortar.eq.1) then
+               jfaces=kon(indexe+nope+2)
+               igauss=kon(indexe+nope+1) 
+               call springstiff_f2f(xl,elas,voldl,s,imat,elcon,nelcon,
+     &              ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
+     &              nplicon,npmat_,iperturb,springarea(1,igauss),
+     &              nmethod,mi,ne0,nstate_,xstateini,xstate,reltime,
+     &              nasym,jfaces,igauss,pslavsurf,
+     &              pmastsurf,clearini,kscale)
+            endif
+         elseif(lakonl(1:4).eq.'MASS') then
 !
-!        as soon as the first contact element is discovered ne0 is
-!        determined and saved
+!           mass matrix contribution
 !
-         endif
-         if((lakonl(7:7).eq.'A').or.(mortar.eq.0)) then
-            call springstiff_n2f(xl,elas,konl,voldl,s,imat,elcon,nelcon,
-     &      ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
-     &      nplicon,npmat_,iperturb,springarea(1,konl(nope+1)),nmethod,
-     &      mi,ne0,nstate_,xstateini,xstate,reltime,nasym)
-         elseif(mortar.eq.1) then
-            iloc=kon(indexe+nope+1)
-            jfaces=kon(indexe+nope+2)
-            igauss=kon(indexe+nope+1) 
-            call springstiff_f2f(xl,elas,voldl,s,imat,elcon,nelcon,
-     &        ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
-     &        nplicon,npmat_,iperturb,springarea(1,iloc),nmethod,
-     &        mi,ne0,nstate_,xstateini,xstate,reltime,
-     &        nasym,iloc,jfaces,igauss,pslavsurf,
-     &        pmastsurf,clearini,kscale)
+            if(mass.eq.1) then
+               do i1=1,3
+                  sm(i1,i1)=rhcon(1,1,imat)
+               enddo
+            endif
+!
+!           contribution to the rhs
+!
+            if(rhsi.eq.1) then
+               if(nbody.ne.0) then
+!
+                  xmass=rhcon(1,1,imat)
+!
+                  do i1=1,3
+                     ff(i1)=bodyfx(i1)*xmass
+                  enddo
+!
+                  if(omx.gt.0.d0) then
+!     
+!                    omega**2 * mass
+!     
+                     om=omx*xmass
+                     do i1=1,3
+!     
+!                   computation of the global coordinates of the
+!                   mass node
+!
+                        if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) 
+     &                     then
+                           q(i1)=xl(i1,1)
+                        else
+                           q(i1)=xl(i1,1)+voldl(i1,1)
+                        endif
+!     
+                        q(i1)=q(i1)-p1(i1)
+                     enddo
+                     const=q(1)*p2(1)+q(2)*p2(2)+q(3)*p2(3)
+!     
+!                 inclusion of the centrifugal force into the body force
+!
+                     do i1=1,3
+                        ff(i1)=ff(i1)+(q(i1)-const*p2(i1))*om
+                     enddo
+                  endif
+               endif
+            endif
+!            
          endif
          return
       endif
@@ -457,7 +544,7 @@ c     Bernhardi end
                   imat=ielmat(ilayer,nelem)
                   amat=matname(imat)
                   if(norien.gt.0) then
-                     iorien=ielorien(ilayer,nelem)
+                     iorien=max(0,ielorien(ilayer,nelem))
                   else
                      iorien=0
                   endif
@@ -484,10 +571,51 @@ c     Bernhardi end
                ze=gauss3d4(3,kk)
                weight=weight3d4(kk)
             elseif(lakonl(4:5).eq.'15') then
-               xi=gauss3d8(1,kk)
-               et=gauss3d8(2,kk)
-               ze=gauss3d8(3,kk)
-               weight=weight3d8(kk)
+               if(lakonl(7:8).ne.'LC') then
+                  xi=gauss3d8(1,kk)
+                  et=gauss3d8(2,kk)
+                  ze=gauss3d8(3,kk)
+                  weight=weight3d8(kk)
+               else
+                  kl=mod(kk,6)
+                  if(kl.eq.0) kl=6
+!
+                  xi=gauss3d10(1,kl)
+                  et=gauss3d10(2,kl)
+                  ze=gauss3d10(3,kl)
+                  weight=weight3d10(kl)
+!
+                  ki=mod(kk,3)
+                  if(ki.eq.0) ki=3
+!
+                  if(kl.eq.1) then
+                     ilayer=ilayer+1
+                     if(ilayer.gt.1) then
+                        do i=1,3
+                           dlayer(i)=dlayer(i)+xlayer(ilayer-1,i)
+                        enddo
+                     endif
+                  endif
+                  ze=2.d0*(dlayer(ki)+(ze+1.d0)/2.d0*xlayer(ilayer,ki))/
+     &                 tlayer(ki)-1.d0
+                  weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!
+!                 material and orientation
+!
+                  imat=ielmat(ilayer,nelem)
+                  amat=matname(imat)
+                  if(norien.gt.0) then
+                     iorien=max(0,ielorien(ilayer,nelem))
+                  else
+                     iorien=0
+                  endif
+!     
+                  if(nelcon(1,imat).lt.0) then
+                     ihyper=1
+                  else
+                     ihyper=0
+                  endif
+               endif
             elseif(lakonl(4:4).eq.'6') then
                xi=gauss3d7(1,kk)
                et=gauss3d7(2,kk)
@@ -527,6 +655,7 @@ c     Bernhardi start
             call shape8hr(xl,xsj,shp,gs,a)
          elseif(lakonl(1:5).eq.'C3D8I') then
             call shape8hu(xi,et,ze,xl,xsj,shp,iflag)
+            call shape8humass(xi,et,ze,xl,xsjmass,shpmass,iflag)
          elseif(nope.eq.20) then
 c     Bernhardi end
             if(lakonl(7:7).eq.'A') then
@@ -558,7 +687,6 @@ c     Bernhardi end
             nmethod=0
          endif
 !
-c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
          if(((iperturb(1).eq.1).or.(iperturb(2).eq.1)).and.
      &           (stiffness.eq.1).and.(buckling.eq.0))then
 !
@@ -577,7 +705,7 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
 !
          t0l=0.d0
          t1l=0.d0
-         if(ithermal.eq.1) then
+         if(ithermal(1).eq.1) then
             if(lakonl(4:5).eq.'8 ') then
                do i1=1,nope
                   t0l=t0l+t0(konl(i1))/8.d0
@@ -585,14 +713,18 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
                enddo
             elseif(lakonl(4:6).eq.'20 ')then
                nopered=20
-               call lintemp(t0,t1,konl,nopered,kk,t0l,t1l)
+               call lintemp(t0,konl,nopered,kk,t0l)
+               call lintemp(t1,konl,nopered,kk,t1l)
+            elseif(lakonl(4:6).eq.'10T') then
+               call linscal10(t0,konl,t0l,null,shp)
+               call linscal10(t1,konl,t1l,null,shp)
             else
                do i1=1,nope
                   t0l=t0l+shp(4,i1)*t0(konl(i1))
                   t1l=t1l+shp(4,i1)*t1(konl(i1))
                enddo
             endif
-         elseif(ithermal.ge.2) then
+         elseif(ithermal(1).ge.2) then
             if(lakonl(4:5).eq.'8 ') then
                do i1=1,nope
                   t0l=t0l+t0(konl(i1))/8.d0
@@ -600,7 +732,11 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
                enddo
             elseif(lakonl(4:6).eq.'20 ')then
                nopered=20
-               call lintemp_th(t0,vold,konl,nopered,kk,t0l,t1l,mi)
+               call lintemp_th0(t0,konl,nopered,kk,t0l,mi)
+               call lintemp_th1(vold,konl,nopered,kk,t1l,mi)
+            elseif(lakonl(4:6).eq.'10T') then
+               call linscal10(t0,konl,t0l,null,shp)
+               call linscal10(vold,konl,t1l,mi(2),shp)
             else
                do i1=1,nope
                   t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -638,7 +774,7 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
      &        nelem,ithermal,alzero,mattyp,t0l,t1l,
      &        ihyper,istiff,elconloc,eth,kode,plicon,
      &        nplicon,plkcon,nplkcon,npmat_,
-     &        plconloc,mi(1),dtime,nelem,kk,
+     &        plconloc,mi(1),dtime,kk,
      &        xstiff,ncmat_)
 !
          if(mattyp.eq.1) then
@@ -688,6 +824,11 @@ c            call orthotropic(elas,anisox)
             shpj(3,i1)=shp(3,i1)*xsjj
             shpj(4,i1)=shp(4,i1)*xsj
          enddo
+         if(lakonl(4:5).eq.'8I') then
+            do i1=1,nope
+               shpjmass(4,i1)=shpmass(4,i1)*xsjmass
+            enddo
+         endif
 !
 !           determination of the stiffness, and/or mass and/or
 !           buckling matrix
@@ -703,7 +844,7 @@ c            call orthotropic(elas,anisox)
 !               dynamic calculations
 !
                if((mass.eq.1).and.(iexpl.gt.1)) then
-                  summass=summass+rho*xsj
+                  summass=summass+rho*xsj*weight
                endif
 !
                jj1=1
@@ -796,8 +937,13 @@ c            call orthotropic(elas,anisox)
 !                     mass matrix
 !
                         if(mass.eq.1) then
-                           sm(ii1,jj1)=sm(ii1,jj1)
-     &                          +rho*shpj(4,ii)*shp(4,jj)*weight
+                           if(lakonl(4:5).ne.'8I') then
+                              sm(ii1,jj1)=sm(ii1,jj1)
+     &                             +rho*shpj(4,ii)*shp(4,jj)*weight
+                           else
+                              sm(ii1,jj1)=sm(ii1,jj1)
+     &                          +rho*shpjmass(4,ii)*shpmass(4,jj)*weight
+                           endif
                            sm(ii1+1,jj1+1)=sm(ii1,jj1)
                            sm(ii1+2,jj1+2)=sm(ii1,jj1)
                         endif
@@ -858,7 +1004,7 @@ c            call orthotropic(elas,anisox)
 !               dynamic calculations
 !
                if((mass.eq.1).and.(iexpl.gt.1)) then
-                  summass=summass+rho*xsj
+                  summass=summass+rho*xsj*weight
                endif
 !
                jj1=1
@@ -927,8 +1073,13 @@ c            call orthotropic(elas,anisox)
 !                   mass matrix
 !
                      if(mass.eq.1) then
-                        sm(ii1,jj1)=sm(ii1,jj1)
-     &                       +rho*shpj(4,ii)*shp(4,jj)*weight
+                        if(lakonl(4:5).ne.'8I') then
+                           sm(ii1,jj1)=sm(ii1,jj1)
+     &                          +rho*shpj(4,ii)*shp(4,jj)*weight
+                        else
+                           sm(ii1,jj1)=sm(ii1,jj1)
+     &                          +rho*shpjmass(4,ii)*shpmass(4,jj)*weight
+                        endif
                         sm(ii1+1,jj1+1)=sm(ii1,jj1)
                         sm(ii1+2,jj1+2)=sm(ii1,jj1)
                      endif
@@ -1023,7 +1174,6 @@ c            call orthotropic(elas,anisox)
 !                   point
 !
                      q(i1)=0.d0
-c                     if(iperturb(1).eq.0) then
                      if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                         do j1=1,nope
                            q(i1)=q(i1)+shp(4,j1)*xl(i1,j1)
@@ -1105,7 +1255,6 @@ c     Bernhardi start
           if((nope.eq.20).or.(nope.eq.8).or.
      &       (nope.eq.11)) then
 c     Bernhardi end
-c             if(iperturb(1).eq.0) then
              if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                 do i=1,nopes
                    do j=1,3
@@ -1128,7 +1277,6 @@ c             if(iperturb(1).eq.0) then
                 enddo
              endif
           elseif((nope.eq.10).or.(nope.eq.4)) then
-c             if(iperturb(1).eq.0) then
              if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                 do i=1,nopes
                    do j=1,3
@@ -1151,7 +1299,6 @@ c             if(iperturb(1).eq.0) then
                 enddo
              endif
           else
-c             if(iperturb(1).eq.0) then
              if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                 do i=1,nopes
                    do j=1,3
@@ -1204,16 +1351,12 @@ c             if(iperturb(1).eq.0) then
              endif
 !
              if(rhsi.eq.1) then
-                if(nopes.eq.9) then
-                   call shape9q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.8) then
+                if(nopes.eq.8) then
                    call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.4) then
                    call shape4q(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.6) then
                    call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.7) then
-                   call shape7tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 else
                    call shape3tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 endif
@@ -1303,19 +1446,14 @@ c    Bernhardi end
 !            three-dimensional thermomechanical Applications,
 !            Wiley, 2004, p 153, eqn. (3.54).
 !
-c             elseif((mass.eq.1).and.(iperturb(1).ne.0)) then
              elseif((mass.eq.1).and.
      &            ((iperturb(1).eq.1).or.(iperturb(2).eq.1))) then
-                if(nopes.eq.9) then
-                   call shape9q(xi,et,xl1,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.8) then
+                if(nopes.eq.8) then
                    call shape8q(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.4) then
                    call shape4q(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.6) then
                    call shape6tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.7) then
-                   call shape7tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 else
                    call shape3tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 endif
@@ -1488,40 +1626,11 @@ c     Bernhardi end
 !     for axially symmetric and plane stress/strain elements: 
 !     complete s and sm
 !
-      if(((lakonl(4:5).eq.'8 ').or.
-     &    ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
-     &   ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'S').or.
-     &    (lakonl(7:7).eq.'E'))) then
-         do i=1,60
-            do j=i,60
-               k=abs(iperm(i))
-               l=abs(iperm(j))
-               if(k.gt.l) then
-                  m=k
-                  k=l
-                  l=m
-               endif
-               sax(i,j)=s(k,l)*iperm(i)*iperm(j)/(k*l)
-            enddo
-         enddo
-         do i=1,60
-            do j=i,60
-               s(i,j)=s(i,j)+sax(i,j)
-            enddo
-         enddo
-!
-         if((nload.ne.0).or.(nbody.ne.0)) then
-            do i=1,60
-               k=abs(iperm(i))
-               ffax(i)=ff(k)*iperm(i)/k
-            enddo
-            do i=1,60
-               ff(i)=ff(i)+ffax(i)
-            enddo
-         endif
-!
-         if(mass.eq.1) then
-            summass=2.d0*summass
+      if(intscheme.eq.0) then
+         if(((lakonl(4:5).eq.'8 ').or.
+     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'S').or.
+     &        (lakonl(7:7).eq.'E'))) then
             do i=1,60
                do j=i,60
                   k=abs(iperm(i))
@@ -1531,18 +1640,49 @@ c     Bernhardi end
                      k=l
                      l=m
                   endif
-                  sax(i,j)=sm(k,l)*iperm(i)*iperm(j)/(k*l)
+                  sax(i,j)=s(k,l)*iperm(i)*iperm(j)/(k*l)
                enddo
             enddo
             do i=1,60
                do j=i,60
-                  sm(i,j)=sm(i,j)+sax(i,j)
+                  s(i,j)=s(i,j)+sax(i,j)
                enddo
             enddo
+!     
+            if((nload.ne.0).or.(nbody.ne.0)) then
+               do i=1,60
+                  k=abs(iperm(i))
+                  ffax(i)=ff(k)*iperm(i)/k
+               enddo
+               do i=1,60
+                  ff(i)=ff(i)+ffax(i)
+               enddo
+            endif
+!     
+            if(mass.eq.1) then
+               summass=2.d0*summass
+               do i=1,60
+                  do j=i,60
+                     k=abs(iperm(i))
+                     l=abs(iperm(j))
+                     if(k.gt.l) then
+                        m=k
+                        k=l
+                        l=m
+                     endif
+                     sax(i,j)=sm(k,l)*iperm(i)*iperm(j)/(k*l)
+                  enddo
+               enddo
+               do i=1,60
+                  do j=i,60
+                     sm(i,j)=sm(i,j)+sax(i,j)
+                  enddo
+               enddo
+            endif
          endif
       endif
-!
-      if((mass.eq.1).and.(iexpl.gt.1)) then
+!     
+      if((mass.eq.1).and.(iexpl.gt.1))then
 !
 !        scaling the diagonal terms of the mass matrix such that the total mass
 !        is right (LUMPING; for explicit dynamic calculations)
@@ -1564,16 +1704,19 @@ c            alp=.2215d0
             alp=0.1203d0
          elseif(nope.eq.15) then
             alp=0.2141d0
+         elseif(nope.eq.11) then
+            alp=0.1852d0
          endif
 !
          if((nope.eq.20).or.(nope.eq.10).or.
-     &      (nope.eq.15)) then
+     &      (nope.eq.15).or.(nope.eq.11)) then
             factore=summass*alp/(1.d0+alp)/sume
             factorm=summass/(1.d0+alp)/summ
          else
             factore=summass/sume
          endif
 !
+c        write(6,*) alp, sume, summ, factore, factorm
          do i=1,3*nopev,3
             sm(i,i)=sm(i,i)*factore
             sm(i+1,i+1)=sm(i,i)
@@ -1585,25 +1728,55 @@ c            alp=.2215d0
             sm(i+2,i+2)=sm(i,i)
          enddo
 !
-      endif
+         if((mscalmethod.ne.1).and.(mscalmethod.ne.3)) then
+!     
+!     setting all off-diagonal terms to zero     
+!     
+            do i=1,3*nope
+               do j=1,3*nope
+                  if(i.eq.j) cycle
+                  sm(i,j)=0.d0
+               enddo
+            enddo
+         else
+!     
+!           CC: Mass Scaling
+!           mscalmethod = 1: selective mass scaling SMS
 !
-c      if(nelem.eq.1) then
-c         open(30,file='nelem1.dat',status='unknown')
-c         write(30,'(8(1x,e11.4))') ((s(i,j),i=1,60),j=1,60)
-c         close(30)
-c      elseif(nelem.eq.2) then
-c         open(31,file='nelem2.dat',status='unknown')
-c         write(31,'(8(1x,e11.4))') ((s(i,j),i=1,60),j=1,60)
-c         close(31)
-c      elseif(nelem.eq.3) then
-c         open(32,file='nelem3.dat',status='unknown')
-c         write(32,'(8(1x,e11.4))') ((s(i,j),i=1,60),j=1,60)
-c         close(32)
-c      else
-c         open(33,file='nelem4.dat',status='unknown')
-c         write(33,'(8(1x,e11.4))') ((s(i,j),i=1,60),j=1,60)
-c         close(33)
-c      endif
+!           beta = smscalel
+!
+            smfactor=smscalel*summass/((nope-1)*nope)
+!
+            do i=1,3*nope
+               do j=1,3*nope
+                  if(i.ne.j) then
+!     set non diagonals to zero           
+                     sm(i,j)=0.d0
+!     diagonal terms of M for SMS
+                  else
+                     sm(i,j)=sm(i,j)+(nope-1)*smfactor
+                  endif
+               enddo
+            enddo
+!     
+!           nondiagonal terms of M for SMS
+!     
+            i=0
+            do j=0,nope*3-1,3
+               i=i+1
+               do k=1,(nope-i)
+                  do l=1,3
+                     sm(j+l+k*3,j+l)=-smfactor
+                     sm(j+l,j+l+k*3)=-smfactor
+                  enddo
+               enddo
+            enddo
+!     to calculate additional energy in resultsmech.f:
+            smscalel=smfactor
+         endif
+!     
+      endif
+!     
       return
       end
 
