@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -28,11 +28,11 @@
      &  nstate_,xstateini,xstate,ne0,ipkon,thicke,
      &  integerglob,doubleglob,tieset,istartset,iendset,ialset,ntie,
      &  nasym,pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,distmin,
-     &  ndesi,nodedesi,dfminds,icoordinate,dxstiff,ne,xdesi,
-     &  istartelem,ialelem,v)
+     &  ndesi,nodedesi,dfl,icoordinate,dxstiff,ne,xdesi,
+     &  istartelem,ialelem,v,sigma,ieigenfrequency)
 !
-!     computation of the element matrix and rhs for the element with
-!     the topology in konl
+!     computation of the sensitivity of the element matrix multiplied by the
+!     displacements for the element with the topology in konl
 !
 !     ff: rhs without temperature and eigenstress contribution
 !
@@ -51,8 +51,8 @@
       character*81 tieset(3,*)
 !
       integer konl(26),ifaceq(8,6),nelemload(2,*),nbody,nelem,
-     &  mi(*),iloc,jfaces,igauss,mortar,kon(*),ielprop(*),null,
-     &  mattyp,ithermal,iperturb(*),nload,idist,i,j,k,l,i1,i2,j1,
+     &  mi(*),jfaces,igauss,mortar,kon(*),ielprop(*),null,
+     &  mattyp,ithermal(*),iperturb(*),nload,idist,i,j,k,l,i1,i2,j1,
      &  nmethod,k1,l1,ii,jj,ii1,jj1,id,ipointer,ig,m1,m2,m3,m4,kk,
      &  nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),six,
      &  ielorien(mi(3),*),ilayer,nlayer,ki,kl,ipkon(*),indexe,
@@ -65,10 +65,11 @@
      &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,nopered,
      &  ndesi,nodedesi(*),idesvar,node,kscale,iactive,ij,
      &  mass,stiffness,buckling,rhsi,coriolis,icoordinate,idir,ne,
-     &  istartelem(*),ialelem(*)
+     &  istartelem(*),ialelem(*),ieigenfrequency,idesloc,
+     &  node1,node2,ifaceqexp(2,20),ifacewexp(2,15) 
 !
       real*8 co(3,*),xl(3,26),shp(4,26),xs2(3,7),veold(0:mi(2),*),
-     &  s(60,60),w(3,3),p1(3),p2(3),bodyf(3),bodyfx(3),
+     &  s(60,60),w(3,3),p1(3),p2(3),bodyf(3),bodyfx(3),sigma,
      &  ff(60),bf(3),q(3),shpj(4,26),elcon(0:ncmat_,ntmat_,*),t(3),
      &  rhcon(0:1,ntmat_,*),xkl(3,3),eknlsign,reltime,prop(*),
      &  alcon(0:6,ntmat_,*),alzero(*),orab(7,*),t0(*),t1(*),
@@ -86,25 +87,10 @@
      &  xstiff(27,mi(1),*),plconloc(802),dtime,ttime,time,tvar(2),
      &  sax(60,60),ffax(60),gs(8,4),a,stress(6),stre(3,3),
      &  pslavsurf(3,*),pmastsurf(6,*),distmin,s0(60,60),xdesi(3,*),
-     &  ds1(60,60),ff0(60),dfminds(ndesi,60),dxstiff(27,mi(1),ne,*),
+     &  ds1(60,60),ff0(60),dfl(20,60),dxstiff(27,mi(1),ne,*),
      &  vl(0:mi(2),26),v(0:mi(2),*)
 !
-      intent(in) co,kon,lakonl,p1,p2,omx,bodyfx,nbody,
-     &  nelem,elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,
-     &  ielmat,ielorien,norien,orab,ntmat_,
-     &  t0,t1,ithermal,vold,iperturb,nelemload,
-     &  sideload,nload,idist,sti,stx,iexpl,plicon,
-     &  nplicon,plkcon,nplkcon,xstiff,npmat_,dtime,
-     &  matname,mi,ncmat_,mass,stiffness,buckling,rhsi,intscheme,
-     &  ttime,time,istep,iinc,coriolis,xloadold,reltime,
-     &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,veold,
-     &  nstate_,xstateini,ne0,ipkon,thicke,
-     &  integerglob,doubleglob,tieset,istartset,iendset,ialset,ntie,
-     &  nasym,pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,
-     &  distmin,ndesi,nodedesi,icoordinate,xdesi,istartelem,ialelem,
-     &  v
 !
-      intent(inout) sm,xload,nmethod,springarea,xstate,dfminds
 !
       include "gauss.f"
 !
@@ -130,6 +116,47 @@
      &            37,38,-39,40,41,-42,43,44,-45,46,47,-48,
      &            25,26,-27,28,29,-30,31,32,-33,34,35,-36,
      &            49,50,-51,52,53,-54,55,56,-57,58,59,-60/)
+!
+!     nodes in expansion direction of hex element 
+!
+      data ifaceqexp /5,17,
+     &                6,18,
+     &                7,19,
+     &                8,20,
+     &                1,17,
+     &                2,18,
+     &                3,19,
+     &                4,20,
+     &                13,0,
+     &                14,0,
+     &                15,0,
+     &                16,0,
+     &                9,0,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                0,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
+!
+!     nodes in expansion direction of wedge element 
+!
+      data ifacewexp /4,13,
+     &                5,14,
+     &                6,16,
+     &                1,13,
+     &                2,14,
+     &                3,15,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                7,0,
+     &                8,0,
+     &                9,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
 !
       tvar(1)=time
       tvar(2)=ttime+time
@@ -176,6 +203,8 @@ c     Bernhardi end
          else
             nope=ichar(lakonl(8:8))-47
          endif
+      elseif(lakonl(1:4).eq.'MASS') then
+         nope=1
       endif
 !
 !     material and orientation
@@ -185,7 +214,7 @@ c     Bernhardi end
          imat=ielmat(1,nelem)
          amat=matname(imat)
          if(norien.gt.0) then
-            iorien=ielorien(1,nelem)
+            iorien=max(0,ielorien(1,nelem))
          else
             iorien=0
          endif
@@ -195,7 +224,7 @@ c     Bernhardi end
          else
             ihyper=0
          endif
-      else
+      elseif(lakonl(4:5).eq.'20') then
 !
 !        composite materials
 !
@@ -233,7 +262,45 @@ c     Bernhardi end
          do i=1,4
             dlayer(i)=0.d0
          enddo
-!     
+      elseif(lakonl(4:5).eq.'15') then
+!
+!        composite materials
+!
+!        determining the number of layers
+!
+         nlayer=0
+         do k=1,mi(3)
+            if(ielmat(k,nelem).ne.0) then
+               nlayer=nlayer+1
+            endif
+         enddo
+         mint2d=3
+!
+!        determining the layer thickness and global thickness
+!        at the shell integration points
+!
+         iflag=1
+         do kk=1,mint2d
+            xi=gauss3d10(1,kk)
+            et=gauss3d10(2,kk)
+            call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+            tlayer(kk)=0.d0
+            do i=1,nlayer
+               thickness=0.d0
+               do j=1,6
+                  thickness=thickness+thicke(i,indexe+j)*shp2(4,j)
+               enddo
+               tlayer(kk)=tlayer(kk)+thickness
+               xlayer(i,kk)=thickness
+            enddo
+         enddo
+         iflag=3
+!
+         ilayer=0
+         do i=1,3
+            dlayer(i)=0.d0
+         enddo
+!
       endif
 !
       if(intscheme.eq.0) then
@@ -272,7 +339,11 @@ c     Bernhardi end
             mint2d=1
             mint3d=1
          elseif(lakonl(4:5).eq.'15') then
-            mint3d=9
+            if(lakonl(7:8).eq.'LC') then
+               mint3d=6*nlayer
+            else
+               mint3d=9
+            endif
          elseif(lakonl(4:4).eq.'6') then
             mint3d=2
          else
@@ -327,6 +398,7 @@ c     Bernhardi end
 !
       do ij=istartelem(nelem),istartelem(nelem+1)-1
          idesvar=ialelem(ij)
+         idesloc=ij+1-istartelem(nelem)
 !
 !     check whether a design variable belongs to the element
 !
@@ -341,7 +413,7 @@ c     Bernhardi end
                if(node.eq.nodedesi(idesvar)) then
                   iactive=i
                   do j=1,60
-                     dfminds(idesvar,j)=0.d0
+                     dfl(idesloc,j)=0.d0
                   enddo
                   exit
                endif
@@ -351,7 +423,7 @@ c     Bernhardi end
 !           the orientations are the design variables
 !
             do j=1,60
-               dfminds(idesvar,j)=0.d0
+               dfl(idesloc,j)=0.d0
             enddo
          endif
       endif
@@ -373,6 +445,59 @@ c     Bernhardi end
          do i=1,3
             xl(i,iactive)=xl(i,iactive)+xdesi(i,idesvar)
          enddo
+         if(lakonl(1:5).eq.'C3D20') then
+            if((lakonl(7:7).eq.'A').or.
+     &         (lakonl(7:7).eq.'L').or.
+     &         (lakonl(7:7).eq.'S').or.
+     &         (lakonl(7:7).eq.'E')) then
+               node1=ifaceqexp(1,iactive)
+               node2=ifaceqexp(2,iactive)
+               do i=1,3
+                  if(iactive.le.8) then
+                     xl(i,node1)=xl(i,node1)+xdesi(i,idesvar)
+                     xl(i,node2)=xl(i,node2)+xdesi(i,idesvar)
+                  else
+                     xl(i,node1)=xl(i,node1)+xdesi(i,idesvar)
+                  endif          
+               enddo  
+            endif    
+         elseif(lakonl(1:5).eq.'C3D15') then
+            if((lakonl(7:7).eq.'A').or.
+     &         (lakonl(7:7).eq.'L').or.
+     &         (lakonl(7:7).eq.'S').or.
+     &         (lakonl(7:7).eq.'E')) then
+               node1=ifacewexp(1,iactive)
+               node2=ifacewexp(2,iactive)
+               do i=1,3
+                  if(iactive.le.6) then
+                     xl(i,node1)=xl(i,node1)+xdesi(i,idesvar)
+                     xl(i,node2)=xl(i,node2)+xdesi(i,idesvar)
+                  else
+                     xl(i,node1)=xl(i,node1)+xdesi(i,idesvar)
+                  endif          
+               enddo  
+            endif    
+         elseif(lakonl(1:4).eq.'C3D8') then
+            if((lakonl(7:7).eq.'A').or.
+     &         (lakonl(7:7).eq.'L').or.
+     &         (lakonl(7:7).eq.'S').or.
+     &         (lakonl(7:7).eq.'E')) then
+               node1=ifaceqexp(1,iactive)
+               do i=1,3
+                  xl(i,node1)=xl(i,node1)+xdesi(i,idesvar)         
+               enddo  
+            endif    
+         elseif(lakonl(1:4).eq.'C3D6') then
+            if((lakonl(7:7).eq.'A').or.
+     &         (lakonl(7:7).eq.'L').or.
+     &         (lakonl(7:7).eq.'S').or.
+     &         (lakonl(7:7).eq.'E')) then
+               node1=ifaceqexp(1,iactive)
+               do j=1,3
+                  xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)       
+               enddo  
+            endif     
+         endif
       endif
 !     Ertl end
 !
@@ -404,7 +529,8 @@ c     Bernhardi end
         endif
       endif
 !
-!     calculating the stiffness matrix for the contact spring elements
+!     calculating the stiffness matrix for the (contact) spring elements
+!     and the mass matrix for the mass elements
 !
       if(mint3d.eq.0) then
 !
@@ -422,37 +548,77 @@ c     Bernhardi end
          endif
 !
          kode=nelcon(1,imat)
-         if(lakonl(7:7).eq.'A') then
-            t0l=0.d0
-            t1l=0.d0
-            if(ithermal.eq.1) then
-               t0l=(t0(konl(1))+t0(konl(2)))/2.d0
-               t1l=(t1(konl(1))+t1(konl(2)))/2.d0
-            elseif(ithermal.ge.2) then
-               t0l=(t0(konl(1))+t0(konl(2)))/2.d0
-               t1l=(vold(0,konl(1))+vold(0,konl(2)))/2.d0
+         if(lakonl(1:2).eq.'ES') then
+            if(lakonl(7:7).ne.'C') then
+               t0l=0.d0
+               t1l=0.d0
+               if(ithermal(1).eq.1) then
+                  t0l=(t0(konl(1))+t0(konl(2)))/2.d0
+                  t1l=(t1(konl(1))+t1(konl(2)))/2.d0
+               elseif(ithermal(1).ge.2) then
+                  t0l=(t0(konl(1))+t0(konl(2)))/2.d0
+                  t1l=(vold(0,konl(1))+vold(0,konl(2)))/2.d0
+               endif
             endif
-         else
+            if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'1').or.
+     &           (lakonl(7:7).eq.'2').or.(mortar.eq.0)) then
+               call springstiff_n2f(xl,elas,konl,voldl,s,imat,elcon,
+     &              nelcon,ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,
+     &              plicon,nplicon,npmat_,iperturb,
+     &              springarea(1,konl(nope+1)),nmethod,mi,ne0,nstate_,
+     &              xstateini,xstate,reltime,nasym,ielorien,orab,norien,
+     &              nelem)
+            elseif(mortar.eq.1) then
+               jfaces=kon(indexe+nope+2)
+               igauss=kon(indexe+nope+1) 
+               call springstiff_f2f(xl,elas,voldl,s,imat,elcon,nelcon,
+     &              ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
+     &              nplicon,npmat_,iperturb,springarea(1,igauss),
+     &              nmethod,mi,ne0,nstate_,xstateini,xstate,reltime,
+     &              nasym,jfaces,igauss,pslavsurf,
+     &              pmastsurf,clearini,kscale)
+            endif
+         elseif(lakonl(1:4).eq.'MASS') then
 !
-!        as soon as the first contact element is discovered ne0 is
-!        determined and saved
+!           mass matrix contribution
 !
-         endif
-         if((lakonl(7:7).eq.'A').or.(mortar.eq.0)) then
-            call springstiff_n2f(xl,elas,konl,voldl,s,imat,elcon,nelcon,
-     &      ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
-     &      nplicon,npmat_,iperturb,springarea(1,konl(nope+1)),nmethod,
-     &      mi,ne0,nstate_,xstateini,xstate,reltime,nasym)
-         elseif(mortar.eq.1) then
-            iloc=kon(indexe+nope+1)
-            jfaces=kon(indexe+nope+2)
-            igauss=kon(indexe+nope+1) 
-            call springstiff_f2f(xl,elas,voldl,s,imat,elcon,nelcon,
-     &        ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
-     &        nplicon,npmat_,iperturb,springarea(1,iloc),nmethod,
-     &        mi,ne0,nstate_,xstateini,xstate,reltime,
-     &        nasym,iloc,jfaces,igauss,pslavsurf,
-     &        pmastsurf,clearini,kscale)
+            if(mass.eq.1) then
+               do i1=1,3
+                  sm(i1,i1)=elcon(1,1,imat)
+               enddo
+            endif
+!
+!           contribution to the rhs
+!
+            if(rhsi.eq.1) then
+               if(om.gt.0.d0) then
+                  do i1=1,3
+!
+!                   computation of the global coordinates of the
+!                   mass node
+!
+                     if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
+                        q(i1)=xl(i1,1)
+                     else
+                        q(i1)=xl(i1,1)+voldl(i1,1)
+                     endif
+!                       
+                     q(i1)=q(i1)-p1(i1)
+                  enddo
+                  const=q(1)*p2(1)+q(2)*p2(2)+q(3)*p2(3)
+!
+!                 inclusion of the centrifugal force into the body force
+!
+                  do i1=1,3
+                     ff(i1)=bodyf(i1)+(q(i1)-const*p2(i1))*om
+                  enddo
+               else
+                  do i1=1,3
+                     ff(i1)=bodyf(i1)
+                  enddo
+               endif
+            endif
+!            
          endif
          return
       endif
@@ -512,7 +678,7 @@ c     Bernhardi end
                   imat=ielmat(ilayer,nelem)
                   amat=matname(imat)
                   if(norien.gt.0) then
-                     iorien=ielorien(ilayer,nelem)
+                     iorien=max(0,ielorien(ilayer,nelem))
                   else
                      iorien=0
                   endif
@@ -539,10 +705,51 @@ c     Bernhardi end
                ze=gauss3d4(3,kk)
                weight=weight3d4(kk)
             elseif(lakonl(4:5).eq.'15') then
-               xi=gauss3d8(1,kk)
-               et=gauss3d8(2,kk)
-               ze=gauss3d8(3,kk)
-               weight=weight3d8(kk)
+               if(lakonl(7:8).ne.'LC') then
+                  xi=gauss3d8(1,kk)
+                  et=gauss3d8(2,kk)
+                  ze=gauss3d8(3,kk)
+                  weight=weight3d8(kk)
+               else
+                  kl=mod(kk,6)
+                  if(kl.eq.0) kl=6
+!
+                  xi=gauss3d10(1,kl)
+                  et=gauss3d10(2,kl)
+                  ze=gauss3d10(3,kl)
+                  weight=weight3d10(kl)
+!
+                  ki=mod(kk,3)
+                  if(ki.eq.0) ki=3
+!
+                  if(kl.eq.1) then
+                     ilayer=ilayer+1
+                     if(ilayer.gt.1) then
+                        do i=1,3
+                           dlayer(i)=dlayer(i)+xlayer(ilayer-1,i)
+                        enddo
+                     endif
+                  endif
+                  ze=2.d0*(dlayer(ki)+(ze+1.d0)/2.d0*xlayer(ilayer,ki))/
+     &                 tlayer(ki)-1.d0
+                  weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!
+!                 material and orientation
+!
+                  imat=ielmat(ilayer,nelem)
+                  amat=matname(imat)
+                  if(norien.gt.0) then
+                     iorien=max(0,ielorien(ilayer,nelem))
+                  else
+                     iorien=0
+                  endif
+!     
+                  if(nelcon(1,imat).lt.0) then
+                     ihyper=1
+                  else
+                     ihyper=0
+                  endif
+               endif
             elseif(lakonl(4:4).eq.'6') then
                xi=gauss3d7(1,kk)
                et=gauss3d7(2,kk)
@@ -567,12 +774,6 @@ c     Bernhardi end
                weight=weight3d8(kk)
             endif
          endif
-c         if(nelem.eq.1) then
-c                  write(*,*) 'kk', kk
-c                  write(*,*) 'coords',xi,et,ze
-c                  write(*,*) 'weight',weight
-c                  write(*,*) 'dlayer',dlayer(ki)
-c         endif
 !
 !           calculation of the shape functions and their derivatives
 !           in the gauss point
@@ -632,7 +833,7 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
 !
          t0l=0.d0
          t1l=0.d0
-         if(ithermal.eq.1) then
+         if(ithermal(1).eq.1) then
             if(lakonl(4:5).eq.'8 ') then
                do i1=1,nope
                   t0l=t0l+t0(konl(i1))/8.d0
@@ -640,14 +841,18 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
                enddo
             elseif(lakonl(4:6).eq.'20 ')then
                nopered=20
-               call lintemp(t0,t1,konl,nopered,kk,t0l,t1l)
+               call lintemp(t0,konl,nopered,kk,t0l)
+               call lintemp(t1,konl,nopered,kk,t1l)
+            elseif(lakonl(4:6).eq.'10T') then
+               call linscal10(t0,konl,t0l,null,shp)
+               call linscal10(t1,konl,t1l,null,shp)
             else
                do i1=1,nope
                   t0l=t0l+shp(4,i1)*t0(konl(i1))
                   t1l=t1l+shp(4,i1)*t1(konl(i1))
                enddo
             endif
-         elseif(ithermal.ge.2) then
+         elseif(ithermal(1).ge.2) then
             if(lakonl(4:5).eq.'8 ') then
                do i1=1,nope
                   t0l=t0l+t0(konl(i1))/8.d0
@@ -655,7 +860,11 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
                enddo
             elseif(lakonl(4:6).eq.'20 ')then
                nopered=20
-               call lintemp_th(t0,vold,konl,nopered,kk,t0l,t1l,mi)
+               call lintemp_th0(t0,konl,nopered,kk,t0l,mi)
+               call lintemp_th1(vold,konl,nopered,kk,t1l,mi)
+            elseif(lakonl(4:6).eq.'10T') then
+               call linscal10(t0,konl,t0l,null,shp)
+               call linscal10(vold,konl,t1l,mi(2),shp)
             else
                do i1=1,nope
                   t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -694,7 +903,7 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
      &        nelem,ithermal,alzero,mattyp,t0l,t1l,
      &        ihyper,istiff,elconloc,eth,kode,plicon,
      &        nplicon,plkcon,nplkcon,npmat_,
-     &        plconloc,mi(1),dtime,nelem,kk,
+     &        plconloc,mi(1),dtime,kk,
      &        xstiff,ncmat_)
          else
             idir=idesvar-3*((idesvar-1)/3)
@@ -703,7 +912,7 @@ c         if((iperturb(1).ne.0).and.stiffness.and.(.not.buckling))
      &        nelem,ithermal,alzero,mattyp,t0l,t1l,
      &        ihyper,istiff,elconloc,eth,kode,plicon,
      &        nplicon,plkcon,nplkcon,npmat_,
-     &        plconloc,mi(1),dtime,nelem,kk,
+     &        plconloc,mi(1),dtime,kk,
      &        dxstiff(1,1,1,idir),ncmat_)
          endif
 !
@@ -1189,7 +1398,7 @@ c             if(iperturb(1).eq.0) then
 !         variation of xl2 and xl1 (quads) for sensitivity analysis
 !         ---------------------------------------------------------
 !
-             if(idesvar.gt.0) then
+             if((idesvar.gt.0).and.(icoordinate.eq.1)) then
                 do i=1,nopes
                    node=konl(ifaceq(i,ig))
                    if(node.eq.nodedesi(idesvar)) then
@@ -1197,6 +1406,37 @@ c             if(iperturb(1).eq.0) then
                          xl2(j,i)=
      &                        xl2(j,i)+xdesi(j,idesvar)
                       enddo
+                      if(lakonl(1:5).eq.'C3D20') then
+                         if((lakonl(7:7).eq.'A').or.
+     &                        (lakonl(7:7).eq.'L').or.
+     &                        (lakonl(7:7).eq.'S').or.
+     &                        (lakonl(7:7).eq.'E')) then
+                            node1=ifaceqexp(1,i)
+                            node2=ifaceqexp(2,i)
+                            do j=1,3
+                               if(iactive.le.8) then
+                                  xl2(j,node1)=xl2(j,node1)+
+     &                                 xdesi(j,idesvar)
+                                  xl2(j,node2)=xl2(j,node2)+
+     &                                 xdesi(j,idesvar)
+                               else
+                                  xl2(j,node1)=xl2(j,node1)+
+     &                                 xdesi(j,idesvar)
+                               endif      
+                            enddo      
+                         endif  
+                      elseif(lakonl(1:4).eq.'C3D8') then
+                         if((lakonl(7:7).eq.'A').or.
+     &                        (lakonl(7:7).eq.'L').or.
+     &                        (lakonl(7:7).eq.'S').or.
+     &                        (lakonl(7:7).eq.'E')) then
+                            node1=ifaceqexp(1,i)
+                            do j=1,3
+                               xl2(j,node1)=xl2(j,node1)+
+     &                              xdesi(j,idesvar)
+                            enddo      
+                         endif  
+                      endif
                       exit
                    endif
                 enddo
@@ -1229,7 +1469,7 @@ c             if(iperturb(1).eq.0) then
 !         variation of xl2 and xl1 (tets) for sensitivity analysis
 !         ---------------------------------------------------------
 !
-             if(idesvar.gt.0) then
+             if((idesvar.gt.0).and.(icoordinate.eq.1)) then
                 do i=1,nopes
                    node=konl(ifacet(i,ig))
                    if(node.eq.nodedesi(idesvar)) then
@@ -1269,7 +1509,7 @@ c             if(iperturb(1).eq.0) then
 !         variation of xl2 and xl1 (wedge) for sensitivity analysis
 !         ---------------------------------------------------------
 !
-             if(idesvar.gt.0) then
+             if((idesvar.gt.0).and.(icoordinate.eq.1)) then
                 do i=1,nopes
                    node=konl(ifacew(i,ig))
                    if(node.eq.nodedesi(idesvar)) then
@@ -1277,6 +1517,26 @@ c             if(iperturb(1).eq.0) then
                          xl2(j,i)=
      &                        xl2(j,i)+xdesi(j,idesvar)
                       enddo
+                      if(lakonl(1:5).eq.'C3D15') then
+                         if((lakonl(7:7).eq.'A').or.
+     &                        (lakonl(7:7).eq.'L').or.
+     &                        (lakonl(7:7).eq.'S').or.
+     &                        (lakonl(7:7).eq.'E')) then
+                            node1=ifacewexp(1,i)
+                            node2=ifacewexp(2,i)
+                            do j=1,3
+                               if(iactive.le.6) then
+                                  xl2(j,node1)=xl2(j,node1)+
+     &                                 xdesi(j,idesvar)
+                                  xl2(j,node2)=xl2(j,node2)+
+     &                                 xdesi(j,idesvar)
+                               else
+                                  xl2(j,node1)=xl2(j,node1)+
+     &                                 xdesi(j,idesvar)
+                               endif      
+                            enddo      
+                         endif  
+                      endif
                       exit
                    endif
                 enddo
@@ -1312,16 +1572,12 @@ c             if(iperturb(1).eq.0) then
              endif
 !
              if(rhsi.eq.1) then
-                if(nopes.eq.9) then
-                   call shape9q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.8) then
+                if(nopes.eq.8) then
                    call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.4) then
                    call shape4q(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.6) then
                    call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.7) then
-                   call shape7tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 else
                    call shape3tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
                 endif
@@ -1411,19 +1667,14 @@ c    Bernhardi end
 !            three-dimensional thermomechanical Applications,
 !            Wiley, 2004, p 153, eqn. (3.54).
 !
-c             elseif((mass.eq.1).and.(iperturb(1).ne.0)) then
              elseif((mass.eq.1).and.
      &            ((iperturb(1).eq.1).or.(iperturb(2).eq.1))) then
-                if(nopes.eq.9) then
-                   call shape9q(xi,et,xl1,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.8) then
+                if(nopes.eq.8) then
                    call shape8q(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.4) then
                    call shape4q(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 elseif(nopes.eq.6) then
                    call shape6tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
-                elseif(nopes.eq.7) then
-                   call shape7tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 else
                    call shape3tri(xi,et,xl1,xsj2,xs2,shp2,iflag)
                 endif
@@ -1438,7 +1689,6 @@ c             elseif((mass.eq.1).and.(iperturb(1).ne.0)) then
                          coords(k)=coords(k)+xl1(k,j)*shp2(4,j)
                       enddo
                    enddo
-c                   read(sideload(id)(2:2),'(i1)') jltyp
                    jltyp=ichar(sideload(id)(2:2))-48
                    jltyp=jltyp+20
                    iscale=1
@@ -1460,7 +1710,6 @@ c                   read(sideload(id)(2:2),'(i1)') jltyp
                          coords(k)=coords(k)+xl2(k,j)*shp2(4,j)
                       enddo
                    enddo
-c                   read(sideload(id)(2:2),'(i1)') jltyp
                    jltyp=ichar(sideload(id)(2:2))-48
 !
                    entity='T'
@@ -1472,7 +1721,6 @@ c                   read(sideload(id)(2:2),'(i1)') jltyp
                    call interpolsubmodel(integerglob,doubleglob,stress,
      &                  coords,iselect,six,iface,tieset,istartset,
      &                  iendset,ialset,ntie,entity)
-c                   write(*,*) 'e_c3d ',(stress(k),k=1,6)
 !
                    stre(1,1)=stress(1)
                    stre(1,2)=stress(4)
@@ -1596,40 +1844,11 @@ c     Bernhardi end
 !     for axially symmetric and plane stress/strain elements: 
 !     complete s and sm
 !
-      if(((lakonl(4:5).eq.'8 ').or.
-     &    ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
-     &   ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'S').or.
-     &    (lakonl(7:7).eq.'E'))) then
-         do i=1,60
-            do j=i,60
-               k=abs(iperm(i))
-               l=abs(iperm(j))
-               if(k.gt.l) then
-                  m=k
-                  k=l
-                  l=m
-               endif
-               sax(i,j)=s(k,l)*iperm(i)*iperm(j)/(k*l)
-            enddo
-         enddo
-         do i=1,60
-            do j=i,60
-               s(i,j)=s(i,j)+sax(i,j)
-            enddo
-         enddo
-!
-         if((nload.ne.0).or.(nbody.ne.0)) then
-            do i=1,60
-               k=abs(iperm(i))
-               ffax(i)=ff(k)*iperm(i)/k
-            enddo
-            do i=1,60
-               ff(i)=ff(i)+ffax(i)
-            enddo
-         endif
-!
-         if(mass.eq.1) then
-            summass=2.d0*summass
+      if(intscheme.eq.0) then
+         if(((lakonl(4:5).eq.'8 ').or.
+     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'S').or.
+     &        (lakonl(7:7).eq.'E'))) then
             do i=1,60
                do j=i,60
                   k=abs(iperm(i))
@@ -1639,17 +1858,48 @@ c     Bernhardi end
                      k=l
                      l=m
                   endif
-                  sax(i,j)=sm(k,l)*iperm(i)*iperm(j)/(k*l)
+                  sax(i,j)=s(k,l)*iperm(i)*iperm(j)/(k*l)
                enddo
             enddo
             do i=1,60
                do j=i,60
-                  sm(i,j)=sm(i,j)+sax(i,j)
+                  s(i,j)=s(i,j)+sax(i,j)
                enddo
             enddo
+!     
+            if((nload.ne.0).or.(nbody.ne.0)) then
+               do i=1,60
+                  k=abs(iperm(i))
+                  ffax(i)=ff(k)*iperm(i)/k
+               enddo
+               do i=1,60
+                  ff(i)=ff(i)+ffax(i)
+               enddo
+            endif
+!     
+            if(mass.eq.1) then
+               summass=2.d0*summass
+               do i=1,60
+                  do j=i,60
+                     k=abs(iperm(i))
+                     l=abs(iperm(j))
+                     if(k.gt.l) then
+                        m=k
+                        k=l
+                        l=m
+                     endif
+                     sax(i,j)=sm(k,l)*iperm(i)*iperm(j)/(k*l)
+                  enddo
+               enddo
+               do i=1,60
+                  do j=i,60
+                     sm(i,j)=sm(i,j)+sax(i,j)
+                  enddo
+               enddo
+            endif
          endif
       endif
-!
+!     
       if((mass.eq.1).and.(iexpl.gt.1)) then
 !
 !        scaling the diagonal terms of the mass matrix such that the total mass
@@ -1698,51 +1948,135 @@ c            alp=.2215d0
 !     Assigning the element stiffness matrix and the external load 
 !     vector to the corresponding matrices/vectors
 !     
-      if(idesvar.eq.0) then
-!     Stiffness matrix
-         do i=1,3*nope
-            do j=1,3*nope
-               s0(i,j)=s(i,j)
-            enddo
-         enddo
+      if(sigma.le.0.d0) then
+!
+         if((ieigenfrequency.ne.1).and.(iperturb(2).eq.1)) then
+!
+!           nonlinear geometric calculation: dK/ds does not have
+!           to be calculated (except for eigenfrequency calculations)
+!
+            if(idesvar.eq.0) then
 !     load vector
-         if((rhsi.eq.1).and.(idist.eq.1)) then
-            do i=1,3*nope
-               ff0(i)=ff(i)
-            enddo
-         endif
-      else
+               if((rhsi.eq.1).and.(idist.eq.1)) then
+                  do i=1,3*nope
+                     ff0(i)=ff(i)
+                  enddo
+               endif
+            else
+!     load vector
+               if((rhsi.eq.1).and.(idist.eq.1)) then
+                  do i=1,3*nope
+                     dfl(idesloc,i)=(ff(i)-ff0(i))/distmin
+                  enddo
+               else
+                  do i=1,3*nope
+                     dfl(idesloc,i)=0.d0
+                  enddo
+               endif
+            endif
+         else
+!
+!           linear geometrical calculation: calculate dK/ds
+!
+            if(idesvar.eq.0) then
 !     Stiffness matrix
-         do i=1,3*nope
-            do j=1,3*nope
-               ds1(i,j)=s(i,j)-s0(i,j)
-            enddo
-         enddo
-         do i=1,3*nope
-            l=0
-            do j=1,nope
-               do k=1,3
-                  l=l+1
-                  if(l.ge.i) then
-                     dfminds(idesvar,i)=dfminds(idesvar,i)
-     &                                 +ds1(i,l)*vl(k,j)
-                  else
-                     dfminds(idesvar,i)=dfminds(idesvar,i)
-     &                                 +ds1(l,i)*vl(k,j)
-                  endif
+               do i=1,3*nope
+                  do j=1,3*nope
+                     s0(i,j)=s(i,j)
+                  enddo
+               enddo
+!     load vector
+               if((rhsi.eq.1).and.(idist.eq.1)) then
+                  do i=1,3*nope
+                     ff0(i)=ff(i)
+                  enddo
+               endif
+            else
+!     Stiffness matrix
+               do i=1,3*nope
+                  do j=1,3*nope
+                     ds1(i,j)=s(i,j)-s0(i,j)
+                  enddo
+               enddo
+               do i=1,3*nope
+                  l=0
+                  do j=1,nope
+                     do k=1,3
+                        l=l+1
+                        if(l.ge.i) then
+                           dfl(idesloc,i)=dfl(idesloc,i)
+     &                          +ds1(i,l)*vl(k,j)
+                        else
+                           dfl(idesloc,i)=dfl(idesloc,i)
+     &                          +ds1(l,i)*vl(k,j)
+                        endif
+                     enddo
+                  enddo
+               enddo
+!     load vector
+               if((rhsi.eq.1).and.(idist.eq.1)) then
+                  do i=1,3*nope
+                     dfl(idesloc,i)=(ff(i)-ff0(i)-dfl(idesloc,i))
+     &                    /distmin
+                  enddo
+               else
+                  do i=1,3*nope
+                     dfl(idesloc,i)=-dfl(idesloc,i)/distmin
+                  enddo
+               endif
+            endif
+         endif
+!     
+      else
+!
+!        eigenfrequency sensitivity: take K-sigma*M instead of K
+!
+         if(idesvar.eq.0) then
+!     Stiffness matrix
+            do i=1,3*nope
+               do j=1,3*nope
+                  s0(i,j)=s(i,j)-sigma*sm(i,j)
                enddo
             enddo
-         enddo
 !     load vector
-         if((rhsi.eq.1).and.(idist.eq.1)) then
-            do i=1,3*nope
-               dfminds(idesvar,i)=(ff(i)-ff0(i)-dfminds(idesvar,i))
-     &                           /distmin
-            enddo
+            if((rhsi.eq.1).and.(idist.eq.1)) then
+               do i=1,3*nope
+                  ff0(i)=ff(i)
+               enddo
+            endif
          else
+!     Stiffness matrix
             do i=1,3*nope
-               dfminds(idesvar,i)=-dfminds(idesvar,i)/distmin
+               do j=1,3*nope
+                  ds1(i,j)=s(i,j)-sigma*sm(i,j)-s0(i,j)
+               enddo
             enddo
+            do i=1,3*nope
+               l=0
+               do j=1,nope
+                  do k=1,3
+                     l=l+1
+                     if(l.ge.i) then
+                        dfl(idesloc,i)=dfl(idesloc,i)
+     &                       +ds1(i,l)*vl(k,j)
+                     else
+                        dfl(idesloc,i)=dfl(idesloc,i)
+     &                       +ds1(l,i)*vl(k,j)
+                     endif
+                  enddo
+               enddo
+            enddo
+!     load vector
+            if((rhsi.eq.1).and.(idist.eq.1)) then
+               do i=1,3*nope
+                  dfl(idesloc,i)=(ff(i)-ff0(i)-dfl(idesloc,i))
+     &                 /distmin
+               enddo
+            else
+               do i=1,3*nope
+                  dfl(idesloc,i)=-dfl(idesloc,i)/distmin
+               enddo
+            endif
          endif
       endif
 !     
