@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -26,7 +26,8 @@
      &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,springarea,
      &  plkcon,nplkcon,npmat_,ncmat_,elcon,nelcon,lakon,
      &  pslavsurf,pmastsurf,mortar,clearini,plicon,nplicon,ipkon,
-     &  ielprop,prop,iponoel,inoel)
+     &  ielprop,prop,iponoel,inoel,sti,xstateini,xstate,nstate_,
+     &  network,ipobody,xbody,ibody)
 !
 !     computation of the element matrix and rhs for the element with
 !     the topology in konl
@@ -48,16 +49,16 @@
       character*80 matname(*),amat
 !
       integer konl(26),ifaceq(8,6),nelemload(2,*),nk,nelem,nmethod,
-     &  mattyp,ithermal(2),iperturb,nload,idist,i,j,k,i1,j1,l,m,
+     &  mattyp,ithermal(*),iperturb(*),nload,idist,i,j,k,i1,j1,l,m,
      &  ii,jj,id,ipointer,ig,kk,istiff,iperm(20),ipompc(*),mi(*),
      &  nrhcon(*),ielmat(mi(3),*),ielorien(mi(3),*),nodempc(3,*),nmpc,
-     &  ntmat_,nope,nopes,norien,iexpl,imat,mint2d,ikmpc(*),iloc,
+     &  ntmat_,nope,nopes,norien,iexpl,imat,mint2d,ikmpc(*),
      &  mint3d,ifacet(6,4),nopev,iorien,ilmpc(*),kode,jfaces,null,
      &  ifacew(8,5),intscheme,ipointeri,ipointerj,ncocon(2,*),
      &  nshcon(*),iinc,istep,jltyp,nfield,node,iflag,iscale,ielprop(*),
      &  nplkcon(0:ntmat_,*),nelcon(2,*),npmat_,ncmat_,i2,ipkon(*),
      &  iemchange,kon(*),mortar,nplicon(0:ntmat_,*),indexe,igauss,
-     &  iponoel(*),inoel(2,*)
+     &  iponoel(*),inoel(2,*),nstate_,network,ipobody(2,*),ibody(3,*)
 !
       real*8 co(3,*),xl(3,26),shp(4,26),xstiff(27,mi(1),*),
      &  s(60,60),w(3,3),ff(60),shpj(4,26),sinktemp,xs2(3,7),
@@ -70,23 +71,13 @@
      &  field,areaj,sax(60,60),ffax(60),coefmpc(*),tl2(8),
      &  voldl(0:mi(2),26),springarea(2,*),plkcon(0:2*npmat_,ntmat_,*),
      &  elcon(0:ncmat_,ntmat_,*),elconloc(21),pslavsurf(3,*),
-     &  pmastsurf(2,*),clearini(3,9,*),plicon(0:2*npmat_,ntmat_,*)
+     &  pmastsurf(2,*),clearini(3,9,*),plicon(0:2*npmat_,ntmat_,*),
+     &  sti(6,mi(1),*),xstate(nstate_,mi(1),*),xbody(7,*),
+     &  xstateini(nstate_,mi(1),*),heatnod,heatfac
 !
       real*8 dtime,physcon(*)
 !
-      intent(in) co,nk,kon,lakonl,
-     &  nelem,rhcon,nrhcon,ielmat,ielorien,norien,orab,
-     &  ntmat_,t0,t1,ithermal,vold,iperturb,nelemload,
-     &  sideload,nload,idist,iexpl,dtime,
-     &  matname,mi,mass,stiffness,buckling,rhsi,intscheme,
-     &  physcon,shcon,nshcon,cocon,ncocon,ttime,time,istep,iinc,
-     &  xstiff,xloadold,reltime,
-     &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,
-     &  plkcon,nplkcon,npmat_,ncmat_,elcon,nelcon,lakon,
-     &  pslavsurf,pmastsurf,mortar,clearini,plicon,nplicon,ipkon,
-     &  ielprop,prop
 !
-      intent(inout) s,sm,ff,xload,nmethod,springarea
 !
       include "gauss.f"
 !
@@ -114,12 +105,13 @@
       timeend(2)=ttime+time
 !
       summass=0.d0
+      heatfac=0.d0
 !
       indexe=ipkon(nelem)
       imat=ielmat(1,nelem)
       amat=matname(imat)
       if(norien.gt.0) then
-         iorien=ielorien(1,nelem)
+         iorien=max(0,ielorien(1,nelem))
       else
          iorien=0
       endif
@@ -161,7 +153,8 @@ c            read(lakonl(8:8),'(i1)') nope
             nope=ichar(lakonl(8:8))-47
 c            nope=nope+1
          endif
-      elseif(lakonl(1:2).eq.'D ') then
+      elseif((lakonl(1:2).eq.'D ').or.
+     &       ((lakonl(1:1).eq.'D').and.(network.eq.1))) then
          nope=3
       endif
 !
@@ -297,17 +290,16 @@ c            nope=nope+1
      &                 npmat_,iperturb,springarea(1,konl(nope+1)),mi,
      &                 timeend,matname,konl(nope),nelem,istep,iinc)
                   elseif(mortar.eq.1) then
-                     iloc=kon(indexe+nope+1)
                      jfaces=kon(indexe+nope+2)
                      igauss=kon(indexe+nope+1) 
                      node=0
                      call springstiff_f2f_th(xl,voldl,s,imat,elcon,
      &                    nelcon,ncmat_,ntmat_,nope,lakonl,kode,
      &                    elconloc,plicon,nplicon,npmat_,
-     &                    springarea(1,iloc),
+     &                    springarea(1,igauss),
      &                    nmethod,mi,reltime,jfaces,igauss,pslavsurf,
      &                    pmastsurf,clearini,matname,plkcon,nplkcon,
-     &                    node,nelem,istep,iinc)
+     &                    node,nelem,istep,iinc,timeend)
                   endif
                endif
             elseif(lakonl(7:7).eq.'F') then
@@ -318,9 +310,11 @@ c            nope=nope+1
      &              nelem,nload,lakon,xload,istep,time,ttime,dtime,
      &              sideload,vold,mi,xloadold,reltime,nmethod,s,
      &              iinc,iponoel,inoel,ielprop,prop,ielmat,shcon,
-     &              nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon)
+     &              nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon,
+     &              ipobody,xbody,ibody)
             endif
-         elseif(lakonl(1:2).eq.'D ') then
+         elseif((lakonl(1:2).eq.'D ').or.
+     &          ((lakonl(1:1).eq.'D').and.(network.eq.1))) then
             do i=1,3
                do j=1,i
                   s(i,j)=0.d0
@@ -580,7 +574,8 @@ c            nope=nope+1
                      call dflux(xload(1,id),t1l,istep,iinc,timeend,
      &                 nelem,kk,pgauss,jltyp,temp,press,sideload(id),
      &                 areaj,vold,co,lakonl,konl,ipompc,nodempc,coefmpc,
-     &                 nmpc,ikmpc,ilmpc,iscale,mi)
+     &                 nmpc,ikmpc,ilmpc,iscale,mi,sti,xstateini,xstate,
+     &                 nstate_,dtime)
                      if((nmethod.eq.1).and.(iscale.ne.0))
      &                    xload(1,id)=xloadold(1,id)+
      &                   (xload(1,id)-xloadold(1,id))*reltime
@@ -722,16 +717,16 @@ c            read(sideload(id)(2:2),'(i1)') ig
                 weight=weight2d4(i)
              endif
 !
-             if(nopes.eq.9) then
-                call shape9q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-             elseif(nopes.eq.8) then
+c             if(nopes.eq.9) then
+c                call shape9q(xi,et,xl2,xsj2,xs2,shp2,iflag)
+             if(nopes.eq.8) then
                 call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
              elseif(nopes.eq.4) then
                 call shape4q(xi,et,xl2,xsj2,xs2,shp2,iflag)
              elseif(nopes.eq.6) then
                 call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
-             elseif(nopes.eq.7) then
-                call shape7tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+c             elseif(nopes.eq.7) then
+c                call shape7tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
              else
                 call shape3tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
              endif
@@ -756,7 +751,6 @@ c            read(sideload(id)(2:2),'(i1)') ig
                       coords(k)=coords(k)+xl2(k,j)*shp2(4,j)
                    enddo
                 enddo
-c                read(sideload(id)(2:2),'(i1)') jltyp
                 jltyp=ichar(sideload(id)(2:2))-48
                 jltyp=jltyp+10
                 if(sideload(id)(1:1).eq.'S') then
@@ -764,25 +758,20 @@ c                read(sideload(id)(2:2),'(i1)') jltyp
                    call dflux(xload(1,id),temp,istep,iinc,timeend,
      &               nelem,i,coords,jltyp,temp,press,sideload(id),
      &               areaj,vold,co,lakonl,konl,
-     &               ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,iscale,mi)
-                   if((nmethod.eq.1).and.(iscale.ne.0))
-     &                   xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
+     &               ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,iscale,mi,
+     &               sti,xstateini,xstate,nstate_,dtime)
                 elseif(sideload(id)(1:1).eq.'F') then
                    node=nelemload(2,id)
                    call film(xload(1,id),sinktemp,temp,istep,
      &               iinc,timeend,nelem,i,coords,jltyp,field,nfield,
      &               sideload(id),node,areaj,vold,mi,
      &               ipkon,kon,lakon,iponoel,inoel,ielprop,prop,ielmat,
-     &               shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon)
-                   if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
+     &               shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon,
+     &               ipobody,xbody,ibody,heatnod,heatfac)
                 elseif(sideload(id)(1:1).eq.'R') then
                    call radiate(xload(1,id),xload(2,id),temp,istep,
      &               iinc,timeend,nelem,i,coords,jltyp,field,nfield,
      &               sideload(id),node,areaj,vold,mi,iemchange)
-                   if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
                 endif
              endif
                 
@@ -803,8 +792,8 @@ c                read(sideload(id)(2:2),'(i1)') jltyp
                    ff(ipointer)=ff(ipointer)+shp2(4,k)*xload(1,id)
      &                  *areaj
                 elseif(sideload(id)(1:1).eq.'F') then
-                   ff(ipointer)=ff(ipointer)-shp2(4,k)*xload(1,id)
-     &                  *(temp-sinktemp)*areaj
+                   ff(ipointer)=ff(ipointer)+shp2(4,k)*areaj*
+     &                  (heatfac-xload(1,id)*(temp-sinktemp))
                 elseif(sideload(id)(1:1).eq.'R') then
                    ff(ipointer)=ff(ipointer)-shp2(4,k)*physcon(2)*
      &                  xload(1,id)*((temp-physcon(1))**4-
@@ -850,41 +839,10 @@ c                read(sideload(id)(2:2),'(i1)') jltyp
 !     for axially symmetric and plane stress/strain elements: 
 !     complete s and sm
 !
-      if(((lakonl(4:5).eq.'8 ').or.
-     &    ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
-     &   ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E'))) then
-         do i=1,20
-            do j=i,20
-               k=iperm(i)
-               l=iperm(j)
-               if(k.gt.l) then
-                  m=k
-                  k=l
-                  l=m
-               endif
-               sax(i,j)=s(k,l)
-            enddo
-         enddo
-         do i=1,20
-            do j=i,20
-               s(i,j)=s(i,j)+sax(i,j)
-            enddo
-         enddo
-!
-!        special treatment of plane stress elements since lateral
-!        heating is allowed (orthogonal to the plane)
-!
-         if(nload.ne.0) then
-            do i=1,20
-               k=iperm(i)
-               ffax(i)=ff(k)
-            enddo
-            do i=1,20
-               ff(i)=ff(i)+ffax(i)
-            enddo
-         endif
-!
-         if(mass.eq.1) then
+      if(intscheme.eq.0) then
+         if(((lakonl(4:5).eq.'8 ').or.
+     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E'))) then
             do i=1,20
                do j=i,20
                   k=iperm(i)
@@ -894,14 +852,68 @@ c                read(sideload(id)(2:2),'(i1)') jltyp
                      k=l
                      l=m
                   endif
-                  sax(i,j)=sm(k,l)
+                  sax(i,j)=s(k,l)
                enddo
             enddo
             do i=1,20
                do j=i,20
-                  sm(i,j)=sm(i,j)+sax(i,j)
+                  s(i,j)=s(i,j)+sax(i,j)
                enddo
             enddo
+!     
+!     special treatment of plane stress elements since lateral
+!     heating is allowed (orthogonal to the plane)
+!     
+            if(nload.ne.0) then
+               do i=1,20
+                  k=iperm(i)
+                  ffax(i)=ff(k)
+               enddo
+               do i=1,20
+                  ff(i)=ff(i)+ffax(i)
+               enddo
+            endif
+!     
+            if(mass.eq.1) then
+               do i=1,20
+                  do j=i,20
+                     k=iperm(i)
+                     l=iperm(j)
+                     if(k.gt.l) then
+                        m=k
+                        k=l
+                        l=m
+                     endif
+                     sax(i,j)=sm(k,l)
+                  enddo
+               enddo
+               do i=1,20
+                  do j=i,20
+                     sm(i,j)=sm(i,j)+sax(i,j)
+                  enddo
+               enddo
+            endif
+         endif
+      else
+!
+!        only 2-D scheme is reduced for intscheme=1
+!
+         if(((lakonl(4:5).eq.'8 ').or.
+     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E'))) then
+!     
+!           special treatment of plane stress elements since lateral
+!           heating is allowed (orthogonal to the plane)
+!     
+            if(nload.ne.0) then
+               do i=1,20
+                  k=iperm(i)
+                  ffax(i)=ff(k)
+               enddo
+               do i=1,20
+                  ff(i)=ff(i)+ffax(i)
+               enddo
+            endif
          endif
       endif
 !
