@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,8 @@
       subroutine elements(inpc,textpart,kon,ipkon,lakon,nkon,ne,ne_,
      &  set,istartset,iendset,ialset,nset,nset_,nalset,nalset_,mi,
      &  ixfree,iponor,xnor,istep,istat,n,iline,ipol,inl,ipoinp,inp,
-     &  iaxial,ipoinpc,solid,cfd,network,filab,nlabel,out3d)
+     &  iaxial,ipoinpc,solid,network,filab,nlabel,out3d,iuel,
+     &  nuel_,ier,iparentel)
 !
 !     reading the input deck: *ELEMENT
 !
@@ -37,19 +38,23 @@
      &  nset_,nalset,nalset_,istep,istat,n,key,i,ielset,js,k,nn,
      &  nteller,j,ipkon(*),nkon,nope,indexe,mi(*),ipos,indexy,ixfree,
      &  iponor(2,*),nopeexp,iline,ipol,inl,ipoinp(2,*),inp(3,*),
-     &  iaxial,ipoinpc(0:*),cfd,nlabel,network
+     &  iaxial,ipoinpc(0:*),nlabel,network,iuel(4,*),nuel_,
+     &  id,four,number,ndof,intpoints,ier,iparentel(*),iparent
 !
       real*8 xnor(*)
 !
-      if(istep.gt.0) then
-         write(*,*) '*ERROR in elements: *ELEMENT should be placed'
-         write(*,*) '  before all step definitions'
-         call exit(201)
-      endif
+c      if(istep.gt.0) then
+c         write(*,*) '*ERROR reading *ELEMENT: *ELEMENT should be placed'
+c         write(*,*) '  before all step definitions'
+c         ier=1
+c         return
+c      endif
 !
       indexy=-1
       ielset=0
       beamshell=.false.
+      four=4
+      iparent=0
 !
       label='        '
 !
@@ -59,10 +64,11 @@
          if(textpart(i)(1:6).eq.'ELSET=') then
             elset=textpart(i)(7:86)
             if(textpart(i)(87:87).ne.' ') then
-               write(*,*) '*ERROR in elements: set name too long'
+               write(*,*) '*ERROR reading *ELEMENT: set name too long'
                write(*,*) '       (more than 80 characters)'
                write(*,*) '       set name:',textpart(i)(1:132)
-               call exit(201)
+               ier=1
+               return
             endif
             elset(81:81)=' '
             ipos=index(elset,' ')
@@ -81,8 +87,10 @@
 !
                      nn=iendset(js)-istartset(js)+1
                      if(nalset+nn.gt.nalset_) then
-                        write(*,*)'*ERROR in elements: increase nalset_'
-                        call exit(201)
+                        write(*,*)
+     &                   '*ERROR reading *ELEMENT: increase nalset_'
+                        ier=1
+                        return
                      endif
                      do k=1,nn
                         ialset(nalset+k)=ialset(istartset(js)+k-1)
@@ -107,8 +115,9 @@
 !
             nset=nset+1
             if(nset.gt.nset_) then
-               write(*,*) '*ERROR in elements: increase nset_'
-               call exit(201)
+               write(*,*) '*ERROR reading *ELEMENT: increase nset_'
+               ier=1
+               return
             endif
             js=nset
             istartset(js)=nalset+1
@@ -123,6 +132,13 @@
             if((label(1:2).eq.'DC').and.(label(1:7).ne.'DCOUP3D')) then
                label(1:7)=label(2:8)
                label(8:8)=' '
+!
+!              mapping 2D beam and truss elements onto 3D elements
+!
+            elseif(label(1:3).eq.'B21') then
+               label='B31     '
+            elseif(label(1:4).eq.'T2D2') then
+               label='T3D2    '
             endif
 !
 !           full integration quadratic hexahedral element
@@ -141,7 +157,6 @@
 !           (including such which are expanded into one)
 !
      &         (label.eq.'C3D20R  ').or.
-c     &         (label.eq.'C3D20RI ').or.
      &         (label.eq.'CPE8R   ').or.
      &         (label.eq.'CPS8R   ').or.
      &         (label.eq.'CAX8R   ').or.
@@ -181,6 +196,7 @@ c    Bernhardi end
 !           quadratic tetrahedral element
 !
      &         (label.eq.'C3D10   ').or.
+     &         (label.eq.'C3D10T  ').or.
 !
 !           linear tetrahedral element
 !
@@ -213,6 +229,8 @@ c    Bernhardi end
 !           spring element
 !
      &         (label.eq.'SPRINGA ').or.
+     &         (label.eq.'SPRING1 ').or.
+     &         (label.eq.'SPRING2 ').or.
 !
 !           dashpot element
 !
@@ -223,9 +241,10 @@ c    Bernhardi end
 !           3D fluid element
 !
             elseif((label.eq.'F3D8    ').or.
+     &             (label.eq.'F3D8R   ').or.
      &             (label.eq.'F3D4    ').or.
      &             (label.eq.'F3D6    ')) then
-               cfd=1
+c               cfd=1
 !
 !           distributing coupling element
 !
@@ -234,27 +253,45 @@ c    Bernhardi end
 !           network element
 !
             elseif(label(1:1).eq.'D') then
-               network=1
+               network=3
+!
+!           mass element
+!
+            elseif(label(1:4).eq.'MASS') then
+!
+!           user element
+!
+            elseif(label(1:1).eq.'U') then
+!
+!           unknown element type
+!
             else
-               write(*,*) '*ERROR in elements:'
+               write(*,*) '*ERROR reading *ELEMENT:'
                write(*,*) label,' is an unknown element type'
-               call exit(201)
+               ier=1
+               return
             endif
 !
             if(label(1:3).eq.'CAX') then
                iaxial=180
-            elseif((label(1:3).eq.'B32').or.
+            elseif((label(1:2).eq.'S3').or.
+     &             (label(1:2).eq.'S4').or.
      &             (label(1:2).eq.'S6').or.
      &             (label(1:2).eq.'S8').or.
-     &             (label(1:3).eq.'B31').or.
-     &             (label(1:2).eq.'S3').or.
-     &             (label(1:2).eq.'S4')) then
+     &             (label(1:1).eq.'B')) then
                beamshell=.true.
             endif
 !
+         elseif(textpart(i)(1:7).eq.'PARENT=') then
+            read(textpart(i)(8:17),'(i10)',iostat=istat) iparent
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*ELEMENT%",ier)
+               return
+            endif
          else
             write(*,*) 
-     &        '*WARNING in elements: parameter not recognized:'
+     &        '*WARNING reading *ELEMENT: parameter not recognized:'
             write(*,*) '         ',
      &                 textpart(i)(1:index(textpart(i),' ')-1)
             call inputwarning(inpc,ipoinpc,iline,
@@ -263,11 +300,11 @@ c    Bernhardi end
       enddo loop
 !
       if(label.eq.'        ') then
-         write(*,*) '*ERROR in elements: element type is lacking'
+         write(*,*) '*ERROR reading *ELEMENT: element type is lacking'
          write(*,*) '       '
          call inputerror(inpc,ipoinpc,iline,
-     &"*ELEMENT%")
-         call exit(201)
+     &        "*ELEMENT%",ier)
+         return
       endif
 !
 !     nope is the number of nodes per element as defined in the input
@@ -307,8 +344,6 @@ c     Bernhardi end
       elseif(label(4:4).eq.'8') then
          nope=8
          nopeexp=8
-c      elseif((label(1:5).eq.'CPE4 ').or.(label(1:5).eq.'CPS4 ').or.
-c     &        (label(1:5).eq.'CAX4 ').or.(label(1:3).eq.'S4 ')) then
       elseif(label(1:3).eq.'S4 ') then
          nope=4
 !        expanded into C3D8I: 11 nodes per element
@@ -353,6 +388,31 @@ c     &        (label(1:5).eq.'CAX4 ').or.(label(1:3).eq.'S4 ')) then
          label='ESPRNGA1'
          nope=2
          nopeexp=2
+      elseif(label(1:7).eq.'SPRING1') then
+         label='ESPRNG10'
+         nope=1
+         nopeexp=1
+      elseif(label(1:7).eq.'SPRING2') then
+         label='ESPRNG21'
+         nope=2
+         nopeexp=2
+      elseif(label(1:4).eq.'MASS') then
+         nope=1
+         nopeexp=1
+      elseif(label(1:1).eq.'U') then
+         number=ichar(label(2:2))*256**3+
+     &          ichar(label(3:3))*256**2+
+     &          ichar(label(4:4))*256+
+     &          ichar(label(5:5))
+c         read(label(2:3),'(i2)',iostat=istat) number
+         call nidentk(iuel,number,nuel_,id,four)
+         intpoints=iuel(2,id)
+         ndof=iuel(3,id)
+         nope=iuel(4,id)
+         nopeexp=nope
+         write(label(6:6),'(a1)') char(intpoints)
+         write(label(7:7),'(a1)') char(ndof)
+         write(label(8:8),'(a1)') char(nope)
       endif
 !
       do
@@ -371,21 +431,26 @@ c     &        (label(1:5).eq.'CAX4 ').or.(label(1:3).eq.'S4 ')) then
             return
          endif
          read(textpart(1)(1:10),'(i10)',iostat=istat) i
-         if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*ELEMENT%")
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*ELEMENT%",ier)
+            return
+         endif
          if(i.gt.ne_) then
-            write(*,*) '*ERROR in elements: increase ne_'
-            call exit(201)
+            write(*,*) '*ERROR reading *ELEMENT: increase ne_'
+            ier=1
+            return
          endif
 !
 !        check whether element was already defined
 !
          if(ipkon(i).ne.-1) then
-            write(*,*) '*ERROR in elements: element',i
+            write(*,*) '*ERROR reading *ELEMENT: element',i
             write(*,*) '       is already defined'
             write(*,*) '       '
             call inputerror(inpc,ipoinpc,iline,
-     &"*ELEMENT%")
+     &           "*ELEMENT%",ier)
+            return
          endif
 !            
 !        new element
@@ -398,8 +463,11 @@ c     &        (label(1:5).eq.'CAX4 ').or.(label(1:3).eq.'S4 ')) then
 !
          do j=2,min(n,nope+1)
             read(textpart(j)(1:10),'(i10)',iostat=istat) kon(indexe+j-1)
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*ELEMENT%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*ELEMENT%",ier)
+               return
+            endif
          enddo
          nteller=n-1
          if(nteller.lt.nope) then
@@ -407,29 +475,37 @@ c     &        (label(1:5).eq.'CAX4 ').or.(label(1:3).eq.'S4 ')) then
                call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &              ipoinp,inp,ipoinpc)
                if((istat.lt.0).or.(key.eq.1)) then
-                  write(*,*) '*ERROR in elements: element definition'
+                  write(*,*) 
+     &              '*ERROR reading *ELEMENT: element definition'
                   write(*,*) '       incomplete for element ',i
-                  call exit(201)
+                  ier=1
+                  return
                endif
                if(nteller+n.gt.nope) n=nope-nteller
                do j=1,n
                   read(textpart(j)(1:10),'(i10)',iostat=istat) 
      &                  kon(indexe+nteller+j)
-                  if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*ELEMENT%")
+                  if(istat.gt.0) then
+                     call inputerror(inpc,ipoinpc,iline,
+     &                    "*ELEMENT%",ier)
+                     return
+                  endif
                enddo
                nteller=nteller+n
                if(nteller.eq.nope) exit
             enddo
          endif
          ne=max(ne,i)
+!
+         if(iparent.gt.0) iparentel(i)=iparent
 !
 !        assigning element to set
 !
          if(ielset.eq.1) then
             if(nalset+1.gt.nalset_) then
-               write(*,*) '*ERROR in elements: increase nalset_'
-               call exit(201)
+               write(*,*) '*ERROR reading *ELEMENT: increase nalset_'
+               ier=1
+               return
             endif
             nalset=nalset+1
             ialset(nalset)=i
