@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
       subroutine elprints(inpc,textpart,set,
      &  nset,nprint,nprint_,jout,prlab,prset,
      &  nmethod,elprint_flag,nener,ithermal,istep,istat,n,iline,ipol,
-     &  inl,ipoinp,inp,amname,nam,itpamp,idrct,ipoinpc,cfd)
+     &  inl,ipoinp,inp,amname,nam,itpamp,idrct,ipoinpc,nef,ier)
 !
 !     reading the *ELEMENT PRINT cards in the input deck
 !
@@ -33,14 +33,15 @@
       character*81 set(*),elset,prset(*)
       character*132 textpart(16)
 !
-      integer nset,nprint,nprint_,istep,istat,n,i,ii,key,
-     &  jout(2),joutl,ipos,nmethod,nener,ithermal,iline,ipol,inl,
-     &  ipoinp(2,*),inp(3,*),nam,itpamp,idrct,ipoinpc(0:*),cfd
+      integer nset,nprint,nprint_,istep,istat,n,i,ii,key,ier,
+     &  jout(2),joutl,ipos,nmethod,nener,ithermal(*),iline,ipol,inl,
+     &  ipoinp(2,*),inp(3,*),nam,itpamp,idrct,ipoinpc(0:*),nef
 !
       if(istep.lt.1) then
          write(*,*) '*ERROR reading *EL PRINT: *EL PRINT should only be'
          write(*,*) '  used within a *STEP definition'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       elemsys='L'
@@ -59,8 +60,12 @@
      &         (prlab(i)(1:4).eq.'ELSE').or.
      &         (prlab(i)(1:4).eq.'ELKE').or.
      &         (prlab(i)(1:4).eq.'EVOL').or.
+     &         (prlab(i)(1:4).eq.'EMAS').or.
+     &         (prlab(i)(1:4).eq.'EBHE').or.
      &         (prlab(i)(1:4).eq.'SVF ').or.
      &         (prlab(i)(1:4).eq.'HFLF').or.
+     &         (prlab(i)(1:4).eq.'COOR').or.
+     &         (prlab(i)(1:4).eq.'CENT').or.
      &         (prlab(i)(1:4).eq.'HFL ')) cycle
             ii=ii+1
             prlab(ii)=prlab(i)
@@ -91,8 +96,11 @@
           endif
         elseif(textpart(ii)(1:10).eq.'FREQUENCY=') then
            read(textpart(ii)(11:20),'(i10)',iostat=istat) joutl
-           if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*EL PRINT%")
+           if(istat.gt.0) then
+              call inputerror(inpc,ipoinpc,iline,
+     &             "*EL PRINT%",ier)
+              return
+           endif
            if(joutl.eq.0) then
               do
                  call getnewline(inpc,textpart,istat,n,key,iline,ipol,
@@ -106,8 +114,11 @@
            endif
         elseif(textpart(ii)(1:11).eq.'FREQUENCYF=') then
            read(textpart(ii)(12:21),'(i10)',iostat=istat) joutl
-           if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*EL PRINT%")
+           if(istat.gt.0) then
+              call inputerror(inpc,ipoinpc,iline,
+     &             "*EL PRINT%",ier)
+              return
+           endif
            if(joutl.eq.0) then
               do
                  call getnewline(inpc,textpart,istat,n,key,iline,ipol,
@@ -139,13 +150,15 @@
               write(*,*) '*ERROR elprints: time'
               write(*,*) '       points definition',
      &               timepointsname,' is unknown'
-              call exit(201)
+              ier=1
+              return
            endif
            if(idrct.eq.1) then
               write(*,*) '*ERROR reading *EL PRINT: the DIRECT option'
               write(*,*) '       collides with a TIME POINTS '
               write(*,*) '       specification'
-              call exit(201)
+              ier=1
+              return
            endif
            jout(1)=1
            jout(2)=1
@@ -182,7 +195,8 @@
                   cycle
                endif
             elseif((textpart(ii)(1:4).eq.'CEEQ').or.
-     &             (textpart(ii)(1:2).eq.'CE').or.
+     &              ((textpart(ii)(1:2).eq.'CE').and.
+     &               (textpart(ii)(1:4).ne.'CENT')).or.
      &             (textpart(ii)(1:2).eq.'PE')) then
                if((nmethod.eq.2).or.(nmethod.eq.3)) then
                   write(*,*) 
@@ -211,7 +225,7 @@
      &             (textpart(ii)(1:4).eq.'ELKE')) then
                nener=1
             elseif(textpart(ii)(1:4).eq.'HFL ') then
-               if(ithermal.lt.2) then
+               if(ithermal(1).lt.2) then
                   write(*,*) 
      &               '*WARNING reading *EL PRINT: HFL only makes '
                   write(*,*) '         sense for heat transfer '
@@ -219,31 +233,37 @@
                   cycle
                endif
             elseif((textpart(ii)(1:4).eq.'SVF ').or.
+     &             (textpart(ii)(1:4).eq.'SF  ').or.
      &             (textpart(ii)(1:4).eq.'HFLF')) then
-               if(cfd.eq.0) then
+               if(nef.eq.0) then
                   write(*,*) 
      &               '*WARNING reading *EL PRINT: SVF or HFLF only'
                   write(*,*) '         make sense for 3D fluid'
                   write(*,*) '         calculations; '
                   call inputerror(inpc,ipoinpc,iline,
-     &"*EL PRINT%")
-                  cycle
+     &                 "*EL PRINT%",ier)
+                  return
                endif
             elseif((textpart(ii)(1:4).ne.'S   ').and.
      &             (textpart(ii)(1:4).ne.'E   ').and.
      &             (textpart(ii)(1:4).ne.'ME  ').and.
-     &             (textpart(ii)(1:4).ne.'EVOL')) then
+     &             (textpart(ii)(1:4).ne.'EVOL').and.
+     &             (textpart(ii)(1:4).ne.'EMAS').and.
+     &             (textpart(ii)(1:4).ne.'COOR').and.
+     &             (textpart(ii)(1:4).ne.'CENT').and.
+     &             (textpart(ii)(1:4).ne.'EBHE')) then
                write(*,*) 
      &             '*WARNING reading *EL PRINT: label not applicable'
                write(*,*) '         or unknown; '
                call inputerror(inpc,ipoinpc,iline,
-     &"*EL PRINT%")
-               cycle
+     &              "*EL PRINT%",ier)
+               return
             endif
             nprint=nprint+1
             if(nprint.gt.nprint_) then
                write(*,*) '*ERROR reading *EL PRINT: increase nprint_'
-               call exit(201)
+               ier=1
+               return
             endif
             prset(nprint)=elset
             prlab(nprint)(1:4)=textpart(ii)(1:4)
