@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-Dimensional finite element program                   */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -62,37 +62,37 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	     char* tieset,ITG* ntie,ITG *imddof,ITG *nmddof,
 	     ITG *imdnode,ITG *nmdnode,ITG *imdboun,ITG *nmdboun,
   	     ITG *imdmpc,ITG *nmdmpc, ITG **izdofp, ITG *nzdof,ITG *nherm,
-	     double *xmr,double *xmi,char *typeboun,ITG *ielprop,double *prop){
+	     double *xmr,double *xmi,char *typeboun,ITG *ielprop,double *prop,
+	    char *orname,ITG *itiefac){
 
   /* calls the Arnoldi Package (ARPACK) for cyclic symmetry calculations */
   
-    char *filabt,*tchar1=NULL,*tchar2=NULL,*tchar3=NULL,lakonl[2]=" \0";
+    char *filabt,lakonl[2]=" \0";
 
-    ITG *inum=NULL,k,idir,lfin,j,iout=0,index,inode,id,i,idof,im,
+    ITG *inum=NULL,k,idir,j,iout=0,index,inode,id,i,idof,im,
         ielas,icmd,kk,l,nkt,icntrl,imag=1,icomplex,kkv,kk6,iterm,
         lprev,ilength,ij,i1,i2,iel,ielset,node,indexe,nope,ml1,nelem,
         *inocs=NULL,*ielcs=NULL,jj,l1,l2,is,nlabel,*nshcon=NULL,
         nodeleft,*noderight=NULL,numnodes,ileft,kflag=2,itr,locdir,
-        neqh,j1,nodenew,mt=mi[1]+1,istep=1,iinc=1,iit=-1,
-	tint=-1,tnstart=-1,tnend=-1,tint2=-1,
-	noderight_,*izdof=*izdofp,iload,iforc,*iznode=NULL,nznode,ll,ne0,
-	icfd=0,*inomat=NULL,mortar=0,*islavact=NULL,
-	*islavnode=NULL,*nslavnode=NULL,*islavsurf=NULL,
-        *iponoel=NULL,*inoel=NULL;
+        neqh,j1,nodenew,mt=mi[1]+1,istep=1,iinc=1,iit=-1,ne0,
+	network=0,noderight_,*izdof=*izdofp,iload,iforc,*iznode=NULL,
+	nznode,ll,icfd=0,*inomat=NULL,mortar=0,*islavact=NULL,*ipobody=NULL,
+	*islavnode=NULL,*nslavnode=NULL,*islavsurf=NULL,idirnew,
+        *iponoel=NULL,*inoel=NULL,mscalmethod=0;
 
     long long lint;
 
     double *stn=NULL,*v=NULL,*temp_array=NULL,*vini=NULL,*csmass=NULL,
-        *een=NULL,cam[5],*f=NULL,*fn=NULL,qa[3],*epn=NULL,summass,
+        *een=NULL,cam[5],*f=NULL,*fn=NULL,qa[4],*epn=NULL,summass,
         *stiini=NULL,*emn=NULL,*emeini=NULL,*clearini=NULL,
 	*xstateini=NULL,theta,pi,*coefmpcnew=NULL,t[3],ctl,stl,
 	*stx=NULL,*enern=NULL,*xstaten=NULL,*eei=NULL,*enerini=NULL,
-	*qfx=NULL,*qfn=NULL,xreal,ximag,*vt=NULL,sum,
+	*qfx=NULL,*qfn=NULL,xreal,ximag,*vt=NULL,sum,*voldt=NULL,
         *coefright=NULL,coef,a[9],ratio,reltime,
         *shcon=NULL,*springarea=NULL,*z=*zp, *zdof=NULL, *thicke=NULL,
-        atrab[9],acs[9],diff,fin[3],fout[3],*sumi=NULL,
+        *sumi=NULL,
         *vti=NULL,*pslavsurf=NULL,*pmastsurf=NULL,*cdn=NULL,
-        *energyini=NULL,*energy=NULL;
+        *energyini=NULL,*energy=NULL,*smscale=NULL;
     
     /* dummy arguments for the results call */
     
@@ -101,6 +101,8 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     pi=4.*atan(1.);
     neqh=neq[1]/2;
 
+    ne0=*ne;
+
     noderight_=10;
     NNEW(noderight,ITG,noderight_);
     NNEW(coefright,double,noderight_);
@@ -113,7 +115,7 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     NNEW(inum,ITG,*nk);
     NNEW(stx,double,6*mi[0]**ne);
     
-    nlabel=47;
+    nlabel=48;
     NNEW(filabt,char,87*nlabel);
     for(i=1;i<87*nlabel;i++) filabt[i]=' ';
     filabt[0]='U';
@@ -160,7 +162,7 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		}else{
 		  nelem=iel+1;
 		  FORTRAN(calcmass,(ipkon,lakon,kon,co,mi,&nelem,ne,thicke,
-                        ielmat,&nope,t0,t1,rhcon,nrhcon,ntmat_,
+                        ielmat,&nope,t0,rhcon,nrhcon,ntmat_,
 			ithermal,&csmass[i],ielprop,prop));
 		}
 		for(i2=0;i2<nope;++i2){
@@ -186,7 +188,7 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 		    }else{
 		      nelem=iel+1;
 		      FORTRAN(calcmass,(ipkon,lakon,kon,co,mi,&nelem,ne,thicke,
-                        ielmat,&nope,t0,t1,rhcon,nrhcon,ntmat_,
+                        ielmat,&nope,t0,rhcon,nrhcon,ntmat_,
 			ithermal,&csmass[i],ielprop,prop));
 		    }
 		    for(i2=0;i2<nope;++i2){
@@ -326,8 +328,8 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     /* body loads */
 
     if(*nbody>0){
-	printf("*ERROR in expand: body loads are not allowed for modal dynamics\n and steady state dynamics calculations in cyclic symmetric structures\n\n");
-	FORTRAN(stop,());
+	printf("\n *WARNING in expand: body loads are not allowed for modal dynamics\n          and steady state dynamics calculations in cyclic symmetric structures;\n          the load is removed\n\n");
+	*nbody=0;
     }
 
   /*  sorting the elements with distributed loads */
@@ -341,17 +343,6 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
     }
     
     /* point loads */
-    
-/*    for(i=0;i<*nforc;i++){
-	if(nodeforc[2*i+1]<*nsectors){
-	    nodeforc[2*i]+=*nk*nodeforc[2*i+1];
-	}else{
-	    nodeforc[2*i]+=*nk*(nodeforc[2*i+1]-(*nsectors));
-	}
-	iforc=i+1;
-	FORTRAN(addizdofcload,(nodeforc,ndirforc,nactdof,mi,izdof,
-	        nzdof,&iforc,iznode,&nznode,nk,imdnode,nmdnode,xforc));
-		}*/
 
     i=0;
     while(i<*nforc){
@@ -359,10 +350,6 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
 	/* checking for a cylindrical transformation;
 	   comparison with the cyclic symmetry system */
-	
-	itr=inotr[2*node-2];
-
-	if(itr==0){
 
             /* carthesian coordinate system */
 
@@ -373,66 +360,8 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    }
 	    i++;iforc=i;
 	    FORTRAN(addizdofcload,(nodeforc,ndirforc,nactdof,mi,izdof,
-		    nzdof,&iforc,iznode,&nznode,nk,imdnode,nmdnode,xforc));
-	}else{
-
-	    /* cylindrical coordinate system */	
-	    
-	    FORTRAN(transformatrix,(&trab[7*(itr-1)],&co[3*(node-1)],atrab));
-	    FORTRAN(transformatrix,(&cs[5],&co[3*(node-1)],acs));
-	    diff=0.; for(j=0;j<9;j++) diff+=(atrab[j]-acs[j])*(atrab[j]-acs[j]);
-	    
-	    if((ndirforc[i]!=1)||
-	       (nodeforc[2*i+2]!=node)||(ndirforc[i+1]!=2)||
-	       (nodeforc[2*i+4]!=node)||(ndirforc[i+2]!=3)||
-	       ((diff>1.e-10)&&(fabs(diff-8.)>1.e-10))){
-		printf("*ERROR: forces in a modal dynamic or steady state dynamics\n");
-		printf("        calculation with cyclic symmetry must be defined in\n");
-		printf("        the cyclic symmetric cylindrical coordinate system\n");
-		printf("        force at fault is applied in node %" ITGFORMAT "\n",node);
-		FORTRAN(stop,());
-	    }
-	    
-	    /* changing the complete force in the node in the basis sector from
-	       the global rectangular system into the cylindrical system */
-	    
-	    fin[0]=xforc[i];
-	    fin[1]=xforc[i+1];
-	    fin[2]=xforc[i+2];
-	    icntrl=2;
-	    FORTRAN(rectcyltrfm,(&node,co,cs,&icntrl,fin,fout));
-	    
-	    /* new node number (= node number in the target sector) */
-	    
-	    if(nodeforc[2*i+1]<*nsectors){
-		nodeforc[2*i]+=*nk*nodeforc[2*i+1];
-	    }else{
-		nodeforc[2*i]+=*nk*(nodeforc[2*i+1]-(*nsectors));
-	    }
-	    nodeforc[2*i+2]=nodeforc[2*i];
-	    nodeforc[2*i+4]=nodeforc[2*i];
-	    
-	    /* changing the complete force in the node in the target sector from
-	       the cylindrical system into the global rectangular system */
-	    
-	    node=nodeforc[2*i];
-	    fin[0]=fout[0];
-	    fin[1]=fout[1];
-	    fin[2]=fout[2];
-	    icntrl=-2;
-	    FORTRAN(rectcyltrfm,(&node,co,cs,&icntrl,fin,fout));
-	    xforc[i]=fout[0];
-	    xforc[i+1]=fout[1];
-	    xforc[i+2]=fout[2];
-	    
-	    /* storing the node and the dof into iznode and izdof */
-	    
-	    for(j=0;j<3;j++){
-		i++;iforc=i;
-		FORTRAN(addizdofcload,(nodeforc,ndirforc,nactdof,mi,izdof,
-			nzdof,&iforc,iznode,&nznode,nk,imdnode,nmdnode,xforc));
-	    }
-	}
+		    nzdof,&iforc,iznode,&nznode,nk,imdnode,nmdnode,xforc,
+		    ntrans,inotr));
     }
     
     /* loop over all eigenvalues; the loop starts from the highest eigenvalue
@@ -447,9 +376,8 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	NNEW(sumi,double,*nev);
     }
 
-    lfin=0;
     for(j=*nev-1;j>-1;--j){
-	lint=2*j*neqh;
+	lint=(long long)2*j*neqh;
 
 	/* calculating the cosine and sine of the phase angle */
 
@@ -531,10 +459,12 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	      xstaten,eei,enerini,cocon,ncocon,set,nset,istartset,iendset,
 	      ialset,nprint,prlab,prset,qfx,qfn,trab,inotr,ntrans,fmpc,
 	      nelemload,nload,ikmpc,ilmpc,&istep,&iinc,springarea,&reltime,
-              &ne0,xforc,nforc,thicke,shcon,nshcon,
+              &ne0,thicke,shcon,nshcon,
               sideload,xload,xloadold,&icfd,inomat,pslavsurf,pmastsurf,
 	      &mortar,islavact,cdn,islavnode,nslavnode,ntie,clearini,
-	      islavsurf,ielprop,prop,energyini,energy,&iit,iponoel,inoel);
+	      islavsurf,ielprop,prop,energyini,energy,&iit,iponoel,
+	      inoel,nener,orname,&network,ipobody,xbody,ibody,typeboun,
+	      itiefac,tieset,smscale,&mscalmethod,nbody);
 	    
 	}
 	SFREE(eei);
@@ -607,7 +537,6 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
 	for(ll=0;ll<nznode;ll++){
 	    i=iznode[ll]-1;
-//	for(i=0;i<*nk;i++){
 	    for(j1=0;j1<mt;j1++){
 
 		for(k=0;k<*nsectors;k++){
@@ -634,10 +563,6 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	
 /* normalizing the eigenvectors with the mass */
 
-/*	if (nm[j]==0||(nm[j]==(ITG)((cs[0]/2))&&(fmod(cs[0],2.)==0.)))
-                 {sum=sqrt(cs[0]);}
-		 else{sum=sqrt(cs[0]/2);}*/
-
 	sum=0.;
 	summass=0.;
 	for(i=0;i<*mcs;i++){
@@ -665,7 +590,70 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    sumi[j]=sqrt(sum);
 	}
     }
-  
+
+    /* mapping the reference displacements to the other sectors 
+       expanding vold */
+
+    if(iperturb[0]==1){
+	NNEW(voldt,double,mt**nk**nsectors);
+	
+	icntrl=2;
+	
+	FORTRAN(rectcylvold,(co,vold,cs,&icntrl,mi,
+			     iznode,&nznode,nsectors,nk));
+	
+	/* basis sector */
+	
+	for(ll=0;ll<nznode;ll++){
+	    l1=iznode[ll]-1;
+	    for(l2=0;l2<mt;l2++){
+		l=mt*l1+l2;
+		voldt[l]=vold[l];
+	    }
+	}
+	
+	/* other sectors */
+	
+	for(jj=0;jj<*mcs;jj++){
+	    ilength=cs[17*jj+3];
+	    lprev=cs[17*jj+13];
+	    for(i=1;i<*nsectors;i++){
+		
+		for(ll=0;ll<nznode;ll++){
+		    l1=iznode[ll]-1;
+		    if(inocs[l1]==jj){
+			
+			/* check whether node lies on axis */
+			
+			ml1=-l1-1;
+			FORTRAN(nident,(&ics[lprev],&ml1,&ilength,&id));
+			if(id!=0){
+			    if(ics[lprev+id-1]==ml1){
+				for(l2=0;l2<mt;l2++){
+				    l=mt*l1+l2;
+				    voldt[l+mt**nk*i]=vold[l];
+				}
+				continue;
+			    }
+			}
+			for(l2=0;l2<mt;l2++){
+			    l=mt*l1+l2;
+			    voldt[l+mt**nk*i]=vold[l];
+			}
+		    }
+		}
+	    }
+	}
+	
+	icntrl=-2;
+	
+	FORTRAN(rectcylvold,(co,voldt,cs,&icntrl,mi,iznode,&nznode,nsectors,nk));
+	
+	memcpy(&vold[0],&voldt[0],sizeof(double)*mt*nkt);
+	SFREE(voldt);
+	
+    }
+
 /* copying zdof into z */
   
     if(*nherm==1){
@@ -705,8 +693,15 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	    ilmpc[*nmpc]=ilmpc[j]+i**nmpcold;
 	    strcpy1(&labmpc[20**nmpc],&labmpcold[20*j],20);
 	    if(strcmp1(&labmpcold[20*j],"CYCLIC")==0){
+
+                /* identifying the nodes on the master side
+                   corresponding to a given slave node */
+
 		index=ipompcold[j]-1;
 		nodeleft=nodempcold[3*index];
+
+                /* slave node term direction */
+
 		idir=nodempcold[3*index+1];
 		index=nodempcold[3*index+2]-1;
 		numnodes=0;
@@ -721,10 +716,52 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 			    RENEW(noderight,ITG,noderight_);
 			    RENEW(coefright,double,noderight_);
 			}
+		    }else{
+
+                        /* master node term direction */
+
+			idirnew=nodempcold[3*index+1];
 		    }
 		    index=nodempcold[3*index+2]-1;
 		    if(index==-1) break;
 		}while(1);
+
+                /* if not successful:
+                   The cyclic MPC consists of one term corresponding
+                   to the slave node and one or more corresponding to
+                   the master node; the direction of the first master
+                   node term can correspond to the direction of the
+                   slave node term, but does not have to. Any master
+                   node term direction will actually do */
+
+		if(numnodes==0){
+
+                    /* identifying the nodes on the master side
+                       corresponding to a given slave node */
+
+		    index=ipompcold[j]-1;
+		    nodeleft=nodempcold[3*index];
+		    idir=idirnew;
+		    index=nodempcold[3*index+2]-1;
+		    numnodes=0;
+		    do{
+			node=nodempcold[3*index];
+			if(nodempcold[3*index+1]==idir){
+			    noderight[numnodes]=node;
+			    coefright[numnodes]=coefmpcold[index];
+			    numnodes++;
+			    if(numnodes>=noderight_){
+				noderight_=(ITG)(1.5*noderight_);
+				RENEW(noderight,ITG,noderight_);
+				RENEW(coefright,double,noderight_);
+			    }
+			}
+			index=nodempcold[3*index+2]-1;
+			if(index==-1) break;
+		    }while(1);
+
+		}
+
 		if(numnodes>0){
 		    sum=0.;
 		    for(k=0;k<numnodes;k++){
@@ -860,18 +897,18 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 
     /* copying the contact definition */
 
-    if(*nmethod==4){
+//    if(*nmethod==4){
       
       /* first find the startposition to append the expanded contact fields*/
       
-      for(j=0; j<*nset; j++){
+/*      for(j=0; j<*nset; j++){
 	if(iendset[j]>tint){
 	  tint=iendset[j];
 	}
       }
-      tint++;
+      tint++;*/
       /* now append and expand the contact definitons*/
-      NNEW(tchar1,char,81);
+/*      NNEW(tchar1,char,81);
       NNEW(tchar2,char,81);
       NNEW(tchar3,char,81);
       for(i=0; i<*ntie; i++){
@@ -879,48 +916,48 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
 	  memcpy(tchar2,&tieset[i*(81*3)+81],81);
 	  tchar2[80]='\0';
 	  memcpy(tchar3,&tieset[i*(81*3)+81+81],81);
-	  tchar3[80]='\0';
+	  tchar3[80]='\0';*/
 	  //a contact constraint was found, so append and expand the information
-	  for(j=0; j<*nset; j++){
+/*	  for(j=0; j<*nset; j++){
 	    memcpy(tchar1,&set[j*81],81);
 	    tchar1[80]='\0';
-	    if(strcmp(tchar1,tchar2)==0){
+	    if(strcmp(tchar1,tchar2)==0){*/
 	      /* dependent nodal surface was found,copy the original information first */
-	      tnstart=tint;
+/*	      tnstart=tint;
 	      for(k=0; k<iendset[j]-istartset[j]+1; k++){
 		ialset[tint-1]=ialset[istartset[j]-1+k];
 		tint++;
-	      }
+	      }*/
 	      /* now append the expanded information */
-	      for(l=1; l<*nsectors; l++){
+/*	      for(l=1; l<*nsectors; l++){
 		for(k=0; k<iendset[j]-istartset[j]+1; k++){
 		  ialset[tint-1]=(ialset[istartset[j]-1+k]!=-1)?ialset[istartset[j]-1+k]+*nk*l:-1;
 		  tint++;
 		}
 	      }
-	      tnend=tint-1;
+	      tnend=tint-1;*/
 	      /* now replace the information in istartset and iendset*/
-	      istartset[j]=tnstart;
+/*	      istartset[j]=tnstart;
 	      iendset[j]=tnend;
 	    }
-	    else if(strcmp(tchar1,tchar3)==0){
+	    else if(strcmp(tchar1,tchar3)==0){*/
 	      /* independent element face surface was found */
-	      tnstart=tint;
+/*	      tnstart=tint;
 	      for(k=0; k<iendset[j]-istartset[j]+1; k++){
 		ialset[tint-1]=ialset[istartset[j]-1+k];
 		tint++;
-	      }
+	      }*/
 	      /* now append the expanded information*/
-	      for(l=1; l<*nsectors; l++){
+/*	      for(l=1; l<*nsectors; l++){
 		for(k=0; k<iendset[j]-istartset[j]+1; k++){
 		  tint2=((ITG)(ialset[istartset[j]-1+k]))/10;
 		  ialset[tint-1]=(ialset[istartset[j]-1+k]!=-1)?(tint2+*ne*l)*10+(ialset[istartset[j]-1+k]-(tint2*10)):-1;
 		  tint++;
 		}
 	      }
-	      tnend=tint-1;
+	      tnend=tint-1;*/
 	      /* now replace the information in istartset and iendset*/
-	      istartset[j]=tnstart;
+/*	      istartset[j]=tnstart;
 	      iendset[j]=tnend;
 	    }
 	  }
@@ -929,7 +966,7 @@ void expand(double *co, ITG *nk, ITG *kon, ITG *ipkon, char *lakon,
       SFREE(tchar1);
       SFREE(tchar2);
       SFREE(tchar3);
-    }    
+    }    */
     
     *nk=nkt;
     (*ne)*=(*nsectors);
