@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -47,11 +47,13 @@
 !
       real*8 yi(ndim,mi(1),*),yn(nfield,*),field(999,20*mi(3)),a8(8,8),
      &  a4(4,4),a27(20,27),a9(6,9),a2(6,2),orab(7,*),co(3,*),prop(*),
-     &  coords(3,27),xi,et,ze,xl(3,20),xsj,shp(4,20),weight,
-     &  yiloc(6,27),a(3,3),b(3,3),c(3,3),vold(0:mi(2),*),tlayer(4),
+     &  coords(3,mi(1)),xi,et,ze,xl(3,20),xsj,shp(4,20),weight,
+     &  yiloc(6,mi(1)),a(3,3),b(3,3),c(3,3),vold(0:mi(2),*),tlayer(4),
      &  dlayer(4),xlayer(mi(3),4),thickness,xs2(3,7),xl2(3,8),
      &  xsj2(3),shp2(7,8),thicke(mi(3),*),coloc(3,8),
-     &  xwedge(2,2,9),a14(8,14)
+     &  xwedge(2,2,9),a14(8,14),a6(6,6)
+!
+!
 !
       include "gauss.f"
 !
@@ -98,152 +100,255 @@
      &  13,5,6,14,6,7,15,7,8,16,8,5,
      &  17,1,5,18,2,6,19,3,7,20,4,8/
 !
-      data a2 /  1.1455,-0.1455,1.1455,-0.1455,1.1455,-0.1455,
-     &           -0.1455,1.1455,-0.1455,1.1455,-0.1455,1.1455/
-      data a4 /  1.92705, -0.30902, -0.30902, -0.30902,
-     &          -0.30902,  1.92705, -0.30902, -0.30902,
-     &          -0.30902, -0.30902,  1.92705, -0.30902,
-     &          -0.30902, -0.30902, -0.30902,  1.92705/
-      data a9 / 1.63138,-0.32628,-0.32628,-0.52027, 0.10405, 0.10405,
-     &         -0.32628, 1.63138,-0.32628, 0.10405,-0.52027, 0.10405,
-     &         -0.32628,-0.32628, 1.63138, 0.10405, 0.10405,-0.52027,
-     &          0.55556,-0.11111,-0.11111, 0.55556,-0.11111,-0.11111,
-     &         -0.11111, 0.55556,-0.11111,-0.11111, 0.55556,-0.11111,
-     &         -0.11111,-0.11111, 0.55556,-0.11111,-0.11111, 0.55556,
-     &         -0.52027, 0.10405, 0.10405, 1.63138,-0.32628,-0.32628,
-     &          0.10405,-0.52027, 0.10405,-0.32628, 1.63138,-0.32628,
-     &          0.10405, 0.10405,-0.52027,-0.32628,-0.32628, 1.63138/
+      data a2 /1.1455d0,-0.1455d0,1.1455d0,-0.1455d0,1.1455d0,-0.1455d0,
+     &        -0.1455d0,1.1455d0,-0.1455d0,1.1455d0,-0.1455d0,1.1455d0/
+      data a4 /  1.92705d0, -0.30902d0, -0.30902d0, -0.30902d0,
+     &          -0.30902d0,  1.92705d0, -0.30902d0, -0.30902d0,
+     &          -0.30902d0, -0.30902d0,  1.92705d0, -0.30902d0,
+     &          -0.30902d0, -0.30902d0, -0.30902d0,  1.92705d0/
+!
+!     extrapolation from a 6 integration point scheme in a wedge to
+!     the vertex nodes
+!
+      data a6 / 2.04904d0, 0.00000d0, 0.00000d0,-0.54904d0, 0.00000d0,
+     &          0.00000d0,
+     &         -0.34151d0, 1.70753d0,-0.34151d0, 0.09151d0,-0.45753d0,
+     &          0.09151d0,
+     &         -0.34151d0,-0.34151d0, 1.70753d0, 0.09151d0, 0.09151d0,
+     &         -0.45753d0,
+     &         -0.54904d0, 0.00000d0, 0.00000d0, 2.04904d0, 0.00000d0,
+     &          0.00000d0,
+     &          0.09151d0,-0.45753d0, 0.09151d0,-0.34151d0, 1.70753d0,
+     &         -0.34151d0,
+     &          0.09151d0, 0.09151d0,-0.45753d0,-0.34151d0,-0.34151d0,
+     &          1.70753d0/
+!
+      data a9 / 1.63138d0,-0.32628d0,-0.32628d0,-0.52027d0, 0.10405d0,
+     &          0.10405d0,
+     &         -0.32628d0, 1.63138d0,-0.32628d0, 0.10405d0,-0.52027d0,
+     &          0.10405d0,
+     &         -0.32628d0,-0.32628d0, 1.63138d0, 0.10405d0, 0.10405d0,
+     &         -0.52027d0,
+     &          0.55556d0,-0.11111d0,-0.11111d0, 0.55556d0,-0.11111d0,
+     &         -0.11111d0,
+     &         -0.11111d0, 0.55556d0,-0.11111d0,-0.11111d0,0.55556d0,
+     &         -0.11111d0,
+     &         -0.11111d0,-0.11111d0, 0.55556d0,-0.11111d0,-0.11111d0,
+     &          0.55556d0,
+     &         -0.52027d0, 0.10405d0, 0.10405d0, 1.63138d0,-0.32628d0,
+     &         -0.32628d0,
+     &          0.10405d0,-0.52027d0, 0.10405d0,-0.32628d0, 1.63138d0,
+     &         -0.32628d0,
+     &          0.10405d0, 0.10405d0,-0.52027d0,-0.32628d0,-0.32628d0,
+     &          1.63138d0/
 !
 !     extrapolation from a 2x2x2=8 integration point scheme in a hex to
 !     the vertex nodes
 !    
-      data a8 /2.549,-.683,.183,-.683,-.683,.183,
-     &        -.04904,.183,-.683,2.549,-.683,.183,
-     &        .183,-.683,.183,-.04904,-.683,.183,
-     &        -.683,2.549,.183,-.04904,.183,-.683,
-     &        .183,-.683,2.549,-.683,-.04904,.183,
-     &        -.683,.183,-.683,.183,-.04904,.183,
-     &        2.549,-.683,.183,-.683,.183,-.683,
-     &        .183,-.04904,-.683,2.549,-.683,.183,
-     &        .183,-.04904,.183,-.683,-.683,.183,
-     &        -.683,2.549,-.04904,.183,-.683,.183,
-     &        .183,-.683,2.549,-.683/  
+      data a8 /2.549d0,-.683d0,.183d0,-.683d0,-.683d0,.183d0,
+     &        -.04904d0,.183d0,-.683d0,2.549d0,-.683d0,.183d0,
+     &        .183d0,-.683d0,.183d0,-.04904d0,-.683d0,.183d0,
+     &        -.683d0,2.549d0,.183d0,-.04904d0,.183d0,-.683d0,
+     &        .183d0,-.683d0,2.549d0,-.683d0,-.04904d0,.183d0,
+     &        -.683d0,.183d0,-.683d0,.183d0,-.04904d0,.183d0,
+     &        2.549d0,-.683d0,.183d0,-.683d0,.183d0,-.683d0,
+     &        .183d0,-.04904d0,-.683d0,2.549d0,-.683d0,.183d0,
+     &        .183d0,-.04904d0,.183d0,-.683d0,-.683d0,.183d0,
+     &        -.683d0,2.549d0,-.04904d0,.183d0,-.683d0,.183d0,
+     &        .183d0,-.683d0,2.549d0,-.683d0/  
 !
 !     extrapolation from a 14 integration point scheme in a hex to
 !     the vertex nodes
 !    
       data a14 /
-     &  0.1396E+01,-0.3026E+00,0.1124E-01,-0.3026E+00,
-     &  -0.3026E+00,0.1124E-01,0.4901E-01,0.1124E-01,
-     &  -0.3026E+00,0.1396E+01,-0.3026E+00,0.1124E-01,
-     &  0.1124E-01,-0.3026E+00,0.1124E-01,0.4901E-01,
-     &  0.1124E-01,-0.3026E+00,0.1396E+01,-0.3026E+00,
-     &  0.4901E-01,0.1124E-01,-0.3026E+00,0.1124E-01,
-     &  -0.3026E+00,0.1124E-01,-0.3026E+00,0.1396E+01,
-     &  0.1124E-01,0.4901E-01,0.1124E-01,-0.3026E+00,
-     &  -0.3026E+00,0.1124E-01,0.4901E-01,0.1124E-01,
-     &  0.1396E+01,-0.3026E+00,0.1124E-01,-0.3026E+00,
-     &  0.1124E-01,-0.3026E+00,0.1124E-01,0.4901E-01,
-     &  -0.3026E+00,0.1396E+01,-0.3026E+00,0.1124E-01,
-     &  0.4901E-01,0.1124E-01,-0.3026E+00,0.1124E-01,
-     &  0.1124E-01,-0.3026E+00,0.1396E+01,-0.3026E+00,
-     &  0.1124E-01,0.4901E-01,0.1124E-01,-0.3026E+00,
-     &  -0.3026E+00,0.1124E-01,-0.3026E+00,0.1396E+01,
-     &  0.2069E+00,0.2069E+00,-0.6408E-01,-0.6408E-01,
-     &  0.2069E+00,0.2069E+00,-0.6408E-01,-0.6408E-01,
-     &  -0.6408E-01,0.2069E+00,0.2069E+00,-0.6408E-01,
-     &  -0.6408E-01,0.2069E+00,0.2069E+00,-0.6408E-01,
-     &  -0.6408E-01,-0.6408E-01,0.2069E+00,0.2069E+00,
-     &  -0.6408E-01,-0.6408E-01,0.2069E+00,0.2069E+00,
-     &  0.2069E+00,-0.6408E-01,-0.6408E-01,0.2069E+00,
-     &  0.2069E+00,-0.6408E-01,-0.6408E-01,0.2069E+00,
-     &  0.2069E+00,0.2069E+00,0.2069E+00,0.2069E+00,
-     &  -0.6408E-01,-0.6408E-01,-0.6408E-01,-0.6408E-01,
-     &  -0.6408E-01,-0.6408E-01,-0.6408E-01,-0.6408E-01,
-     &  0.2069E+00,0.2069E+00,0.2069E+00,0.2069E+00/
+     &  0.1396d+01,-0.3026d+00,0.1124d-01,-0.3026d+00,
+     &  -0.3026d+00,0.1124d-01,0.4901d-01,0.1124d-01,
+     &  -0.3026d+00,0.1396d+01,-0.3026d+00,0.1124d-01,
+     &  0.1124d-01,-0.3026d+00,0.1124d-01,0.4901d-01,
+     &  0.1124d-01,-0.3026d+00,0.1396d+01,-0.3026d+00,
+     &  0.4901d-01,0.1124d-01,-0.3026d+00,0.1124d-01,
+     &  -0.3026d+00,0.1124d-01,-0.3026d+00,0.1396d+01,
+     &  0.1124d-01,0.4901d-01,0.1124d-01,-0.3026d+00,
+     &  -0.3026d+00,0.1124d-01,0.4901d-01,0.1124d-01,
+     &  0.1396d+01,-0.3026d+00,0.1124d-01,-0.3026d+00,
+     &  0.1124d-01,-0.3026d+00,0.1124d-01,0.4901d-01,
+     &  -0.3026d+00,0.1396d+01,-0.3026d+00,0.1124d-01,
+     &  0.4901d-01,0.1124d-01,-0.3026d+00,0.1124d-01,
+     &  0.1124d-01,-0.3026d+00,0.1396d+01,-0.3026d+00,
+     &  0.1124d-01,0.4901d-01,0.1124d-01,-0.3026d+00,
+     &  -0.3026d+00,0.1124d-01,-0.3026d+00,0.1396d+01,
+     &  0.2069d+00,0.2069d+00,-0.6408d-01,-0.6408d-01,
+     &  0.2069d+00,0.2069d+00,-0.6408d-01,-0.6408d-01,
+     &  -0.6408d-01,0.2069d+00,0.2069d+00,-0.6408d-01,
+     &  -0.6408d-01,0.2069d+00,0.2069d+00,-0.6408d-01,
+     &  -0.6408d-01,-0.6408d-01,0.2069d+00,0.2069d+00,
+     &  -0.6408d-01,-0.6408d-01,0.2069d+00,0.2069d+00,
+     &  0.2069d+00,-0.6408d-01,-0.6408d-01,0.2069d+00,
+     &  0.2069d+00,-0.6408d-01,-0.6408d-01,0.2069d+00,
+     &  0.2069d+00,0.2069d+00,0.2069d+00,0.2069d+00,
+     &  -0.6408d-01,-0.6408d-01,-0.6408d-01,-0.6408d-01,
+     &  -0.6408d-01,-0.6408d-01,-0.6408d-01,-0.6408d-01,
+     &  0.2069d+00,0.2069d+00,0.2069d+00,0.2069d+00/
 !
 !     extrapolation from a 3x3x3=27 integration point scheme in a hex to
 !     the all nodes in a 20-node element
 !    
       data a27 /
-     &  2.37499,-0.12559,-0.16145,-0.12559,-0.12559,-0.16145, 0.11575,
-     & -0.16145, 0.32628, 0.11111, 0.11111, 0.32628, 0.11111,-0.10405,
-     & -0.10405, 0.11111, 0.32628, 0.11111,-0.10405, 0.11111,-0.31246,
-     & -0.31246, 0.31481, 0.31481, 0.31481, 0.31481,-0.16902,-0.16902,
-     &  1.28439,-0.27072,-0.19444,-0.27072,-0.19444, 0.15961,-0.00661,
-     &  0.15961,-0.27072,-0.27072, 0.15961, 0.15961,-0.12559, 2.37499,
-     & -0.12559,-0.16145,-0.16145,-0.12559,-0.16145, 0.11575, 0.32628,
-     &  0.32628, 0.11111, 0.11111, 0.11111, 0.11111,-0.10405,-0.10405,
-     &  0.11111, 0.32628, 0.11111,-0.10405,-0.31246, 0.31481, 0.31481,
-     & -0.31246, 0.31481,-0.16902,-0.16902, 0.31481,-0.27072,-0.19444,
-     & -0.27072, 1.28439, 0.15961,-0.00661, 0.15961,-0.19444,-0.27072,
-     &  0.15961, 0.15961,-0.27072,-0.48824,-0.48824,-0.48824,-0.48824,
-     &  0.22898, 0.22898, 0.22898, 0.22898, 0.05556, 0.05556, 0.05556,
-     &  0.05556, 0.05556, 0.05556, 0.05556, 0.05556,-0.22222,-0.22222,
-     & -0.22222,-0.22222, 0.31481,-0.31246,-0.31246, 0.31481,-0.16902,
-     &  0.31481, 0.31481,-0.16902,-0.27072, 1.28439,-0.27072,-0.19444,
-     &  0.15961,-0.19444, 0.15961,-0.00661, 0.15961,-0.27072,-0.27072,
-     &  0.15961,-0.12559,-0.16145,-0.12559, 2.37499,-0.16145, 0.11575,
-     & -0.16145,-0.12559, 0.11111, 0.11111, 0.32628, 0.32628,-0.10405,
-     & -0.10405, 0.11111, 0.11111, 0.11111,-0.10405, 0.11111, 0.32628,
-     &  0.31481, 0.31481,-0.31246,-0.31246,-0.16902,-0.16902, 0.31481,
-     &  0.31481,-0.19444,-0.27072, 1.28439,-0.27072,-0.00661, 0.15961,
-     & -0.19444, 0.15961, 0.15961, 0.15961,-0.27072,-0.27072,-0.16145,
-     & -0.12559, 2.37499,-0.12559, 0.11575,-0.16145,-0.12559,-0.16145,
-     &  0.11111, 0.32628, 0.32628, 0.11111,-0.10405, 0.11111, 0.11111,
-     & -0.10405,-0.10405, 0.11111, 0.32628, 0.11111,-0.31246, 0.31481,
-     & -0.16902, 0.31481,-0.31246, 0.31481,-0.16902, 0.31481,-0.27072,
-     &  0.15961, 0.15961,-0.27072,-0.27072, 0.15961, 0.15961,-0.27072,
-     &  1.28439,-0.19444,-0.00661,-0.19444,-0.48824,-0.48824, 0.22898,
-     &  0.22898,-0.48824,-0.48824, 0.22898, 0.22898, 0.05556,-0.22222,
-     &  0.05556,-0.22222, 0.05556,-0.22222, 0.05556,-0.22222, 0.05556,
-     &  0.05556, 0.05556, 0.05556, 0.31481,-0.31246, 0.31481,-0.16902,
-     &  0.31481,-0.31246, 0.31481,-0.16902,-0.27072,-0.27072, 0.15961,
-     &  0.15961,-0.27072,-0.27072, 0.15961, 0.15961,-0.19444, 1.28439,
-     & -0.19444,-0.00661,-0.48824, 0.22898, 0.22898,-0.48824,-0.48824,
-     &  0.22898, 0.22898,-0.48824,-0.22222, 0.05556,-0.22222, 0.05556,
-     & -0.22222, 0.05556,-0.22222, 0.05556, 0.05556, 0.05556, 0.05556,
-     &  0.05556,-0.29630,-0.29630,-0.29630,-0.29630,-0.29630,-0.29630,
-     & -0.29630,-0.29630,-0.11111,-0.11111,-0.11111,-0.11111,-0.11111,
-     & -0.11111,-0.11111,-0.11111,-0.11111,-0.11111,-0.11111,-0.11111,
-     &  0.22898,-0.48824,-0.48824, 0.22898, 0.22898,-0.48824,-0.48824,
-     &  0.22898,-0.22222, 0.05556,-0.22222, 0.05556,-0.22222, 0.05556,
-     & -0.22222, 0.05556, 0.05556, 0.05556, 0.05556, 0.05556, 0.31481,
-     & -0.16902, 0.31481,-0.31246, 0.31481,-0.16902, 0.31481,-0.31246,
-     &  0.15961, 0.15961,-0.27072,-0.27072, 0.15961, 0.15961,-0.27072,
-     & -0.27072,-0.19444,-0.00661,-0.19444, 1.28439, 0.22898, 0.22898,
-     & -0.48824,-0.48824, 0.22898, 0.22898,-0.48824,-0.48824, 0.05556,
-     & -0.22222, 0.05556,-0.22222, 0.05556,-0.22222, 0.05556,-0.22222,
-     &  0.05556, 0.05556, 0.05556, 0.05556,-0.16902, 0.31481,-0.31246,
-     &  0.31481,-0.16902, 0.31481,-0.31246, 0.31481, 0.15961,-0.27072,
-     & -0.27072, 0.15961, 0.15961,-0.27072,-0.27072, 0.15961,-0.00661,
-     & -0.19444, 1.28439,-0.19444,-0.12559,-0.16145, 0.11575,-0.16145,
-     &  2.37499,-0.12559,-0.16145,-0.12559, 0.11111,-0.10405,-0.10405,
-     &  0.11111, 0.32628, 0.11111, 0.11111, 0.32628, 0.32628, 0.11111,
-     & -0.10405, 0.11111, 0.31481, 0.31481,-0.16902,-0.16902,-0.31246,
-     & -0.31246, 0.31481, 0.31481,-0.19444, 0.15961,-0.00661, 0.15961,
-     &  1.28439,-0.27072,-0.19444,-0.27072,-0.27072,-0.27072, 0.15961,
-     &  0.15961,-0.16145,-0.12559,-0.16145, 0.11575,-0.12559, 2.37499,
-     & -0.12559,-0.16145, 0.11111, 0.11111,-0.10405,-0.10405, 0.32628,
-     &  0.32628, 0.11111, 0.11111, 0.11111, 0.32628, 0.11111,-0.10405,
-     &  0.31481,-0.16902,-0.16902, 0.31481,-0.31246, 0.31481, 0.31481,
-     & -0.31246, 0.15961,-0.00661, 0.15961,-0.19444,-0.27072,-0.19444,
-     & -0.27072, 1.28439,-0.27072, 0.15961, 0.15961,-0.27072, 0.22898,
-     &  0.22898, 0.22898, 0.22898,-0.48824,-0.48824,-0.48824,-0.48824,
-     &  0.05556, 0.05556, 0.05556, 0.05556, 0.05556, 0.05556, 0.05556,
-     &  0.05556,-0.22222,-0.22222,-0.22222,-0.22222,-0.16902, 0.31481,
-     &  0.31481,-0.16902, 0.31481,-0.31246,-0.31246, 0.31481, 0.15961,
-     & -0.19444, 0.15961,-0.00661,-0.27072, 1.28439,-0.27072,-0.19444,
-     &  0.15961,-0.27072,-0.27072, 0.15961,-0.16145, 0.11575,-0.16145,
-     & -0.12559,-0.12559,-0.16145,-0.12559, 2.37499,-0.10405,-0.10405,
-     &  0.11111, 0.11111, 0.11111, 0.11111, 0.32628, 0.32628, 0.11111,
-     & -0.10405, 0.11111, 0.32628,-0.16902,-0.16902, 0.31481, 0.31481,
-     &  0.31481, 0.31481,-0.31246,-0.31246,-0.00661, 0.15961,-0.19444,
-     &  0.15961,-0.19444,-0.27072, 1.28439,-0.27072, 0.15961, 0.15961,
-     & -0.27072,-0.27072, 0.11575,-0.16145,-0.12559,-0.16145,-0.16145,
-     & -0.12559, 2.37499,-0.12559,-0.10405, 0.11111, 0.11111,-0.10405,
-     &  0.11111, 0.32628, 0.32628, 0.11111,-0.10405, 0.11111, 0.32628,
-     &  0.11111/
+     &  2.37499d0,-0.12559d0,-0.16145d0,-0.12559d0,-0.12559d0,
+     & -0.16145d0, 0.11575d0,
+     & -0.16145d0, 0.32628d0, 0.11111d0, 0.11111d0, 0.32628d0,
+     &  0.11111d0,-0.10405d0,
+     & -0.10405d0, 0.11111d0, 0.32628d0, 0.11111d0,-0.10405d0,
+     &  0.11111d0,-0.31246d0,
+     & -0.31246d0, 0.31481d0, 0.31481d0, 0.31481d0, 0.31481d0,
+     & -0.16902d0,-0.16902d0,
+     &  1.28439d0,-0.27072d0,-0.19444d0,-0.27072d0,-0.19444d0,
+     &  0.15961d0,-0.00661d0,
+     &  0.15961d0,-0.27072d0,-0.27072d0, 0.15961d0, 0.15961d0,
+     & -0.12559d0, 2.37499d0,
+     & -0.12559d0,-0.16145d0,-0.16145d0,-0.12559d0,-0.16145d0,
+     &  0.11575d0, 0.32628d0,
+     &  0.32628d0, 0.11111d0, 0.11111d0, 0.11111d0, 0.11111d0,
+     & -0.10405d0,-0.10405d0,
+     &  0.11111d0, 0.32628d0, 0.11111d0,-0.10405d0,-0.31246d0,
+     &  0.31481d0, 0.31481d0,
+     & -0.31246d0, 0.31481d0,-0.16902d0,-0.16902d0, 0.31481d0,
+     & -0.27072d0,-0.19444d0,
+     & -0.27072d0, 1.28439d0, 0.15961d0,-0.00661d0, 0.15961d0,
+     & -0.19444d0,-0.27072d0,
+     &  0.15961d0, 0.15961d0,-0.27072d0,-0.48824d0,-0.48824d0,
+     & -0.48824d0,-0.48824d0,
+     &  0.22898d0, 0.22898d0, 0.22898d0, 0.22898d0, 0.05556d0,
+     &  0.05556d0, 0.05556d0,
+     &  0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0,
+     & -0.22222d0,-0.22222d0,
+     & -0.22222d0,-0.22222d0, 0.31481d0,-0.31246d0,-0.31246d0,
+     &  0.31481d0,-0.16902d0,
+     &  0.31481d0, 0.31481d0,-0.16902d0,-0.27072d0, 1.28439d0,
+     & -0.27072d0,-0.19444d0,
+     &  0.15961d0,-0.19444d0, 0.15961d0,-0.00661d0, 0.15961d0,
+     & -0.27072d0,-0.27072d0,
+     &  0.15961d0,-0.12559d0,-0.16145d0,-0.12559d0, 2.37499d0,
+     & -0.16145d0, 0.11575d0,
+     & -0.16145d0,-0.12559d0, 0.11111d0, 0.11111d0, 0.32628d0,
+     &  0.32628d0,-0.10405d0,
+     & -0.10405d0, 0.11111d0, 0.11111d0, 0.11111d0,-0.10405d0,
+     &  0.11111d0, 0.32628d0,
+     &  0.31481d0, 0.31481d0,-0.31246d0,-0.31246d0,-0.16902d0,
+     & -0.16902d0, 0.31481d0,
+     &  0.31481d0,-0.19444d0,-0.27072d0, 1.28439d0,-0.27072d0,
+     & -0.00661d0, 0.15961d0,
+     & -0.19444d0, 0.15961d0, 0.15961d0, 0.15961d0,-0.27072d0,
+     & -0.27072d0,-0.16145d0,
+     & -0.12559d0, 2.37499d0,-0.12559d0, 0.11575d0,-0.16145d0,
+     & -0.12559d0,-0.16145d0,
+     &  0.11111d0, 0.32628d0, 0.32628d0, 0.11111d0,-0.10405d0,
+     &  0.11111d0, 0.11111d0,
+     & -0.10405d0,-0.10405d0, 0.11111d0, 0.32628d0, 0.11111d0,
+     & -0.31246d0, 0.31481d0,
+     & -0.16902d0, 0.31481d0,-0.31246d0, 0.31481d0,-0.16902d0,
+     &  0.31481d0,-0.27072d0,
+     &  0.15961d0, 0.15961d0,-0.27072d0,-0.27072d0, 0.15961d0,
+     &  0.15961d0,-0.27072d0,
+     &  1.28439d0,-0.19444d0,-0.00661d0,-0.19444d0,-0.48824d0,
+     & -0.48824d0, 0.22898d0,
+     &  0.22898d0,-0.48824d0,-0.48824d0, 0.22898d0, 0.22898d0,
+     &  0.05556d0,-0.22222d0,
+     &  0.05556d0,-0.22222d0, 0.05556d0,-0.22222d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0,
+     &  0.05556d0, 0.05556d0, 0.05556d0, 0.31481d0,-0.31246d0,
+     &  0.31481d0,-0.16902d0,
+     &  0.31481d0,-0.31246d0, 0.31481d0,-0.16902d0,-0.27072d0,
+     & -0.27072d0, 0.15961d0,
+     &  0.15961d0,-0.27072d0,-0.27072d0, 0.15961d0, 0.15961d0,
+     & -0.19444d0, 1.28439d0,
+     & -0.19444d0,-0.00661d0,-0.48824d0, 0.22898d0, 0.22898d0,
+     & -0.48824d0,-0.48824d0,
+     &  0.22898d0, 0.22898d0,-0.48824d0,-0.22222d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0,-0.22222d0, 0.05556d0, 0.05556d0,
+     &  0.05556d0, 0.05556d0,
+     &  0.05556d0,-0.29630d0,-0.29630d0,-0.29630d0,-0.29630d0,
+     & -0.29630d0,-0.29630d0,
+     & -0.29630d0,-0.29630d0,-0.11111d0,-0.11111d0,-0.11111d0,
+     & -0.11111d0,-0.11111d0,
+     & -0.11111d0,-0.11111d0,-0.11111d0,-0.11111d0,-0.11111d0,
+     & -0.11111d0,-0.11111d0,
+     &  0.22898d0,-0.48824d0,-0.48824d0, 0.22898d0, 0.22898d0,
+     & -0.48824d0,-0.48824d0,
+     &  0.22898d0,-0.22222d0, 0.05556d0,-0.22222d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0,
+     &  0.05556d0, 0.31481d0,
+     & -0.16902d0, 0.31481d0,-0.31246d0, 0.31481d0,-0.16902d0,
+     &  0.31481d0,-0.31246d0,
+     &  0.15961d0, 0.15961d0,-0.27072d0,-0.27072d0, 0.15961d0,
+     &  0.15961d0,-0.27072d0,
+     & -0.27072d0,-0.19444d0,-0.00661d0,-0.19444d0, 1.28439d0,
+     &  0.22898d0, 0.22898d0,
+     & -0.48824d0,-0.48824d0, 0.22898d0, 0.22898d0,-0.48824d0,
+     & -0.48824d0, 0.05556d0,
+     & -0.22222d0, 0.05556d0,-0.22222d0, 0.05556d0,-0.22222d0,
+     &  0.05556d0,-0.22222d0,
+     &  0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0,-0.16902d0,
+     &  0.31481d0,-0.31246d0,
+     &  0.31481d0,-0.16902d0, 0.31481d0,-0.31246d0, 0.31481d0,
+     &  0.15961d0,-0.27072d0,
+     & -0.27072d0, 0.15961d0, 0.15961d0,-0.27072d0,-0.27072d0
+     & , 0.15961d0,-0.00661d0,
+     & -0.19444d0, 1.28439d0,-0.19444d0,-0.12559d0,-0.16145d0,
+     &  0.11575d0,-0.16145d0,
+     &  2.37499d0,-0.12559d0,-0.16145d0,-0.12559d0, 0.11111d0,
+     & -0.10405d0,-0.10405d0,
+     &  0.11111d0, 0.32628d0, 0.11111d0, 0.11111d0, 0.32628d0,
+     &  0.32628d0, 0.11111d0,
+     & -0.10405d0, 0.11111d0, 0.31481d0, 0.31481d0,-0.16902d0,
+     & -0.16902d0,-0.31246d0,
+     & -0.31246d0, 0.31481d0, 0.31481d0,-0.19444d0, 0.15961d0,
+     & -0.00661d0, 0.15961d0,
+     &  1.28439d0,-0.27072d0,-0.19444d0,-0.27072d0,-0.27072d0,
+     & -0.27072d0, 0.15961d0,
+     &  0.15961d0,-0.16145d0,-0.12559d0,-0.16145d0, 0.11575d0,
+     & -0.12559d0, 2.37499d0,
+     & -0.12559d0,-0.16145d0, 0.11111d0, 0.11111d0,-0.10405d0,
+     & -0.10405d0, 0.32628d0,
+     &  0.32628d0, 0.11111d0, 0.11111d0, 0.11111d0, 0.32628d0,
+     &  0.11111d0,-0.10405d0,
+     &  0.31481d0,-0.16902d0,-0.16902d0, 0.31481d0,-0.31246d0,
+     &  0.31481d0, 0.31481d0,
+     & -0.31246d0, 0.15961d0,-0.00661d0, 0.15961d0,-0.19444d0,
+     & -0.27072d0,-0.19444d0,
+     & -0.27072d0, 1.28439d0,-0.27072d0, 0.15961d0, 0.15961d0,
+     & -0.27072d0, 0.22898d0,
+     &  0.22898d0, 0.22898d0, 0.22898d0,-0.48824d0,-0.48824d0,
+     & -0.48824d0,-0.48824d0,
+     &  0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0, 0.05556d0,
+     &  0.05556d0, 0.05556d0,
+     &  0.05556d0,-0.22222d0,-0.22222d0,-0.22222d0,-0.22222d0,
+     & -0.16902d0, 0.31481d0,
+     &  0.31481d0,-0.16902d0, 0.31481d0,-0.31246d0,-0.31246d0,
+     &  0.31481d0, 0.15961d0,
+     & -0.19444d0, 0.15961d0,-0.00661d0,-0.27072d0, 1.28439d0,
+     & -0.27072d0,-0.19444d0,
+     &  0.15961d0,-0.27072d0,-0.27072d0, 0.15961d0,-0.16145d0,
+     &  0.11575d0,-0.16145d0,
+     & -0.12559d0,-0.12559d0,-0.16145d0,-0.12559d0, 2.37499d0,
+     & -0.10405d0,-0.10405d0,
+     &  0.11111d0, 0.11111d0, 0.11111d0, 0.11111d0, 0.32628d0,
+     &  0.32628d0, 0.11111d0,
+     & -0.10405d0, 0.11111d0, 0.32628d0,-0.16902d0,-0.16902d0,
+     &  0.31481d0, 0.31481d0,
+     &  0.31481d0, 0.31481d0,-0.31246d0,-0.31246d0,-0.00661d0,
+     &  0.15961d0,-0.19444d0,
+     &  0.15961d0,-0.19444d0,-0.27072d0, 1.28439d0,-0.27072d0,
+     &  0.15961d0, 0.15961d0,
+     & -0.27072d0,-0.27072d0, 0.11575d0,-0.16145d0,-0.12559d0,
+     & -0.16145d0,-0.16145d0,
+     & -0.12559d0, 2.37499d0,-0.12559d0,-0.10405d0, 0.11111d0,
+     &  0.11111d0,-0.10405d0,
+     &  0.11111d0, 0.32628d0, 0.32628d0, 0.11111d0,-0.10405d0,
+     &  0.11111d0, 0.32628d0,
+     &  0.11111d0/
 !
       data iflag /1/
 !
@@ -293,7 +398,9 @@
             nope=15
          elseif(lakonl(4:4).eq.'6') then
             nope=6
-         elseif((lakonl(1:1).eq.'E').and.(lakonl(7:7).eq.'A'))then
+         elseif((lakonl(1:1).eq.'E').and.
+     &          ((lakonl(7:7).eq.'A').or.
+     &           (lakonl(7:7).eq.'2'))) then
 !
             inum(kon(indexe+1))=inum(kon(indexe+1))+1
             inum(kon(indexe+2))=inum(kon(indexe+2))+1
@@ -303,6 +410,11 @@
             nope=nope+1
             inum(kon(indexe+nope))=-1
             cycle
+         elseif(lakonl(1:1).eq.'U') then
+            call extrapolate_u(yi,yn,ipkon,inum,kon,lakon,nfield,nk,
+     &           ne,mi,ndim,orab,ielorien,co,iorienloc,cflag,
+     &           vold,force,ielmat,thicke,ielprop,prop,i)
+            return
          else
             cycle
          endif
@@ -316,25 +428,22 @@
      &        ((lakonl(7:8).eq.'LC').or.(ielorien(1,i).ne.0))) then
 !
             if(lakonl(7:8).ne.'LC') then
-               iorien=ielorien(1,i)
-            else
+               iorien=max(0,ielorien(1,i))
+!david -start
+            elseif(lakonl(4:5).eq.'20') then
 !     
 !     composite materials
 !     
                mint2d=4
                nopes=8
-!     determining the number of layers
-!     
-               nlayer=0
-               do k=1,mi(3)
-                  if(ielmat(k,i).ne.0) then
-                     nlayer=nlayer+1
-                  endif
-               enddo
 !     
 !     determining the layer thickness and global thickness
 !     at the shell integration points
 !     
+!     xlayer: actual layer thickness
+!     tlayer: total thickness of all layers
+!     dlayer: total thickness of all layers up to the actual one
+!
                indexe=ipkon(i)
                do kk=1,mint2d
                   xi=gauss3d2(1,kk)
@@ -352,7 +461,44 @@
                enddo
 !     
                ilayer=0
-               do k=1,4
+               do k=1,mint2d
+                  dlayer(k)=0.d0
+               enddo
+!
+!     S6-composite shell
+!
+            elseif(lakonl(4:5).eq.'15') then
+!     
+!     composite materials
+!     
+               mint2d=3
+               nopes=6
+!     
+!     determining the layer thickness and global thickness
+!     at the shell integration points
+!     
+!     xlayer: actual layer thickness
+!     tlayer: total thickness of all layers
+!     dlayer: total thickness of all layers up to the actual one
+!    
+               indexe=ipkon(i)
+               do kk=1,mint2d
+                  xi=gauss3d10(1,kk)
+                  et=gauss3d10(2,kk)
+                  call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                  tlayer(kk)=0.d0
+                  do k=1,nlayer
+                     thickness=0.d0
+                     do j=1,nopes
+                        thickness=thickness+thicke(k,indexe+j)*shp2(4,j)
+                     enddo
+                     tlayer(kk)=tlayer(kk)+thickness
+                     xlayer(k,kk)=thickness
+                  enddo
+               enddo
+!     
+               ilayer=0
+               do k=1,mint2d
                   dlayer(k)=0.d0
                enddo
             endif
@@ -378,7 +524,11 @@
             elseif(lakon(i)(4:4).eq.'4') then
                mint3d=1
             elseif(lakon(i)(4:5).eq.'15') then
-               mint3d=9
+               if(lakonl(7:8).eq.'LC') then
+                  mint3d=6*nlayer
+               else
+                  mint3d=9
+               endif
             elseif(lakon(i)(4:4).eq.'6') then
                mint3d=2
             endif
@@ -411,15 +561,20 @@
                      ze=gauss3d2(3,j)
                      weight=weight3d2(j)
                   else
-                     kl=mod(jj,8)
+!
+!                    kl: number of the integration point within the layer
+!
+                     kl=mod(j,8)
                      if(kl.eq.0) kl=8
 !     
                      xi=gauss3d2(1,kl)
                      et=gauss3d2(2,kl)
                      ze=gauss3d2(3,kl)
                      weight=weight3d2(kl)
-!     
-                     ki=mod(jj,4)
+!    
+!                    ki: position of the integration point (1...4)
+! 
+                     ki=mod(j,4)
                      if(ki.eq.0) ki=4
 !     
                      if(kl.eq.1) then
@@ -436,7 +591,7 @@
 !     
 !                    material and orientation
 !     
-                     iorien=ielorien(ilayer,i)
+                     iorien=max(0,ielorien(ilayer,i))
                   endif
                elseif(lakon(i)(4:4).eq.'2') then
                   xi=gauss3d3(1,j)
@@ -454,10 +609,44 @@
                   ze=gauss3d4(3,j)
                   weight=weight3d4(j)
                elseif(lakon(i)(4:5).eq.'15') then
-                  xi=gauss3d8(1,j)
-                  et=gauss3d8(2,j)
-                  ze=gauss3d8(3,j)
-                  weight=weight3d8(j)
+                  if(lakonl(7:8).ne.'LC') then
+                    xi=gauss3d8(1,j)
+                    et=gauss3d8(2,j)
+                    ze=gauss3d8(3,j)
+                    weight=weight3d8(j)
+                  else
+!
+!                    kl: number of the integration point within the layer
+!
+                     kl=mod(j,6)
+                     if(kl.eq.0) kl=6
+!     
+                     xi=gauss3d10(1,kl)
+                     et=gauss3d10(2,kl)
+                     ze=gauss3d10(3,kl)
+                     weight=weight3d10(kl)
+!    
+!                    ki: position of the integration point (1...3)
+! 
+                     ki=mod(j,3)
+                     if(ki.eq.0) ki=3
+!     
+                     if(kl.eq.1) then
+                        ilayer=ilayer+1
+                        if(ilayer.gt.1) then
+                           do k=1,3
+                              dlayer(k)=dlayer(k)+xlayer(ilayer-1,k)
+                           enddo
+                        endif
+                     endif
+                     ze=2.d0*(dlayer(ki)+(ze+1.d0)/2.d0*
+     &                    xlayer(ilayer,ki))/tlayer(ki)-1.d0
+                     weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!     
+!                    material and orientation
+!     
+                     iorien=max(0,ielorien(ilayer,i))
+                  endif
                elseif(lakon(i)(1:4).eq.'C3D6') then
                   xi=gauss3d7(1,j)
                   et=gauss3d7(2,j)
@@ -498,6 +687,8 @@
                   endif
                   cycle
                endif
+!
+!              coordinates of the integration point
 !
                do k=1,3
                   coords(k,j)=0.d0
@@ -505,6 +696,8 @@
                      coords(k,j)=coords(k,j)+xl(k,l)*shp(4,l)
                   enddo
                enddo
+!
+!              transforming the vector or tensor field
 !
                if(nfield.eq.3) then
                   call transformatrix(orab(1,iorien),coords(1,j),a)
@@ -620,14 +813,30 @@ c     Bernhardi end
                   enddo
                enddo
             elseif(lakonl(4:4).eq.'1') then
-               do j=1,6
-                  do k=1,nfield
-                     field(k,j)=0.d0
-                     do l=1,9
-                        field(k,j)=field(k,j)+a9(j,l)*yiloc(k,l)
+               if(lakonl(7:8).ne.'LC') then
+                  do j=1,6
+                     do k=1,nfield
+                        field(k,j)=0.d0
+                        do l=1,9
+                           field(k,j)=field(k,j)+a9(j,l)*yiloc(k,l)
+                        enddo
                      enddo
                   enddo
-               enddo
+               else
+                  do m=1,nlayer
+                     jj=15*(m-1)
+                     ll=6*(m-1)
+                     do j=1,6
+                        do k=1,nfield
+                           field(k,jj+j)=0.d0
+                           do l=1,6
+                              field(k,jj+j)=
+     &                           field(k,jj+j)+a6(j,l)*yiloc(k,ll+l)
+                           enddo
+                        enddo
+                     enddo
+                  enddo
+               endif
             else
                do j=1,6
                   do k=1,nfield
@@ -722,14 +931,30 @@ c     Bernhardi end
                   enddo
                enddo
             elseif(lakonl(4:4).eq.'1') then
-               do j=1,6
-                  do k=1,nfield
-                     field(k,j)=0.d0
-                     do l=1,9
-                        field(k,j)=field(k,j)+a9(j,l)*yi(k,l,i)
+               if(lakonl(7:8).ne.'LC') then
+                  do j=1,6
+                     do k=1,nfield
+                        field(k,j)=0.d0
+                        do l=1,9
+                           field(k,j)=field(k,j)+a9(j,l)*yi(k,l,i)
+                        enddo
                      enddo
                   enddo
-               enddo
+               else
+                  do m=1,nlayer
+                     jj=15*(m-1)
+                     ll=6*(m-1)
+                     do j=1,6
+                        do k=1,nfield
+                           field(k,jj+j)=0.d0
+                           do l=1,6
+                              field(k,jj+j)=
+     &                           field(k,jj+j)+a6(j,l)*yi(k,ll+l,i)
+                           enddo
+                        enddo
+                     enddo
+                  enddo
+               endif
             else
                do j=1,6
                   do k=1,nfield
@@ -771,12 +996,24 @@ c     Bernhardi end
                enddo
             enddo
          elseif(lakonl(4:5).eq.'15') then
-            do j=7,15
-               do k=1,nfield
-                  field(k,j)=(field(k,nonei15(2,j-6))+
-     &                 field(k,nonei15(3,j-6)))/2.d0
+            if(lakonl(7:8).ne.'LC') then
+               do j=7,15
+                  do k=1,nfield
+                     field(k,j)=(field(k,nonei15(2,j-6))+
+     &                    field(k,nonei15(3,j-6)))/2.d0
+                  enddo
                enddo
-            enddo
+            else
+               do m=1,nlayer
+                  jj=15*(m-1)
+                  do j=7,15
+                     do k=1,nfield
+                        field(k,jj+j)=(field(k,jj+nonei15(2,j-6))
+     &                     +field(k,jj+nonei15(3,j-6)))/2.d0
+                     enddo
+                  enddo
+               enddo
+            endif
          endif
 !
 !        transferring the field values into yn
@@ -795,7 +1032,8 @@ c     Bernhardi end
                   yn(k,kon(indexe+nopeexp+j))=
      &            yn(k,kon(indexe+nopeexp+j))+field(k,j)
                enddo
-               inum(kon(indexe+nopeexp+j))=inum(kon(indexe+nopeexp+j))+1
+               inum(kon(indexe+nopeexp+j))=
+     &              inum(kon(indexe+nopeexp+j))+1
             enddo
          endif
 !
@@ -827,8 +1065,8 @@ c     Bernhardi end
 !     finding the solution in the original nodes
 !
       if((cflag.ne.' ').and.(cflag.ne.'E')) then
-         call map3dto1d2d(yn,ipkon,inum,kon,lakon,nfield,nk,ne,cflag,co,
-     &         vold,force,mi)
+         call map3dto1d2d(yn,ipkon,inum,kon,lakon,nfield,nk,ne,cflag,
+     &         co,vold,force,mi,ielprop,prop)
       endif
 !
       return
