@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@
      &     node1,node2,node3
 !     
       real*8 cam(5),vold(0:mi(2),*),v(0:mi(2),*),xloadact(2,*)
+!
+!
 !     
 !     calculating the change of gas temperature: is taken
 !     into account in the global convergence for purely
@@ -36,7 +38,7 @@
 !     the network solution is not iterated to speed up
 !     the calculation)
 !
-      if(network.eq.0) then
+      if(network.le.1) then
          do i=1,ntg
             node=itg(i)
             if(nactdog(0,node).eq.0) cycle
@@ -57,7 +59,7 @@
          enddo
       enddo
 !
-!     determining all outflowing mass flow in the end nodes and
+!     determining all inflowing mass flow in the end nodes and
 !     assigning it to the end nodes
 !
       do i=1,ne
@@ -89,14 +91,17 @@
          node2=kon(indexe+2)
          if(kon(indexe+1).ne.0)  then
             node1=kon(indexe+1)
-            if(vold(1,node2).gt.0.d0) 
-     &         vold(1,node1)=vold(1,node1)+vold(1,node2)
+c            if(vold(1,node2).gt.0.d0) 
+c     &         vold(1,node1)=vold(1,node1)+vold(1,node2)
+            if(vold(1,node2).lt.0.d0) 
+     &         vold(1,node1)=vold(1,node1)-vold(1,node2)
          endif
          if(kon(indexe+3).ne.0) then
             node3=kon(indexe+3)
-            if(vold(1,node2).lt.0.d0) 
-     &         vold(1,node3)=vold(1,node3)+vold(1,node2)
+c            if(vold(1,node2).lt.0.d0) 
 c     &         vold(1,node3)=vold(1,node3)-vold(1,node2)
+            if(vold(1,node2).gt.0.d0) 
+     &         vold(1,node3)=vold(1,node3)+vold(1,node2)
          endif
       enddo
 !     
