@@ -1,7 +1,6 @@
-
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -21,7 +20,7 @@
      &  ialset,nset,ielmat,matname,nmat,
      &  irstrt,istep,istat,n,iline,ipol,
      &  inl,ipoinp,inp,lakon,ielprop,nprop,nprop_,prop,
-     &  ipoinpc,mi)
+     &  ipoinpc,mi,ier)
 !
 !     reading the input deck: *FLUID SECTION
 !
@@ -36,11 +35,11 @@
       character*81 set(*),elset
       character*132 textpart(16)
 !
-      integer mi(*),istartset(*),iendset(*),ialset(*),
-     &  ielmat(mi(3),*),irstrt,nset,nmat,ndprop,npropstart,ndpropread,
+      integer mi(*),istartset(*),iendset(*),ialset(*),nconstants,
+     &  ielmat(mi(3),*),irstrt(*),nset,nmat,ndprop,npropstart,
      &  istep,istat,n,key,i,j,k,imaterial,ipos,lprop,ipoinpc(0:*),
      &  iline,ipol,inl,ipoinp(2,*),inp(3,*),ielprop(*),nprop,nprop_,
-     &  nodea,nodeb,noil _mat,npu,nfix,nstart
+     &  nodea,nodeb,noil _mat,npu,nfix,nstart,iset,ier,ndpropread
 !
       real*8 prop(*)
 
@@ -48,11 +47,12 @@
       liquid=.false.
       manning=.false.
 !
-      if((istep.gt.0).and.(irstrt.ge.0)) then
+      if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) 
      &      '*ERROR reading *FLUID SECTION: *FLUID SECTION should'
          write(*,*) '  be placed before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       typename='
@@ -69,8 +69,11 @@
 !           
          elseif(textpart(i)(1:5).eq.'TYPE=') then
             read(textpart(i)(6:85),'(a80)',iostat=istat) typename
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*FLUID SECTION%",ier)
+               return
+            endif
 !           
          elseif(textpart(i)(1:4).eq.'OIL=') then
             read(textpart(i)(5:85),'(a80)',iostat=istat) typename_oil
@@ -84,13 +87,21 @@
                write(*,*) 
      &            '*ERROR reading *FLUID SECTION: no oil with the'
                write(*,*) '       name',typename_oil,'has been defined'
-               call exit(201)
+               ier=1
+               return
             endif
 !
          elseif(textpart(i)(1:6).eq.'LIQUID') then
             liquid=.true.
          elseif(textpart(i)(1:7).eq.'MANNING') then
             manning=.true.
+         elseif(textpart(i)(1:10).eq.'CONSTANTS=') then
+            read(textpart(i)(11:20),'(i10)',iostat=istat) nconstants
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*FLUID SECTION%",ier)
+               return
+            endif
          else
             write(*,*) 
      &      '*WARNING reading *FLUID SECTION: parameter not recognized:'
@@ -105,19 +116,26 @@
 !
       if(typename(1:18).eq.'ABSOLUTETORELATIVE') then
          elname='ATR    '
-         ndprop=3
+         ndprop=4
 !
 !     version of Yavor Dobrev
 !
       elseif(typename(1:10).eq.'ACCTUBEONE') then
          elname ='ACCTUBO'
-         ndprop=19 
+         ndprop=60 
 !
 !     version of Stefanie Asenbauer
 !
       elseif(typename(1:7).eq.'ACCTUBE') then
          elname ='ACCTUBE'
          ndprop=24
+!
+!	Air Valve
+!
+      elseif(typename(1:8).eq.'AIRVALVE') then
+         elname ='AVLV'
+!
+         ndprop=4
 !
       elseif(typename(1:12).eq.'BLEEDTAPPING') then
          elname='ORBT   '
@@ -204,16 +222,17 @@
             write(*,*) '*ERROR reading *FLUID SECTION:'
             write(*,*) '       unknown channel section'
             call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
+     &           "*FLUID SECTION%",ier)
+            return
          endif
 !
       elseif(typename(1:14).eq.'CHARACTERISTIC') then
-         ndprop=22
+         ndprop=65
          elname='CHAR   '
 !      
       elseif(typename(1:10).eq.'CROSSSPLIT')then
          elname='CROSPL'
-         ndprop=12
+         ndprop=13
 !     
       elseif (typename(1:18).eq.'FREECONVECTIONFLOW') then
          elname='FCVF '
@@ -225,7 +244,7 @@
 !
       elseif(typename(1:12).eq.'GASPIPEFANNO') then
 !     
-!     version Fanno(friction and oil)
+!     gaspipe version Fanno(friction and oil)
 !
          if(typename(13:21).eq.'ADIABATIC') then
             if(typename(22:27).eq.'ALBERS') then
@@ -234,9 +253,12 @@
             elseif(typename(22:28).eq.'FRIEDEL') then
                elname='GAPFAF '
                ndprop=9
-            elseif(typename(22:29).eq.'FLEXIBLE') then
-               elname='GAPFB '
-               ndprop=11 
+            elseif(typename(22:35).eq.'FLEXIBLERADIUS') then
+               elname='GAPFAFR'
+               ndprop=9 
+            elseif(typename(22:44).eq.'FLEXIBLERADIUSANDLENGTH') then
+               elname='GAPFARL'
+               ndprop=9 
             else
                elname='GAPFA  '
                ndprop=9
@@ -249,40 +271,18 @@
             elseif(typename(23:29).eq.'FRIEDEL') then
                elname='GAPFIF '
                ndprop=9
+            elseif(typename(22:35).eq.'FLEXIBLERADIUS') then
+               elname='GAPFIFR'
+               ndprop=9 
+            elseif(typename(22:44).eq.'FLEXIBLERADIUSANDLENGTH') then
+               elname='GAPFIRL'
+               ndprop=9 
             else
                elname='GAPFI  '
                ndprop=9
             endif
          endif
 !
-!     generalised version(friction,rotation,oil and section variation)
-!
-      elseif(typename(1:7).eq.'GASPIPE') then
-!     
-         if(typename(8:16).eq.'ADIABATIC') then
-            if(typename(17:22).eq.'ALBERS') then
-               elname='GAPIAA '
-               ndprop=10
-            elseif(typename(17:23).eq.'FRIEDEL') then
-               elname='GAPIAF '
-               ndprop=10
-            else
-               elname='GAPIA  '
-               ndprop=10
-            endif
-         elseif(typename(8:17).eq.'ISOTHERMAL') then
-            if(typename(18:23).eq.'ALBERS') then
-               elname='GAPIIA '
-               ndprop=10
-            elseif(typename(18:24).eq.'FRIEDEL') then
-               elname='GAPIIF '
-               ndprop=10
-            else
-               elname='GAPII  '
-               ndprop=10
-            endif
-         endif
-!
 !     inlet for the general case (flows, pressures and
 !     temperatures unknown)
 !
@@ -296,7 +296,7 @@
 !     
       elseif(typename(1:9).eq.'LABYRINTH') then
          if(typename(10:17).eq.'FLEXIBLE') then
-            ndprop=22 
+            ndprop=23 
             if(typename(18:23).eq.'SINGLE') then
                elname='LABFSN '
             elseif(typename(18:25).eq.'STRAIGHT') then
@@ -319,7 +319,8 @@
          endif
 !     
       elseif(typename(1:10).eq.'LIQUIDPUMP') then
-         ndprop=20
+C         ndprop=20
+         ndprop=19
          elname='LIPU   '
 !     
       elseif (typename(1:15).eq.'MASSFLOWPERCENT') then
@@ -359,7 +360,11 @@
          elseif(typename(8:22).eq.'GEOMETRICALAREA') then
              elname='ORGA   '
          elseif(typename(8:20).eq.'BRUSHFLEXIBLE') then
-             elname='ORBF   '     
+             elname='ORBF   ' 
+         elseif(typename(8:15).eq.'BRUSHGAP') then
+             elname='ORB1   ' 
+         elseif(typename(8:21).eq.'BRISTLEPACKAGE') then
+             elname='ORB2   '
          else 
             elname='ORC1   '
          endif
@@ -410,16 +415,16 @@
          elseif(typename(5:19).eq.'WHITE-COLEBROOK') then
             if(typename(20:27).eq.'FLEXIBLE') then
                elname='LIPIWCF'
-               ndprop=5
+               ndprop=6
             else
                elname='LIPIWC '
-               ndprop=5
+               ndprop=6
             endif
          endif
 !
       elseif(typename(1:14).eq.'PRESWIRLNOZZLE') then
          elname='ORPN   '
-         ndprop=40
+         ndprop=41
 !
       elseif(typename(1:13).eq.'RADIALOUTFLOW') then
          if(typename(14:24).eq.'RADIALINLET')then
@@ -461,7 +466,7 @@
          elseif(typename(11:23).eq.'BENDIDELCIRC') then
             elname='REBEIDC'
             ndprop=10
-         elseif(typename(11:23).eq.'BENDIDELRECT') then
+         elseif(typename(11:22).eq.'BENDIDELRECT') then
             elname='REBEIDR'
             ndprop=10
          elseif(typename(11:20).eq.'BENDMILLER') then
@@ -494,6 +499,10 @@
             elname='RCVN   '
          endif
          ndprop=6
+!
+      elseif(typename(1:15).eq.'ROTATINGGASPIPE') then
+         elname='GAPR   '
+         ndprop=10
 !
       else if (typename(1:6).eq.'S-PUMP') then
          elname='SPUMP  '
@@ -511,10 +520,14 @@
       elseif(typename(1:1).eq.' ') then
          elname='       '
          ndprop=0
+      elseif(typename(1:1).eq.'U') then
+         elname(1:7)=typename(1:7)
+         ndprop=nconstants
       else
          write(*,*) '*ERROR reading *FLUID SECTION: ',typename
          write(*,*) '       is an unknown fluid section type'
-         call exit(201)
+         ier=1
+         return
       endif
 !
 !     check for the existence of the set and the material
@@ -527,8 +540,8 @@
      &     '*ERROR reading *FLUID SECTION: nonexistent material'
          write(*,*) '  '
          call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
-         call exit(201)
+     &        "*FLUID SECTION%",ier)
+         return
       endif
       imaterial=i
 !
@@ -540,9 +553,10 @@
          write(*,*) '*ERROR reading *FLUID SECTION: element set ',elset
          write(*,*) '  has not yet been defined. '
          call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
-         call exit(201)
+     &        "*FLUID SECTION%",ier)
+         return
       endif
+      iset=i
 !
       npropstart=nprop
 !
@@ -550,12 +564,14 @@
 !
 !     liquid pump / gas characteristic /
 !     preswirl nozzle / bleed tapping
+!     these network elements contain curves with a variable
+!     amount of data points
 !
       if((elname(1:4).eq.'LIPU').or.(elname(1:4).eq.'CHAR').or.
      &   (elname(1:4).eq.'ORPN').or.(elname(1:4).eq.'ORBT')) then
 !
 !        determine the number of fixed inputs (before entering
-!        a curve with arbitary many data points)
+!        a curve with arbitrary many data points)
 !
          if(elname(1:4).eq.'LIPU') then
             nfix=1
@@ -568,48 +584,92 @@
          endif
          ndprop=0
          npu=0
+!
+!        npu is the number of data points, i.e. the number of
+!        abscissa-values + the number of ordinate values
+!
          do
             call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &        ipoinp,inp,ipoinpc)
             if((istat.lt.0).or.(key.eq.1)) exit
-            if(ndprop.eq.0) then
-               do j=1,nfix
-                  ndprop=ndprop+1
-                  read(textpart(j),'(f40.0)',iostat=istat) 
-     &                 prop(nprop+ndprop)
-                  if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
-               enddo
-               nstart=nfix+1
-!
-!              adding the point (0.,0.) for characteristics
-!
-               if(elname(1:4).eq.'CHAR') then
-                  prop(nprop+3)=0.d0
-                  prop(nprop+4)=0.d0
-                  npu=2
-                  nfix=2
-               endif
-            else
-               nstart=1
-            endif
-            do j=nstart,n
-               npu=npu+1
+            do j=1,n
                ndprop=ndprop+1
+               if(ndprop.eq.nfix) then
+                  if(elname(1:4).eq.'CHAR') then
+                     prop(nprop+3)=0.d0
+                     prop(nprop+4)=0.d0
+                     npu=2
+                     nfix=2
+                     cycle
+                  endif
+               elseif(ndprop.gt.nfix) then
+                  npu=npu+1
+               endif
+!
+!              check whether number of data points is smaller than allowed
+!
+               if(elname(1:4).eq.'LIPU') then
+                  if(ndprop.gt.19) then
+                     write(*,*) 'ERROR reading *FLUID SECTION: more'
+                     write(*,*) '      than 9 pairs were defined for'
+                     write(*,*) '      fluid section type LIQUID PUMP'
+                     ier=1
+                     return
+                  endif
+               elseif(elname(1:4).eq.'CHAR') then
+                  if(ndprop.gt.65) then
+                     write(*,*) 'ERROR reading *FLUID SECTION: more'
+                     write(*,*) '      than 30 pairs were defined for'
+                     write(*,*) 
+     &              '      fluid section type CHARACTERISTIC'
+                     ier=1
+                     return
+                  endif
+               elseif(elname(1:4).eq.'ORPN') then
+                  if(ndprop.gt.41) then
+                     write(*,*) 'ERROR reading *FLUID SECTION: more'
+                     write(*,*) '      than 17 pairs were defined for'
+                     write(*,*) 
+     &              '      fluid section type PRESWIRL NOZZLE'
+                     ier=1
+                     return
+                  endif
+               elseif(elname(1:4).eq.'ORBT') then
+                  if(ndprop.gt.41) then
+                     write(*,*) 'ERROR reading *FLUID SECTION: more'
+                     write(*,*) '      than 18 pairs were defined for'
+                     write(*,*) '      fluid section type BLEED TAPPING'
+                     ier=1
+                     return
+                  endif
+               endif
+!
                read(textpart(j),'(f40.0)',iostat=istat) 
      &              prop(nprop+ndprop)
-               if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
+               if(istat.gt.0) then
+                  call inputerror(inpc,ipoinpc,iline,
+     &                 "*FLUID SECTION%",ier)
+                  return
+               endif
             enddo
          enddo
 !
+!        filling up to nfix if less values were specified
+!
+         if(ndprop.lt.nfix) then
+            do j=ndprop+1,nfix
+               prop(nprop+j)=0.d0
+            enddo
+            ndprop=nfix
+         endif
+!
 !        check whether data points are paired
 !
          if(2*(npu/2).ne.npu) then
-            write(*,*) '*ERROR reading *FLUID SECTION: less y data'
-            write(*,*) '       points x data points for fluid'
-            write(*,*) '       section type',elname(1:4)
-            call exit(201)
+            write(*,*) '*WARNING reading *FLUID SECTION: less y data'
+            write(*,*) '         points x data points for fluid'
+            write(*,*) '         section type',elname(1:4)
+            npu=npu-1
          endif
 !
          prop(nprop+nfix)=npu/2
@@ -623,13 +683,15 @@
                   write(*,*) '*ERROR reading *FLUID SECTION: volumetric'
                   write(*,*) '       flow data points must be in'
                   write(*,*) '       strict increasing order'
-                  call exit(201)
+                  ier=1
+                  return
                elseif(prop(nprop+nfix+2*j).gt.prop(nprop+nfix+2*j-2))
      &              then
                   write(*,*) '*ERROR reading *FLUID SECTION: total'
                   write(*,*) '       head data points must be '
                   write(*,*) '       decreasing'
-                  call exit(201)
+                  ier=1
+                  return
                endif
             enddo
          endif
@@ -637,61 +699,14 @@
          nprop=nprop+ndprop+1
 ! 
       elseif(elname.eq.'ACCTUBE') then
-!         
-!        Read the first 20 elements
-! 
-         lprop=0
-         ndpropread=ndprop
-!         
-         do j=1,(ndpropread-1)/8+1
-            call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
-     &              ipoinp,inp,ipoinpc)
-            if((istat.lt.0).or.(key.eq.1)) exit
-            do k=1,8
-               lprop=lprop+1
-               if(lprop.gt.ndpropread) exit
-               read(textpart(k),'(f40.0)',iostat=istat)
-     &                 prop(nprop+lprop)
-               
-               if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
-               
-!              If 20 elements have been read, check how many more
-!              are to be read
-               if (lprop.eq.20) then
-                  ndpropread = lprop +
-     &                        (prop(nprop+19)+prop(nprop+20))*2
-                  ndprop = ndpropread
-               endif
-            enddo
-         enddo
-!         
-!        Until now 3 lines have been read
-!        If necessary read the rest
 !
-         if(ndpropread.gt.lprop) then
-            do j=3,(ndpropread-1)/8+1
-               call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
-     &              ipoinp,inp,ipoinpc)
-               if((istat.lt.0).or.(key.eq.1)) exit
-               do k=1,8
-                  lprop=lprop+1
-                  if(lprop.gt.ndpropread) exit
-                  read(textpart(k),'(f40.0)',iostat=istat)
-     &                 prop(nprop+lprop)
-               
-                  if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
-               enddo
-            enddo
-         endif
-!
-         nprop=nprop+ndprop
-!
-         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
-     &              ipoinp,inp,ipoinpc)
+!        acc-tube (proprietary)
 !         
       elseif(ndprop.gt.0) then
+!
+!        reading the element properties
+!
+!        general case
 !
          lprop=0
 !
@@ -704,8 +719,11 @@
                if(lprop.gt.ndprop) exit
                read(textpart(k),'(f20.0)',iostat=istat) 
      &             prop(nprop+lprop)
-               if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*FLUID SECTION%")
+               if(istat.gt.0) then
+                  call inputerror(inpc,ipoinpc,iline,
+     &                 "*FLUID SECTION%",ier)
+                  return
+               endif
             enddo
          enddo
          nprop=nprop+ndprop
@@ -716,7 +734,8 @@
 !
       if(nprop.gt.nprop_) then
          write(*,*) '*ERROR reading *FLUID SECTION: increase nprop_'
-         call exit(201)
+         ier=1
+         return
       endif
 !
 !     complementing the properties of restrictor elements
@@ -734,57 +753,30 @@
       elseif(elname(1:4).eq.'REEX') then
 !       
             prop(npropstart+6)=noil_mat
-            prop(npropstart+2)=100000*prop(npropstart+1)
 !     
       elseif(elname(1:6).eq.'REWAOR') then
 !        
             prop(npropstart+6)=noil_mat
-            prop(npropstart+1)=100000*prop(npropstart+2)
 !
       elseif(elname(1:4).eq.'REEN') then
 !        
             prop(npropstart+6)=noil_mat
 !
 !           zeta (loss coefficient for an entry)
-!
-            prop(npropstart+4)=0.5d0
-            prop(npropstart+1)=100000*prop(npropstart+2)
 !
       elseif(elname(1:7).eq.'REBRJI1') then
          prop(npropstart+11)=noil_mat
 !
       elseif(elname(1:7).eq.'REBRJI2') then
          prop(npropstart+11)=noil_mat
-         if(1.d0-(prop(npropstart+5)+prop(npropstart+6))/
-     &        prop(npropstart+4).gt.0.01d0)then
-            write(*,*) '*ERROR: reading *FLUID SECTION:'
-            write(*,*) '        in element type RESTRICTOR 
-     &                           BRANCH JOINT IDELCHIK2'
-            write(*,*) '        A0 ist not equal to A1+A2'
-            call exit(201)
-         endif
       elseif(elname(1:6).eq.'REBRJG') then
          prop(npropstart+11)=noil_mat
       elseif(elname(1:6).eq.'REBRSG') then
          prop(npropstart+11)=noil_mat
       elseif(elname(1:7).eq.'REBRSI1') then
          prop(npropstart+15)=noil_mat
-         if(dabs(prop(npropstart+13)).le.1E-5) then
-            prop(npropstart+13)=1.d0
-         endif
-         if(dabs(prop(npropstart+14)).le.1E-5) then
-            prop(npropstart+14)=1.d0
-         endif
-!     
       elseif(elname(1:7).eq.'REBRSI2') then
          prop(npropstart+13)=noil_mat
-         if(dabs(prop(npropstart+11)).le.1E-5) then
-            prop(npropstart+11)=1.d0
-         endif
-         if(dabs(prop(npropstart+12)).le.1E-5) then
-            prop(npropstart+12)=1.d0
-         endif
-!     
       endif
 !
 !     k_oil for restrictors type USER, ENTRENCE, LONG ORIFICE IDELCHIK,
@@ -809,7 +801,7 @@
 !
 !     k_oil for pipe elements
 !
-      elseif(elname(1:3).eq.'GAP') then
+      elseif(elname(1:4).eq.'GAPF') then
          prop(npropstart+7)=noil_mat
       elseif(elname(1:4).eq.'LICH') then
 !
@@ -830,12 +822,13 @@
 !     assigning the elements of the set the appropriate material
 !     and property pointer
 !
-      do j=istartset(i),iendset(i)
+      do j=istartset(iset),iendset(iset)
          if(ialset(j).gt.0) then
             if(lakon(ialset(j))(1:1).ne.'D') then
                write(*,*) '*ERROR reading *FLUID SECTION: element ',
      &            ialset(j),' is no fluid element.'
-               call exit(201)
+               ier=1
+               return
             endif
             lakon(ialset(j))(2:8)=elname(1:7)
 !
@@ -867,7 +860,8 @@
                write(*,*) ' Please change element type', 
      &' and/or definition.'
                write(*,*) ' Calculation stopped.'
-               call exit(201)
+               ier=1
+               return
             endif
 !
             ielmat(1,ialset(j))=imaterial
@@ -880,7 +874,8 @@
                if(lakon(k)(1:1).ne.'D') then
                   write(*,*) '*ERROR reading *FLUID SECTION: element ',
      &                 k,' is no fluid element.'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                lakon(k)(2:8)=elname(1:7)
 !
@@ -912,7 +907,8 @@
                   write(*,*) ' Please change element type ',
      &'and/or definition.'
                   write(*,*) ' Calculation stopped.'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                ielmat(1,k)=imaterial
                if(ndprop.gt.0) ielprop(k)=npropstart
