@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,10 +20,15 @@
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,R,rho,physcon,g,co,dvi,numf,
      &     vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider,
-     &     ttime,time,iaxial)
+     &     ttime,time,iaxial,iplausi)
 !
-!     determine whether the flux in the element is an unknown 
+!     gas element routines 
 !     
+!     mass flow input for all gas element routines is the gas
+!     flow for a 2 degrees segment with the correct sign
+!     (positive if from node 1 to node2 of the element,
+!      negative if from node 2 to node1 of the element)
+!
       implicit none
 !     
       logical identity
@@ -32,7 +37,7 @@
 !      
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
      &     ielprop(*),nodef(8),idirf(8),kflag,ipkon(*),kon(*),
-     &     nshcon(*), nrhcon(*),ntmat_,mi(*),ider,iaxial
+     &     nshcon(*), nrhcon(*),ntmat_,mi(*),ider,iaxial,iplausi
 !      
       real*8 prop(*),v(0:mi(2),*),xflow,f,df(8),R,cp,physcon(*),rho,
      &     g(3),co(3,*),dvi,vold(0:mi(2),*),shcon(0:3,ntmat_,*),
@@ -45,7 +50,8 @@
 !     
          call absolute_relative(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &        nodef,idirf,df,cp,r,physcon,numf,set,mi,ttime,time,iaxial)
+     &        nodef,idirf,df,cp,r,physcon,numf,set,mi,ttime,time,iaxial,
+     &        iplausi)
 !     
       elseif(lakon(nelem)(2:8).eq.'ACCTUBO') then
 !
@@ -54,7 +60,7 @@
          call acctube_one(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,mi,ider,
-     &        ttime,time,iaxial)
+     &        ttime,time,iaxial,iplausi)
 !
       elseif(lakon(nelem)(2:8).eq.'ACCTUBE') then
 !
@@ -63,7 +69,16 @@
          call acctube(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,mi,ider,
-     &        ttime,time,iaxial)
+     &        ttime,time,iaxial,iplausi)
+!
+!	 proprietary
+!
+      elseif(lakon(nelem)(2:5).eq.'AVLV') then 
+!
+         call air_valve(node1,node2,nodem,nelem,lakon,kon,ipkon,
+     &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
+     &     nodef,idirf,df,cp,R,physcon,dvi,numf,set,co,vold,mi,
+     &     ttime,time,iaxial,iplausi)
 !
       elseif(lakon(nelem)(2:6).eq.'CARBS') then  
 !
@@ -71,7 +86,8 @@
 !
          call carbon_seal(node1,node2,nodem,nelem,lakon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &     nodef,idirf,df,R,physcon,dvi,numf,set,mi,ttime,time,iaxial)
+     &     nodef,idirf,df,R,physcon,dvi,numf,set,mi,ttime,time,
+     &     iaxial,iplausi)
 !     
       elseif(lakon(nelem)(2:5).eq.'CHAR') then 
 !
@@ -80,7 +96,7 @@
          call characteristic(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,
-     &        mi,ttime,time,iaxial)
+     &        mi,ttime,time,iaxial,iplausi)
 !
       elseif(lakon(nelem)(2:5).eq.'CROS') then 
 !         
@@ -89,7 +105,7 @@
          call cross_split(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,r,physcon,numf,set,mi,ider,ttime,time,
-     &     iaxial)
+     &     iaxial,iplausi)
 !
 !     proprietary
 !
@@ -97,7 +113,8 @@
          call free_disc_pumping(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !  
 !     proprietary
 ! 
@@ -105,16 +122,8 @@
          call free_convection(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
-! 
-!     gas pipe
-!
-      elseif(lakon(nelem)(2:5).eq.'GAPI') then 
-!
-         call gaspipe(node1,node2,nodem,nelem,lakon,kon,ipkon,
-     &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !
 !     gas pipe fanno
 !
@@ -123,7 +132,18 @@
          call gaspipe_fanno(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
+!
+!     rotating gas pipe
+!
+      elseif(lakon(nelem)(2:5).eq.'GAPR') then 
+!
+         call gaspipe_rot(node1,node2,nodem,nelem,lakon,kon,ipkon,
+     &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
+     &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !
 !     straight and stepped labyrinth
 !
@@ -132,7 +152,7 @@
          call labyrinth(node1,node2,nodem,nelem,lakon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,R,physcon,co,dvi,numf,vold,set,
-     &     kon,ipkon,mi,ttime,time,iaxial)
+     &     kon,ipkon,mi,ttime,time,iaxial,iplausi)
 !
 !     liquid pipes including loss elements (hydraulic elements)
 !         
@@ -141,7 +161,7 @@
          call liquidpipe(node1,node2,nodem,nelem,lakon,nactdog,identity,
      &           ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
      &           rho,g,co,dvi,numf,vold,mi,ipkon,kon,set,ttime,time,
-     &           iaxial)
+     &           iaxial,iplausi)
 !
 !     liquid channel (flow with free surface) including all loss elements
 !
@@ -149,7 +169,7 @@
 !         
          call liquidchannel(node1,node2,nodem,nelem,lakon,nactdog,
      &           identity,ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
-     &           rho,g,co,dvi,numf,mi,ipkon,kon)
+     &           rho,g,co,dvi,numf,mi,ipkon,kon,iplausi)
 !
 !     liquid pipes including loss elements (types derived from their
 !     compressible equivalent)
@@ -159,7 +179,7 @@
          call liquidpipe(node1,node2,nodem,nelem,lakon,nactdog,identity,
      &           ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
      &           rho,g,co,dvi,numf,vold,mi,ipkon,kon,set,ttime,time,
-     &           iaxial)
+     &           iaxial,iplausi)
 !
 !     liquid pump
 !
@@ -167,15 +187,17 @@
 !        
          call liquidpump(node1,node2,nodem,nelem,nactdog,identity,
      &           ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
-     &           rho,g,co,numf,mi,ttime,time,iaxial)
+     &           rho,g,co,numf,mi,ttime,time,iaxial,iplausi)
 !
-!     ?????
+!     element that fixes the mass flow as a specific percentage of the
+!     sum of the massflow of up to 10 other elements
 !
       elseif(lakon(nelem)(2:5).eq.'MFPC') then 
          call massflow_percent(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !     
 !     Moehring
 ! 
@@ -183,7 +205,8 @@
 !     
          call moehring(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &        nodef,idirf,df,cp,r,dvi,numf,set,mi,ttime,time,iaxial)
+     &        nodef,idirf,df,cp,r,dvi,numf,set,mi,ttime,time,
+     &        iaxial,iplausi)
 !
 !     Bleed tapping, orifice and pre-swirl nozzle
 !
@@ -192,7 +215,7 @@
          call orifice(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,co,vold,mi,
-     &        ttime,time,iaxial)
+     &        ttime,time,iaxial,iplausi)
 ! 
 !     proprietary
 !
@@ -200,7 +223,8 @@
 !
          call rcavi(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &     nodef,idirf,df,cp,R,dvi,numf,set,mi,ttime,time,iaxial)
+     &     nodef,idirf,df,cp,R,dvi,numf,set,mi,ttime,time,
+     &     iaxial,iplausi)
 !
 !     proprietary
 !
@@ -208,27 +232,31 @@
 !
          call rcavi2(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &     nodef,idirf,df,cp,R,dvi,numf,set,mi,ttime,time,iaxial)
+     &     nodef,idirf,df,cp,R,dvi,numf,set,mi,ttime,time,
+     &     iaxial,iplausi)
 !
 !     restrictors
 !
-      elseif((lakon(nelem)(2:3).eq.'RE').and.
+      elseif(((lakon(nelem)(2:3).eq.'RE').or.
+     &        (lakon(nelem)(2:3).eq.'RB')).and.
      &       (lakon(nelem)(2:8).ne.'REBRSI1').and.
      &       (lakon(nelem)(2:8).ne.'REBRSI2')) then 
-!         
+!          
          call restrictor(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,mi,ttime,time,iaxial,
+     &        co,vold,iplausi)
 !
 !     proprietary
 !
-      elseif(lakon(nelem)(2:5).eq.'RIMS') then   
+      elseif((lakon(nelem)(2:5).eq.'RIMS').or.
+     &       (lakon(nelem)(2:8).eq.'RIMFLEX')) then   
 !
          call rimseal(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,mi,ttime,time,
-     &        iaxial)
+     &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,mi,
+     &        ttime,time,iaxial,co,vold,iplausi)
 !     
 !     proprietary
 !
@@ -237,7 +265,7 @@
         call scavenge_pump(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,ntmat_,mi,
-     &        ttime,time,iaxial)   
+     &        ttime,time,iaxial,iplausi)   
 !   
 !     branch split Idelchik2
 !  
@@ -246,7 +274,16 @@
          call tee(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,r,physcon,numf,set,mi,ider,ttime,time,
-     &     iaxial)
+     &     iaxial,iplausi)
+!
+!     user element
+!
+      elseif(lakon(nelem)(2:2).eq.'U') then
+!         
+         call user_network_element(node1,node2,nodem,nelem,lakon,kon,
+     &        ipkon,nactdog,identity,ielprop,prop,kflag,v,xflow,f,
+     &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,co,vold,mi,
+     &        ttime,time,iaxial,iplausi)
 !
 !     vortex
 !
@@ -254,7 +291,8 @@
 !     
          call vortex(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,kflag,v,xflow,f,
-     &        nodef,idirf,df,cp,r,numf,set,mi,ttime,time,iaxial) 
+     &        nodef,idirf,df,cp,r,numf,set,mi,ttime,time,iaxial,
+     &        iplausi) 
 !
 !     branch split Idelchik1
 !  
@@ -263,7 +301,7 @@
          call wye(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,r,physcon,numf,set,mi,ider,ttime,time,
-     &     iaxial)
+     &     iaxial,iplausi,dvi)
 !
       else
          identity=.true.
