@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,8 @@
       subroutine free_convection(node1,node2,nodem,nelem,lakon,kon,
      &        ipkon,nactdog,identity,ielprop,prop,iflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !          
 !     Free-convection-Flow
 !     
@@ -30,12 +31,14 @@
       character*81 set(*)
 !     
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
-     &     ielprop(*),nodef(4),idirf(4),iflag,iaxial,
-     &     ipkon(*),kon(*),mi(*),nrhcon(*),ntmat_,nshcon(*)
+     &     ielprop(*),nodef(*),idirf(*),iflag,iaxial,
+     &     ipkon(*),kon(*),mi(*),nrhcon(*),ntmat_,nshcon(*),iplausi
 !     
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),cp,r,dvi,
-     &     physcon(3),co(3,*),vold(0:mi(2),*),ttime,time,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),cp,r,dvi,
+     &     physcon(*),co(3,*),vold(0:mi(2),*),ttime,time,
      &     shcon(0:3,ntmat_,*),rhcon(0:1,ntmat_,*)
+!
+!
 !  
       if (iflag.eq.0) then
          identity=.true.
