@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,8 @@
       subroutine free_disc_pumping(node1,node2,nodem,nelem,lakon,kon,
      &        ipkon,nactdog,identity,ielprop,prop,iflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !     
 !     Free-Disc-Pump-Flow
 !     
@@ -30,12 +31,14 @@
       character*81 set(*)
 !     
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,iaxial,
-     &     ipkon(*),kon(*),mi(*),nrhcon(*),ntmat_,nshcon(*)
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,iaxial,
+     &     ipkon(*),kon(*),mi(*),nrhcon(*),ntmat_,nshcon(*),iplausi
 !     
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),ttime,time,
-     &     physcon(3),co(3,*),vold(0:mi(2),*),r,cp,dvi,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),ttime,time,
+     &     physcon(*),co(3,*),vold(0:mi(2),*),r,cp,dvi,
      &     shcon(0:3,ntmat_,*),rhcon(0:1,ntmat_,*)
+!
+!
 !
       if (iflag.eq.0) then
          identity=.true.
@@ -48,7 +51,7 @@
             identity=.false.
          endif
 !     
-      elseif ((iflag.eq.1) .or.(iflag.eq.2).or.(iflag.eq.3))then
+      elseif ((iflag.eq.1).or.(iflag.eq.2).or.(iflag.eq.3))then
 !
 !    User defined flow element
 !
