@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -46,7 +46,8 @@
 !     npred              not used
 !     kstep              step number
 !     kinc               increment number
-!     area               slave area
+!     area               slave area corresponding to the contact spring
+!                        element
 !
 !     OUTPUT:
 !
@@ -62,10 +63,7 @@
       real*8 ak(5),d(2),flowm(2),temp(2),predef(2,*),time(*),coords(3),
      &  area
 !
-      intent(in) d,flowm,temp,predef,time,ciname,slname,
-     &   msname,coords,noel,node,npred,kstep,kinc,area
 !
-      intent(inout) ak
 !
 !     insert code here
 !
