@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -25,7 +25,8 @@
      &  nload,ithermal,ntrans,co,ixfree,ikfree,inoelfree,iponoelmax,
      &  iperturb,tinc,tper,tmin,tmax,ctrl,typeboun,nmethod,nset,set,
      &  istartset,iendset,ialset,prop,ielprop,vold,mi,nkon,ielmat,
-     &  icomposite,t0g,t1g,idefforc,iamt1)
+     &  icomposite,t0g,t1g,idefforc,iamt1,orname,orab,norien,norien_,
+     &  ielorien,jobnamec,ne2boun)
 !
 !     generates three-dimensional elements:
 !         for plane stress
@@ -37,39 +38,66 @@
 !
       character*1 typeboun(*)
       character*8 lakon(*)
-      character*20 labmpc(*),sideload(*),label
+      character*20 labmpc(*),sideload(*)
+      character*80 orname(*)
       character*81 set(*)
+      character*132 jobnamec(*),fn
 !
       integer ipompc(*),nodempc(3,*),nmpc,nmpc_,mpcfree,ikmpc(*),
-     &  ilmpc(*),kon(*),ipkon(*),ne,mpc,indexe,i,j,k,node,idof,
-     &  id,mpcfreeold,ikboun(*),ilboun(*),nboun,nboun_,kflag,idummy,
-     &  iterm(500),nterm,neigh(7,8),l,m,nodeboun(*),ndirboun(*),nk,
-     &  nk_,index,iponoel(*),inoel(3,*),inoelfree,istep,nmpcold,
+     &  ilmpc(*),kon(*),ipkon(*),ne,indexe,i,j,node,index,
+     &  ikboun(*),ilboun(*),nboun,nboun_,ishift,iexpand,
+     &  neigh(7,8),nodeboun(*),ndirboun(*),nk,iflagpl,
+     &  nk_,iponoel(*),inoel(3,*),inoelfree,istep,nmpcold,
      &  ikforc(*),ilforc(*),nodeforc(2,*),ndirforc(*),iamforc(*),
-     &  nforc,nforc_,ithermal(2),nload,iamboun(*),
-     &  ntrans,inotr(2,*),nam,iponoelmax,iperturb,numnod,itransaxial,
+     &  nforc,nforc_,ithermal(*),nload,iamboun(*),
+     &  ntrans,inotr(2,*),nam,iponoelmax,iperturb(*),numnod,itransaxial,
      &  rig(*),nmethod,nset,istartset(*),iendset(*),ialset(*),nkon,
-     &  ielprop(*),idir,indexref,indexold,idofold,idold,indexnew,
-     &  idofnew,idnew,ksol,lsol,nmpc0,nmpc01,nmpcdif,mi(*),nope,
+     &  ielprop(*),mi(*),nope,ilen,
      &  ielmat(mi(3),*),iponor(2,*),knor(*),ixfree,ikfree,icomposite,
-     &  idefforc(*),idim,iamt1(*)
+     &  idefforc(*),idim,iamt1(*),norien,norien_,ielorien(mi(3),*),
+     &  ne2boun(2,*)
 !
       real*8 coefmpc(*),thicke(mi(3),*),xnor(*),thickn(2,*),tinc,
-     &  tper,tmin,t0g(2,*),t1g(2,*),e1(3),e2(3),xt1(3),
-     &  tmax,offset(2,*),t0(*),t1(*),xforc(*),trab(7,*),co(3,*),b(3,3),
-     &  xboun(*),pi,ctrl(*),prop(*),vold(0:mi(2),*),xlag(3,20),
-     &  xeul(3,20),a(3,3),xi,et,ze,coloc(3,8),xj
+     &  tper,tmin,t0g(2,*),t1g(2,*),e1(3),e2(3),xt1(3),orab(7,*),
+     &  tmax,offset(2,*),t0(*),t1(*),xforc(*),trab(7,*),co(3,*),
+     &  xboun(*),pi,ctrl(*),prop(*),vold(0:mi(2),*),
+     &  coloc(3,8)
 !
       data neigh /1,9,2,12,4,17,5,2,9,1,10,3,18,6,
      &            3,11,4,10,2,19,7,4,11,3,12,1,20,8,
      &            5,13,6,16,8,17,1,6,13,5,14,7,18,2,
      &            7,15,8,14,6,19,3,8,15,7,16,5,20,4/
 !
-      data coloc /-1.,-1.,-1.,1.,-1.,-1.,1.,1.,-1.,-1.,1.,-1.,
-     &            -1.,-1.,1.,1.,-1.,1.,1.,1.,1.,-1.,1.,1./
+      data coloc /-1.d0,-1.d0,-1.d0,1.d0,-1.d0,-1.d0,1.d0,1.d0,-1.d0,
+     &            -1.d0,1.d0,-1.d0,
+     &            -1.d0,-1.d0,1.d0,1.d0,-1.d0,1.d0,1.d0,1.d0,1.d0,-1.d0,
+     &             1.d0,1.d0/
 !
+      iflagpl=0
       pi=4.d0*datan(1.d0)
 !
+!     open file for 2d-information
+!
+      ilen=index(jobnamec(1),char(0))
+      if(ilen.gt.129) then
+         write(*,*) '*ERROR in gen3delem:'
+         write(*,*) '       name of file for storing the 1d/2d-info'
+         write(*,*) '       is too long (> 128 char); name = '
+         write(*,*) jobnamec(1)(1:132)
+         call exit(201)
+      else
+         fn(1:ilen-1)=jobnamec(1)(1:ilen-1)
+         fn(ilen:ilen+3)='.12d'
+         do i=ilen+4,132
+            fn(i:i)=' '
+         enddo
+      endif
+      if(istep.eq.1) then
+         open(27,file=fn,status='unknown')
+      else
+         open(27,file=fn,status='unknown',position='append')
+      endif
+!
 !     catalogueing the element per node relationship for shell/beam
 !     elements and transferring the nodal thickness to the elements
 !
@@ -89,7 +117,8 @@
          do i=1,ne
             if(ipkon(i).lt.0) cycle
             if((lakon(i)(1:2).ne.'C3').and.(lakon(i)(1:1).ne.'D').and.
-     &         (lakon(i)(1:1).ne.'G').and.(lakon(i)(1:1).ne.'E')) then
+     &         (lakon(i)(1:1).ne.'G').and.(lakon(i)(1:1).ne.'E').and.
+     &         (lakon(i)(1:4).ne.'MASS')) then
 !
 !              number of nodes belonging to the element
 !
@@ -99,7 +128,7 @@
                elseif((lakon(i)(1:1).eq.'B').or.
      &                (lakon(i)(1:1).eq.'T')) then
                   numnod=3
-               elseif((lakon(i)(2:2).eq.'3').or.
+               elseif((lakon(i)(1:2).eq.'S3').or.
      &                (lakon(i)(4:4).eq.'3')) then
                   numnod=3
                elseif((lakon(i)(2:2).eq.'4').or.
@@ -122,12 +151,37 @@
                   inoel(3,inoelfree)=iponoel(node)
                   iponoel(node)=inoelfree
                   inoelfree=inoelfree+1
+!
+!                 default is element thickness unless NODAL THICKNESS
+!                 was specified on the *SHELL SECTION or *BEAM SECTION
+!                 card. In the latter case thicke(1,indexe+j) was set
+!                 to -1.d0 in shellsections.f and beamsections.f for
+!                 all nodes j belonging to the element
+!
                   if(lakon(i)(1:2).ne.'CA') then
-                     if(thickn(1,node).gt.0.d0)
-     &                    thicke(1,indexe+j)=thickn(1,node)
+                     if(thicke(1,indexe+j).lt.-0.5d0) then
+                        if(thickn(1,node).le.0.d0) then
+                           write(*,*) '*ERROR in gen3delem:'
+                           write(*,*) 
+     &                      '       first thickness in node',node
+                           write(*,*) '       is nonpositive'
+                           call exit(201)
+                        else
+                           thicke(1,indexe+j)=thickn(1,node)
+                        endif
+                     endif
                      if(mi(3).gt.1) then
-                        if(thickn(2,node).gt.0.d0)
-     &                       thicke(2,indexe+j)=thickn(2,node)
+                        if(thicke(2,indexe+j).lt.-0.5d0) then
+                           if(thickn(2,node).le.0.d0) then
+                              write(*,*) '*ERROR in gen3delem:'
+                              write(*,*) 
+     &                          '       second thickness in node',node
+                              write(*,*) '       is nonpositive'
+                              call exit(201)
+                           else
+                              thicke(2,indexe+j)=thickn(2,node)
+                           endif
+                        endif
                      endif
                   endif
                   if(thicke(1,indexe+j).le.0.d0) then
@@ -153,33 +207,17 @@
                enddo
             endif
          enddo
-c!
-c!        checking whether any rotational degrees of freedom are fixed
-c!        by SPC's, MPC's or loaded by bending moments or torques
-c!        in the end, rig(i)=0 if no rigid knot is defined in node i,
-c!        else rig(i)=the rotational node of the knot. The value -1 is
-c!        a dummy. 
-c!
-c!        only for nonlinear dynamic calculations
-c!
-c         if((nmethod.eq.4).and.(iperturb.gt.1)) then
-c            do i=1,nboun
-c               if(ndirboun(i).gt.4) rig(nodeboun(i))=-1
-c            enddo
-c            do i=1,nforc
-c               if(ndirforc(i).gt.4) rig(nodeforc(1,i))=-1
-c            enddo
-c            do i=1,nmpc
-c               index=ipompc(i)
-c               do
-c                  if(index.eq.0) exit
-c                  if(nodempc(2,index).gt.4) then
-c                     rig(nodempc(1,index))=-1
-c                  endif
-c                 index=nodempc(3,index)
-c              enddo
-c            enddo
-c         endif
+!
+!        creating a flag to indicate that there is at least one
+!        plane stress, plane strain or axisymmetric element
+!
+         do i=1,ne
+            if(ipkon(i).lt.0) cycle
+            if((lakon(i)(1:2).eq.'CP').or.
+     &           (lakon(i)(1:2).eq.'CA')) then
+               iflagpl=1
+            endif
+         enddo
 !
 !     calculating the normals in nodes belonging to shells/beams
 !
@@ -191,7 +229,7 @@ c         endif
      &     mpcfree,ikmpc,ilmpc,labmpc,ikboun,ilboun,nboun,nboun_,
      &     nodeboun,ndirboun,xboun,iamboun,typeboun,nam,ntrans,inotr,
      &     trab,ikfree,ixfree,nmethod,ithermal,istep,mi,icomposite,
-     &     ielmat,vold)
+     &     ielmat,vold,iflagpl)
 !
       endif
 !
@@ -207,11 +245,6 @@ c         endif
      &              (lakon(i)(1:2).eq.'CA')) then
                   indexe=ipkon(i)
                   read(lakon(i)(4:4),'(i1)') nope
-c                  if(lakon(i)(4:4).eq.'6') then
-c                     nope=6
-c                  else
-c                     nope=8
-c                  endif
                   do j=1,nope
                      node=kon(indexe+j)
                      if(dabs(co(3,node)).gt.0.d0) then
@@ -236,86 +269,138 @@ c                  endif
                if(ipkon(i).lt.0) cycle
                if((lakon(i)(1:2).eq.'CP').or.
      &              (lakon(i)(1:1).eq.'S').or.
-     &              (lakon(i)(1:1).eq.'M').or.
+     &              (lakon(i)(1:2).eq.'M3').or.
      &              (lakon(i)(1:2).eq.'CA')) then
 !
+                 iexpand=1
                  call gen3dfrom2d(i,kon,ipkon,lakon,ne,iponor,xnor,knor,
      &           thicke,offset,ntrans,inotr,trab,ikboun,ilboun,nboun,
      &           nboun_,nodeboun,ndirboun,xboun,iamboun,typeboun,ipompc,
      &           nodempc,coefmpc,nmpc,nmpc_,mpcfree,ikmpc,ilmpc,labmpc,
      &           nk,nk_,co,rig,nmethod,iperturb,ithermal,mi,nam,
-     &           icomposite,ielmat,vold)
+     &           icomposite,ielmat,vold,orname,orab,norien,norien_,
+     &           ielorien)
 !            
               elseif((lakon(i)(1:1).eq.'B').or.
      &               (lakon(i)(1:1).eq.'T')) then
+                 iexpand=1
                  call gen3dfrom1d(i,kon,ipkon,lakon,ne,iponor,xnor,knor,
      &                thicke,ntrans,inotr,trab,nk,nk_,co,offset,mi)
+              else
+                 iexpand=0
               endif
 !     
+              if(iexpand.eq.1) then
+                 write(27,*) 'ELEMENT ',i,'with label "',lakon(i),
+     &              '" and with nodes:'
+              endif
               if(lakon(i)(1:4).eq.'CPE3') then
                  lakon(i)(1:7)='C3D6  E'
+                 nope=3
               elseif(lakon(i)(1:5).eq.'CPE4R') then
                  lakon(i)(1:7)='C3D8R E'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CPE4') then
                  lakon(i)(1:7)='C3D8  E'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CPE6') then
                  lakon(i)(1:7)='C3D15 E'
+                 nope=6
               elseif(lakon(i)(1:5).eq.'CPE8R') then
                  lakon(i)(1:7)='C3D20RE'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'CPE8') then
                  lakon(i)(1:7)='C3D20 E'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'CPS3') then
                  lakon(i)(1:7)='C3D6  S'
+                 nope=3
               elseif(lakon(i)(1:5).eq.'CPS4R') then
                  lakon(i)(1:7)='C3D8R S'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CPS4') then
                  lakon(i)(1:7)='C3D8  S'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CPS6') then
                  lakon(i)(1:7)='C3D15 S'
+                 nope=6
               elseif(lakon(i)(1:5).eq.'CPS8R') then
                  lakon(i)(1:7)='C3D20RS'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'CPS8') then
                  lakon(i)(1:7)='C3D20 S'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'CAX3') then
                  lakon(i)(1:7)='C3D6  A'
+                 nope=3
               elseif(lakon(i)(1:5).eq.'CAX4R') then
                  lakon(i)(1:7)='C3D8R A'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CAX4') then
                  lakon(i)(1:7)='C3D8  A'
+                 nope=4
               elseif(lakon(i)(1:4).eq.'CAX6') then
                  lakon(i)(1:7)='C3D15 A'
+                 nope=6
               elseif(lakon(i)(1:5).eq.'CAX8R') then
                  lakon(i)(1:7)='C3D20RA'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'CAX8') then
                  lakon(i)(1:7)='C3D20 A'
+                 nope=8
               elseif((lakon(i)(1:2).eq.'S3').or.
      &               (lakon(i)(1:4).eq.'M3D3')) then
                  lakon(i)(1:7)='C3D6  L'
+                 nope=3
               elseif((lakon(i)(1:3).eq.'S4R').or.
      &               (lakon(i)(1:5).eq.'M3D4R')) then
                  lakon(i)(1:7)='C3D8R L'
-              elseif((lakon(i)(1:2).eq.'S4').or.
-     &               (lakon(i)(1:4).eq.'M3D4')) then
+                 nope=4
+              elseif(lakon(i)(1:2).eq.'S4') then
                  lakon(i)(1:7)='C3D8I L'
+                 nope=4
+              elseif(lakon(i)(1:4).eq.'M3D4') then
+                 lakon(i)(1:7)='C3D8  L'
+                 nope=4
               elseif((lakon(i)(1:2).eq.'S6').or.
      &               (lakon(i)(1:4).eq.'M3D6')) then
                  lakon(i)(1:7)='C3D15 L'
+                 nope=6
               elseif((lakon(i)(1:3).eq.'S8R').or.
      &               (lakon(i)(1:5).eq.'M3D8R')) then
                  lakon(i)(1:7)='C3D20RL'
+                 nope=8
               elseif((lakon(i)(1:2).eq.'S8').or.
      &               (lakon(i)(1:4).eq.'M3D8')) then
                  lakon(i)(1:7)='C3D20 L'
+                 nope=8
               elseif(lakon(i)(1:4).eq.'B31R') then
                  lakon(i)(1:7)='C3D8R B'
+                 nope=2
               elseif((lakon(i)(1:3).eq.'B31').or.
      &               (lakon(i)(1:4).eq.'T3D2')) then
                  lakon(i)(1:7)='C3D8I B'
+                 nope=2
               elseif(lakon(i)(1:4).eq.'B32R') then
                  lakon(i)(1:7)='C3D20RB'
+                 nope=3
               elseif((lakon(i)(1:3).eq.'B32').or.
      &               (lakon(i)(1:4).eq.'T3D3')) then
                  lakon(i)(1:7)='C3D20 B'
+                 nope=3
+              endif
+              if(iexpand.eq.1) then
+                 read(lakon(i)(4:4),'(i1)') ishift
+                 if(lakon(i)(4:5).eq.'8I') ishift=11
+                 if(ishift.le.2) then
+                    read(lakon(i)(4:5),'(i2)') ishift
+                 endif
+                 write(27,'(10(1x,i10))')
+     &              (kon(ipkon(i)+ishift+j),j=1,nope)
+                 write(27,*) ' is expanded into a "',lakon(i),
+     &          '" element with topology:'
+                 write(27,'(10(1x,i10))') (kon(ipkon(i)+j),j=1,ishift)
+                 write(27,*)
               endif
            enddo
 c     Bernhardi start
@@ -333,21 +418,19 @@ c     Bernhardi end
          if(inoelfree.ne.0) then
             call fillknotmpc(co,ipompc,nodempc,coefmpc,labmpc,
      &           nmpc,nmpcold,mpcfree,idim,e1,e2,xt1)
-            call gen3dprop(prop,ielprop,iponoel,inoel,iponoelmax,kon,
-     &           ipkon,lakon,ne,iponor,xnor,knor,ipompc,nodempc,coefmpc,
-     &           nmpc,nmpc_,mpcfree,ikmpc,ilmpc,labmpc,rig,ntrans,inotr,
-     &           trab,nam,nk,nk_,co,nmethod,iperturb)
          endif
 !     
       endif
 !
 !        generating MPC's to connect shells and beams with solid
-!        elements
+!        elements or spring elements or mass elements
 !
       if((inoelfree.ne.0).and.(istep.eq.1)) then
          call gen3dconnect(kon,ipkon,lakon,ne,iponoel,inoel,
      &     iponoelmax,rig,iponor,xnor,knor,ipompc,nodempc,coefmpc,nmpc,
-     &     nmpc_,mpcfree,ikmpc,ilmpc,labmpc)
+     &     nmpc_,mpcfree,ikmpc,ilmpc,labmpc,vold,ikboun,ilboun,nboun,
+     &     nboun_,nodeboun,ndirboun,xboun,iamboun,typeboun,ithermal,
+     &     mi,trab,ntrans,nmethod,nk,nk_,nam,inotr,iperturb,co)
       endif
 !
       if(inoelfree.ne.0) then
@@ -358,7 +441,7 @@ c     Bernhardi end
      &     xboun,iamboun,typeboun,iponoel,inoel,iponoelmax,kon,ipkon,
      &     lakon,ne,iponor,xnor,knor,ipompc,nodempc,coefmpc,nmpc,nmpc_,
      &     mpcfree,ikmpc,ilmpc,labmpc,rig,ntrans,inotr,trab,nam,nk,nk_,
-     &     co,nmethod,iperturb,istep,vold,mi)
+     &     co,nmethod,iperturb,istep,vold,mi,ne2boun)
 !
 !        updating the nodal surfaces: establishing links between the user
 !        defined nodes and the newly generated nodes (mid-nodes
@@ -405,6 +488,8 @@ c         endif
      &     iamboun,typeboun,xboun,nmethod,iperturb,istep,vold,mi,
      &     idefforc)
       endif
+!
+      close(27)
 !
       return
       end
