@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,7 +20,8 @@
      &  thicke,offset,ntrans,inotr,trab,ikboun,ilboun,nboun,nboun_,
      &  nodeboun,ndirboun,xboun,iamboun,typeboun,ipompc,nodempc,coefmpc,
      &  nmpc,nmpc_,mpcfree,ikmpc,ilmpc,labmpc,nk,nk_,co,rig,nmethod,
-     &  iperturb,ithermal,mi,nam,icomposite,ielmat,vold)
+     &  iperturb,ithermal,mi,nam,icomposite,ielmat,vold,orname,orab,
+     &  norien,norien_,ielorien)
 !
 !     expands 2d element i into a 3d element
 !
@@ -34,20 +35,25 @@
       character*1 type,typeboun(*)
       character*8 lakon(*)
       character*20 labmpc(*),label
+      character*80 orname(*)
 !
       integer kon(*),ipkon(*),ne,iponor(2,*),knor(*),ntrans,
-     &  inotr(2,*),nodefixz,
+     &  inotr(2,*),nodefixz,norien,norien_,norieninput,iflag,
      &  ikboun(*),ilboun(*),nboun,nboun_,nodeboun(*),ndirboun(*),
      &  iamboun(*),nam,ipompc(*),nodempc(3,*),nmpc,nmpc_,mpcfree,
-     &  ikmpc(*),ilmpc(*),nk,nk_,i,rig(*),nmethod,iperturb,ishift,
+     &  ikmpc(*),ilmpc(*),nk,nk_,i,rig(*),nmethod,iperturb(*),ishift,
      &  indexe,j,nodel(8),indexx,indexk,k,nedge,nodes(3,8),nodec(3,8),
-     &  iamplitude,l,newnode,idir,idof,id,m,mpcfreenew,node,ithermal(2),
+     &  iamplitude,l,newnode,idir,idof,id,m,mpcfreenew,node,ithermal(*),
      &  jmin,jmax,idummy,mi(*),indexc,indexl,icomposite,ielmat(mi(3),*),
-     &  nope
+     &  nope,neworien(0:norien),iorien,ipos,ielorien(mi(3),*)
 ! 
       real*8 xnor(*),thicke(mi(3),*),offset(2,*),trab(7,*),xboun(*),
-     &  coefmpc(*),co(3,*),thicks(8),xnors(3,8),dc,ds,val,
-     &  x,y,thickness(8),vold(0:mi(2),*)
+     &  coefmpc(*),co(3,*),thicks(8),xnors(3,8),dc,ds,val,orab(7,*),
+     &  x,y,thickness(8),vold(0:mi(2),*),xi,et,xl(3,8),xno(3),xs(3,7),
+     &  shp(7,8),p(3),a(3,3),dd,e1(3),e2(3)
+!
+      norieninput=norien
+      iflag=2
 !
       label='                    '
       fixed=.false.
@@ -63,7 +69,7 @@
 !           localizing the nodes, thicknesses and normals for the
 !           2-D element
 !            
-      if((lakon(i)(2:2).eq.'3').or.
+      if((lakon(i)(1:2).eq.'S3').or.
      &     (lakon(i)(4:4).eq.'3')) then
          quadratic=.false.
          nedge=3
@@ -75,7 +81,7 @@
          nedge=4
          nope=4
 !
-!        CPE4R,CPS4R, CAX4R and S4R elements are expanded into C3D8R
+!        CPE4R, CPS4R, CAX4R and S4R elements are expanded into C3D8R
 !
          if((lakon(i)(3:3).eq.'R').or.
      &      (lakon(i)(5:5).eq.'R')) then
@@ -83,10 +89,10 @@
 !
 !           S4 elements are expanded into C3D8I
 !
-         elseif((lakon(i)(1:1).eq.'S').or.(lakon(i)(1:1).eq.'M')) then
+         elseif(lakon(i)(1:1).eq.'S') then
             ishift=11
 !
-!           CPE4, CPS4 and CAX4 elements are expanded into C3D8
+!           CPE4, CPS4, CAX4 and M3D4 elements are expanded into C3D8
 !
          else
             ishift=8
@@ -435,7 +441,7 @@ c               call bounadd(nodes(2,j),k,k,val,nodeboun,
                            mpcfree=nodempc(3,mpcfree)
                            if(mpcfree.eq.0) then
                               write(*,*) 
-     &                          '*ERROR in gen3dfrom2d: increase nmpc_'
+     &                        '*ERROR in gen3dfrom2d: increase memmpc_'
                               call exit(201)
                            endif
                            nodempc(1,mpcfree)=nodes(3,j)
@@ -444,7 +450,7 @@ c               call bounadd(nodes(2,j),k,k,val,nodeboun,
                            mpcfreenew=nodempc(3,mpcfree)
                            if(mpcfreenew.eq.0) then
                               write(*,*) 
-     &                          '*ERROR in gen3dfrom2d: increase nmpc_'
+     &                        '*ERROR in gen3dfrom2d: increase memmpc_'
                               call exit(201)
                            endif
                            nodempc(3,mpcfree)=0
@@ -477,7 +483,7 @@ c               call bounadd(nodes(2,j),k,k,val,nodeboun,
                      mpcfree=nodempc(3,mpcfree)
                      if(mpcfree.eq.0) then
                         write(*,*) 
-     &                       '*ERROR in gen3dfrom2d: increase nmpc_'
+     &                       '*ERROR in gen3dfrom2d: increase memmpc_'
                         call exit(201)
                      endif
                      nodempc(1,mpcfree)=nodes(2,j)
@@ -505,7 +511,7 @@ c               call bounadd(nodes(2,j),k,k,val,nodeboun,
                      mpcfreenew=nodempc(3,mpcfree)
                      if(mpcfreenew.eq.0) then
                         write(*,*) 
-     &                       '*ERROR in gen3dfrom2d: increase nmpc_'
+     &                       '*ERROR in gen3dfrom2d: increase memmpc_'
                         call exit(201)
                      endif
                      nodempc(3,mpcfree)=0
@@ -515,6 +521,176 @@ c               call bounadd(nodes(2,j),k,k,val,nodeboun,
             enddo
          endif
       enddo
+!
+!     create a local coordinate system for each layer in a shell
+!     element
+!
+      if(lakon(i)(1:1).eq.'S') then
+!
+!        initialize neworien
+!
+         do j=0,norieninput
+            neworien(j)=0
+         enddo
+!
+!        loop over the layers
+!
+         do j=1,mi(3)
+!
+!           check whether layer exists (i.e. whether a material
+!           was assigned to the layer)
+!            
+            if(ielmat(j,i).le.0) exit
+!
+            iorien=max(0,ielorien(j,i))
+!
+!           check whether this orientation already occurred within
+!           this element (i.e. for a previous layer)
+!
+            if(neworien(iorien).gt.0) then
+               ielorien(j,i)=neworien(iorien)
+               cycle
+            endif
+!
+!           calculate the coordinates of the point at
+!           (xi,et)=(0,0) and the normal at that point
+!
+            xi=0.d0
+            et=0.d0
+!
+            if(lakon(i)(2:2).eq.'3') then
+               nope=3
+            elseif(lakon(i)(2:2).eq.'4') then
+               nope=4
+            elseif(lakon(i)(2:2).eq.'6') then
+               nope=6
+            elseif(lakon(i)(2:2).eq.'8') then
+               nope=8
+            else
+               exit
+            endif
+!
+            do k=1,nope
+               node=kon(indexe+ishift+k)
+               do l=1,3
+                  xl(l,k)=co(l,node)
+               enddo
+            enddo
+!
+            if(nope.eq.3) then
+               call shape3tri(xi,et,xl,xno,xs,shp,iflag)
+            elseif(nope.eq.4) then
+               call shape4q(xi,et,xl,xno,xs,shp,iflag)
+            elseif(nope.eq.6) then
+               call shape6tri(xi,et,xl,xno,xs,shp,iflag)
+            else
+               call shape8q(xi,et,xl,xno,xs,shp,iflag)
+            endif
+!
+            dd=dsqrt(xno(1)*xno(1)+xno(2)*xno(2)+xno(3)*xno(3))
+            do l=1,3
+               xno(l)=xno(l)/dd
+            enddo
+c            write(*,*) 'gen3dfrom2d xnor',(xnor(l),l=1,3)
+!
+!           coordinates of the point at (xi,et)=(0.,0.)
+!
+            do l=1,3
+               p(l)=0.d0
+               do k=1,nope
+                  p(l)=p(l)+shp(4,k)*xl(l,k)
+               enddo
+            enddo
+!
+            if(iorien.eq.0) then
+!
+!              unit matrix
+!
+               do k=1,3
+                  do l=1,3
+                     a(k,l)=0.d0
+                  enddo
+                  a(k,k)=1.d0
+               enddo
+            else
+               call transformatrix(orab(1,iorien),p,a)
+            endif
+!
+!           dd=e1.n
+!            
+            dd=a(1,1)*xno(1)+a(2,1)*xno(2)+a(3,1)*xno(3)
+c            write(*,*) 'gen3dfrom2d ',i,dd
+!
+!           check whether e1 is within 0.1° from n
+!           e1 is the local 1-direction in the shell plane
+!
+            if(dabs(dd).gt.0.999999999536d0) then
+!
+!              project the z-axis
+!
+c               write(*,*) 'gen3dfrom2d project z'
+               dd=a(1,3)*xno(1)+a(2,3)*xno(2)+a(3,3)*xno(3)
+               do l=1,3
+                  e1(l)=a(l,3)-dd*xno(l)
+               enddo
+            else
+!
+!              project the x-axis
+!
+c               write(*,*) 'gen3dfrom2d project x'
+               do l=1,3
+                  e1(l)=a(l,1)-dd*xno(l)
+               enddo
+            endif
+!
+            dd=dsqrt(e1(1)*e1(1)+e1(2)*e1(2)+e1(3)*e1(3))
+            do l=1,3
+               e1(l)=e1(l)/dd
+            enddo
+!
+!           e2=nxe1
+!
+            e2(1)=xno(2)*e1(3)-e1(2)*xno(3)
+            e2(2)=xno(3)*e1(1)-e1(3)*xno(1)
+            e2(3)=xno(1)*e1(2)-e1(1)*xno(2)
+!
+            norien=norien+1
+            if(norien.gt.norien_) then
+               write(*,*) '*ERROR in gen3dfrom2d: increase norien_'
+               call exit(201)
+            endif
+!
+!           creating the name of the new orientation =
+!           old name + _shell_ + element number (10 digits;
+!           this corresponds to the maximum number of digits
+!           for element numbers in the frd-format)
+!
+            if(iorien.gt.0) then
+               ipos=index(orname(iorien),' ')
+               orname(norien)(1:ipos-1)=orname(iorien)(1:ipos-1)
+            else
+               ipos=1
+            endif
+            orname(norien)(ipos:ipos+6)='_shell_'
+            write(orname(norien)(ipos+7:ipos+16),'(i10.10)') i
+            do l=ipos+17,80
+               orname(norien)(l:l)=' '
+            enddo
+!
+            do l=1,3
+               orab(l,norien)=e1(l)
+               orab(l+3,norien)=e2(l)
+            enddo
+            orab(7,norien)=1.d0
+c            write(*,*) 'gen3dfrom2d o',(orab(l,iorien),l=1,7)
+c            write(*,*) 'gen3dfrom2d n',(orab(l,norien),l=1,7)
+!
+            neworien(iorien)=norien
+            ielorien(j,i)=norien
+!
+         enddo
+!
+      endif
 !            
       return
       end
