@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
      &  tper,tmin,tmax,ctrl,ipompc,nodempc,coefmpc,nmpc,nmpc_,mpcfree,
      &  ikmpc,ilmpc,labmpc,ikboun,ilboun,nboun,nboun_,nodeboun,ndirboun,
      &  xboun,iamboun,typeboun,nam,ntrans,inotr,trab,ikfree,ixfree,
-     &  nmethod,ithermal,istep,mi,icomposite,ielmat,vold)
+     &  nmethod,ithermal,istep,mi,icomposite,ielmat,vold,iflagpl)
 !
 !     calculates normals on 1-D and 2-D elements
 !
@@ -30,10 +30,9 @@
 !     1: tra_in_x
 !     2: tra_in_y
 !     3: tra_in_z
-!     4: pressure
-!     5: rot_abou_x
-!     6: rot_abou_y
-!     7: rot_about_z
+!     4: pressure (only CFD) or rot_about_x (else)
+!     5: rot_about_y
+!     6: rot_about_z
 !
       implicit none
 !
@@ -44,14 +43,15 @@
       character*20 labmpc(*),label
 !
       integer nk,nk_,iponoel(*),inoel(3,*),iponoelmax,kon(*),ipkon(*),
-     &  ne,iponor(2,*),knor(*),rig(*),iperturb,ipompc(*),nodempc(3,*),
+     &  ne,iponor(2,*),knor(*),rig(*),iperturb(*),ipompc(*),
      &  nmpc,nmpc_,mpcfree,ikmpc(*),ilmpc(*),ikboun(*),ilboun(*),nboun,
      &  nboun_,nodeboun(*),ndirboun(*),iamboun(*),nam,ntrans,inotr(2,*),
      &  isol,istep,idummy,mi(*),icomposite,ielmat(mi(3),*),nkold,
      &  i,ndepnodes,index,nexp,nnor,nel,ielem,indexe,j,iel(100),idmpc,
      &  jl(100),ial(100),ifi(100),idepnodes(800),indexx,k,l,ifix,nemin,
      &  jact,ixfree,ikfree,node,nelshell,irefnode,idof,id,mpcfreeold,
-     &  irotnode,imax,iamplitude,nmethod,ithermal(2),iexpnode,idim
+     &  irotnode,imax,iamplitude,nmethod,ithermal(*),iexpnode,idim,
+     &  iflagpl,nodempc(3,*)
 !
       real*8 co(3,*),thicke(mi(3),*),offset(2,*),xnor(*),tinc,tper,tmin,
      &  tmax,ctrl(*),coefmpc(*),xboun(*),trab(7,*),vold(0:mi(2),*),
@@ -98,7 +98,8 @@
             ielem=inoel(1,index)
             if((lakon(ielem)(1:1).ne.'B').and.
      &         (lakon(ielem)(1:1).ne.'T')) then
-               if(lakon(ielem)(1:1).eq.'S') nnor=1
+               if((lakon(ielem)(1:1).eq.'S').or.
+     &            (lakon(ielem)(1:1).eq.'M')) nnor=1
                indexe=ipkon(ielem)
                nel=nel+1
                if(nel.gt.100) then
@@ -145,7 +146,7 @@
 !
 !              local normal on the element (Jacobian)
 !
-               if((lakon(iel(j))(2:2).eq.'3').or.
+               if((lakon(iel(j))(1:2).eq.'S3').or.
      &              (lakon(iel(j))(4:4).eq.'3')) then
                   xi=coloc6(1,jl(j))
                   et=coloc6(2,jl(j))
@@ -280,7 +281,7 @@ c
 !                                direction, the expanded nodes are on a
 !                                straight line
 !
-                              if(dot.gt.-0.999962) then
+                              if(dot.gt.-0.999962d0) then
                                  nnor=2
                               else
                                  write(*,*) '*INFO in gen3dnor: in some 
@@ -312,7 +313,7 @@ c
 !                                direction, the expanded nodes are on a
 !                                straight line
 !
-                              if(dot.gt.-0.999962) then
+                              if(dot.gt.-0.999962d0) then
                                  nnor=2
                               else
                                  write(*,*) '*INFO in gen3dnor: in some
@@ -420,7 +421,7 @@ c
 !
 !                       membrane: insert hinge
 !
-                        call gen3membrane(ipompc,nodempc,coefmpc,nmpc,
+                        call gen3dmembrane(ipompc,nodempc,coefmpc,nmpc,
      &                       nmpc_,mpcfree,ikmpc,ilmpc,labmpc,nk,
      &                       ithermal,i)
                      endif
@@ -816,7 +817,7 @@ c
      &                 ithermal,i,nodeboun,ndirboun,ikboun,ilboun,
      &                 nboun,nboun_,typeboun,xboun,xta,jact,co,
      &                 knor,ntrans,inotr,trab,vold,mi,nmethod,nk_,
-     &                 nam,iperturb,ikfree,iamboun)
+     &                 nam,iperturb,ikfree,iamboun,iflagpl)
                endif
                ikfree=ikfree+8
             enddo
@@ -826,12 +827,12 @@ c
 !           of freedom (in that case rig(i)=-1 was assigned in 
 !           subroutine gen3delem); if so, a rigid MPC must be defined
 !
-         if(rig(i).ne.0) then
-            rig(i)=0
-            if(nexp.le.1) then
-               nexp=2
-            endif
-         endif
+c         if(rig(i).ne.0) then
+c            rig(i)=0
+c            if(nexp.le.1) then
+c               nexp=2
+c            endif
+c         endif
 !
 !        storing the expanded nodes
 !
@@ -839,23 +840,6 @@ c
 !        generate rigid MPC's if necessary
 !
          if(nexp.gt.1) then
-cc            write(*,*) i,'= KNOT !'
-cc            if(iperturb.eq.0) then
-c            if((iperturb.eq.0).and.(nmethod.eq.1)) then
-c               iperturb=2
-c               tinc=1.d0
-c               tper=1.d0
-c               tmin=1.d-5
-c               tmax=1.d+30
-c            elseif(iperturb.eq.1) then
-c               write(*,*) '*ERROR in gen3dnor: the expansion of'
-c               write(*,*) '       1D/2D elements has led to the'
-c               write(*,*) '       creation of rigid body MPCs.'
-c               write(*,*) '       This is not allowed in a'
-c               write(*,*) '       perturbation analysis. Please'
-c               write(*,*) '       generate a truely 3D structure'
-c               call exit(201)
-c            endif
             irefnode=i
 !
             rig(i)=-1
@@ -870,9 +854,10 @@ c            endif
 !                 rig(i)=-1 to indicate that a knot has
 !                 been generated without rotational node
 !
-c                  rig(i)=-1
-c                  changed for purely thermal calculations
-!     
+                  write(27,*) 'a KNOT without rotation was generated'
+                  write(27,*) '       in node ',i
+                  write(27,*)
+!
                   do k=1,ndepnodes
                      node=idepnodes(k)
                      do j=1,2
@@ -896,7 +881,6 @@ c                  changed for purely thermal calculations
                         ilmpc(id+1)=nmpc
 !     
                         nodempc(1,mpcfree)=node
-c     write(*,*) 'dependent node: ',node
                         nodempc(2,mpcfree)=j
                         coefmpc(mpcfree)=1.d0
                         mpcfree=nodempc(3,mpcfree)
@@ -920,6 +904,8 @@ c     write(*,*) 'dependent node: ',node
                   endif
                   irotnode=nk
                   rig(i)=irotnode
+                  write(27,*) 'a KNOT was generated in node ',i
+                  write(27,*)
                   nk=nk+1
                   if(nk.gt.nk_) then
                      write(*,*) '*ERROR in rigidbodies: increase nk_'
@@ -999,7 +985,7 @@ c     write(*,*) 'dependent node: ',node
 !                 check whether the dof has been used as dependent
 !                 term of a MPC
 !
-                  idof=8*(i-1)+4+imax
+                  idof=8*(i-1)+3+imax
                   call nident(ikmpc,idof,nmpc,idmpc)
 !
                   if((idmpc.le.0).or.(ikmpc(idmpc).ne.idof)) then
@@ -1018,7 +1004,7 @@ c     write(*,*) 'dependent node: ',node
 !
                   isol=0
                   do l=1,3
-                     idof=8*(i-1)+4+imax
+                     idof=8*(i-1)+3+imax
                      call nident(ikboun,idof,nboun,id)
                      call nident(ikmpc,idof,nmpc,idmpc)
                      if(((idmpc.gt.0).and.(ikmpc(idmpc).eq.idof)).or.
