@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
      &  koncont,co,xo,yo,zo,x,y,z,nx,ny,nz,nset,
      &  ifaceslave,istartfield,iendfield,ifield,
      &  ipompc,nodempc,coefmpc,nmpc,nmpctied,mpcfree,ikmpc,ilmpc,
-     &  labmpc,ithermal,tietol,cfd,ncont,imastop,ikboun,nboun,kind)
+     &  labmpc,ithermal,tietol,nef,ncont,imastop,ikboun,nboun,kind)
 !
 !     generates MPC's for the slave tied contact nodes
 !
@@ -40,10 +40,10 @@
      &  nnodelem,nface,nope,nodef(8),idof,kstart,kend,jstart,id,
      &  jend,ifield(*),istartfield(*),iendfield(*),ifaceslave(*),
      &  ipompc(*),nodempc(3,*),nmpc,nmpctied,mpcfree,ikmpc(*),
-     &  ilmpc(*),ithermal(2),cfd,ncont,mpcfreeold,m,id1,ikboun(*),
+     &  ilmpc(*),ithermal(*),nef,ncont,mpcfreeold,m,id1,ikboun(*),
      &  itriold,itrinew,ntriangle,ntriangle_,itriangle(100)
 !
-      real*8 cg(3,*),straight(16,*),co(3,*),p(3),istartwrite,
+      real*8 cg(3,*),straight(16,*),co(3,*),p(3),
      &  dist,xo(*),yo(*),zo(*),x(*),y(*),z(*),pl(3,9),
      &  ratio(9),xi,et,coefmpc(*),tietol(3,*),tolloc
 !
@@ -103,7 +103,7 @@
      &               straight(3,i)*cg(3,i)+
      &               straight(4,i))
       enddo
-      tolloc=0.025*tolloc/ncont
+      tolloc=0.025d0*tolloc/ncont
 !
 !     maximum number of neighboring master triangles for a slave node
 !
@@ -118,7 +118,7 @@
 !
 !           thermomechanical ties or CFD
 !
-            if(cfd.eq.1) then
+            if(nef.gt.0) then
                if(ithermal(2).le.1) then
                   kstart=1
                   kend=4
@@ -216,8 +216,6 @@
      &             n,neigh,kneigh)
 !     
                isol=0
-!     
-               isol=0
 !
                itriold=0
                itri=neigh(1)+itietri(1,i)-1
@@ -232,7 +230,8 @@
      &                    straight(ll+2,itri)*p(3)+
      &                    straight(ll+3,itri)
 c                     if(dist.gt.1.d-6) then
-                     if(dist.gt.tietol(1,i)) then
+c                     if(dist.gt.tietol(1,i)) then
+                     if(dist.gt.tolloc) then
                         itrinew=imastop(l,itri)
                         if(itrinew.eq.0) then
 c                           write(*,*) '**border reached'
@@ -297,7 +296,8 @@ c                              write(*,*) '**regular solution'
                   else
                      write(*,*) '         no corresponding master face'
                      write(*,*) '         found; tolerance: ',
-     &                       tietol(1,i)
+     &                       tolloc
+c     &                       tietol(1,i)
                   endif
                   write(40,*) node
                 else
@@ -368,10 +368,17 @@ c                              write(*,*) '**regular solution'
                         pl(l,k)=co(l,nodef(k))
                      enddo
                   enddo
-                  call attach(pl,p,nnodelem,ratio,dist,xi,et)
-                  do k=1,3
-                     co(k,node)=p(k)
-                  enddo
+                  call attach_2d(pl,p,nnodelem,ratio,dist,xi,et)
+!
+!                 adjusting the coordinates of the node (only
+!                 if the user did not specify ADJUST=NO on the
+!                 *TIE card)
+!
+                  if(tietol(2,i).gt.0.d0) then
+                     do k=1,3
+                        co(k,node)=p(k)
+                     enddo
+                  endif
 !
 !                 generating MPC's
 !
@@ -457,8 +464,6 @@ c                              write(*,*) '**regular solution'
 !     
                   call near3d(xo,yo,zo,x,y,z,nx,ny,nz,p(1),p(2),p(3),
      &                 n,neigh,kneigh)
-!     
-                  isol=0
 !     
                isol=0
 !
@@ -475,7 +480,8 @@ c                              write(*,*) '**regular solution'
      &                    straight(ll+2,itri)*p(3)+
      &                    straight(ll+3,itri)
 c                     if(dist.gt.1.d-6) then
-                     if(dist.gt.tietol(1,i)) then
+c                     if(dist.gt.tietol(1,i)) then
+                     if(dist.gt.tolloc) then
                         itrinew=imastop(l,itri)
                         if(itrinew.eq.0) then
 c                           write(*,*) '**border reached'
@@ -530,25 +536,26 @@ c                              write(*,*) '**regular solution'
 !     check whether distance is larger than tietol(1,i):
 !     no element is generated
 !     
-                  if(isol.eq.0) then
-!
-!                 no MPC is generated
-!
-                     write(*,*) '*WARNING in gentiedmpc: no tied MPC'
-                     write(*,*) '         generated for node ',node
-                     if(isol.eq.0) then
-                        write(*,*) '         master face too far away'
-                        write(*,*) '         distance: ',dist
-                        write(*,*) '         tolerance: ',tietol(1,i)
-                     else
-                      write(*,*) '         no corresponding master face'
-                      write(*,*) '         found; tolerance: ',
-     &                                tietol(1,i)
-                     endif
-                     write(40,*) node
-                  else
+               if(isol.le.0) then
 !     
-                     nelem=int(koncont(4,itri)/10.d0)
+!     no MPC is generated
+!     
+                  write(*,*) '*WARNING in gentiedmpc: no tied MPC'
+                  write(*,*) '         generated for node ',node
+                  if(isol.eq.0) then
+                     write(*,*) '         master face too far away'
+                     write(*,*) '         distance: ',dist
+                     write(*,*) '         tolerance: ',tietol(1,i)
+                  else
+                     write(*,*) '         no corresponding master face'
+                     write(*,*) '         found; tolerance: ',
+     &                    tolloc
+c     &                                tietol(1,i)
+                  endif
+                  write(40,*) node
+               else
+!     
+                  nelem=int(koncont(4,itri)/10.d0)
                      jface=koncont(4,itri)-10*nelem
 !     
                      indexe=ipkon(nelem)
@@ -614,10 +621,17 @@ c                              write(*,*) '**regular solution'
                            pl(l,k)=co(l,nodef(k))
                         enddo
                      enddo
-                     call attach(pl,p,nnodelem,ratio,dist,xi,et)
-                     do k=1,3
-                        co(k,node)=p(k)
-                     enddo
+                     call attach_2d(pl,p,nnodelem,ratio,dist,xi,et)
+!
+!                    adjusting the coordinates of the node (only
+!                    if the user did not specify ADJUST=NO on the
+!                    *TIE card)
+!
+                     if(tietol(2,i).gt.0.d0) then
+                        do k=1,3
+                           co(k,node)=p(k)
+                        enddo
+                     endif
 !     
 !                    generating MPC's
 !     
