@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -30,7 +30,7 @@
 !
 !     change following line if nlabel is increased
 !     
-      character*87 label(47)
+      character*87 label(48)
 !     
       integer i,j,l,mi(*),kontri(4,*),i1,mcs,inocs(*),i2,i3,
      &     ntrit,jj,is,m,nkt,icntrl,imag,nk,nlabel
@@ -46,7 +46,7 @@
 !     change following line if nlabel is increased and the dimension
 !     of field label above!
 !     
-      nlabel=47
+      nlabel=48
 !     
 !     updating the displacements for cyclic symmetric structures
 !     
@@ -68,7 +68,9 @@
      &        label,imag,mi,emn)
          
          do jj=0,mcs-1
-            is=cs(1,jj+1)
+           is=int(cs(1,jj+1))
+c           write(*,*) 'geomview ',cs(1,jj+1)
+c            is=cs(1,jj+1)
 !     
             do i=1,is-1
                do l=1,nk
