@@ -26,11 +26,11 @@
 #include "CalculiX.h"
 #include "readfrd.h"
 
-void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
+void getglobalresults (char *masterfile,ITG **integerglobp,double **doubleglobp,
                        ITG *nboun,ITG *iamboun,double *xboun, ITG *nload,
                        char *sideload,ITG *iamload, ITG *iglob,ITG *nforc,
                        ITG *iamforc,double *xforc,ITG *ithermal,ITG *nk,
-                       double *t1,ITG *iamt1)
+                       double *t1,ITG *iamt1,double *sigma,ITG *irefine)
 {
  
     char  datin[MAX_LINE_LENGTH],text[13]="            ";
@@ -43,7 +43,7 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 	*nnx=NULL,*nny=NULL,*nnz=NULL,*kon=NULL,*ipkon=NULL,*kontyp=NULL,
 	*iparent=NULL,ifreefa=1,kflag=2,ne,netet,numnodes,nkon,
 	indexe,istep,loadcase,nfaces,netet_,nktet=0,nfield,j,nodes[4],i,
-	read_mode=1,nodenr,*integerglob=NULL,*ielemnr=NULL,istep_global;
+	read_mode=0,nodenr,*integerglob=NULL,*ielemnr=NULL,istep_global;
     
     ITG i1[24]={3,7,8,6,4,3,8,1,3,8,5,6,3,5,8,1,2,3,5,6,2,5,3,1};
     ITG i2[12]={1,2,3,5,1,5,3,4,4,5,3,6};
@@ -77,8 +77,8 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     /* reading the global coordinates and the topology from file
        (if any, else return) */
     
-    if(strcmp1(&jobnamec[396]," ")==0)return;
-    strcpy1(datin,&jobnamec[396],132); 
+    if(strcmp1(&masterfile[0]," ")==0)return;
+    strcpy1(datin,&masterfile[0],132); 
     for(i=0;i<MAX_LINE_LENGTH;i++){
 	if(strcmp1(&datin[i]," ")==0){
 	    datin[i]='\0';
@@ -88,14 +88,18 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     /* determining the appropriate step number: scanning the SPC
        boundary conditions and distribed facial loads 
-       if no global data is needed return*/
-    
-    istep=0;
-    for(i=0;i<*nboun;i++){
+       if no global data is needed return 
+       (only for submodel applications, i.e. if *refine==0 )*/
+
+    istep=*irefine;
+
+    if(istep==0){
+      for(i=0;i<*nboun;i++){
 	if((xboun[i]<1.9232931375)&&(xboun[i]>1.9232931373)){
-	    istep=iamboun[i];
-	    break;
+	  istep=iamboun[i];
+	  break;
 	}
+      }
     }
     if(istep==0){
 	for(i=0;i<*nforc;i++){
@@ -124,7 +128,19 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     if(istep==0){
 	return;
     }else{
-	*iglob=1;
+
+        /* iglob=-1 if global results are from a frequency analysis
+           iglob=1  if global results are from a static analysis */
+
+	if(istep<0){
+	    *iglob=-1;
+
+            /* mode number is reverted into a positive number */
+
+	    istep=-istep;
+	}else{
+	    *iglob=1;
+	}
     }
     
     /* initialization of the size of fields used in readfrd.c */
@@ -145,8 +161,8 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     anz->sets=0;
     anz->mats=0;
     anz->amps=0;
-    anz->noffs=0;
-    anz->eoffs=0;
+//    anz->nnext=0;
+//    anz->enext=0;
     
     readfrd( datin, anz, &node, &elem, &lcase, read_mode);
     
@@ -229,7 +245,7 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     /* initialization of fields */
     
-    FORTRAN(init,(&nktet,inodfa,ipofa,&netet_));
+    FORTRAN(init_submodel,(&nktet,inodfa,ipofa,&netet_));
     
     for(i=0;i<ne;i++){
 	type=kontyp[i];
@@ -245,8 +261,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 		nodes[3]=kon[indexe+i1[4*j+3]];
 		iparent[netet]=i+1;
 		netet++;
-		FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				     &ifreefa,planfa,ipofa,nodes,cotet));
+		FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				     &ifreefa,planfa,ipofa,nodes,cotet,
+				     &iparent[netet]));
 	    }
 	}
 	else if(type==2){
@@ -260,8 +277,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 		nodes[3]=kon[indexe+i2[4*j+3]];
 		iparent[netet]=i+1;
 		netet++;
-		FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				     &ifreefa,planfa,ipofa,nodes,cotet));
+		FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				     &ifreefa,planfa,ipofa,nodes,cotet,
+				     &iparent[netet]));
 	    }
 	}
 	else if(type==3){
@@ -274,8 +292,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 	    nodes[3]=kon[indexe+4];
 	    iparent[netet]=i+1;
 	    netet++;
-	    FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				 &ifreefa,planfa,ipofa,nodes,cotet));
+	    FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				 &ifreefa,planfa,ipofa,nodes,cotet,
+				 &iparent[netet]));
 	}
 	else if(type==4){
 	    
@@ -288,8 +307,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 		nodes[3]=kon[indexe+i4[4*j+3]];
 		iparent[netet]=i+1;
 		netet++;
-		FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				     &ifreefa,planfa,ipofa,nodes,cotet));
+		FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				     &ifreefa,planfa,ipofa,nodes,cotet,
+				     &iparent[netet]));
 	    }
 	}
 	else if(type==5){
@@ -303,8 +323,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 		nodes[3]=kon[indexe+i5[4*j+3]];
 		iparent[netet]=i+1;
 		netet++;
-		FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				     &ifreefa,planfa,ipofa,nodes,cotet));
+		FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				     &ifreefa,planfa,ipofa,nodes,cotet,
+				     &iparent[netet]));
 	    }
 	}
 	else if(type==6){
@@ -318,8 +339,9 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
 		nodes[3]=kon[indexe+i6[4*j+3]];
 		iparent[netet]=i+1;
 		netet++;
-		FORTRAN(generatetet,(kontet,ifatet,&netet,inodfa,
-				     &ifreefa,planfa,ipofa,nodes,cotet));
+		FORTRAN(createtet,(kontet,ifatet,&netet,inodfa,
+				     &ifreefa,planfa,ipofa,nodes,cotet,
+				     &iparent[netet]));
 	    }
 	}
     }
@@ -385,16 +407,21 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     loadcase=-1;
     for(i=0;i<anz[0].l;i++){
-	for(j=0;j<lcase[i].npheader;j++){
-	    if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
-		strcpy1(text,&lcase[i].pheader[j][48],12);
-		istep_global=atoi(text);
-		break;
+	if(*iglob>0){
+	    for(j=0;j<lcase[i].npheader;j++){
+		if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
+		    strcpy1(text,&lcase[i].pheader[j][48],12);
+		    istep_global=atoi(text);
+		    break;
+		}
 	    }
+	}else if(*iglob<0){
+	    istep_global=lcase[i].step_number;
 	}
 	if((istep_global==istep)&&
 	   (strcmp1(lcase[i].name,"NDTEMP")==0)){
 	    loadcase=i;
+	    if(*iglob<0) *sigma=(6.283185307*(double)lcase[i].value)*(6.283185307*(double)lcase[i].value);
 	}else if(istep_global>istep){
 	    break;
 	}
@@ -425,17 +452,22 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     loadcase=-1;
     for(i=0;i<anz[0].l;i++){
-	for(j=0;j<lcase[i].npheader;j++){
-	    if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
-		strcpy1(text,&lcase[i].pheader[j][48],12);
-		istep_global=atoi(text);
-		break;
+	if(*iglob>0){
+	    for(j=0;j<lcase[i].npheader;j++){
+		if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
+		    strcpy1(text,&lcase[i].pheader[j][48],12);
+		    istep_global=atoi(text);
+		    break;
+		}
 	    }
+	}else if(*iglob<0){
+	    istep_global=(ITG)lcase[i].step_number;
 	}
 	if((istep_global==istep)&&
-//	   (strcmp1(lcase[i].name,"DISPR")==0)){
+	   (strcmp1(lcase[i].name,"DISPR")!=0)&&
 	   (strcmp1(lcase[i].name,"DISP")==0)){
 	    loadcase=i;
+	    if(*iglob<0) *sigma=(6.283185307*(double)lcase[i].value)*(6.283185307*(double)lcase[i].value);
 	}else if(istep_global>istep){
 	    break;
 	}
@@ -468,16 +500,21 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     loadcase=-1;
     for(i=0;i<anz[0].l;i++){
-	for(j=0;j<lcase[i].npheader;j++){
-	    if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
-		strcpy1(text,&lcase[i].pheader[j][48],12);
-		istep_global=atoi(text);
-		break;
+	if(*iglob>0){
+	    for(j=0;j<lcase[i].npheader;j++){
+		if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
+		    strcpy1(text,&lcase[i].pheader[j][48],12);
+		    istep_global=atoi(text);
+		    break;
+		}
 	    }
+	}else if(*iglob<0){
+	    istep_global=(ITG)lcase[i].step_number;
 	}
 	if((istep_global==istep)&&
 	   (strcmp1(lcase[i].name,"STRESS")==0)){
 	    loadcase=i;
+	    if(*iglob<0) *sigma=(6.283185307*(double)lcase[i].value)*(6.283185307*(double)lcase[i].value);
 	}else if(istep_global>istep){
 	    break;
 	}
@@ -513,16 +550,21 @@ void getglobalresults (char *jobnamec,ITG **integerglobp,double **doubleglobp,
     
     loadcase=-1;
     for(i=0;i<anz[0].l;i++){
-	for(j=0;j<lcase[i].npheader;j++){
-	    if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
-		strcpy1(text,&lcase[i].pheader[j][48],12);
-		istep_global=atoi(text);
-		break;
+	if(*iglob>0){
+	    for(j=0;j<lcase[i].npheader;j++){
+		if(strcmp1(&lcase[i].pheader[j][5],"PSTEP")==0){
+		    strcpy1(text,&lcase[i].pheader[j][48],12);
+		    istep_global=atoi(text);
+		    break;
+		}
 	    }
+	}else if(*iglob<0){
+	    istep_global=(ITG)lcase[i].step_number;
 	}
 	if((istep_global==istep)&&
 	   (strcmp1(lcase[i].name,"FORC")==0)){
 	    loadcase=i;
+	    if(*iglob<0) *sigma=(6.283185307*(double)lcase[i].value)*(6.283185307*(double)lcase[i].value);
 	}else if(istep_global>istep){
 	    break;
 	}
