@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -41,18 +41,18 @@
       integer mi(*),ieg(*),nflow,i,j,ntg,ielmat(mi(3),*),ntmat_,id,
      &     node1,node2,
      &     nelem,index,nshcon(*),ipkon(*),kon(*),ikboun(*),nboun,idof,
-     &     nodem,idirf(5),nactdog(0:3,*),imat,ielprop(*),id1,id2,
-     &     nodef(5),ndirboun(*),nodeboun(*),itg(*),node,kflag,ipiv(*),
+     &     nodem,idirf(8),nactdog(0:3,*),imat,ielprop(*),id1,id2,
+     &     nodef(8),ndirboun(*),nodeboun(*),itg(*),node,kflag,ipiv(*),
      &     nrhs,info,idof1,idof2,nteq,nrhcon(*),ipobody(2,*),ibody(3,*),
      &     nbody,numf,network,iin_abs,icase,index2,index1,nelem1,nelem2,
      &     node11,node21,node12,node22,istep,iit,ineighe(*),
-     &     ilboun(*),nelemup,k,node2up,ider,iaxial
+     &     ilboun(*),nelemup,k,node2up,ider,iaxial,iplausi
 !     
       real*8 ac(nteq,nteq), bc(nteq),prop(*),shcon(0:3,ntmat_,*),
-     &     f,df(5),xflow,xbounact(*),v(0:mi(2),*),cp,r,tg1,
+     &     f,df(8),xflow,xbounact(*),v(0:mi(2),*),cp,r,tg1,
      &     tg2,gastemp,physcon(*),pressmin,dvi,rho,g(3),z1,z2,
      &     rhcon(0:1,ntmat_,*),co(3,*),xbodyact(7,*),kappa,
-     &     a,Tt,Pt,Ts,pressmax,constant,vold(0:mi(2),*),href,
+     &     a,Tt,Ts,pressmax,constant,vold(0:mi(2),*),href,
      &     ttime,time
 !
       kflag=1
@@ -67,7 +67,7 @@
 !     and identifying the chamber and gas pipe nodes (only for gas
 !     networks)
 !
-      if(network.ne.0) then
+      if(network.gt.2) then
 !   
 !        determining whether pressure initial conditions 
 !        are provided for all nodes
@@ -95,7 +95,8 @@
          enddo
 !         
          if(pressmin.lt.0.d0) then
-            write(*,*) '*ERROR in initialgas: minimum initial pressure'
+            write(*,*) 
+     &        '*ERROR in initialchannel: minimum initial pressure'
             write(*,*) '       is smaller than zero'
             call exit(201)
          endif
@@ -116,8 +117,7 @@
             call nident(itg,node1,ntg,id1)
             call nident(itg,node2,ntg,id2)
 !     
-            if ((((lakon(nelem)(1:5).eq.'DGAPF')
-     &           .or.(lakon(nelem)(1:5).eq.'DGAPI')).and.(iin_abs.eq.0))
+            if (((lakon(nelem)(1:5).eq.'DGAPF').and.(iin_abs.eq.0))
      &           .or.((lakon(nelem)(1:3).eq.'DRE')
      &           .and.(lakon(nelem)(1:7).ne.'DREWAOR')
      &           .and.(iin_abs.eq.0))) then 
@@ -205,14 +205,14 @@
       do i=1,ntg
          node=itg(i)
          if (nactdog(0,node).eq.0) cycle
-         if (v(0,node)+physcon(1).lt.1.d-10) then
+         if (v(0,node)-physcon(1).lt.1.d-10) then
             write(*,*)
-     &           '*WARNING in initialgas : the initial temperature for n
-     &ode',node
+     &           '*WARNING in initialchannel : the initial temperature f
+     &or node',node
             write(*,*) 
      &           'is O Kelvin or less; the default is taken (293 K)'
             write(*,*)
-            v(0,node)=293.d0+physcon(1)
+            v(0,node)=293.d0-physcon(1)
          endif
       enddo
 !    
@@ -226,7 +226,7 @@
 !     flux boundary conditions are defined
 !     liquid channels are treated separately
 !   
-      if(network.ne.0)then
+      if(network.gt.2) then
 !     
 !     calculate the initial mass flow
 !     
@@ -294,7 +294,8 @@
                   enddo
                endif
                if(.not.gravity) then
-                  write(*,*)'*ERROR in initialgas: no gravity vector'
+                  write(*,*)
+     &              '*ERROR in initialchannel: no gravity vector'
                   write(*,*) '       was defined for liquid element',
      &                 nelem
                   call exit(201)
@@ -311,7 +312,7 @@
      &              nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &              nodef,idirf,df,cp,r,rho,physcon,g,co,dvi,numf,
      &              vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider,
-     &              ttime,time,iaxial)
+     &              ttime,time,iaxial,iplausi)
 !     
                if(dabs(xflow).gt.1.d-30) exit
             enddo
@@ -327,7 +328,7 @@
                   if(nactdog(1,nodem).ne.0) v(1,nodem)=xflow
                enddo
             else
-               write(*,*) '*ERROR in initialgas: initial mass flow'
+               write(*,*) '*ERROR in initialchannel: initial mass flow'
                write(*,*) '       cannot be determined'
                call exit(201)
             endif
@@ -373,7 +374,8 @@
                   enddo
                endif
                if(.not.gravity) then
-                  write(*,*)'*ERROR in initialgas: no gravity vector'
+                  write(*,*)
+     &              '*ERROR in initialchannel: no gravity vector'
                   write(*,*) '       was defined for liquid element',
      &                 nelem
                   call exit(201)
@@ -390,7 +392,7 @@
      &              nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &              nodef,idirf,df,cp,r,rho,physcon,g,co,dvi,numf,
      &              vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider,
-     &              ttime,time,iaxial)
+     &              ttime,time,iaxial,iplausi)
 !     
             enddo
 !     
