@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@
      &     nactdog,ndirboun,nodeboun,xbounact,
      &     ielmat,ntmat_,shcon,nshcon,physcon,ipiv,nteq,
      &     rhcon,nrhcon,ipobody,ibody,xbodyact,co,nbody,network,
-     &     iin_abs,vold,set,istep,iit,mi,ineighe,ilboun,channel,
+     &     iin_abs,vold,set,istep,iit,mi,ineighe,ilboun,ichannel,
      &     iaxial,nmpc,labmpc,ipompc,nodempc,coefmpc,ttime,time,
      &     iponoel,inoel)
 !     
@@ -40,32 +40,35 @@
       character*20 labmpc(*)
       character*81 set(*)
 !           
-      integer mi(*),ieg(*),nflow,i,j,ntg,ielmat(mi(3),*),ntmat_,id,
-     &     node1,node2,ider,iaxial,nmpc,nodempc(3,*),ipompc(*),
+      integer mi(*),ieg(*),nflow,i,j,ntg,ielmat(mi(3),*),ntmat_,
+     &     node1,node2,ider,iaxial,nmpc,ipompc(*),nodempc(3,*),
      &     nelem,index,nshcon(*),ipkon(*),kon(*),ikboun(*),nboun,idof,
      &     nodem,idirf(8),nactdog(0:3,*),imat,ielprop(*),id1,id2,
      &     nodef(8),ndirboun(*),nodeboun(*),itg(*),node,kflag,ipiv(*),
      &     nrhs,info,idof1,idof2,nteq,nrhcon(*),ipobody(2,*),ibody(3,*),
      &     nbody,numf,network,iin_abs,icase,index2,index1,nelem1,nelem2,
      &     node11,node21,node12,node22,istep,iit,ineighe(*),iponoel(*),
-     &     ilboun(*),nelemup,k,node2up,idir,channel,inoel(2,*)
+     &     ilboun(*),idir,ichannel,inoel(2,*),indexe,iel,iplausi
 !     
       real*8 ac(nteq,nteq), bc(nteq),prop(*),shcon(0:3,ntmat_,*),
      &     f,df(8),xflow,xbounact(*),v(0:mi(2),*),cp,r,tg1,
      &     tg2,gastemp,physcon(*),pressmin,dvi,rho,g(3),z1,z2,
      &     rhcon(0:1,ntmat_,*),co(3,*),xbodyact(7,*),kappa,
-     &     a,Tt,Pt,Ts,pressmax,constant,vold(0:mi(2),*),href,
-     &     coefmpc(*),ttime,time,xflow360
+     &     A,Tt,pt,Ts,pressmax,constant,vold(0:mi(2),*),
+     &     coefmpc(*),ttime,time,xflow360,A2,d,l,s
+!
+!
 !
       kflag=1
       ider=0
-      channel=0
+      ichannel=0
       gaspipe=.false.
 !
 !     applying the boundary conditions
 !
       do j=1,nboun
          v(ndirboun(j),nodeboun(j))=xbounact(j)
+         vold(ndirboun(j),nodeboun(j))=xbounact(j)
       enddo
 !
 !     check for channel elements
@@ -73,7 +76,7 @@
       do i=1,nflow
          nelem=ieg(i)
          if(lakon(nelem)(2:5).eq.'LICH') then
-            channel=1
+            ichannel=1
             return
          endif
       enddo
@@ -91,11 +94,11 @@
 !     determining the initial pressure
 !     and identifying the chamber and gas pipe nodes
 !     
-      if(network.ne.0) then
+      if(network.gt.2) then
 !
          call preinitialnet(ieg,lakon,v,ipkon,kon,nflow,prop,ielprop,
      &     ielmat,ntmat_,shcon,nshcon,rhcon,nrhcon,mi,iponoel,inoel,
-     &     itg,ntg)
+     &        itg,ntg)
 !     
 !     determining whether pressure initial conditions 
 !     are provided for all nodes
@@ -142,12 +145,12 @@
             call nident(itg,node1,ntg,id1)
             call nident(itg,node2,ntg,id2)
 !     
-            if ((((lakon(nelem)(1:5).eq.'DGAPF')
-     &           .or.(lakon(nelem)(1:5).eq.'DGAPI')).and.
-     &           (iin_abs.eq.0))
-     &           .or.((lakon(nelem)(1:3).eq.'DRE')
-     &           .and.(lakon(nelem)(1:7).ne.'DREWAOR')
-     &           .and.(iin_abs.eq.0))) then 
+            if(iin_abs.eq.0) then
+               if ((lakon(nelem)(2:5).eq.'GAPF').or.
+     &             (lakon(nelem)(2:5).eq.'GAPR').or.
+     &             ((lakon(nelem)(2:3).eq.'RE')
+     &              .and.(lakon(nelem)(2:7).ne.'REWAOR')).or.
+     &             (lakon(nelem)(2:3).eq.'UP')) then 
 !     
 !     In the case of a element of type GASPIPE or RESTRICTOR 
 !     (except TYPE= RESTRICTOR WALL ORIFICE)
@@ -155,24 +158,23 @@
 !     are computed and stored in ineighe(id1)
 !     respectively ineighe(id2)
 !     
-               gaspipe=.true.
-               if(node1.ne.0) then
-                  if (ineighe(id1).ge.0) then
+                  gaspipe=.true.
+                  if(node1.ne.0) then
+                     if (ineighe(id1).ge.0) then
 !     
-                     if(node2.ne.0)then
-                        ineighe(id1)=ineighe(id1)+1
+                        if(node2.ne.0)then
+                           ineighe(id1)=ineighe(id1)+1
+                        endif
                      endif
                   endif
-               endif
-               if(node2.ne.0) then
-                  if (ineighe(id2).ge.0) then
-                     if(node1.ne.0) then
-                        ineighe(id2)=ineighe(id2)+1
+                  if(node2.ne.0) then
+                     if (ineighe(id2).ge.0) then
+                        if(node1.ne.0) then
+                           ineighe(id2)=ineighe(id2)+1
+                        endif
                      endif
                   endif
-               endif
-            else
-               if(iin_abs.eq.0) then
+               else
 !     
 !     for all other elements (different from GASPIPE or 
 !     RESTRICTOR), including RESTRICTOR WALL ORIFICE 
@@ -208,25 +210,60 @@
             node=itg(i)
 !     boundary condition nodes
             if(nactdog(2,node).eq.0) then 
-               v(2,node)=dtan(v(2,node)
-     &              *constant/pressmax) 
                cycle 
             endif
 !     initial condition nodes
             if((nactdog(2,node).ne.0)
      &           .and.(v(2,node).gt.0d0)) then
-               v(2,node) = dtan(v(2,node)
-     &              *constant/pressmax) 
                cycle
             endif
 !     nodes neither defined as *BOUNDARY nor as *INITIAL CONDITIONS
             if(abs(v(2,node)+1.d0).lt.1.d-10) then
-               v(2,node)=0.95d0*pressmin
-               pressmin=0.95d0*pressmin
-               v(2,node)=dtan(v(2,node)
-     &              *constant/pressmax)
+!
+!              if a nonzero flux is leaving the node increase
+!              the maximum pressure, else decrease the minimum
+!              pressure
+!
+!              determining the exit element (= element with one
+!              zero node)
+!
+               index=iponoel(node)
+               do
+                  iel=inoel(1,index)
+                  indexe=ipkon(iel)
+                  node1=kon(indexe+1)
+                  nodem=kon(indexe+2)
+                  node2=kon(indexe+3)
+                  if((node1.eq.0).or.(node2.eq.0)) exit
+                  index=inoel(2,index)
+               enddo
+!
+               if(((node1.eq.node).and.(v(1,nodem).ge.0.d0)).or.
+     &            ((node2.eq.node).and.(v(1,nodem).le.0.d0))) then
+                  v(2,node)=0.95d0*pressmin
+                  pressmin=0.95d0*pressmin
+               else
+                  v(2,node)=1.05d0*pressmax
+                  pressmax=1.05d0*pressmax
+               endif
             endif              
          enddo
+!
+!        transforming the pressures using the tangent
+!        function
+!
+         do i=1,ntg
+            node=itg(i)
+!     boundary condition nodes
+            if(nactdog(2,node).eq.0) then 
+               v(2,node)=dtan(v(2,node)
+     &              *constant/pressmax) 
+!     initial condition nodes
+            elseif(v(2,node).gt.0d0) then
+               v(2,node)=dtan(v(2,node)
+     &              *constant/pressmax) 
+            endif
+         enddo
 !     
 !     mass flow and geometry dofs: 1 on the diagonal
 !     
@@ -305,7 +342,7 @@
 !     temperature conditions for all but purely hydrodynamic
 !     networks
 !     
-      if(network.ne.2) then
+      if(network.ne.4) then
 !     
 !     temperature dofs
 !     
@@ -378,35 +415,6 @@
       do j=1,nteq
          if(dabs(ac(j,j)).lt.1.d-20) ac(j,j)=1.d0
       enddo
-cc      j=nteq+1
-c      j=0
-c      do i=nmpc,1,-1
-c         if(labmpc(i)(1:7).ne.'NETWORK') cycle
-c!
-c!        looking of a row in the matrix which has not been used yet
-c!
-c         do
-c            j=j+1
-c            if(dabs(ac(j,j)).lt.1.d-20) exit
-c            if(j.eq.nteq) then
-c               write(*,*) '*ERROR in initialnet: MPC cannot'
-c               write(*,*) '       be inserted'
-c               call stop()
-c            endif
-c         enddo
-c         j=j-1
-c         index=ipompc(i)
-c!
-c         do
-c            node=nodempc(1,index)
-c            idir=nodempc(2,index)
-c            if(nactdog(idir,node).ne.0) then
-c               ac(j,nactdog(idir,node))=coefmpc(index)
-c            endif
-c            index=nodempc(3,index)
-c            if(index.eq.0) exit
-c         enddo
-c      enddo
 !     
 !     solving the system
 !     
@@ -422,16 +430,14 @@ c      enddo
 !     storing the initial pressure in v
 !     (not for purely thermal networks)
 !     
-      if(network.ne.0) then
+      if(network.gt.2) then
          do i=1,ntg
             node=itg(i)
             if(nactdog(2,node).eq.0) then
-c               write(*,*) 'initialnet ',node,v(2,node)
                v(2,node)=pressmax
      &              *datan(v(2,node))/constant
                cycle
             endif
-c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
             v(2,node)=pressmax
      &           *datan(bc(nactdog(2,node)))/constant
          enddo
@@ -440,7 +446,7 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
 !     storing the initial temperature in v
 !     (not for purely hydrodynamic networks)
 !     
-      if(network.ne.2) then
+      if(network.ne.4) then
          do i=1,ntg
             node=itg(i)
             if(nactdog(0,node).ne.0) v(0,node)=bc(nactdog(0,node))
@@ -456,12 +462,12 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
          if(lakon(i)(2:5).ne.'REBR') cycle
          index=ielprop(nelem)
 !     
-         nelem1=prop(index+2)
+         nelem1=nint(prop(index+2))
          index1=ipkon(nelem1)
          node11=kon(index1+1)
          node21=kon(index1+3)
 !     
-         nelem2=prop(index+3)
+         nelem2=nint(prop(index+3))
          index2=ipkon(nelem2)
          node12=kon(index2+1)
          node22=kon(index2+3)
@@ -505,7 +511,8 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
 !     gas networks (needed to determine the static
 !     temperature which is used for the material properties)
 !     
-      if(network.eq.0) then
+c      if(network.le.2) then
+      if((network.le.2).and.(iin_abs.eq.0)) then
          do i=1,nflow
             nelem=ieg(i)
             if((lakon(nelem)(2:3).eq.'LP').or.
@@ -533,7 +540,7 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
 !     determining the initial mass flow in those nodes for which no
 !     flux boundary conditions are defined
 !     
-      if(network.ne.0)then
+      if(network.gt.2)then
          do i=1,nflow
             nelem=ieg(i)
 !     
@@ -550,7 +557,8 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
 !     
 !     for liquids: determine the gravity vector
 !     
-            if(lakon(nelem)(2:3).eq.'LI') then
+            if((lakon(nelem)(2:3).eq.'LI').or.
+     &         (lakon(nelem)(2:3).eq.'LP')) then
                gravity=.false.
                do j=1,3
                   g(j)=0.d0
@@ -576,10 +584,12 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
                endif
                if(.not.gravity) then
                   write(*,*) 
-     &                 '*ERROR in initialnet: no gravity vector'
+     &                 '*WARNING in initialnet: no gravity vector'
                   write(*,*) 
      &                 '       was defined for liquid element',nelem
-                  call exit(201)
+                  z1=0.d0
+                  z2=0.d0
+c                  call exit(201)
                endif
             endif
 !     
@@ -623,13 +633,14 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
                if((nactdog(2,node1).eq.0)
      &           .and.(nactdog(2,node2).eq.0)) then
                   WRITE(*,*) '**************************************'
-                  write(*,*) '*ERROR:in subroutine initialnet.f'
-                  write(*,*) '       in element', nelem
-                  write(*,*) '       Inlet and outlet pressures are '
-                  write(*,*) '       boundary conditions '
-                  write(*,*) '       node1',node1,' pressure',
+                  write(*,*) '*ERROR: in subroutine initialnet.f'
+                  write(*,*) '        no pressure gradient'
+                  write(*,*) '        in element', nelem
+                  write(*,*) '        Inlet and outlet pressures are '
+                  write(*,*) '        boundary conditions '
+                  write(*,*) '        node1',node1,' pressure',
      &                 v(2,node1)
-                  write(*,*) '       node2',node2,' pressure',
+                  write(*,*) '        node2',node2,' pressure',
      &                 v(2,node2)
                   call exit(201)
 !     
@@ -638,17 +649,18 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
                else if((nactdog(2,node1).ne.0)
      &                 .and.(nactdog(2,node2).eq.0))then
                   WRITE(*,*) '**************************************'
-                  write(*,*) '*WARNING:in subroutine initialnet.f'
-                  write(*,*) '       in element', nelem
+                  write(*,*) '*WARNING: in subroutine initialnet.f'
+                  write(*,*) '          no pressure gradient'
+                  write(*,*) '          in element', nelem
                   write(*,*) 
-     &                 '       Inlet pressure initial condition '
-                  write(*,*) '       is changed '
-                  write(*,*) '       node1',node1,
+     &               '          Inlet pressure initial condition '
+                  write(*,*) '          is changed '
+                  write(*,*) '          node1',node1,
      &                 ' given initial pressure',v(2,node1)
-                  v(2,node1)=1.1*v(2,node1)
-                  write(*,*) '       node1',node1,
+                  v(2,node1)=1.1d0*v(2,node1)
+                  write(*,*) '          node1',node1,
      &                 ' new initial pressure',v(2,node1)
-                  write(*,*) '       node2',node2,' pressure',
+                  write(*,*) '          node2',node2,' pressure',
      &                 v(2,node2)
 !     
 !     if outlet pressure is an active D.O.F.
@@ -656,18 +668,19 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
                else if((nactdog(2,node1).eq.0)
      &                 .and.(nactdog(2,node2).ne.0))then
                   WRITE(*,*) '**************************************'
-                  write(*,*) '*WARNING:in subroutine initialnet.f'
-                  write(*,*) '       in element', nelem
+                  write(*,*) '*WARNING: in subroutine initialnet.f'
+                  write(*,*) '          no pressure gradient'
+                  write(*,*) '          in element', nelem
                   write(*,*) 
-     &                 '       Outlet pressure initial condition '
-                  write(*,*) '       is changed '
-                  write(*,*) '       node1',node1,' pressure'
+     &                 '          Outlet pressure initial condition '
+                  write(*,*) '          is changed '
+                  write(*,*) '          node1',node1,' pressure'
      &                 ,v(2,node1)    
-                  write(*,*) '       node2',node2,
+                  write(*,*) '          node2',node2,
      &                 'given intial pressure',
      &                 v(2,node2)
-                  v(2,node2)=0.9*v(2,node2)
-                  write(*,*) '       node2',node2,
+                  v(2,node2)=0.9d0*v(2,node2)
+                  write(*,*) '          node2',node2,
      &                 ' new initial pressure',v(2,node2)
 !     
 !     if both inlet and outlet pressures are active D.O.F.
@@ -675,62 +688,74 @@ c               write(*,*) 'initialnet ',node,bc(nactdog(2,node))
                else if((nactdog(2,node1).ne.0)
      &                 .and.(nactdog(2,node2).ne.0))then
                   WRITE(*,*) '**************************************'
-                  write(*,*) '*WARNING:in subroutine initialnet.f'
-                  write(*,*) '       in element', nelem
-                  write(*,*) '       Inlet and outlet pressure '
-                  write(*,*) '       initial condition are changed '
-                  write(*,*) '       node1',node1,
+                  write(*,*) '*WARNING: in subroutine initialnet.f'
+                  write(*,*) '          no pressure gradient'
+                  write(*,*) '          in element', nelem
+                  write(*,*) '          Inlet and outlet pressure '
+                  write(*,*) '          initial condition are changed '
+                  write(*,*) '          node1',node1,
      &                 ' given initial pressure',v(2,node1)  
-                  v(2,node1)=1.05*v(2,node2)
-                  write(*,*) '       node1',node1,
+                  v(2,node1)=1.05d0*v(2,node2)
+                  write(*,*) '          node1',node1,
      &                 ' new intial pressure',v(2,node1)
-                  write(*,*) '       node2',node2,
+                  write(*,*) '          node2',node2,
      &                 ' given initial pressure',v(2,node2)
-                  v(2,node2)=0.95*v(2,node2)
-                  write(*,*) '       node2',node2,
+                  v(2,node2)=0.95d0*v(2,node2)
+                  write(*,*) '          node2',node2,
      &                 ' new intial pressure',v(2,node2)
                endif
             endif
 !     
-!           calculating flux if the flux is an unknown AND there was
-!           no initial flux defined by the user
+!            calculating flux if the flux is an unknown; any initial
+!            flux defined by the user is checked on plausibility            
 !
-            if((nactdog(1,nodem).ne.0).and.(v(1,nodem).eq.0.d0)) then
+c!     
+c!           calculating flux if the flux is an unknown AND there was
+c!           no initial flux defined by the user
+c!
+c            if((nactdog(1,nodem).ne.0).and.(v(1,nodem).eq.0.d0)) then
+            if(nactdog(1,nodem).ne.0) then
                call flux(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &           nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &           nodef,idirf,df,cp,r,rho,physcon,g,co,dvi,numf,
      &           vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider,
-     &           ttime,time,iaxial)
+     &           ttime,time,iaxial,iplausi)
                v(1,nodem)=xflow
             endif
 !     
-c            if(nactdog(1,nodem).ne.0) v(1,nodem)=xflow
-!     
-c            if(lakon(nelem)(2:4).ne.'LIP') then
             if((lakon(nelem)(2:4).ne.'LIP').and.
-     &         (lakon(nelem)(2:3).ne.'VO')) then
+     &         (lakon(nelem)(2:3).ne.'VO').and.
+     &         (lakon(nelem)(2:5).ne.'GAPR').and.
+     &         (lakon(nelem)(2:4).ne.'ATR').and.
+     &         (lakon(nelem)(2:4).ne.'RTA')) then
                if(v(1,nodem).eq.0d0) then
                   WRITE(*,*) '**************************************'
-                  write(*,*) '*ERROR:in subroutine initialnet.f'
-                  write(*,*) '       in element', nelem,
+                  write(*,*) '*ERROR: in subroutine initialnet.f'
+                  write(*,*) '        in element', nelem,
      &                 lakon(nelem)(1:6)
-                  write(*,*) '       mass flow rate value = 0 !'
-                  write(*,*) '       node1',node1,' pressure',
+                  write(*,*) '        mass flow rate value = 0 !'
+                  write(*,*) '        node1',node1,' pressure',
      &                 v(2,node1)
-                  write(*,*) '       node2',node2,' pressure',
+                  write(*,*) '        node2',node2,' pressure',
      &                 v(2,node2)
                   call exit(201)
                endif
-               if (v(1,nodem).lt.0) then
+               if (v(1,nodem)*(v(2,node1)-v(2,node2)).lt.0) then
                   WRITE(*,*) '**************************************'
                   write(*,*) '*WARNING: in subroutine initialnet.f'
-                  write(*,*) '        in element', nelem
-                  write(*,*) '        mass flow rate value .le. 0 !'
-                  write(*,*) '        node1',node1,'pressure',
+                  write(*,*) '          in element', nelem
+                  write(*,*) 
+     &                 '          initial mass flow rate value does not'
+                  write(*,*) '          correspond to pressure gradient'
+                  write(*,*) '          node1',node1,'pressure',
      &                 v(2,node1)
-                  write(*,*) '        node2',node2,'pressure',
+                  write(*,*) '          node2',node2,'pressure',
      &                 v(2,node2)
-                  write(*,*) '        check element definition'
+                  write(*,*) '          nodem',nodem,'mass flow',
+     &                 v(1,nodem)
+                  write(*,*) 
+     &               '          the sign of the initial mass flow'
+                  write(*,*) '          rate will be changed'
                endif
             endif
          enddo
@@ -738,7 +763,7 @@ c            if(lakon(nelem)(2:4).ne.'LIP') then
 !
       call postinitialnet(ieg,lakon,v,ipkon,kon,nflow,prop,ielprop,
      &     ielmat,ntmat_,shcon,nshcon,rhcon,nrhcon,mi,iponoel,inoel,
-     &     itg,ntg)
+     &     itg,ntg,nactdog)
 !     
 !     calculating the static temperature for nodes belonging to gas pipes
 !     and restrictors (except RESTRICTOR WALL ORIFICE)
@@ -814,31 +839,51 @@ c            if(lakon(nelem)(2:4).ne.'LIP') then
                kappa=cp/(cp-R)
                xflow=v(1,nodem)
                Tt=v(0,node)
-               Pt=v(2,node)
+               pt=v(2,node)
 !     
-               if((lakon(nelem)(2:5).eq.'GAPF')
-     &              .or.(lakon(nelem)(2:5).eq.'GAPI')) then
-                  A=prop(index+1)
-                  if((lakon(nelem)(2:6).eq.'GAPFA') 
-     &                 .or.(lakon(nelem)(2:6).eq.'GAPIA')) then
+               if(lakon(nelem)(2:5).eq.'GAPF') then
+!
+!                 distinguish between standard pipes and flexible
+!                 pipes
+! 
+                  if((lakon(nelem)(6:8).eq.'AFR').or.
+     &               (lakon(nelem)(6:8).eq.'ARL').or.
+     &               (lakon(nelem)(6:8).eq.'ARG').or.
+     &               (lakon(nelem)(6:8).eq.'IFR').or.
+     &               (lakon(nelem)(6:8).eq.'IRL')) then
+                     call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                   ttime,time,ielprop,mi,A,A2,d,l,s)
+                  else
+                     A=prop(index+1)
+                  endif
+!
+                  if(lakon(nelem)(2:6).eq.'GAPFA') then
                      icase=0
-                  elseif((lakon(nelem)(2:6).eq.'GAPFI')
-     &                    .or.(lakon(nelem)(2:6).eq.'GAPII')) then
+                  elseif(lakon(nelem)(2:6).eq.'GAPFI') then
                      icase=1
                   endif     
-!     
+!
+               elseif(lakon(nelem)(2:5).eq.'GAPR') then
+!
+!                 rotating adiabatic gas pipe with variable cross section:
+!                 check whether section 1 or 2                  
+!
+                  if(kon(ipkon(nelem)+1).eq.node) then
+                     A=prop(index+1)
+                  else
+                     A=prop(index+2)
+                  endif
+                  icase=0
                elseif(lakon(nelem)(2:3).eq.'RE') then
                   index2=ipkon(nelem)
                   node1=kon(index2+1)
                   node2=kon(index2+3)
                   if(lakon(nelem)(4:5).eq.'EX') then
-                     if((lakon(nint(prop(index+4)))(2:6).eq.'GAPFA')
-     &                 .or.(lakon(nint(prop(index+4)))(2:6).eq.'GAPIA'))
+                     if(lakon(nint(prop(index+4)))(2:6).eq.'GAPFA')
      &                    then
                         icase=0
                      elseif
-     &                   ((lakon(nint(prop(index+4)))(2:6).eq.'GAPFI')
-     &                 .or.(lakon(nint(prop(index+4)))(2:6).eq.'GAPII')) 
+     &                   (lakon(nint(prop(index+4)))(2:6).eq.'GAPFI') 
      &                       then
                         icase=1
                      endif
@@ -847,7 +892,7 @@ c            if(lakon(nelem)(2:4).ne.'LIP') then
                   endif
 !     
                   if(lakon(nelem)(4:5).eq.'BE') then
-                     a=prop(index+1)
+                     A=prop(index+1)
 !     
                   elseif(lakon(nelem)(4:5).eq.'BR') then
                      if(lakon(nelem)(4:6).eq.'BRJ') then
@@ -865,17 +910,34 @@ c            if(lakon(nelem)(2:4).ne.'LIP') then
                      endif
 !     
                   else
-                     if(node.eq.node1) then
-                        a=prop(index+1)
-                     elseif(node.eq.node2) then
-                        a=prop(index+2)
+!
+                     if((lakon(nelem)(2:5).eq.'RBSF')) then
+                        call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                       ttime,time,ielprop,mi,A,A2,d,l,s)
+                        if(node.eq.node2) then
+                           A=A2
+                        endif
+                     else
+                        if(node.eq.node1) then
+                           A=prop(index+1)
+                        elseif(node.eq.node2) then
+                           A=prop(index+2)
+                        endif
                      endif
                   endif
+               elseif(lakon(nelem)(2:3).eq.'UP') then
+!
+!                 user elements whose names start with UP are assumed
+!                 to be Pipe-like (static and total temperatures differ)
+!
+                  call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                       ttime,time,ielprop,mi,A,A2,d,l,s)
+                  icase=0
                endif
 !     
-               if(v(3,node).eq.0) then
+               if(v(3,node).eq.0.d0) then
                   xflow360=xflow*iaxial
-                  call ts_calc(xflow360,Tt,Pt,kappa,r,a,Ts,icase)
+                  call ts_calc(xflow360,Tt,pt,kappa,r,A,Ts,icase)
                   v(3,node)=Ts
                endif
 !     
