@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !     
       subroutine liquidchannel(node1,node2,nodem,nelem,lakon,
      &     nactdog,identity,ielprop,prop,iflag,v,xflow,f,
-     &     nodef,idirf,df,rho,g,co,dvi,numf,mi,ipkon,kon)
+     &     nodef,idirf,df,rho,g,co,dvi,numf,mi,ipkon,kon,iplausi)
 !
 !     open channel for incompressible media
 !
@@ -37,10 +37,11 @@
       character*8 lakon(*)
 !      
       integer nelem,nactdog(0:3,*),node1,node2,nodem,indexup,i,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,mi(*),nsol,
-     &     inv,numf,nodesg,nelemdown,nelemup,node0,kon(*),ipkon(*)
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,mi(*),nsol,
+     &     inv,numf,nodesg,nelemdown,nelemup,node0,kon(*),ipkon(*),
+     &     iplausi
 !      
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),b,d,c,p,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),b,d,c,p,
      &     h1,h2,rho,dvi,friction,reynolds,dg,b1,b2,
      &     g(3),dl,xks,z1,z2,co(3,*),xflow2,dyg3dbj,dyg4dbj,
      &     s0,sqrts0,hk,form_fact,h1ns,h2ns,h0,dyg3deta,dyg4deta,
@@ -52,6 +53,8 @@
      &     dyg3dm,dyg4dm,dA3dm,dA4dm,dyg3dh1,dyg4dh2,
      &     dA3dh1,dA4dh2,solreal(3),solimag(3),dist
 !
+!
+!
 !     iflag=0: check whether all parameters in the element equation
 !              are known => equation is not needed
 !     iflag=1: calculation of the initial flux
@@ -1103,7 +1106,7 @@ c     &              'liquidchannel: h3,h4,eta ',h3,h4,eta
          endif
       elseif(iflag.eq.3) then
 !
-!        only if called from resultgas in case the element contains
+!        only if called from resultnet in case the element contains
 !        a hydraulic jump and eta<0 or eta>1. This means that the 
 !        jump does not take place in the element itself. By adjusting
 !        h1 or h2 the jump is forced into a neighboring element
