@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
       subroutine liquidpipe(node1,node2,nodem,nelem,lakon,
      &     nactdog,identity,ielprop,prop,iflag,v,xflow,f,
      &     nodef,idirf,df,rho,g,co,dvi,numf,vold,mi,ipkon,kon,set,
-     &     ttime,time,iaxial)
+     &     ttime,time,iaxial,iplausi)
 !
 !     pipe element for incompressible media
 !     
@@ -36,11 +36,11 @@
       character*81 set(*)
 !      
       integer nelem,nactdog(0:3,*),node1,node2,nodem,iaxial,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,mi(*),
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,mi(*),
      &     inv,ncoel,ndi,nbe,id,nen,ngv,numf,nodea,nodeb,
-     &     ipkon(*),isothermal,kon(*),nelemswirl
+     &     ipkon(*),isothermal,kon(*),nelemswirl,iplausi
 !      
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),a,d,pi,radius,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),a,d,pi,radius,
      &     p1,p2,rho,dvi,friction,reynolds,vold(0:mi(2),*),
      &     g(3),a1,a2,xn,xk,xk1,xk2,zeta,dl,dg,rh,a0,alpha,
      &     coarseness,rd,xks,z1,z2,co(3,*),xcoel(11),yel(11),
@@ -49,28 +49,38 @@
      &     dh,kappa,r,dkda,form_fact,dzetadalpha,t_chang,
      &     xflow_vol,r1d,r2d,r1,r2,eta, K1, Kr, U1,Ui, ciu, c1u, 
      &     c2u, omega,cinput,un,T
+!
+!
 !
       data ncoel /11/
-      data xcoel /0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0/
-      data yco /0.5,0.46,0.41,0.36,0.30,0.24,0.18,0.12,0.06,0.02,0./
-      data yel /1.,0.81,0.64,0.49,0.36,0.25,0.16,0.09,0.04,0.01,0./
+      data xcoel /0d0,0.1d0,0.2d0,0.3d0,0.4d0,0.5d0,0.6d0,0.7d0,0.8d0,
+     &            0.9d0,1.0d0/
+      data yco /0.5d0,0.46d0,0.41d0,0.36d0,0.30d0,0.24d0,0.18d0,0.12d0,
+     &          0.06d0,0.02d0,0.d0/
+      data yel /1.d0,0.81d0,0.64d0,0.49d0,0.36d0,0.25d0,0.16d0,0.09d0,
+     &          0.04d0,0.01d0,0.d0/
 !
       data ndi /10/
-      data xdi /0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1./
-      data ydi /226.,47.5,17.5,7.8,3.75,1.80,0.8,0.29,0.06,0./
+      data xdi /0.1d0,0.2d0,0.3d0,0.4d0,0.5d0,0.6d0,0.7d0,0.8d0,0.9d0,
+     &          1.d0/
+      data ydi /226.d0,47.5d0,17.5d0,7.8d0,3.75d0,1.80d0,0.8d0,0.29d0,
+     &          0.06d0,0.d0/
 !
       data nbe /7/
-      data xbe /1.,1.5,2.,3.,4.,6.,10./
-      data ybe /0.21,0.12,0.10,0.09,0.09,0.08,0.2/
-      data zbe /0.51,0.32,0.29,0.26,0.26,0.17,0.31/
+      data xbe /1.d0,1.5d0,2.d0,3.d0,4.d0,6.d0,10.d0/
+      data ybe /0.21d0,0.12d0,0.10d0,0.09d0,0.09d0,0.08d0,0.2d0/
+      data zbe /0.51d0,0.32d0,0.29d0,0.26d0,0.26d0,0.17d0,0.31d0/
 !
       data nen /10/
-      data xen /0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1./
-      data yen /232.,51.,18.8,9.6,5.26,3.08,1.88,1.17,0.734,0.46/
+      data xen /0.1d0,0.2d0,0.3d0,0.4d0,0.5d0,0.6d0,0.7d0,0.8d0,0.9d0,
+     &          1.d0/
+      data yen /232.d0,51.d0,18.8d0,9.6d0,5.26d0,3.08d0,1.88d0,1.17d0,
+     &          0.734d0,0.46d0/
 !
       data ngv /8/
-      data xgv /0.125,0.25,0.375,0.5,0.625,0.75,0.875,1./
-      data ygv /98.,17.,5.52,2.,0.81,0.26,0.15,0.12/
+      data xgv /0.125d0,0.25d0,0.375d0,0.5d0,0.625d0,0.75d0,0.875d0,
+     &          1.d0/
+      data ygv /98.d0,17.d0,5.52d0,2.d0,0.81d0,0.26d0,0.15d0,0.12d0/
 !
       numf=4
 !
@@ -90,7 +100,13 @@
             identity=.false.
          endif
 !     
-      elseif((iflag.eq.1).or.(iflag.eq.2).or.(iflag.eq.3))then
+      elseif((iflag.eq.1).or.(iflag.eq.2).or.(iflag.eq.3)) then
+         if(iflag.eq.1) then
+            if(v(1,nodem).ne.0.d0) then
+               xflow=v(1,nodem)
+               return
+            endif
+         endif
 !     
          index=ielprop(nelem)
 !     
@@ -136,18 +152,13 @@
                nodea=nint(prop(index+1))
                nodeb=nint(prop(index+2))
                xn=prop(index+3)
-c               iaxial=nint(prop(index+4))
                radius=dsqrt((co(1,nodeb)+vold(1,nodeb)-
      &                       co(1,nodea)-vold(1,nodea))**2+
      &                      (co(2,nodeb)+vold(2,nodeb)-
      &                       co(2,nodea)-vold(2,nodea))**2+
      &                      (co(3,nodeb)+vold(3,nodeb)-
      &                       co(3,nodea)-vold(3,nodea))**2)
-c               if(iaxial.ne.0) then
-c                  a=pi*radius*radius/iaxial
-c               else
-                  a=pi*radius*radius
-c               endif
+               a=pi*radius*radius
                rh=radius/2.d0
             else
                a=prop(index+1)
@@ -174,18 +185,13 @@ c               endif
                nodea=nint(prop(index+1))
                nodeb=nint(prop(index+2))
                xn=prop(index+3)
-c               iaxial=nint(prop(index+4))
                radius=dsqrt((co(1,nodeb)+vold(1,nodeb)-
      &                       co(1,nodea)-vold(1,nodea))**2+
      &                       (co(2,nodeb)+vold(2,nodeb)-
      &                       co(2,nodea)-vold(2,nodea))**2+
      &                       (co(3,nodeb)+vold(3,nodeb)-
      &                       co(3,nodea)-vold(3,nodea))**2)
-c               if(iaxial.ne.0) then
-c                  a=pi*radius*radius/iaxial
-c               else
-                  a=pi*radius*radius
-c               endif
+               a=pi*radius*radius
                d=2.d0*radius
             else
                a=prop(index+1)
@@ -205,7 +211,7 @@ c               endif
 !
 !              assuming large reynolds number
 !
-               friction=1.d0/(2.03*dlog10(xks/(d*3.7)))**2
+               friction=1.d0/(2.03d0*dlog10(xks/(d*3.7d0)))**2.d0
             else
 !
 !              solving the implicit White-Colebrook equation
@@ -278,7 +284,7 @@ c               endif
             endif
             if(inv.ge.0) then
                call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &              isothermal,kon,ipkon,R,Kappa,v,mi)
+     &              isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
                if(inv.ne.0) then
                   xk=zeta/(a1*a1)
                else
@@ -294,7 +300,7 @@ c               endif
                prop(index+5)=0.d0
                lakon(nelem)(4:5)='CO'
                call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &              isothermal,kon,ipkon,R,Kappa,v,mi)
+     &              isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
                lakon(nelem)(4:5)='EL'
                if(inv.ne.0) then
                   xk=zeta/(a1*a1)
@@ -359,7 +365,7 @@ c               endif
             endif
             if(inv.ge.0) then
                call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &              isothermal,kon,ipkon,R,Kappa,v,mi)
+     &              isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
                if(inv.ne.0) then
                   xk=zeta/(a2*a2)
                else
@@ -370,7 +376,7 @@ c               endif
                reynolds=-reynolds
                lakon(nelem)(4:5)='EL'
                call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &              isothermal,kon,ipkon,R,Kappa,v,mi)
+     &              isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
                lakon(nelem)(4:5)='CO'
                if(inv.ne.0) then
                   xk=zeta/(a2*a2)
@@ -439,7 +445,7 @@ c               endif
             a1=prop(index+1)
             a2=prop(index+2)
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
 !
 !           check for negative flow: in that case the loss
 !           coefficient is wrong
@@ -474,7 +480,7 @@ c               endif
             a1=prop(index+1)
             a2=prop(index+2)
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
             if(inv.lt.0) then
                write(*,*) '*ERROR in liquidpipe: loss coefficients'
                write(*,*) '       for exit (Idelchik) do not apply to'
@@ -505,7 +511,7 @@ c               endif
             a1=prop(index+1)
             a2=prop(index+2)
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
             if(inv.lt.0) then
                write(*,*) '*ERROR in liquidpipe: loss coefficients'
                write(*,*) '       for a user element do not apply to'
@@ -614,7 +620,7 @@ c               endif
                reynolds=dabs(xflow)*dh/(dvi*a)
             endif
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
             if(inv.ne.0) then
                xk=zeta/(a*a)
             else
@@ -635,7 +641,7 @@ c               endif
                reynolds=dabs(xflow)*dh/(dvi*a1)
             endif
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
             if(inv.ne.0) then
                xk=zeta/(a1*a1)
             else
@@ -661,7 +667,7 @@ c               endif
                reynolds=dabs(xflow)*dh/(dvi*a2)
             endif
             call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &           isothermal,kon,ipkon,R,Kappa,v,mi)
+     &           isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
 !
 !           check for negative flow: in that case the loss
 !           coefficient is wrong
@@ -701,7 +707,7 @@ c               endif
             endif
             if(inv.ne.0) then
                call zeta_calc(nelem,prop,ielprop,lakon,reynolds,zeta,
-     &              isothermal,kon,ipkon,R,Kappa,v,mi)
+     &              isothermal,kon,ipkon,R,Kappa,v,mi,iaxial)
                xk=zeta/(a*a)
             else
 !
@@ -849,7 +855,7 @@ c               endif
                endif
 !            
 !               if (iflag.eq.1) then
-                  a1=1E-6
+                  a1=1d-6
                   a2=a1 
                if(inv.ne.0) then                         
                   xkn=rho/2*ciu**2*(1-(R1/R2)**2)
@@ -885,7 +891,7 @@ c               endif
                   prop(index+7)=c1u
                endif
 !     
-               a1=1E-6
+               a1=1d-6
                a2=a1 
                if(iflag.eq.1)then
                   xflow=0.5d0
@@ -945,12 +951,12 @@ c               endif
             else if (lakon(nelem)(4:5).eq.'VF') then
                numf=3  
                if(R2.ge.R1) then
-                  f=P1-P2+xkp
+                  f=p1-p2+xkp
                   df(1)=1
                   df(2)=0
                   df(3)=-1
                elseif(R2.lt.R1) then 
-                  f=P1-P2-xkp
+                  f=p1-p2-xkp
                   df(1)=1
                   df(2)=0
                   df(3)=-1
@@ -999,7 +1005,7 @@ c               endif
          
             if(inv.eq.1) then
                write(1,56)'       Inlet node  ',node1,':   Tt1=',T,
-     &              ', Pt1=',P1
+     &              ', Pt1=',p1
                if(lakon(nelem)(4:5).eq.'EL'.or.
      &            lakon(nelem)(4:5).eq.'CO'.or.
      &            lakon(nelem)(4:5).eq.'EN'.or.
@@ -1031,7 +1037,7 @@ c               endif
                endif
 !
                write(1,56)'       Outlet node ',node2,':   Tt2=',T,
-     &              ', Pt2=',P2
+     &              ', Pt2=',p2
 !     
             else if(inv.eq.-1) then
                
