@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,8 @@
 !     
       subroutine liquidpump(node1,node2,nodem,nelem,
      &     nactdog,identity,ielprop,prop,iflag,v,xflow,f,
-     &     nodef,idirf,df,rho,g,co,numf,mi,ttime,time,iaxial)
+     &     nodef,idirf,df,rho,g,co,numf,mi,ttime,time,
+     &     iaxial,iplausi)
 !
 !     pump for incompressible media
 !     
@@ -27,12 +28,14 @@
       logical identity
 !      
       integer nelem,nactdog(0:3,*),node1,node2,nodem,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,
-     &     inv,id,numf,npu,i,mi(*),iaxial
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,
+     &     inv,id,numf,npu,i,mi(*),iaxial,iplausi
 !      
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),ttime,time,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),ttime,time,
      &     p1,p2,rho,g(3),dg,z1,z2,co(3,*),
      &     xpu(10),ypu(10),xxpu(10),yypu(10),dh
+!
+!
 !
       numf=3
 !
@@ -47,7 +50,13 @@
             identity=.false.
          endif
 !     
-      elseif((iflag.eq.1).or.(iflag.eq.2))then
+      elseif((iflag.eq.1).or.(iflag.eq.2)) then
+         if(iflag.eq.1) then
+            if(v(1,nodem).ne.0.d0) then
+               xflow=v(1,nodem)
+               return
+            endif
+         endif
 !     
          index=ielprop(nelem)
 !
