@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -42,7 +42,7 @@
      &  irow(*),nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),
      &  ielorien(mi(3),*),ipkon(*),ipobody(2,*),nbody,ibody(3,*),
      &  nk,ne,nboun,nmpc,nforc,nload,neq(2),nzl,nmethod,icolumn,
-     &  ithermal,iprestr,iperturb,nzs(3),i,j,k,l,m,idist,jj,
+     &  ithermal(*),iprestr,iperturb(*),nzs(3),i,j,k,l,m,idist,jj,
      &  ll,id,id1,id2,ist,ist1,ist2,index,jdof1,jdof2,idof1,idof2,
      &  mpc1,mpc2,index1,index2,node1,node2,ielprop(*),
      &  ntmat_,indexe,nope,norien,iexpl,i0,ncmat_,istep,iinc,
@@ -117,7 +117,7 @@ c     Bernhardi end
         do
            j=ipobody(1,index)
            if(j.eq.0) exit
-           if(ibody(1,j).eq.1) then
+           if(ibody(1,j).eq.4) then
               om=xbody(1,j)
               p1(1)=xbody(2,j)
               p1(2)=xbody(3,j)
