@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -43,7 +43,7 @@
      &  irow(*),nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),
      &  ielorien(mi(3),*),ipkon(*),ipobody(2,*),nbody,ibody(3,*),
      &  nk,ne,nboun,nmpc,nforc,nload,neq(2),nzl,nmethod,icolumn,
-     &  ithermal,iprestr,iperturb,nzs(3),i,j,k,l,m,idist,jj,iloc,
+     &  ithermal(*),iprestr,iperturb(*),nzs(3),i,j,k,l,m,idist,jj,
      &  ll,id,id1,id2,ist,ist1,ist2,index,jdof1,jdof2,idof1,idof2,
      &  mpc1,mpc2,index1,index2,node1,node2,ne0,igauss,imat,
      &  ntmat_,indexe,nope,norien,iexpl,i0,ncmat_,istep,iinc,
@@ -82,7 +82,7 @@ c      do i=1,nzs(2)
 c         au(i)=0.d0
 c      enddo
 !
-      if((ithermal.le.1).or.(ithermal.eq.3)) then
+      if((ithermal(1).le.1).or.(ithermal(1).eq.3)) then
 !     
 !     mechanical analysis: loop over all elements
 !     
@@ -149,12 +149,11 @@ c      enddo
      &                 springarea(1,konl(nope+1)),nmethod,
      &                 mi,reltime,nasym)
                elseif(mortar.eq.1) then
-                  iloc=kon(indexe+nope+1)
                   jfaces=kon(indexe+nope+2)
                   igauss=kon(indexe+nope+1) 
                   call springdamp_f2f(xl,elas,voldl,s,imat,elcon,
      &                 ncmat_,ntmat_,nope,lakon(i),iperturb,
-     &                 springarea(1,iloc),
+     &                 springarea(1,igauss),
      &                 nmethod,mi,reltime,nasym,jfaces,igauss,pslavsurf,
      &                 pmastsurf,clearini)
                endif
@@ -183,6 +182,7 @@ c      enddo
 !     check whether one of the DOF belongs to a SPC or MPC
 !     
                   if((jdof1.gt.0).and.(jdof2.gt.0)) then
+c                     write(*,*) 'mafilldm1',jdof1,jdof2,jj,ll,s(jj,ll)
                      call add_sm_st(au,ad,jq,irow,jdof1,jdof2,
      &                    s(jj,ll),jj,ll)
                   elseif((jdof1.gt.0).or.(jdof2.gt.0)) then
@@ -218,6 +218,7 @@ c                           id=ilmpc(id)
      &                                        coefmpc(ist)
                               if(idof1.eq.idof2) value=2.d0*value
                               if(idof2.gt.0) then
+c                     write(*,*) 'mafilldm2',idof1,idof2,i0,i0,value
                                  call add_sm_st(au,ad,jq,irow,idof1,
      &                                idof2,value,i0,i0)
                               endif
@@ -264,6 +265,7 @@ c                        id2=ilmpc(id2)
                                  value=coefmpc(index1)*coefmpc(index2)*
      &                                s(jj,ll)/coefmpc(ist)/coefmpc(ist)
                                  if((idof1.gt.0).and.(idof2.gt.0)) then
+c                     write(*,*) 'mafilldm3',idof1,idof2,i0,i0,value
                                     call add_sm_st(au,ad,jq,irow,
      &                                   idof1,idof2,value,i0,i0)
                                  endif
@@ -302,6 +304,7 @@ c                        id2=ilmpc(id2)
      &                                         coefmpc(ist2)
                                  if(idof1.eq.idof2) value=2.d0*value
                                  if((idof1.gt.0).and.(idof2.gt.0)) then
+c                     write(*,*) 'mafilldm4',idof1,idof2,i0,i0,value
                                     call add_sm_st(au,ad,jq,irow,
      &                                   idof1,idof2,value,i0,i0)
                                  endif
@@ -370,6 +373,7 @@ c                          id=ilmpc(id)
      &                              nodempc(1,index))
                              value=-coefmpc(index)*s(jj,ll)/coefmpc(ist)
                              if(idof1.gt.0) then
+c                     write(*,*) 'mafilldm6',idof1,idof2,i0,i0,value
                                 call add_sm_st_as(au,ad,jq,irow,idof1,
      &                               idof2,value,i0,i0,nzs)
                              endif
@@ -400,6 +404,7 @@ c                          id=ilmpc(id)
      &                             nodempc(1,index))
                              value=-coefmpc(index)*s(jj,ll)/coefmpc(ist)
                              if(idof2.gt.0) then
+c                     write(*,*) 'mafilldm7',idof1,idof2,i0,i0,value
                                 call add_sm_st_as(au,ad,jq,irow,idof1,
      &                               idof2,value,i0,i0,nzs)
                              endif
@@ -459,6 +464,7 @@ c     &                            s(jj,ll)/coefmpc(ist)/coefmpc(ist)
                              value=coefmpc(index1)*coefmpc(index2)*
      &                            s(jj,ll)/coefmpc(ist1)/coefmpc(ist2)
                              if((idof1.gt.0).and.(idof2.gt.0)) then
+c                     write(*,*) 'mafilldm8',idof1,idof2,i0,i0,value
                                 call add_sm_st_as(au,ad,jq,irow,
      &                               idof1,idof2,value,i0,i0,nzs)
                              endif
@@ -495,6 +501,7 @@ c     &                            s(jj,ll)/coefmpc(ist)/coefmpc(ist)
                              value=coefmpc(index1)*coefmpc(index2)*
      &                            s(jj,ll)/coefmpc(ist1)/coefmpc(ist2)
                              if((idof1.gt.0).and.(idof2.gt.0)) then
+c                     write(*,*) 'mafilldm9',idof1,idof2,i0,i0,value
                                 call add_sm_st_as(au,ad,jq,irow,
      &                               idof1,idof2,value,i0,i0,nzs)
                              endif
