@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@
      &  xstateini,xstate,thicke,integerglob,doubleglob,
      &  tieset,istartset,iendset,ialset,ntie,nasym,iactive,h0,
      &  pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,
-     &  iponoel,inoel)
+     &  iponoel,inoel,network)
 !
 !     filling the stiffness matrix in spare matrix format (sm)
 !
@@ -58,12 +58,12 @@
      &  ielorien(mi(3),*),integerglob(*),istartset(*),iendset(*),
      &  ipkon(*),intscheme,ncocon(2,*),nshcon(*),ipobody(2,*),nbody,
      &  ibody(3,*),nk,ne,nboun,nmpc,nforc,nload,neq(2),nzl,nmethod,
-     &  ithermal(2),iprestr,iperturb(*),nzs(3),i,j,k,l,m,idist,jj,
+     &  ithermal(*),iprestr,iperturb(*),nzs(3),i,j,k,l,m,idist,jj,
      &  ll,id,id1,id2,ist,ist1,ist2,index,jdof1,jdof2,idof1,idof2,
      &  mpc1,mpc2,index1,index2,jdof,node1,node2,kflag,icalccg,
      &  ntmat_,indexe,nope,norien,iexpl,i0,ncmat_,istep,iinc,mortar,
      &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,iactive(3),
-     &  ielprop(*),iponoel(*),inoel(2,*)
+     &  ielprop(*),iponoel(*),inoel(2,*),network
 !
       real*8 co(3,*),xboun(*),coefmpc(*),xforc(*),xload(2,*),p1(3),
      &  p2(3),ad(*),au(*),bodyf(3),fext(*),xloadold(2,*),reltime,
@@ -438,7 +438,8 @@ c                         id=ilmpc(id)
 !
            read(lakon(i)(8:8),'(i1)') nope
            nope=nope+1
-        elseif(lakon(i)(1:2).eq.'D ') then
+        elseif((lakon(i)(1:2).eq.'D ').or.
+     &         ((lakon(i)(1:1).eq.'D').and.(network.eq.1))) then
 !
 !          asymmetrical contribution -> mafillsmas.f
 !
@@ -456,7 +457,8 @@ c                         id=ilmpc(id)
      &  xstiff,xloadold,reltime,ipompc,nodempc,coefmpc,nmpc,ikmpc,
      &  ilmpc,springarea,plkcon,nplkcon,npmat_,ncmat_,elcon,nelcon,
      &  lakon,pslavsurf,pmastsurf,mortar,clearini,plicon,nplicon,
-     &  ipkon,ielprop,prop,iponoel,inoel)
+     &  ipkon,ielprop,prop,iponoel,inoel,sti,xstateini,xstate,
+     &  nstate_,network,ipobody,xbody,ibody)
 !
         do jj=1,nope
 !
