@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -33,7 +33,8 @@ static ITG *nk1,*kon1,*ipkon1,*ne1,*nodeboun1,*ndirboun1,*nboun1,
     *mi1,*ncmat1_,*mass1,*stiffness1,*buckling1,*rhsi1,*intscheme1,
     *nshcon1,*ncocon1,*istep1,*iinc1,*coriolis1,*ibody1,*nstate1_,
     *integerglob1,*istartset1,*iendset1,*ialset1,*ntie1,*nasym1,
-    *mortar1,*ielprop1,*ne01,num_cpus,*kscale1,*iponoel1,*inoel1;
+    *mortar1,*ielprop1,*ne01,num_cpus,*kscale1,*iponoel1,*inoel1,
+    *network1,*neapar=NULL,*nebpar=NULL,*mscalmethod1;
 
 static double *co1,*xboun1,*coefmpc1,*xforc1,*xload1,*xbody1,*cgr1,
     *ad1=NULL,*au1=NULL,*fext1=NULL,*elcon1,*rhcon1,*alcon1,*alzero1,
@@ -41,7 +42,7 @@ static double *co1,*xboun1,*coefmpc1,*xforc1,*xload1,*xbody1,*cgr1,
     *plicon1,*plkcon1,*xstiff1,*dtime1,*physcon1,*shcon1,*cocon1,
     *ttime1,*time1,*xloadold1,*reltime1,*veold1,*springarea1,
     *xstateini1,*xstate1,*thicke1,*doubleglob1,*pslavsurf1,*pmastsurf1,
-    *clearini1,*prop1,*fnext1=NULL;
+    *clearini1,*prop1,*fnext1=NULL,*smscale1;
 
 void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 	       ITG *ne,ITG *nodeboun,ITG *ndirboun,double *xboun, 
@@ -75,8 +76,13 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 	       ITG *istartset,ITG *iendset,ITG *ialset,ITG *ntie,
 	       ITG *nasym,double *pslavsurf,double *pmastsurf,ITG *mortar,
 	       double *clearini,ITG *ielprop,double *prop,ITG *ne0,
-	       double *fnext,ITG *kscale,ITG *iponoel,ITG *inoel){
+	       double *fnext,ITG *kscale,ITG *iponoel,ITG *inoel,
+	       ITG *network,ITG *ntrans,ITG *inotr,double *trab,
+	       double *smscale,ITG *mscalmethod){
 
+  /* mafillsmmain = main program for MAtrix FILLing of the Stiffnes
+     Matrix */
+  
     ITG i,j,mt=mi[1]+1;
       
     /* variables for multithreading procedure */
@@ -135,6 +141,28 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
     if(*ne<num_cpus) num_cpus=*ne;
     
     pthread_t tid[num_cpus];
+    
+    /* determining the element bounds in each thread */
+    
+    NNEW(neapar,ITG,num_cpus);
+    NNEW(nebpar,ITG,num_cpus);
+    if((*nasym==0)||(*ithermal>1)){
+
+        /* symmetric mechanical calculations or
+           thermal/thermomechanical calculations:
+           include contact elements (symmetric
+           thermal contributions are not covered by
+           mafillsmas.f) */
+
+	elementcpuload(neapar,nebpar,ne,ipkon,&num_cpus);
+
+    }else{
+
+        /* asymmetric mechanical calculations:
+           do not include contact elements */
+
+	elementcpuload(neapar,nebpar,ne0,ipkon,&num_cpus);
+    }
 
     /* determine nzl */
 
@@ -148,10 +176,8 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 
     /* allocating fields for mass and stiffness matrix */
 
-//    if(*buckling!=1){
-	NNEW(ad1,double,num_cpus*neq[1]);
-	NNEW(au1,double,(long long)num_cpus*nzs[2]);
-//    }
+    NNEW(ad1,double,num_cpus*neq[1]);
+    NNEW(au1,double,(long long)num_cpus*nzs[2]);
 
     if(*rhsi==1){
 	NNEW(fext1,double,num_cpus*neq[1]);
@@ -204,7 +230,8 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
     iendset1=iendset;ialset1=ialset;ntie1=ntie;nasym1=nasym;
     pslavsurf1=pslavsurf;pmastsurf1=pmastsurf;mortar1=mortar;
     clearini1=clearini;ielprop1=ielprop;prop1=prop;ne01=ne0;kscale1=kscale;
-    iponoel1=iponoel;inoel1=inoel;  
+    iponoel1=iponoel;inoel1=inoel;network1=network;
+    smscale1=smscale;mscalmethod1=mscalmethod;  
 
     /* calculating the stiffness/mass */
     
@@ -217,15 +244,10 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 	ithread[i]=i;
 	pthread_create(&tid[i], NULL, (void *)mafillsmmt, (void *)&ithread[i]);
     }
-    for(i=0; i<num_cpus; i++)  pthread_join(tid[i], NULL);
+    for(i=0; i<num_cpus; i++)
+      pthread_join(tid[i], NULL);
     
-    SFREE(ithread);
-
-    /*      for(i=0;i<num_cpus;i++){
-      for(k=i*neq[1];k<i*neq[1]+neq[1];++k){printf("fext=%" ITGFORMAT ",%f\n",k-i*neq[1],fext1[k]);}
-      for(k=i*neq[1];k<i*neq[1]+neq[1];++k){printf("ad=%" ITGFORMAT ",%f\n",k-i*neq[1],ad1[k]);}
-      for(k=i*nzs[2];k<i*nzs[2]+nzs[2];++k){printf("au=%" ITGFORMAT ",%f\n",k-i*nzs[2],au1[k]);}
-      }*/
+    SFREE(ithread);SFREE(neapar);SFREE(nebpar);
 
     /* copying and accumulating the stiffnes and/or mass matrix 
        for buckling the matrices have to be added*/
@@ -358,16 +380,12 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 	}
     }
     SFREE(nmethod1);
-      
-    /*     for(k=0;k<neq[1];++k){printf("fext=%" ITGFORMAT ",%f\n",k,fext[k]);}
-      for(k=0;k<neq[1];++k){printf("ad=%" ITGFORMAT ",%f\n",k,ad[k]);}
-      for(k=0;k<nzs[1];++k){printf("au=%" ITGFORMAT ",%f\n",k,au[k]);}*/
 
     /* taking point forces into account in fext */
 
     FORTRAN(mafillsmforc,(nforc,ndirforc,nodeforc,xforc,nactdof,
-			  fext,nmpc,ipompc,nodempc,ikmpc,ilmpc,
-		          coefmpc,mi,rhsi,fnext,nmethod));
+			  fext,ipompc,nodempc,coefmpc,mi,rhsi,fnext,
+                          nmethod,ntrans,inotr,trab,co));
   
   return;
 
@@ -377,7 +395,7 @@ void mafillsmmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,
 
 void *mafillsmmt(ITG *i){
 
-    ITG indexad,indexfext,indexadb,nea,neb,nedelta,indexfnext;
+    ITG indexad,indexfext,indexadb,nea,neb,indexfnext;
     long long indexau,indexaub;
 
     indexad=0;
@@ -387,10 +405,9 @@ void *mafillsmmt(ITG *i){
     indexaub=0;
     indexfnext=0;
 
-//    if(*buckling1!=1){
-	indexad=*i*neq1[1];
-	indexau=(long long)*i*nzs1[2];
-//    }
+    indexad=*i*neq1[1];
+    indexau=(long long)*i*nzs1[2];
+    
     if(*rhsi1==1){
 	indexfext=*i*neq1[1];
     }
@@ -404,24 +421,9 @@ void *mafillsmmt(ITG *i){
     if(nmethod1[0]==4){
 	indexfnext=*i*(mi1[1]+1)**nk1;
     }
-    
-// ceil -> floor
 
-    if(*nasym1==0){
-	nedelta=(ITG)floor(*ne1/(double)num_cpus);
-	nea=*i*nedelta+1;
-	neb=(*i+1)*nedelta;
-// next line! -> all parallel sections
-	if((*i==num_cpus-1)&&(neb<*ne1)) neb=*ne1;
-    }else{
-	nedelta=(ITG)floor(*ne01/(double)num_cpus);
-	nea=*i*nedelta+1;
-	neb=(*i+1)*nedelta;
-// next line! -> all parallel sections
-	if((*i==num_cpus-1)&&(neb<*ne01)) neb=*ne01;
-    }
-
-//    printf("i=%d,nea=%d,neb=%d\n",*i,nea,neb);
+    nea=neapar[*i]+1;
+    neb=nebpar[*i]+1;
 
     FORTRAN(mafillsm,(co1,nk1,kon1,ipkon1,lakon1,ne1,nodeboun1,ndirboun1,
 	    xboun1,nboun1,
@@ -442,7 +444,8 @@ void *mafillsmmt(ITG *i){
             xstateini1,xstate1,thicke1,integerglob1,doubleglob1,
 	    tieset1,istartset1,iendset1,ialset1,ntie1,nasym1,pslavsurf1,
 	    pmastsurf1,mortar1,clearini1,ielprop1,prop1,ne01,
-	    &fnext1[indexfnext],&nea,&neb,kscale1,iponoel1,inoel1));
+	    &fnext1[indexfnext],&nea,&neb,kscale1,iponoel1,inoel1,network1,
+	    smscale1,mscalmethod1));
 
     return NULL;
 }
