@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -27,12 +27,11 @@
      &  springarea,nstate_,xstateini,xstate,thicke,integerglob,
      &  doubleglob,tieset,istartset,iendset,ialset,ntie,nasym,
      &  pslavsurf,pmastsurf,mortar,clearini,ielprop,prop,ne0,nea,
-     &  neb,distmin,ndesi,nodedesi,df,jqs,irows,dfminds,
-     &  icoordinate,dxstiff,xdesi,istartelem,ialelem,v)
+     &  neb,distmin,ndesi,nodedesi,df,jqs,irows,dfl,
+     &  icoordinate,dxstiff,xdesi,istartelem,ialelem,v,sigma,
+     &  ieigenfrequency)
 !
       implicit none
-!
-      integer mass(2),stiffness,buckling,rhsi,stiffonly(2),coriolis
 !
       character*8 lakon(*)
       character*20 sideload(*)
@@ -44,17 +43,18 @@
      &  ielprop(*),nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),
      &  ntie,ielorien(mi(3),*),integerglob(*),istartset(*),iendset(*),
      &  ipkon(*),intscheme,ipobody(2,*),nbody,ibody(3,*),ne,nmpc,
-     &  nload,neq(2),nmethod,ithermal(2),iprestr,iperturb(*),i,j,k,
+     &  nload,neq(2),nmethod,ithermal(*),iprestr,iperturb(*),i,j,k,
      &  idist,jj,id,ist,index,jdof1,idof1,node1,kflag,icalccg,ntmat_,
      &  indexe,nope,norien,iexpl,i0,ncmat_,istep,iinc,jqs(*),irows(*),
      &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,mortar,nea,
      &  neb,ndesi,nodedesi(*),idesvar,istartelem(*),ialelem(*),
-     &  icoordinate,ii
+     &  icoordinate,ii,ieigenfrequency,mass(2),stiffness,buckling,rhsi,
+     &  stiffonly(2),coriolis,idesloc
 !
       real*8 co(3,*),coefmpc(*),xload(2,*),p1(3),p2(3),bodyf(3),
      &  xloadold(2,*),reltime,t0(*),t1(*),vold(0:mi(2),*),
      &  s(60,60),ff(60),sti(6,mi(1),*),sm(60,60),xdesi(3,*),
-     &  stx(6,mi(1),*),elcon(0:ncmat_,ntmat_,*),val,
+     &  stx(6,mi(1),*),elcon(0:ncmat_,ntmat_,*),val,sigma,
      &  rhcon(0:1,ntmat_,*),springarea(2,*),alcon(0:6,ntmat_,*),
      &  physcon(*),prop(*),xstate(nstate_,mi(1),*),
      &  xstateini(nstate_,mi(1),*),alzero(*),orab(7,*),
@@ -62,23 +62,10 @@
      &  plkcon(0:2*npmat_,ntmat_,*),xstiff(27,mi(1),*),
      &  veold(0:mi(2),*),om,dtime,ttime,time,thicke(mi(3),*),
      &  doubleglob(*),clearini(3,9,*),pslavsurf(3,*),
-     &  pmastsurf(6,*),distmin,dfminds(ndesi,60),
+     &  pmastsurf(6,*),distmin,dfl(20,60),
      &  df(*),dxstiff(27,mi(1),ne,*),v(0:mi(2),*)
 !
-      intent(in) co,kon,ipkon,lakon,ne,ipompc,nodempc,coefmpc,nmpc,
-     &  nelemload,sideload,nload,xbody,ipobody,nbody,nactdof,neq,
-     &  ikmpc,ilmpc,elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,
-     &  ielmat,ielorien,norien,orab,ntmat_,t0,t1,ithermal,iprestr,
-     &  vold,iperturb,sti,stx,iexpl,plicon,nplicon,plkcon,nplkcon,
-     &  xstiff,npmat_,dtime,matname,mi,ncmat_,mass,stiffness,
-     &  buckling,rhsi,intscheme,physcon,ttime,time,istep,iinc,
-     &  coriolis,ibody,xloadold,reltime,veold,nstate_,xstateini,
-     &  thicke,integerglob,doubleglob,tieset,istartset,iendset,
-     &  ialset,ntie,nasym,pslavsurf,pmastsurf,mortar,clearini,
-     &  ielprop,prop,ne0,nea,neb,ndesi,nodedesi,distmin,
-     &  icoordinate,jqs,irows,dxstiff,xdesi,istartelem,ialelem,v
 !
-      intent(inout) xload,nmethod,cgr,springarea,xstate,df,dfminds
 !
       kflag=2
       i0=0
@@ -159,9 +146,9 @@ c     Bernhardi end
 !     
 !     assigning centrifugal forces
 !     
-               bodyf(1)=0.
-               bodyf(2)=0.
-               bodyf(3)=0.
+               bodyf(1)=0.d0
+               bodyf(2)=0.d0
+               bodyf(3)=0.d0
 !     
                index=i
                do
@@ -208,12 +195,13 @@ c     Bernhardi end
      &           integerglob,doubleglob,tieset,istartset,
      &           iendset,ialset,ntie,nasym,pslavsurf,pmastsurf,mortar,
      &           clearini,ielprop,prop,distmin,ndesi,nodedesi,
-     &           dfminds,icoordinate,dxstiff,ne,xdesi,istartelem,
-     &           ialelem,v)
+     &           dfl,icoordinate,dxstiff,ne,xdesi,istartelem,
+     &           ialelem,v,sigma,ieigenfrequency)
 !     
             do ii=istartelem(i),istartelem(i+1)-1
                idesvar=ialelem(ii)
                if(idesvar.eq.0) cycle
+               idesloc=ii-istartelem(i)+1
 !
                do jj=1,3*nope
 !     
@@ -225,12 +213,8 @@ c     Bernhardi end
 !     
                   if(jdof1.le.0) then
                      if(nmpc.ne.0) then
-c     idof1=(node1-1)*8+k
                         idof1=jdof1
-c     call nident(ikmpc,idof1,nmpc,id)
-c     if((id.gt.0).and.(ikmpc(id).eq.idof1)) then
                         if(idof1.ne.2*(idof1/2)) then
-c     id=ilmpc(id)
                            id=(-idof1+1)/2
                            ist=ipompc(id)
                            index=nodempc(3,ist)
@@ -239,14 +223,10 @@ c     id=ilmpc(id)
                               jdof1=nactdof(nodempc(2,index),
      &                             nodempc(1,index))
                               if(jdof1.gt.0) then
-                                 val=-coefmpc(index)*dfminds(idesvar,jj)
+                                 val=-coefmpc(index)*dfl(idesloc,jj)
      &                                /coefmpc(ist)
                                  call add_bo_st(df,jqs,irows,jdof1,
      &                                          idesvar,val)
-c     dfextminds(idesvar,jdof1)=
-c     &                            dfextminds(idesvar,jdof1)
-c     &                            -coefmpc(index)*dfminds(idesvar,jj)
-c     &                            /coefmpc(ist)
                               endif
                               index=nodempc(3,index)
                               if(index.eq.0) exit
@@ -255,10 +235,8 @@ c     &                            /coefmpc(ist)
                      endif
                      cycle
                   endif  
-c     dfextminds(idesvar,jdof1)=dfextminds(idesvar,jdof1)+
-c     &          dfminds(idesvar,jj)
                   call add_bo_st(df,jqs,irows,jdof1,idesvar,
-     &                 dfminds(idesvar,jj))
+     &                 dfl(idesloc,jj))
                enddo
             enddo
          enddo
