@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,8 @@
       subroutine massflow_percent(node1,node2,nodem,nelem,lakon,kon,
      &        ipkon,nactdog,identity,ielprop,prop,iflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,shcon,
-     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,iaxial)
+     &        nshcon,rhcon,nrhcon,ntmat_,co,vold,mi,ttime,time,
+     &        iaxial,iplausi)
 !     
 !     partial massflow  element
 !
@@ -32,19 +33,22 @@
       character*81 set(*)
 !     
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,
      &     inv,ipkon(*),kon(*),number,kgas,iaxial,
      &     nodea,nodeb,mi(*),i,itype,nodemup,
-     &     nrhcon(*),ntmat_,nshcon(*)
+     &     nrhcon(*),ntmat_,nshcon(*),iplausi
 !     
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),kappa,R,a,d,xl,
-     &     p1,p2,T1,physcon(3),pi,xflow_oil,T2,co(3,*),vold(0:mi(2),*),
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),kappa,R,a,d,xl,
+     &     p1,p2,T1,physcon(*),pi,xflow_oil,T2,co(3,*),vold(0:mi(2),*),
      &     xflow_sum,percent_xflow,cp,dvi,pt1,pt2,Tt1,Tt2,ttime,time,
      &     shcon(0:3,ntmat_,*),rhcon(0:1,ntmat_,*)
+!
+!
 !
       pi=4.d0*datan(1.d0) 
-  
-      if (iflag.eq.0) then
+      index=ielprop(nelem)
+!  
+      if(iflag.eq.0) then
          identity=.true.
 !     
          if(nactdog(2,node1).ne.0)then
@@ -55,9 +59,12 @@
             identity=.false.
          endif
 !     
-      elseif (iflag.eq.1)then
+      elseif(iflag.eq.1)then
+         if(v(1,nodem).ne.0.d0) then
+            xflow=v(1,nodem)
+            return
+         endif
 !     
-         index=ielprop(nelem)
          percent_xflow=prop(index+1)
          xflow_sum=0
 !         
@@ -76,14 +83,11 @@
 !
          xflow=xflow_sum*percent_xflow
 !     
-      elseif ((iflag.eq.2).or.(iflag.eq.3))then
+      elseif((iflag.eq.2).or.(iflag.eq.3))then
 !     
-        
-         index=ielprop(nelem)
          percent_xflow=prop(index+1)
          xflow_sum=0
          do i=2,10
-            nodemup=kon(ipkon(nint(prop(index+i)))+2)
             if(nint(prop(index+i)).ne.0) then
                nodemup=kon(ipkon(nint(prop(index+i)))+2)
                if(v(1,nodemup).gt.0)then        
@@ -93,7 +97,7 @@
          enddo
          
          if(xflow_sum.eq.0.d0) then
-            xflow_sum=1E-5
+            xflow_sum=1d-5
          endif
 !
          inv=1
@@ -101,8 +105,8 @@
          pt1=v(2,node1)
          pt2=v(2,node2)
          xflow=v(1,nodem)*iaxial
-         Tt1=v(0,node1)+physcon(1)
-         Tt2=v(0,node2)+physcon(1)
+         Tt1=v(0,node1)-physcon(1)
+         Tt2=v(0,node2)-physcon(1)
 !
          nodef(1)=node1
          nodef(2)=node1
@@ -138,14 +142,14 @@
  55         format(1X,A,I6,A,I6,A,e11.4,A,A,e11.4,A)
 !            
             write(1,56)'       Inlet node ',node1,' :   Tt1 = ',Tt1,
-     &           '  , Ts1 = ',Tt1,'  , Pt1 = ',Pt1
+     &           '  , Ts1 = ',Tt1,'  , Pt1 = ',pt1
 !            
             write(1,*)'             Element ',nelem,lakon(nelem)
             write(1,57)'        Massflow upstream = ',xflow_sum,
      &        ' [kg/s]'
             write(1,58)'        Massflow fraction = ', percent_xflow
-            write(1,56)'       Outlet node ',node2,':    Tt2=',Tt2,
-     &           ', Ts2=',Tt2,', Pt2=',Pt2
+            write(1,56)'      Outlet node ',node2,':    Tt2=',Tt2,
+     &           ', Ts2=',Tt2,', Pt2=',pt2
 !            
          endif
       endif
