@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !
       subroutine materials(inpc,textpart,matname,nmat,nmat_,
      &  irstrt,istep,istat,n,iline,ipol,inl,ipoinp,inp,ipoinpc,
-     &  imat)
+     &  imat,ier)
 !
 !     reading the input deck: *MATERIAL
 !
@@ -28,20 +28,22 @@
       character*80 matname(*)
       character*132 textpart(16)
 !
-      integer nmat,nmat_,istep,istat,n,key,i,irstrt,iline,ipol,inl,
-     &  ipoinp(2,*),inp(3,*),ipoinpc(0:*),imat
+      integer nmat,nmat_,istep,istat,n,key,i,irstrt(*),iline,ipol,inl,
+     &  ipoinp(2,*),inp(3,*),ipoinpc(0:*),imat,ier
 !
-      if((istep.gt.0).and.(irstrt.ge.0)) then
+      if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) 
      &       '*ERROR reading *MATERIAL: *MATERIAL should be placed'
          write(*,*) '  before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       nmat=nmat+1
       if(nmat.gt.nmat_) then
          write(*,*) '*ERROR reading *MATERIAL: increase nmat_'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       imat=nmat
@@ -54,7 +56,8 @@
      &             '*ERROR reading *MATERIAL: material name too long'
                write(*,*) '       (more than 80 characters)'
                write(*,*) '       material name:',textpart(i)(1:132)
-               call exit(201)
+               ier=1
+               return
             endif
             exit
          else
