@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -29,10 +29,7 @@
       real*8 voldl(0:mi(2),9),s(60,60),gastemp,shcon(0:3,ntmat_,*),
      &  cp,r,dvi,rhcon(0:1,ntmat_,*),rho
 !
-      intent(in) voldl,imat,konl,mi,ntmat_,shcon,
-     &  nshcon,rhcon,nrhcon
 !
-      intent(inout) s
 !
       gastemp=(voldl(0,1)+voldl(0,3))/2.d0
 !
