@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -30,9 +30,7 @@
 !
       dimension x(n)
 !
-      intent(in) x,px,n
 !
-      intent(out) id
 !
       id=0
       if(n.eq.0) return
