@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -35,7 +35,8 @@
      &  mt,i,ii,j,k,jj,kk,indexe,nope,norien,ihyper,iactive,
      &  imat,mint3d,iorien,ilayer,nlayer,ki,kl,istartdesi(*),
      &  ielprop(*),mortar,idesvar,node,ndesi,nodedesi(*),
-     &  nobject,iobject,ialdesi(*),ij
+     &  nobject,iobject,ialdesi(*),ij,node1,node2,
+     &  ifaceqexp(2,20),ifacewexp(2,15) 
 !
       real*8 co(3,*),shp(4,26),xl(3,26),vl(0:mi(2),26),
      &  prop(*),rhcon(0:1,ntmat_,*),xs2(3,7),thickness,rho,xl2(3,8),
@@ -43,6 +44,8 @@
      &  xsj,weight,gs(8,4),a,tlayer(4),dlayer(4),xlayer(mi(3),4),
      &  thicke(mi(3),*),distmin,xmassel,g0(*),xmass(*),xdesi(3,*),
      &  dgdx(ndesi,nobject)
+!
+!
 !
       include "gauss.f"
 !
@@ -51,6 +54,47 @@
 !
       mt=mi(2)+1
 !
+!     nodes in expansion direction of hex element 
+!
+      data ifaceqexp /5,17,
+     &                6,18,
+     &                7,19,
+     &                8,20,
+     &                1,17,
+     &                2,18,
+     &                3,19,
+     &                4,20,
+     &                13,0,
+     &                14,0,
+     &                15,0,
+     &                16,0,
+     &                9,0,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                0,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
+!
+!     nodes in expansion direction of wedge element 
+!
+      data ifacewexp /4,13,
+     &                5,14,
+     &                6,16,
+     &                1,13,
+     &                2,14,
+     &                3,15,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                7,0,
+     &                8,0,
+     &                9,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
+!
 !     -------------------------------------------------------------
 !     Initialisation of the loop for the variation of 
 !     the internal forces
@@ -82,7 +126,7 @@
                imat=ielmat(1,i)
                amat=matname(imat)
                if(norien.gt.0) then
-                  iorien=ielorien(1,i)
+                  iorien=max(0,ielorien(1,i))
                else
                   iorien=0
                endif
@@ -95,9 +139,7 @@
             else
 !     
 !     composite materials
-!     
-               mint2d=4
-               nopes=8
+!
 !     determining the number of layers
 !     
                nlayer=0
@@ -110,28 +152,63 @@
 !     determining the layer thickness and global thickness
 !     at the shell integration points
 !     
-               iflag=1
-               indexe=ipkon(i)
-               do kk=1,mint2d
-                  xi=gauss3d2(1,kk)
-                  et=gauss3d2(2,kk)
-                  call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                  tlayer(kk)=0.d0
-                  do k=1,nlayer
-                     thickness=0.d0
-                     do j=1,nopes
-                        thickness=thickness+thicke(k,indexe+j)*shp2(4,j)
+               if(lakonl(4:5).eq.'20') then
+!
+                  mint2d=4
+                  nopes=8
+!
+                  iflag=1
+                  indexe=ipkon(i)
+                  do kk=1,mint2d
+                     xi=gauss3d2(1,kk)
+                     et=gauss3d2(2,kk)
+                     call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                     tlayer(kk)=0.d0
+                     do k=1,nlayer
+                        thickness=0.d0
+                        do j=1,nopes
+                           thickness=thickness+thicke(k,indexe+j)*
+     &                        shp2(4,j)
+                        enddo
+                        tlayer(kk)=tlayer(kk)+thickness
+                        xlayer(k,kk)=thickness
                      enddo
-                     tlayer(kk)=tlayer(kk)+thickness
-                     xlayer(k,kk)=thickness
                   enddo
-               enddo
-               iflag=3
+                  iflag=3
 !     
-               ilayer=0
-               do k=1,4
-                  dlayer(k)=0.d0
-               enddo
+                  ilayer=0
+                  do k=1,4
+                     dlayer(k)=0.d0
+                  enddo
+               elseif(lakonl(4:5).eq.'15') then
+!
+                  mint2d=3
+                  nopes=6
+!
+                  iflag=1
+                  indexe=ipkon(i)
+                  do kk=1,mint2d
+                     xi=gauss3d10(1,kk)
+                     et=gauss3d10(2,kk)
+                     call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                     tlayer(kk)=0.d0
+                     do k=1,nlayer
+                        thickness=0.d0
+                        do j=1,nopes
+                           thickness=thickness+thicke(k,indexe+j)*
+     &                        shp2(4,j)
+                        enddo
+                        tlayer(kk)=tlayer(kk)+thickness
+                        xlayer(k,kk)=thickness
+                     enddo
+                  enddo
+                  iflag=3
+!     
+                  ilayer=0
+                  do k=1,3
+                     dlayer(k)=0.d0
+                  enddo
+               endif
 !     
             endif
 !     
@@ -222,7 +299,11 @@ c     Bernhardi end
             elseif(lakonl(4:4).eq.'4') then
                mint3d=1
             elseif(lakonl(4:5).eq.'15') then
-               mint3d=9
+               if(lakonl(7:8).eq.'LC') then
+                  mint3d=6*nlayer
+               else
+                  mint3d=9
+               endif
             elseif(lakonl(4:4).eq.'6') then
                mint3d=2
             elseif(lakonl(1:1).eq.'E') then
@@ -244,6 +325,59 @@ c     Bernhardi end
                do j=1,3
                   xl(j,iactive)=xl(j,iactive)+xdesi(j,idesvar)
                enddo
+               if(lakonl(1:5).eq.'C3D20') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     node2=ifaceqexp(2,iactive)
+                     do j=1,3
+                        if(iactive.le.8) then
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                           xl(j,node2)=xl(j,node2)+xdesi(j,idesvar)
+                        else
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                        endif         
+                     enddo  
+                  endif     
+               elseif(lakonl(1:5).eq.'C3D15') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifacewexp(1,iactive)
+                     node2=ifacewexp(2,iactive)
+                     do j=1,3
+                        if(iactive.le.6) then
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                           xl(j,node2)=xl(j,node2)+xdesi(j,idesvar)
+                        else
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                        endif         
+                     enddo  
+                  endif     
+               elseif(lakonl(1:4).eq.'C3D8') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     do j=1,3
+                        xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)       
+                     enddo  
+                  endif     
+               elseif(lakonl(1:4).eq.'C3D6') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     do j=1,3
+                        xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)       
+                     enddo  
+                  endif     
+               endif
             endif
 !     
             do jj=1,mint3d
@@ -299,7 +433,7 @@ c     Bernhardi end
                      imat=ielmat(ilayer,i)
                      amat=matname(imat)
                      if(norien.gt.0) then
-                        iorien=ielorien(ilayer,i)
+                        iorien=max(0,ielorien(ilayer,i))
                      else
                         iorien=0
                      endif
@@ -327,10 +461,51 @@ c     Bernhardi end
                   ze=gauss3d4(3,jj)
                   weight=weight3d4(jj)
                elseif(lakonl(4:5).eq.'15') then
-                  xi=gauss3d8(1,jj)
-                  et=gauss3d8(2,jj)
-                  ze=gauss3d8(3,jj)
-                  weight=weight3d8(jj)
+                  if(lakonl(7:8).ne.'LC') then
+                     xi=gauss3d8(1,jj)
+                     et=gauss3d8(2,jj)
+                     ze=gauss3d8(3,jj)
+                     weight=weight3d8(jj)
+                  else
+                     kl=mod(jj,6)
+                     if(kl.eq.0) kl=6
+!     
+                     xi=gauss3d10(1,kl)
+                     et=gauss3d10(2,kl)
+                     ze=gauss3d10(3,kl)
+                     weight=weight3d10(kl)
+!     
+                     ki=mod(jj,3)
+                     if(ki.eq.0) ki=3
+!     
+                     if(kl.eq.1) then
+                        ilayer=ilayer+1
+                        if(ilayer.gt.1) then
+                           do k=1,3
+                              dlayer(k)=dlayer(k)+xlayer(ilayer-1,k)
+                           enddo
+                        endif
+                     endif
+                     ze=2.d0*(dlayer(ki)+(ze+1.d0)/
+     &                    2.d0*xlayer(ilayer,ki))/tlayer(ki)-1.d0
+                     weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!     
+!     material and orientation
+!     
+                     imat=ielmat(ilayer,i)
+                     amat=matname(imat)
+                     if(norien.gt.0) then
+                        iorien=max(0,ielorien(ilayer,i))
+                     else
+                        iorien=0
+                     endif
+!     
+                     if(nelcon(1,imat).lt.0) then
+                        ihyper=1
+                     else
+                        ihyper=0
+                     endif
+                  endif
                elseif(lakonl(4:4).eq.'6') then
                   xi=gauss3d7(1,jj)
                   et=gauss3d7(2,jj)
@@ -353,14 +528,14 @@ c     Bernhardi end
                   else
                      call shape20h(xi,et,ze,xl,xsj,shp,iflag)
                   endif
-               elseif(nope.eq.26) then
-                  call shape26h(xi,et,ze,xl,xsj,shp,iflag,konl)
+c               elseif(nope.eq.26) then
+c                  call shape26h(xi,et,ze,xl,xsj,shp,iflag,konl)
                elseif(nope.eq.8) then
                   call shape8h(xi,et,ze,xl,xsj,shp,iflag)
                elseif(nope.eq.10) then
                   call shape10tet(xi,et,ze,xl,xsj,shp,iflag)
-               elseif(nope.eq.14) then
-                  call shape14tet(xi,et,ze,xl,xsj,shp,iflag,konl)
+c               elseif(nope.eq.14) then
+c                  call shape14tet(xi,et,ze,xl,xsj,shp,iflag,konl)
                elseif(nope.eq.4) then
                   call shape4tet(xi,et,ze,xl,xsj,shp,iflag)
                elseif(nope.eq.15) then
@@ -373,6 +548,11 @@ c     Bernhardi end
 !     
                rho=rhcon(1,1,imat)
                xmassel=xmassel+weight*xsj*rho
+!
+!               write(5,'(i5,3x,i5,3x,i5,3x,e14.8,3x,e14.7,3x,e14.8,
+!     &                    3x,e14.8,3x,e14.8,3x,e20.14)') 
+!     &                    idesvar,i,jj,weight,xsj,xi,et,ze,
+!     &                    xmassel
 !     
 !     end of loop over all integration points
             enddo
