@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -23,7 +23,7 @@
      &  veold,dtime,time,ttime,plicon,nplicon,plkcon,nplkcon,
      &  xstateini,xstiff,xstate,npmat_,matname,mi,ielas,icmd,
      &  ncmat_,nstate_,stiini,vini,ener,enerini,istep,iinc,
-     &  springarea,reltime,calcul_qa,iener,ikin,ne0,thicke,
+     &  springarea,reltime,calcul_qa,nener,ikin,ne0,thicke,
      &  emeini,pslavsurf,pmastsurf,mortar,clearini,nea,neb,ielprop,
      &  prop,distmin,ndesi,nodedesi,nobject,g0,dgdx,
      &  iobject,sti,xener,istartdesi,ialdesi,xdesi,idesvar)
@@ -41,19 +41,20 @@
       integer kon(*),konl(26),nea,neb,mi(*),mint2d,nopes,
      &  nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),
      &  ielorien(mi(3),*),ntmat_,ipkon(*),ne0,iflag,null,
-     &  istep,iinc,mt,ne,mattyp,ithermal(2),iprestr,i,ii,j,k,m1,m2,jj,
-     &  i1,m3,kk,iener,indexe,nope,norien,iperturb(*),iout,
+     &  istep,iinc,mt,ne,mattyp,ithermal(*),iprestr,i,ii,j,k,m1,m2,jj,
+     &  i1,m3,kk,nener,indexe,nope,norien,iperturb(*),iout,
      &  icmd,ihyper,nmethod,kode,imat,mint3d,iorien,ielas,
      &  istiff,ncmat_,nstate_,ikin,ilayer,nlayer,ki,kl,ielprop(*),
      &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,calcul_qa,
-     &  nopered,mortar,jfaces,iloc,igauss,idesvar,ndesi,nodedesi(*),
-     &  nobject,iobject,iactive,node,istartdesi(*),ialdesi(*),ij
+     &  nopered,mortar,jfaces,igauss,idesvar,ndesi,nodedesi(*),
+     &  nobject,iobject,iactive,node,istartdesi(*),ialdesi(*),ij,
+     &  nlgeom_undo,node1,node2,ifaceqexp(2,20),ifacewexp(2,15) 
 !
       real*8 co(3,*),shp(4,26),stiini(6,mi(1),*),xener(*),
      &  stx(6,mi(1),*),xl(3,26),vl(0:mi(2),26),stre(6),prop(*),
      &  elcon(0:ncmat_,ntmat_,*),rhcon(0:1,ntmat_,*),xs2(3,7),
      &  alcon(0:6,ntmat_,*),vini(0:mi(2),*),thickness,
-     &  alzero(*),orab(7,*),elas(21),rho,qa(3),
+     &  alzero(*),orab(7,*),elas(21),rho,qa(4),
      &  fnl(3,10),beta(6),q(0:mi(2),26),xl2(3,8),
      &  vkl(0:3,3),t0(*),t1(*),prestr(6,mi(1),*),
      &  ckl(3,3),vold(0:mi(2),*),eloc(9),veold(0:mi(2),*),
@@ -69,14 +70,59 @@
      &  thicke(mi(3),*),emeini(6,mi(1),*),clearini(3,9,*),
      &  pslavsurf(3,*),pmastsurf(6,*),distmin,xenerel,g0(iobject),
      &  dgdx(ndesi,nobject),sti(6,mi(1),*),xdesi(3,*)
+!
+!
 !
       include "gauss.f"
 !
       iflag=3
       null=0
+      qa(3)=-1.d0
+      qa(4)=0.d0
 !
       mt=mi(2)+1
 !
+!     nodes in expansion direction of hex element 
+!
+      data ifaceqexp /5,17,
+     &                6,18,
+     &                7,19,
+     &                8,20,
+     &                1,17,
+     &                2,18,
+     &                3,19,
+     &                4,20,
+     &                13,0,
+     &                14,0,
+     &                15,0,
+     &                16,0,
+     &                9,0,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                0,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
+!
+!     nodes in expansion direction of wedge element 
+!
+      data ifacewexp /4,13,
+     &                5,14,
+     &                6,16,
+     &                1,13,
+     &                2,14,
+     &                3,15,
+     &                10,0,
+     &                11,0,
+     &                12,0,
+     &                7,0,
+     &                8,0,
+     &                9,0,
+     &                0,0,
+     &                0,0,
+     &                0,0/
+!
 !     -------------------------------------------------------------
 !     Initialisation of the loop for the variation of 
 !     the internal forces
@@ -108,7 +154,7 @@
                imat=ielmat(1,i)
                amat=matname(imat)
                if(norien.gt.0) then
-                  iorien=ielorien(1,i)
+                  iorien=max(0,ielorien(1,i))
                else
                   iorien=0
                endif
@@ -121,9 +167,7 @@
             else
 !     
 !     composite materials
-!     
-               mint2d=4
-               nopes=8
+!
 !     determining the number of layers
 !     
                nlayer=0
@@ -136,29 +180,64 @@
 !     determining the layer thickness and global thickness
 !     at the shell integration points
 !     
-               iflag=1
-               indexe=ipkon(i)
-               do kk=1,mint2d
-                  xi=gauss3d2(1,kk)
-                  et=gauss3d2(2,kk)
-                  call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
-                  tlayer(kk)=0.d0
-                  do k=1,nlayer
-                     thickness=0.d0
-                     do j=1,nopes
-                        thickness=thickness+thicke(k,indexe+j)*shp2(4,j)
+               if(lakonl(4:5).eq.'20') then
+!
+                  mint2d=4
+                  nopes=8
+   !
+                  iflag=1
+                  indexe=ipkon(i)
+                  do kk=1,mint2d
+                     xi=gauss3d2(1,kk)
+                     et=gauss3d2(2,kk)
+                     call shape8q(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                     tlayer(kk)=0.d0
+                     do k=1,nlayer
+                        thickness=0.d0
+                        do j=1,nopes
+                           thickness=thickness+thicke(k,indexe+j)*
+     &                        shp2(4,j)
+                        enddo
+                        tlayer(kk)=tlayer(kk)+thickness
+                        xlayer(k,kk)=thickness
                      enddo
-                     tlayer(kk)=tlayer(kk)+thickness
-                     xlayer(k,kk)=thickness
                   enddo
-               enddo
-               iflag=3
-!     
-               ilayer=0
-               do k=1,4
-                  dlayer(k)=0.d0
-               enddo
+                  iflag=3
+   !     
+                  ilayer=0
+                  do k=1,4
+                     dlayer(k)=0.d0
+                  enddo
+               elseif(lakonl(4:5).eq.'15') then
+!
+                  mint2d=3
+                  nopes=6
+!
+                  iflag=1
+                  indexe=ipkon(i)
+                  do kk=1,mint2d
+                     xi=gauss3d10(1,kk)
+                     et=gauss3d10(2,kk)
+                     call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+                     tlayer(kk)=0.d0
+                     do k=1,nlayer
+                        thickness=0.d0
+                        do j=1,nopes
+                           thickness=thickness+thicke(k,indexe+j)*
+     &                        shp2(4,j)
+                        enddo
+                        tlayer(kk)=tlayer(kk)+thickness
+                        xlayer(k,kk)=thickness
+                     enddo
+                  enddo
+                  iflag=3
+   !     
+                  ilayer=0
+                  do k=1,3
+                     dlayer(k)=0.d0
+                  enddo
 !     
+               endif
             endif
 !     
             indexe=ipkon(i)
@@ -248,7 +327,11 @@ c     Bernhardi end
             elseif(lakonl(4:4).eq.'4') then
                mint3d=1
             elseif(lakonl(4:5).eq.'15') then
-               mint3d=9
+               if(lakonl(7:8).eq.'LC') then
+                  mint3d=6*nlayer
+               else
+                  mint3d=9
+               endif
             elseif(lakonl(4:4).eq.'6') then
                mint3d=2
             elseif(lakonl(1:1).eq.'E') then
@@ -271,6 +354,59 @@ c     Bernhardi end
                   xl(j,iactive)=
      &                 xl(j,iactive)+xdesi(j,idesvar)
                enddo
+               if(lakonl(1:5).eq.'C3D20') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     node2=ifaceqexp(2,iactive)
+                     do j=1,3
+                        if(iactive.le.8) then
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                           xl(j,node2)=xl(j,node2)+xdesi(j,idesvar)
+                        else
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                        endif         
+                     enddo  
+                  endif     
+               elseif(lakonl(1:5).eq.'C3D15') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifacewexp(1,iactive)
+                     node2=ifacewexp(2,iactive)
+                     do j=1,3
+                        if(iactive.le.6) then
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                           xl(j,node2)=xl(j,node2)+xdesi(j,idesvar)
+                        else
+                           xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)
+                        endif         
+                     enddo  
+                  endif     
+               elseif(lakonl(1:4).eq.'C3D8') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     do j=1,3
+                        xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)       
+                     enddo  
+                  endif     
+               elseif(lakonl(1:4).eq.'C3D6') then
+                  if((lakonl(7:7).eq.'A').or.
+     &               (lakonl(7:7).eq.'L').or.
+     &               (lakonl(7:7).eq.'S').or.
+     &               (lakonl(7:7).eq.'E')) then
+                     node1=ifaceqexp(1,iactive)
+                     do j=1,3
+                        xl(j,node1)=xl(j,node1)+xdesi(j,idesvar)       
+                     enddo  
+                  endif     
+               endif
             endif
 !     
 !     calculating the forces for the contact elements
@@ -295,27 +431,27 @@ c     Bernhardi end
 !     spring elements (including contact springs)
 !     
                if(lakonl(2:2).eq.'S') then
-                  if((lakonl(7:7).eq.'A').or.((mortar.eq.0).and.
+                  if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'1').or.
+     &                 (lakonl(7:7).eq.'2').or.((mortar.eq.0).and.
      &                 ((nmethod.ne.1).or.(iperturb(1).ge.2).or.
      &                  (iout.ne.-1))))then
                      call springforc_n2f(xl,konl,vl,imat,elcon,nelcon,
      &                    elas,fnl,ncmat_,ntmat_,nope,lakonl,t1l,kode,
      &                    elconloc,plicon,nplicon,npmat_,ener(1,i),
-     &                    iener,stx(1,1,i),mi,springarea(1,konl(nope+1))
+     &                    nener,stx(1,1,i),mi,springarea(1,konl(nope+1))
      &                    ,nmethod,ne0,nstate_,xstateini,xstate,reltime,
      &                    ielas,ener(1,i+ne))
                   elseif((mortar.eq.1).and.
      &                    ((nmethod.ne.1).or.(iperturb(1).ge.2).or.
      &                     (iout.ne.-1)))then
-                     iloc=kon(indexe+nope+1)
                      jfaces=kon(indexe+nope+2)
                      igauss=kon(indexe+nope+1)
                      call springforc_f2f(xl,vl,imat,elcon,nelcon,elas,
      &                    fnl,ncmat_,ntmat_,nope,lakonl,t1l,kode,
      &                    elconloc,plicon,nplicon,npmat_,ener(1,i),
-     &                    iener,stx(1,1,i),mi,springarea(1,iloc),
+     &                    nener,stx(1,1,i),mi,springarea(1,igauss),
      &                    nmethod,ne0,nstate_,xstateini,xstate,reltime,
-     &                    ielas,iloc,jfaces,igauss,pslavsurf,pmastsurf,
+     &                    ielas,jfaces,igauss,pslavsurf,pmastsurf,
      &                    clearini,ener(1,i+ne))
                   endif
 !     
@@ -382,7 +518,7 @@ c               enddo
                      imat=ielmat(ilayer,i)
                      amat=matname(imat)
                      if(norien.gt.0) then
-                        iorien=ielorien(ilayer,i)
+                        iorien=max(0,ielorien(ilayer,i))
                      else
                         iorien=0
                      endif
@@ -410,10 +546,51 @@ c               enddo
                   ze=gauss3d4(3,jj)
                   weight=weight3d4(jj)
                elseif(lakonl(4:5).eq.'15') then
-                  xi=gauss3d8(1,jj)
-                  et=gauss3d8(2,jj)
-                  ze=gauss3d8(3,jj)
-                  weight=weight3d8(jj)
+                  if(lakonl(7:8).ne.'LC') then
+                     xi=gauss3d8(1,jj)
+                     et=gauss3d8(2,jj)
+                     ze=gauss3d8(3,jj)
+                     weight=weight3d8(jj)
+                  else
+                     kl=mod(jj,6)
+                     if(kl.eq.0) kl=6
+!     
+                     xi=gauss3d10(1,kl)
+                     et=gauss3d10(2,kl)
+                     ze=gauss3d10(3,kl)
+                     weight=weight3d10(kl)
+!     
+                     ki=mod(jj,3)
+                     if(ki.eq.0) ki=3
+!     
+                     if(kl.eq.1) then
+                        ilayer=ilayer+1
+                        if(ilayer.gt.1) then
+                           do k=1,3
+                              dlayer(k)=dlayer(k)+xlayer(ilayer-1,k)
+                           enddo
+                        endif
+                     endif
+                     ze=2.d0*(dlayer(ki)+(ze+1.d0)/
+     &                    2.d0*xlayer(ilayer,ki))/tlayer(ki)-1.d0
+                     weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!     
+!     material and orientation
+!     
+                     imat=ielmat(ilayer,i)
+                     amat=matname(imat)
+                     if(norien.gt.0) then
+                        iorien=max(0,ielorien(ilayer,i))
+                     else
+                        iorien=0
+                     endif
+!     
+                     if(nelcon(1,imat).lt.0) then
+                        ihyper=1
+                     else
+                        ihyper=0
+                     endif
+                  endif
                elseif(lakonl(4:4).eq.'6') then
                   xi=gauss3d7(1,jj)
                   et=gauss3d7(2,jj)
@@ -436,14 +613,14 @@ c     Bernhardi end
                   else
                      call shape20h(xi,et,ze,xl,xsj,shp,iflag)
                   endif
-               elseif(nope.eq.26) then
-                  call shape26h(xi,et,ze,xl,xsj,shp,iflag,konl)
+c               elseif(nope.eq.26) then
+c                  call shape26h(xi,et,ze,xl,xsj,shp,iflag,konl)
                elseif(nope.eq.8) then
                   call shape8h(xi,et,ze,xl,xsj,shp,iflag)
                elseif(nope.eq.10) then
                   call shape10tet(xi,et,ze,xl,xsj,shp,iflag)
-               elseif(nope.eq.14) then
-                  call shape14tet(xi,et,ze,xl,xsj,shp,iflag,konl)
+c               elseif(nope.eq.14) then
+c                  call shape14tet(xi,et,ze,xl,xsj,shp,iflag,konl)
                elseif(nope.eq.4) then
                   call shape4tet(xi,et,ze,xl,xsj,shp,iflag)
                elseif(nope.eq.15) then
@@ -649,9 +826,9 @@ c     Bernhardi end
 !     calculating the deformation gradient of the old
 !     fields
 !     
-                  xikl(1,1)=vikl(1,1)+1
-                  xikl(2,2)=vikl(2,2)+1.
-                  xikl(3,3)=vikl(3,3)+1.
+                  xikl(1,1)=vikl(1,1)+1.d0
+                  xikl(2,2)=vikl(2,2)+1.d0
+                  xikl(3,3)=vikl(3,3)+1.d0
                   xikl(1,2)=vikl(1,2)
                   xikl(1,3)=vikl(1,3)
                   xikl(2,3)=vikl(2,3)
@@ -705,7 +882,11 @@ c     Bernhardi end
                      elseif((lakonl(4:6).eq.'20 ').or.
      &                       (lakonl(4:6).eq.'26 ')) then
                         nopered=20
-                        call lintemp(t0,t1,konl,nopered,jj,t0l,t1l)
+                        call lintemp(t0,konl,nopered,jj,t0l)
+                        call lintemp(t1,konl,nopered,jj,t1l)
+                     elseif(lakonl(4:6).eq.'10T') then
+                        call linscal10(t0,konl,t0l,null,shp)
+                        call linscal10(t1,konl,t1l,null,shp)
                      else
                         do i1=1,nope
                            t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -722,8 +903,11 @@ c     Bernhardi end
                      elseif((lakonl(4:6).eq.'20 ').or.
      &                       (lakonl(4:6).eq.'26 ')) then
                         nopered=20
-                        call lintemp_th(t0,vold,konl,nopered,jj,t0l,t1l,
-     &                         mi)
+                        call lintemp_th0(t0,konl,nopered,jj,t0l,mi)
+                        call lintemp_th1(vold,konl,nopered,jj,t1l,mi)
+                     elseif(lakonl(4:6).eq.'10T') then
+                        call linscal10(t0,konl,t0l,null,shp)
+                        call linscal10(vold,konl,t1l,mi(2),shp)
                      else
                         do i1=1,nope
                            t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -757,15 +941,16 @@ c     Bernhardi end
      &              nalcon,imat,amat,iorien,pgauss,orab,ntmat_,
      &              elas,rho,i,ithermal,alzero,mattyp,t0l,t1l,ihyper,
      &              istiff,elconloc,eth,kode,plicon,nplicon,
-     &              plkcon,nplkcon,npmat_,plconloc,mi(1),dtime,i,jj,
+     &              plkcon,nplkcon,npmat_,plconloc,mi(1),dtime,jj,
      &              xstiff,ncmat_)
 !     
 !     determining the mechanical strain
 !     
                if(ithermal(1).ne.0) then
-                  do m1=1,6
-                     emec(m1)=eloc(m1)-eth(m1)
-                  enddo
+                  call calcmechstrain(vkl,vokl,emec,eth,iperturb)
+c                  do m1=1,6
+c                     emec(m1)=eloc(m1)-eth(m1)
+c                  enddo
                else
                   do m1=1,6
                      emec(m1)=eloc(m1)
@@ -787,12 +972,13 @@ c     Bernhardi end
 !     
 !     calculating the local stiffness and stress
 !     
+               nlgeom_undo=0
                call mechmodel(elconloc,elas,emec,kode,emec0,ithermal,
      &              icmd,beta,stre,xkl,ckl,vj,xikl,vij,
      &              plconloc,xstate,xstateini,ielas,
      &              amat,t1l,dtime,time,ttime,i,jj,nstate_,mi(1),
      &              iorien,pgauss,orab,eloc,mattyp,qa(3),istep,iinc,
-     &              ipkon,nmethod,iperturb)
+     &              ipkon,nmethod,iperturb,qa(4),nlgeom_undo)
 !     
                if(((nmethod.ne.4).or.(iperturb(1).ne.0)).and.
      &              (nmethod.ne.5)) then
@@ -801,7 +987,7 @@ c     Bernhardi end
                   enddo
                endif
 !     
-               if(iperturb(1).eq.-1) then
+               if((iperturb(1).eq.-1).and.(nlgeom_undo.eq.0)) then
 !     
 !     if the forced displacements were changed at
 !     the start of a nonlinear step, the nodal
@@ -867,18 +1053,35 @@ c     Bernhardi end
 !     calculation of the shape energy for the objective 
 !     function: SHAPE ENERGY
 !     
-               ener(jj,i)=enerini(jj,i)+
-     &              ((eloc(1)-eth(1)-emeini(1,jj,i))*
+               if(idesvar.eq.0) then
+                  xenerel=xenerel+weight*xsj*enerini(jj,i)
+               else
+!
+!                 only change of energy is stored
+!
+c                  ener(jj,i)=
+c     &              ((eloc(1)-eth(1)-emeini(1,jj,i))*
+c     &              (stre(1)+stiini(1,jj,i))+
+c     &              (eloc(2)-eth(2)-emeini(2,jj,i))*
+c     &              (stre(2)+stiini(2,jj,i))+
+c     &              (eloc(3)-eth(3)-emeini(3,jj,i))*
+c     &              (stre(3)+stiini(3,jj,i)))/2.d0+
+c     &         (eloc(4)-eth(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
+c     &         (eloc(5)-eth(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
+c     &         (eloc(6)-eth(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
+                  ener(jj,i)=
+     &              ((emec(1)-emeini(1,jj,i))*
      &              (stre(1)+stiini(1,jj,i))+
-     &              (eloc(2)-eth(2)-emeini(2,jj,i))*
+     &              (emec(2)-emeini(2,jj,i))*
      &              (stre(2)+stiini(2,jj,i))+
-     &              (eloc(3)-eth(3)-emeini(3,jj,i))*
+     &              (emec(3)-emeini(3,jj,i))*
      &              (stre(3)+stiini(3,jj,i)))/2.d0+
-     &         (eloc(4)-eth(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
-     &         (eloc(5)-eth(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
-     &         (eloc(6)-eth(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
-               
-               xenerel=xenerel+weight*xsj*ener(jj,i)
+     &         (emec(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
+     &         (emec(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
+     &         (emec(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
+!               
+                  xenerel=xenerel+weight*xsj*ener(jj,i)
+               endif
 !     
 !     end of loop over all integration points
             enddo
@@ -890,7 +1093,7 @@ c     Bernhardi end
                g0(iobject)=g0(iobject)+xenerel
             else
                dgdx(idesvar,iobject)=dgdx(idesvar,iobject)
-     &              +(xenerel-xener(i))/distmin
+     &              +xenerel/distmin
             endif
 !     
 !     end of loop over all elements in thread
