@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,13 +16,11 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine openfile(jobname,output)
+      subroutine openfile(jobname)
 !
       implicit none
 !
-      logical exi
-      character*3 output
-      character*132 jobname,fnin,fndat,fnfrd,fnsta,fncvg,fncel
+      character*132 jobname,fndat,fnfrd,fnsta,fncvg,fncel
       integer i
 !
 !     opening the input and output file
@@ -37,16 +35,6 @@
          write(*,*) '       exceeds 128 characters'
          call exit(201)
       endif
-!
-c      fnin=jobname(1:i)//'.inp'
-c      inquire(file=fnin(1:i+4),exist=exi)
-c      if(exi) then
-c         open(1,file=fnin(1:i+4),status='old',err=1)
-c      else
-c         write(*,*) '*ERROR in openfile: input file ',fnin
-c         write(*,*) 'does not exist'
-c         call exit(201)
-c      endif
 !
       fndat=jobname(1:i)//'.dat'
       open(5,file=fndat(1:i+4),status='unknown',err=51)
@@ -59,16 +47,10 @@ c      endif
       open(7,file=fnfrd(1:i+4),status='unknown',err=71)
       close(7,status='delete',err=72)
 !
-!     delete the f.frd file (it is reopened in openfilefluid.f)
+!     delete the .fcv file (it is reopened in compfluid.c)
 !
-      fnfrd=jobname(1:i)//'f.frd'
-      open(13,file=fnfrd(1:i+5),status='unknown',err=71)
-      close(13,status='delete',err=73)
-!
-!     delete the f.cvg file (it is reopened in compfluid.c)
-!
-      fnfrd=jobname(1:i)//'f.cvg'
-      open(12,file=fnfrd(1:i+5),status='unknown',err=71)
+      fnfrd=jobname(1:i)//'.fcv'
+      open(12,file=fnfrd(1:i+4),status='unknown',err=71)
       close(12,status='delete',err=73)
 !
 !     .sta-file
@@ -103,14 +85,12 @@ c      endif
 !
 !     contact elements
 !
-      fncel='contactelements.inp'
-      open(27,file=fncel(1:19),status='unknown',err=93)
+      fncel=jobname(1:i)//'.cel'
+      open(27,file=fncel(1:i+4),status='unknown',err=93)
       close(27,status='delete',err=94)
 !
       return
 !
- 1    write(*,*) '*ERROR in openfile: could not open file ',fnin(1:i+4)
-      call exit(201)
  51   write(*,*) '*ERROR in openfile: could not open file ',fndat(1:i+4)
       call exit(201)
  52   write(*,*) '*ERROR in openfile: could not delete file ',
@@ -132,8 +112,7 @@ c      endif
       call exit(201)
  92   write(*,*) '*ERROR in openfile: could not delete file ',
      &  fncvg(1:i+4)
- 93   write(*,*) '*ERROR in openfile: could not open file ',fncel(1:19),
-     &    ' which is a reserved file name for ccx contact output'
+ 93   write(*,*) '*ERROR in openfile: could not open file ',fncel(1:19)
       call exit(201)
  94   write(*,*) '*ERROR in openfile: could not delete file ',
      &  fncel(1:19)
