@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
 !
       implicit none
 !
-      character*132 jobname,fnfrd,fncvg
+      character*132 jobname,fnfrd
       integer i
 !
 !     opening frd file
@@ -37,7 +37,7 @@
          call exit(201)
       endif
 !
-      fnfrd=jobname(1:i)//'f.frd'
+      fnfrd=jobname(1:i)//'.frd'
       open(13,file=fnfrd(1:i+5),status='unknown',position='append')
 !
       return
