@@ -1,6 +1,6 @@
-
+!
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,7 +20,9 @@
      &  prlab,prset,v,t1,fn,ipkon,lakon,stx,eei,xstate,ener,
      &  mi,nstate_,ithermal,co,kon,qfx,ttime,trab,inotr,ntrans,
      &  orab,ielorien,norien,nk,ne,inum,filab,vold,ikin,ielmat,thicke,
-     &  eme,islavsurf,mortar,time,ielprop,prop,veold)
+     &  eme,islavsurf,mortar,time,ielprop,prop,veold,orname,
+     &  nelemload,nload,sideload,xload,rhcon,nrhcon,ntmat_,ipobody,
+     &  ibody,xbody,nbody)
 !
 !     stores results in the .dat file
 !
@@ -31,21 +33,25 @@
       character*1 cflag
       character*6 prlab(*)
       character*8 lakon(*)
-      character*80 noset,elset
+      character*20 sideload(*)
+      character*80 noset,elset,orname(*)
       character*81 set(*),prset(*)
       character*87 filab(*)
 !
       integer nset,istartset(*),iendset(*),ialset(*),nprint,ipkon(*),
-     &  mi(*),nstate_,ii,jj,iset,l,limit,node,ipos,ithermal,ielem,
+     &  mi(*),nstate_,ii,jj,iset,l,limit,node,ipos,ithermal(*),ielem,
      &  nelem,kon(*),inotr(2,*),ntrans,ielorien(mi(3),*),norien,nk,ne,
      &  inum(*),nfield,ikin,nodes,ne0,nope,mt,ielmat(mi(3),*),iface,
-     &  jfaces,mortar,islavsurf(2,*),ielprop(*)
+     &  jfaces,mortar,islavsurf(2,*),ielprop(*),nload,i,ntmat_,
+     &  nelemload(2,*),nrhcon(*),ipobody(2,*),ibody(3,*),nbody
 !
-      real*8 v(0:mi(2),*),t1(*),fn(0:mi(2),*),stx(6,mi(1),*),
+      real*8 v(0:mi(2),*),t1(*),fn(0:mi(2),*),stx(6,mi(1),*),bhetot,
      &  eei(6,mi(1),*),xstate(nstate_,mi(1),*),ener(mi(1),*),energytot,
      &  volumetot,co(3,*),qfx(3,mi(1),*),rftot(0:3),ttime,time,
      &  trab(7,*),orab(7,*),vold(0:mi(2),*),enerkintot,thicke(mi(3),*),
-     &  eme(6,mi(1),*),prop(*),veold(0:mi(2),*)
+     &  eme(6,mi(1),*),prop(*),veold(0:mi(2),*),xload(2,*),xmasstot,
+     &  xinertot(6),cg(3),rhcon(0:1,ntmat_,*),xbody(7,*)
+!
 !
       mt=mi(2)+1
 !
@@ -53,25 +59,25 @@
 !
       do ii=1,nprint
          if((prlab(ii)(1:4).eq.'U   ').or.
-     &      ((prlab(ii)(1:4).eq.'NT  ').and.(ithermal.gt.1))) then
+     &      ((prlab(ii)(1:4).eq.'NT  ').and.(ithermal(1).gt.1))) then
             if(filab(1)(5:5).ne.' ') then
                nfield=mt
                cflag=' '
                force=.false.
                call map3dto1d2d(v,ipkon,inum,kon,lakon,nfield,nk,
-     &              ne,cflag,co,vold,force,mi)
+     &              ne,cflag,co,vold,force,mi,ielprop,prop)
             endif
             exit
           endif
       enddo
       do ii=1,nprint
-         if((prlab(ii)(1:4).eq.'NT  ').and.(ithermal.le.1)) then
+         if((prlab(ii)(1:4).eq.'NT  ').and.(ithermal(1).le.1)) then
             if(filab(2)(5:5).ne.' ') then
                nfield=1
                cflag=' '
                force=.false.
                call map3dto1d2d(t1,ipkon,inum,kon,lakon,nfield,nk,
-     &              ne,cflag,co,vold,force,mi)
+     &              ne,cflag,co,vold,force,mi,ielprop,prop)
             endif
             exit
           endif
@@ -83,7 +89,7 @@
                cflag=' '
                force=.true.
                call map3dto1d2d(fn,ipkon,inum,kon,lakon,nfield,nk,
-     &              ne,cflag,co,vold,force,mi)
+     &              ne,cflag,co,vold,force,mi,ielprop,prop)
             endif
             exit
           endif
@@ -108,8 +114,14 @@
 !
             if(prlab(ii)(1:4).eq.'U   ') then
                write(5,*)
-               write(5,100) noset(1:ipos-2),ttime+time
+               if(mi(2).eq.3) then
+                  write(5,100) noset(1:ipos-2),ttime+time
+               else
+                  write(5,133) noset(1:ipos-2),ttime+time
+               endif
  100           format(' displacements (vx,vy,vz) for set ',A,
+     &             ' and time ',e14.7)
+ 133           format(' displacements (v(i),i=1..ndof) for set ',A,
      &             ' and time ',e14.7)
                write(5,*)
             elseif((prlab(ii)(1:4).eq.'NT  ').or.
@@ -211,6 +223,7 @@
      &          (prlab(ii)(1:4).eq.'PEEQ').or.
      &          (prlab(ii)(1:4).eq.'ENER').or.
      &          (prlab(ii)(1:4).eq.'SDV ').or.
+     &          (prlab(ii)(1:4).eq.'COOR').or.
      &          (prlab(ii)(1:4).eq.'HFL ')) then
 !
             ipos=index(prset(ii),' ')
@@ -233,7 +246,7 @@
                   write(5,*)
                   write(5,107) elset(1:ipos-2),ttime+time
  107              format(' strains (elem, integ.pnt.,exx,eyy,ezz,exy,exz
-     &,eyz) forset ',A,' and time ',e14.7)
+     &,eyz) for set ',A,' and time ',e14.7)
                   write(5,*)
                elseif(prlab(ii)(1:4).eq.'PEEQ') then
                   write(5,*)
@@ -244,8 +257,8 @@
                elseif(prlab(ii)(1:4).eq.'ENER') then
                   write(5,*)
                   write(5,109) elset(1:ipos-2),ttime+time
- 109              format(' internal energy density (elem, integ.pnt.,energy) for 
-     &set ',A,' and time ',e14.7)
+ 109              format(' internal energy density (elem, integ.pnt.,ene
+     &rgy) for set ',A,' and time ',e14.7)
                   write(5,*)
                elseif(prlab(ii)(1:4).eq.'SDV ') then
                   write(5,*)
@@ -264,7 +277,13 @@
                   write(5,*)
                   write(5,130) elset(1:ipos-2),ttime+time
  130              format(' mechanical strains (elem, integ.pnt.,exx,eyy,
-     &ezz,exy,exz,eyz) forset ',A,' and time ',e14.7)
+     &ezz,exy,exz,eyz) for set ',A,' and time ',e14.7)
+                  write(5,*)
+               elseif(prlab(ii)(1:4).eq.'COOR') then
+                  write(5,*)
+                  write(5,139) elset(1:ipos-2),ttime+time
+ 139              format(' global coordinates (elem, integ.pnt.,x,y,z) f
+     &or set ',A,' and time ',e14.7)
                   write(5,*)
                endif
 !
@@ -280,20 +299,21 @@
                      call printoutint(prlab,ipkon,lakon,stx,eei,xstate,
      &                    ener,mi(1),nstate_,ii,nelem,qfx,
      &                    orab,ielorien,norien,co,kon,ielmat,thicke,eme,
-     &                    ielprop,prop,nelem,ithermal)
+     &                    ielprop,prop,nelem,ithermal,orname)
                   elseif(ialset(jj+1).gt.0) then
                      nelem=ialset(jj)
                      call printoutint(prlab,ipkon,lakon,stx,eei,xstate,
      &                    ener,mi(1),nstate_,ii,nelem,qfx,orab,
      &                    ielorien,norien,co,kon,ielmat,thicke,eme,
-     &                    ielprop,prop,nelem,ithermal)
+     &                    ielprop,prop,nelem,ithermal,orname)
                   else
                      do nelem=ialset(jj-1)-ialset(jj+1),ialset(jj),
      &                    -ialset(jj+1)
                         call printoutint(prlab,ipkon,lakon,stx,eei,
      &                       xstate,ener,mi(1),nstate_,ii,nelem,
      &                       qfx,orab,ielorien,norien,co,kon,ielmat,
-     &                       thicke,eme,ielprop,prop,nelem,ithermal)
+     &                       thicke,eme,ielprop,prop,nelem,ithermal,
+     &                       orname)
                      enddo
                   endif
                enddo
@@ -305,6 +325,9 @@
          elseif((prlab(ii)(1:4).eq.'ELSE').or.
      &           (prlab(ii)(1:4).eq.'ELKE').or.
      &           (prlab(ii)(1:4).eq.'EVOL').or.
+     &           (prlab(ii)(1:4).eq.'EMAS').or.
+     &           (prlab(ii)(1:4).eq.'EBHE').or.
+     &           (prlab(ii)(1:4).eq.'CENT').or.
      &           (prlab(ii)(1:4).eq.'CSTR').or.
      &           (prlab(ii)(1:4).eq.'CDIS').or.
      &           (prlab(ii)(1:4).eq.'CNUM').or.
@@ -335,6 +358,21 @@
                write(5,*)
                write(5,114) elset(1:ipos-2),ttime+time
  114           format(' volume (element, volume) for set ',A,
+     &                ' and time ',e14.7)
+               write(5,*)
+            elseif((prlab(ii)(1:5).eq.'EMAS ').or.
+     &             (prlab(ii)(1:5).eq.'EMAST')) then
+               write(5,*)
+               write(5,136) elset(1:ipos-2),ttime+time
+ 136           format(' mass (element, mass) and mass moment of itertia 
+     &(xx,yy,zz,xy,xz,yz) for set ',A,
+     &                ' and time ',e14.7)
+               write(5,*)
+            elseif((prlab(ii)(1:5).eq.'EBHE ').or.
+     &             (prlab(ii)(1:5).eq.'EBHET')) then
+               write(5,*)
+               write(5,131) elset(1:ipos-2),ttime+time
+ 131           format(' body heating (element, volume) for set ',A,
      &                ' and time ',e14.7)
                write(5,*)
             elseif((prlab(ii)(1:5).eq.'CSTR ').or.
@@ -381,15 +419,29 @@
      &              ' contact print energy (slave element+face,energy)'  
      &              'for all contact elements and time',e14.7)
                endif
+               write(5,*)
+            elseif(prlab(ii)(1:4).eq.'CENT') then
+               write(5,*)
+               write(5,140)  elset(1:ipos-2),ttime+time
+ 140           format(' centrifugal force(element,omega square) for '  
+     &              'set ',A,' and time ',e14.7)
+               write(5,*)
             endif
 !     
 !     printing the data
 !     
-            
             volumetot=0.d0
+            bhetot=0.d0
             energytot=0.d0
             enerkintot=0.d0
-            
+            xmasstot=0.d0
+            do jj=1,6
+               xinertot(jj)=0.d0
+            enddo
+            do jj=1,3
+               cg(jj)=0.d0
+            enddo
+!            
             if ((prlab(ii)(1:4).eq.'CSTR').or.
      &           (prlab(ii)(1:4).eq.'CDIS').or.
      &           (prlab(ii)(1:4).eq.'CNUM').or.
@@ -413,8 +465,11 @@
                         nodes=kon(ipkon(nelem)+nope)
                         call printoutelem(prlab,ipkon,lakon,kon,co,
      &                       ener,mi(1),ii,nelem,energytot,volumetot,
-     &                       enerkintot,ikin,ne,stx,nodes,thicke,ielmat,
-     &                       ielem,iface,mortar,ielprop,prop)
+     &                       enerkintot,ne,stx,nodes,thicke,ielmat,
+     &                       ielem,iface,mortar,ielprop,prop,
+     &                       sideload,nload,nelemload,xload,bhetot,
+     &                       xmasstot,xinertot,cg,ithermal,rhcon,nrhcon,
+     &                       ntmat_,t1,vold,ipobody,ibody,xbody,nbody)
                      enddo
                   elseif(mortar.eq.1) then
                      do nelem=ne0,ne
@@ -424,8 +479,11 @@
                         iface=jfaces-10*ielem
                         call printoutelem(prlab,ipkon,lakon,kon,co,
      &                       ener,mi(1),ii,nelem,energytot,volumetot,
-     &                       enerkintot,ikin,ne,stx,nodes,thicke,ielmat,
-     &                       ielem,iface,mortar,ielprop,prop)
+     &                       enerkintot,ne,stx,nodes,thicke,ielmat,
+     &                       ielem,iface,mortar,ielprop,prop,
+     &                       sideload,nload,nelemload,xload,bhetot,
+     &                       xmasstot,xinertot,cg,ithermal,rhcon,nrhcon,
+     &                       ntmat_,t1,vold,ipobody,ibody,xbody,nbody)
                      enddo
                   endif
                endif
@@ -439,21 +497,30 @@
                      nelem=ialset(jj)
                      call printoutelem(prlab,ipkon,lakon,kon,co,
      &                    ener,mi(1),ii,nelem,energytot,volumetot,
-     &                    enerkintot,ikin,ne,stx,nodes,thicke,ielmat,
-     &                    ielem,iface,mortar,ielprop,prop)
+     &                    enerkintot,ne,stx,nodes,thicke,ielmat,
+     &                    ielem,iface,mortar,ielprop,prop,
+     &                    sideload,nload,nelemload,xload,bhetot,
+     &                    xmasstot,xinertot,cg,ithermal,rhcon,nrhcon,
+     &                    ntmat_,t1,vold,ipobody,ibody,xbody,nbody)
                   elseif(ialset(jj+1).gt.0) then
                      nelem=ialset(jj)
                      call printoutelem(prlab,ipkon,lakon,kon,co,
      &                    ener,mi(1),ii,nelem,energytot,volumetot,
-     &                    enerkintot,ikin,ne,stx,nodes,thicke,ielmat,
-     &                    ielem,iface,mortar,ielprop,prop)
+     &                    enerkintot,ne,stx,nodes,thicke,ielmat,
+     &                    ielem,iface,mortar,ielprop,prop,
+     &                    sideload,nload,nelemload,xload,bhetot,
+     &                    xmasstot,xinertot,cg,ithermal,rhcon,nrhcon,
+     &                    ntmat_,t1,vold,ipobody,ibody,xbody,nbody)
                   else
                      do nelem=ialset(jj-1)-ialset(jj+1),ialset(jj),
      &                    -ialset(jj+1)
                         call printoutelem(prlab,ipkon,lakon,kon,co,
      &                       ener,mi(1),ii,nelem,energytot,volumetot,
-     &                    enerkintot,ikin,ne,stx,nodes,thicke,ielmat,
-     &                    ielem,iface,mortar,ielprop,prop)
+     &                       enerkintot,ne,stx,nodes,thicke,ielmat,
+     &                       ielem,iface,mortar,ielprop,prop,
+     &                       sideload,nload,nelemload,xload,bhetot,
+     &                       xmasstot,xinertot,cg,ithermal,rhcon,nrhcon,
+     &                       ntmat_,t1,vold,ipobody,ibody,xbody,nbody)
                      enddo
                   endif
                enddo
@@ -484,6 +551,47 @@
  121           format(' total volume for set ',A,' and time ',e14.7)
                write(5,*)
                write(5,'(6x,1p,1x,e13.6)') volumetot
+            elseif((prlab(ii)(1:5).eq.'EMASO').or.
+     &              (prlab(ii)(1:5).eq.'EMAST')) then
+               write(5,*)
+               write(5,137) elset(1:ipos-2),ttime+time
+ 137           format(' total mass for set ',A,' and time ',e14.7)
+               write(5,*)
+               write(5,'(6x,1p,1x,e13.6)') xmasstot
+               write(5,*)
+               write(5,134) elset(1:ipos-2),ttime+time
+ 134           format(
+     &     ' total mass moment of inertia (xx,yy,zz,xy,xz,yz) for set ',
+     &            A,' and time ',e14.7)
+               write(5,*)
+               write(5,'(6x,1p,6(1x,e13.6))') (xinertot(i),i=1,6)
+               write(5,*)
+               write(5,135) elset(1:ipos-2),ttime+time
+ 135           format(' center of gravity for set ',A,
+     &                ' and time ',e14.7)
+               write(5,*)
+               write(5,'(6x,1p,3(1x,e13.6))') (cg(i)/xmasstot,i=1,3)
+               write(5,*)
+               write(5,138) elset(1:ipos-2),ttime+time
+ 138           format(' total mass moment of inertia about the center of
+     & gravity (xx,yy,zz,xy,xz,yz) for set ',
+     &            A,' and time ',e14.7)
+               write(5,*)
+               write(5,'(6x,1p,6(1x,e13.6))')
+     &              xinertot(1)-cg(1)*cg(1)/xmasstot,
+     &              xinertot(2)-cg(2)*cg(2)/xmasstot,
+     &              xinertot(3)-cg(3)*cg(3)/xmasstot,
+     &              xinertot(4)-cg(1)*cg(2)/xmasstot,
+     &              xinertot(5)-cg(1)*cg(3)/xmasstot,
+     &              xinertot(6)-cg(2)*cg(3)/xmasstot
+            elseif((prlab(ii)(1:5).eq.'EBHEO').or.
+     &              (prlab(ii)(1:5).eq.'EBHET')) then
+               write(5,*)
+               write(5,132) elset(1:ipos-2),ttime+time
+ 132           format(' total body heating for set ',A,' and time ',
+     &                e14.7)
+               write(5,*)
+               write(5,'(6x,1p,1x,e13.6)') bhetot
             elseif((prlab(ii)(1:5).eq.'CELSO').or.
      &              (prlab(ii)(1:5).eq.'CELST')) then
                write(5,*)
