@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -23,7 +23,8 @@
 #include "CalculiX.h"
 
 void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
-   ITG *ipoinp, ITG **inpp, ITG **ipoinpcp, ITG *ithermal){
+	       ITG *ipoinp, ITG **inpp, ITG **ipoinpcp, ITG *ithermal,
+               ITG *nuel,ITG *inp_size){
 
   /*   reads and stores the input deck in inpcp; determines the
        number of sets  */
@@ -37,10 +38,11 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
       icntrl,nload,nforc,nboun,nk,ne,nmpc,nalset,nmat,ntmat,npmat,
       norien,nam,nprint,mi[3],ntrans,ncs,namtot,ncmat,memmpc,ne1d,
       ne2d,nflow,*meminset=NULL,*rmeminset=NULL, *inp=NULL,ntie,
-      nener,nstate,nentries=15,ifreeinp,ikey,lincludefn,nslavs,
+      nener,nstate,nentries=18,ifreeinp,ikey,lincludefn,nslavs,
       nbody,ncharmax=1000000,*ipoinpc=NULL,ichangefriction=0,nkon,
       ifile,mcs,initialtemperature=0,nprop,mortar,ifacecount,
-      nintpoint,infree[4],iheading=0,ichangesurfacebehavior=0; 
+      nintpoint,infree[4],iheading=0,ichangesurfacebehavior=0,
+      nef,mpcend; 
 
   /* initialization */
 
@@ -117,6 +119,9 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
 	  j=0;
 	  ifile=0;
 	  do{
+	      if(j>=5){
+		  if(strcmp1(&buff[j-5],"FILE=")==0) ifile=1;
+	      }
 	      if(j>=6){
 		  if(strcmp1(&buff[j-6],"INPUT=")==0) ifile=1;
 	      }
@@ -147,6 +152,17 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
       if(strcmp1(&buff[0],"*HEADING")==0){
 	  iheading=1;
       }
+
+      /* check for a *KINEMATIC or *DISTRIBUTING card and change
+         the asterisk into a C (is a "dependent" card of the 
+         *COUPLING card */
+
+      if((strcmp1(&buff[0],"*KINEMATIC")==0)||
+         ((strcmp1(&buff[0],"*DISTRIBUTING")==0)&&
+          (strcmp1(&buff[0],"*DISTRIBUTINGCOUPLING")!=0)))
+         {
+	  buff[0]='C';
+      }
 	  
       /* check for include statements */
 	  
@@ -193,29 +209,33 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
       if(strcmp1(&buff[0],"*AMPLITUDE")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"AMPLITUDE",
                           nline,&ikey));
-			  }
+      }
       else if(strcmp1(&buff[0],"*CHANGEFRICTION")==0){
 	ichangefriction=1;
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"REST",
                           nline,&ikey));
-			  }
+      }
       else if(strcmp1(&buff[0],"*CHANGESURFACEBEHAVIOR")==0){
 	ichangesurfacebehavior=1;
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"REST",
                           nline,&ikey));
-			  }
+      }
       else if(strcmp1(&buff[0],"*CONDUCTIVITY")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*CONTACTDAMPING")==0){
-        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*CONTACTPAIR")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"CONTACTPAIR",
                           nline,&ikey));
-			  }
+      }
+      else if(strcmp1(&buff[0],"*COUPLING")==0){
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"COUPLING",
+                          nline,&ikey));
+      }
       else if(strcmp1(&buff[0],"*CREEP")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
                           nline,&ikey));
@@ -224,6 +244,18 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
                           nline,&ikey));
       }
+      else if(strcmp1(&buff[0],"*DAMPING")==0){
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
+                          nline,&ikey));
+      }
+      else if(strcmp1(&buff[0],"*DISTRIBUTION")==0){
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"DISTRIBUTION",
+                          nline,&ikey));
+      }
+      else if(strcmp1(&buff[0],"*ELASTIC")==0){
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
+                          nline,&ikey));
+      }
       else if(strcmp1(&buff[0],"*DEFORMATIONPLASTICITY")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
                           nline,&ikey));
@@ -264,15 +296,15 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
                           nline,&ikey));
       }
       else if((strcmp1(&buff[0],"*FRICTION")==0)&&(ichangefriction==0)){
-        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*GAPCONDUCTANCE")==0){
-        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*GAPHEATGENERATION")==0){
-        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*HYPERELASTIC")==0){
@@ -343,7 +375,7 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
               &ne1d,&ne2d,&nflow,set,meminset,rmeminset,jobnamec,
 	      &irestartstep,&icntrl,ithermal,&nener,&nstate,&ntie,
 	      &nslavs,&nkon,&mcs,&nprop,&mortar,&ifacecount,&nintpoint,
-              infree));
+	      infree,&nef,&mpcend));
             FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"RESTART,READ",
                               nline,&ikey));
 	  }
@@ -367,12 +399,12 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*SURFACEINTERACTION")==0){
-        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
       }
       else if(strcmp1(&buff[0],"*SURFACEBEHAVIOR")==0){
 	  if(ichangesurfacebehavior==0){
-	      FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"SURFACEINTERACTION",
+	      FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"INTERACTION",
                           nline,&ikey));
 	  }else{
 	      FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"REST",
@@ -392,6 +424,11 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"TRANSFORM",
                           nline,&ikey));
       }
+      else if(strcmp1(&buff[0],"*USERELEMENT")==0){
+        FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"USERELEMENT",
+                          nline,&ikey));
+	(*nuel)++;
+      }
       else if(strcmp1(&buff[0],"*USERMATERIAL")==0){
         FORTRAN(keystart,(&ifreeinp,ipoinp,inp,"MATERIAL",
                           nline,&ikey));
@@ -426,12 +463,15 @@ void readinput(char *jobnamec, char **inpcp, ITG *nline, ITG *nset,
 
   inp[3*ipoinp[2*ikey-1]-2]=*nline;
   RENEW(inpc,char,(long long)132**nline);
-  RENEW(inp,ITG,3*ipoinp[2*ikey-1]);
+  
+  *inp_size=3*ipoinp[2*ikey-1];
+  RENEW(inp,ITG,*inp_size);
+  
   *inpcp=inpc;
   *ipoinpcp=ipoinpc;
   *inpp=inp;
   
-  //  FORTRAN(writeinput,(inpc,ipoinp,inp,nline,&ipoinp[2*ikey-1],ipoinpc));
+//  FORTRAN(writeinput,(inpc,ipoinp,inp,nline,&ipoinp[2*ikey-1],ipoinpc));
 
   return;
 
