@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -30,14 +30,15 @@ void remastructar(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
               ITG *nactdof, ITG *icol, ITG *jq, ITG **irowp, ITG *isolver,
               ITG *neq, ITG *nzs,ITG *nmethod, ITG *ithermal,
 	      ITG *iperturb, ITG *mass, ITG *mi, ITG *ics, double *cs,
-	      ITG *mcs,ITG *mortar,char *typeboun,ITG *iit){
+	      ITG *mcs,ITG *mortar,char *typeboun,ITG *iit,
+	      ITG *network,ITG *iexpl){
 
     /* reconstructs the nonzero locations in the stiffness and mass
        matrix after a change in MPC's or the generation of contact
        spring elements: version for frequency calculations (called
-       by arpack and arpackcs)  */
+       by arpack, arpackcs and linstatic)  */
 
-    ITG *nodempc=NULL,*mast1=NULL,*ipointer=NULL,mpcend,mpcmult,
+    ITG *nodempc=NULL,*mast1=NULL,*ipointer=NULL,mpcend,
         callfrommain,i,*irow=NULL,mt;
 
     double *coefmpc=NULL;
@@ -53,7 +54,7 @@ void remastructar(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     callfrommain=0;
     cascade(ipompc,&coefmpc,&nodempc,nmpc,
 	    mpcfree,nodeboun,ndirboun,nboun,ikmpc,
-	    ilmpc,ikboun,ilboun,&mpcend,&mpcmult,
+	    ilmpc,ikboun,ilboun,&mpcend,
 	    labmpc,nk,memmpc_,icascade,maxlenmpc,
             &callfrommain,iperturb,ithermal);
 
@@ -73,7 +74,7 @@ void remastructar(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
 	     nodempc,nmpc,nactdof,icol,jq,&mast1,&irow,isolver,neq,
 	     ikmpc,ilmpc,ipointer,nzs,nmethod,ithermal,
 	     ikboun,ilboun,iperturb,mi,mortar,typeboun,labmpc,
-	     iit,icascade);
+	     iit,icascade,network,iexpl);
 
     }else{
       
