@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -32,12 +32,12 @@ void remastructem(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
               double **fextp, double **bp, double **aux2p, double **finip,
               double **fextinip,double **adbp, double **aubp, ITG *ithermal,
 	      ITG *iperturb, ITG *mass, ITG *mi, ITG *ielmat, double *elcon,
-	      ITG *ncmat_, ITG *ntmat_,ITG *inomat){
+	      ITG *ncmat_, ITG *ntmat_,ITG *inomat,ITG *network){
 
     /* reconstructs the nonzero locations in the stiffness and mass
        matrix after a change in MPC's */
 
-    ITG *nodempc=NULL,*mast1=NULL,*ipointer=NULL,mpcend,mpcmult,
+    ITG *nodempc=NULL,*mast1=NULL,*ipointer=NULL,mpcend,
         callfrommain,i,*irow=NULL,mt;
 
     double *coefmpc=NULL,*f=NULL,*fext=NULL,*b=NULL,*aux2=NULL,
@@ -56,7 +56,7 @@ void remastructem(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     callfrommain=0;
     cascade(ipompc,&coefmpc,&nodempc,nmpc,
 	    mpcfree,nodeboun,ndirboun,nboun,ikmpc,
-	    ilmpc,ikboun,ilboun,&mpcend,&mpcmult,
+	    ilmpc,ikboun,ilboun,&mpcend,
 	    labmpc,nk,memmpc_,icascade,maxlenmpc,
             &callfrommain,iperturb,ithermal);
 
@@ -72,7 +72,7 @@ void remastructem(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     mastructem(nk,kon,ipkon,lakon,ne,nodeboun,ndirboun,nboun,ipompc,
 	     nodempc,nmpc,nactdof,icol,jq,&mast1,&irow,isolver,neq,
 	     ikmpc,ilmpc,ipointer,nzs,ithermal,mi,ielmat,elcon,
-	     ncmat_,ntmat_,inomat);
+	     ncmat_,ntmat_,inomat,network);
 
     SFREE(ipointer);SFREE(mast1);
     RENEW(irow,ITG,nzs[2]);
@@ -86,7 +86,7 @@ void remastructem(ITG *ipompc, double **coefmpcp, ITG **nodempcp, ITG *nmpc,
     RENEW(b,double,neq[1]);for(i=0;i<neq[1];i++) b[i]=0.;
     RENEW(fini,double,neq[1]);for(i=0;i<neq[1];i++) fini[i]=0.;
 
-    if(*nmethod==4){
+    if((*nmethod==2)||(*nmethod==4)){
 	RENEW(aux2,double,neq[1]);for(i=0;i<neq[1];i++) aux2[i]=0.;
 	RENEW(fextini,double,neq[1]);for(i=0;i<neq[1];i++) fextini[i]=0.;
 	RENEW(adb,double,neq[1]);for(i=0;i<neq[1];i++) adb[i]=0.;
