@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !
       subroutine restartread(istep,nset,nload,nforc, nboun,nk,ne,
      &  nmpc,nalset,nmat,ntmat_,npmat_,norien,nam,nprint,mi,
-     &  ntrans,ncs_,namtot_,ncmat_,mpcfree,maxlenmpc,
+     &  ntrans,ncs_,namtot,ncmat_,mpcfree,maxlenmpc,
      &  ne1d,ne2d,nflow,nlabel,iplas,
      &  nkon,ithermal,nmethod,iperturb,nstate_,nener,set,istartset,
      &  iendset,ialset,co,kon,ipkon,lakon,nodeboun,ndirboun,iamboun,
@@ -35,12 +35,13 @@
      &  cbody,ibody,xbody,nbody,xbodyold,ttime,qaold,cs,mcs,
      &  output,physcon,ctrl,typeboun,fmpc,tieset,ntie,tietol,nslavs,
      &  t0g,t1g,nprop,ielprop,prop,mortar,nintpoint,ifacecount,
-     &  islavsurf,pslavsurf,clearini)
+     &  islavsurf,pslavsurf,clearini,irstrt,vel,nef,velo,veloo,
+     &  ne2boun)
 !
       implicit none
 !
       character*1 typeboun(*)
-      character*3 output
+      character*4 output
       character*6 prlab(*)
       character*8 lakon(*)
       character*20 labmpc(*),sideload(*)
@@ -51,8 +52,8 @@
 !
       integer istep,nset,nload,nforc,nboun,nk,ne,nmpc,nalset,nmat,
      &  ntmat_,npmat_,norien,nam,nprint,mi(*),ntrans,ncs_,
-     &  namtot_,ncmat_,mpcfree,ne1d,ne2d,nflow,nlabel,iplas,nkon,
-     &  ithermal,nmethod,iperturb(*),nstate_,istartset(*),iendset(*),
+     &  namtot,ncmat_,mpcfree,ne1d,ne2d,nflow,nlabel,iplas,nkon,
+     &  ithermal(*),nmethod,iperturb(*),nstate_,istartset(*),iendset(*),
      &  ialset(*),kon(*),ipkon(*),nodeboun(*),ndirboun(*),iamboun(*),
      &  ikboun(*),ilboun(*),ipompc(*),nodempc(*),ikmpc(*),ilmpc(*),
      &  nodeforc(*),ndirforc(*),iamforc(*),ikforc(*),ilforc(*),
@@ -62,8 +63,9 @@
      &  namta(*),iamt1(*),ielmat(*),nodebounold(*),ndirbounold(*),
      &  iponor(*),knor(*),iponoel(*),inoel(*),rig(*),
      &  nshcon(*),ncocon(*),ics(*),infree(*),i,ipos,
-     &  nener,irestartstep,istat,iprestr,
-     &  maxlenmpc,mcs,mpcend,ntie,ibody(*),nbody,nslavs
+     &  nener,irestartstep,istat,iprestr,irstrt(*),
+     &  maxlenmpc,mcs,mpcend,ntie,ibody(*),nbody,nslavs,nef,
+     &  ne2boun(*),memmpc_
 !
       real*8 co(*),xboun(*),coefmpc(*),xforc(*),xload(*),elcon(*),
      &  rhcon(*),alcon(*),alzero(*),plicon(*),plkcon(*),orab(*),
@@ -72,7 +74,7 @@
      &  xnor(*),thicke(*),offset(*),t0g(*),t1g(*),clearini(*),
      &  shcon(*),cocon(*),sti(*),ener(*),xstate(*),prestr(*),ttime,
      &  qaold(2),physcon(*),ctrl(*),cs(*),fmpc(*),xbody(*),
-     &  xbodyold(*),prop(*)
+     &  xbodyold(*),prop(*),vel(*),velo(*),veloo(*)
 !
       ipos=index(jobnamec(1),char(0))
       fnrstrt(1:ipos-1)=jobnamec(1)(1:ipos-1)
@@ -83,12 +85,11 @@
 !
       open(15,file=fnrstrt,ACCESS='SEQUENTIAL',FORM='UNFORMATTED',
      &  err=15)
-!
-      read(15) version
 !
       do
 !
-         read(15,iostat=istat)istep
+         read(15,iostat=istat) version
+!
          if(istat.lt.0) then
             if(irestartstep.eq.0) then
 !
@@ -99,13 +100,13 @@
                open(15,file=fnrstrt,ACCESS='SEQUENTIAL',
      &              FORM='UNFORMATTED',err=15)
                read(15) version
-               read(15,iostat=istat)istep
             else
                write(*,*) '*ERROR reading *RESTART,READ: requested step'
                write(*,*) '       is not in the restart file'
                call exit(201)
             endif
          endif
+         read(15)istep
 !
 !        set size
 !
@@ -124,6 +125,7 @@
 !
          read(15)nk
          read(15)ne
+         read(15)nef
          read(15)nkon
          read(15)(mi(i),i=1,3)
          mt=mi(2)+1
@@ -132,6 +134,7 @@
 !
          read(15)nmpc
          read(15)mpcend
+         read(15)memmpc_
          read(15)maxlenmpc
 !
 !        material size
@@ -153,7 +156,7 @@
 !        amplitude size
 !
          read(15)nam
-         read(15)namtot_
+         read(15)namtot
 !
 !        print size
 !
@@ -181,7 +184,7 @@
          read(15)(iperturb(i),i=1,2)
          read(15)nener
          read(15)iplas
-         read(15)ithermal
+         read(15)ithermal(1)
          read(15)nstate_
          read(15)nslavs
          read(15)iprestr
@@ -199,7 +202,7 @@
      &     mi(1),nmpc,mpcend,nmat,ntmat_,npmat_,ncmat_,norien,ntrans,
      &     nam,nprint,nlabel,ncs_,ne1d,ne2d,infree,nmethod,
      &     iperturb,nener,ithermal,nstate_,iprestr,mcs,ntie,
-     &     nslavs,nprop,mortar,ifacecount,nintpoint)
+     &     nslavs,nprop,mortar,ifacecount,nintpoint,nef)
 !
       enddo
 !
@@ -303,7 +306,7 @@
 !
 !     physical constants
 !
-      read(15)(physcon(i),i=1,3)
+      read(15)(physcon(i),i=1,10)
 !
 !     plastic data
 !
@@ -347,7 +350,7 @@
 !
 !     temperatures
 !
-      if(ithermal.gt.0)then
+      if(ithermal(1).gt.0)then
          read(15)(t0(i),i=1,nk)
          read(15)(t1(i),i=1,nk)
          if((ne1d.gt.0).or.(ne2d.gt.0))then
@@ -370,6 +373,14 @@
          read(15)(veold(i),i=1,mt*nk)
       endif
 !
+!     CFD results at the element centers
+!
+      if(nef.gt.0) then
+         read(15)(vel(i),i=1,8*nef)
+         read(15)(velo(i),i=1,8*nef)
+         read(15)(veloo(i),i=1,8*nef)
+      endif
+!
 !     1d and 2d elements
 !
       if((ne1d.gt.0).or.(ne2d.gt.0))then
@@ -381,6 +392,7 @@
          read(15)(iponoel(i),i=1,infree(4))
          read(15)(inoel(i),i=1,3*(infree(3)-1))
          read(15)(rig(i),i=1,infree(4))
+         read(15)(ne2boun(i),i=1,2*infree(4))
       endif
 !
 !     tie constraints
@@ -424,10 +436,14 @@
 !
 !     control parameters
 !
-      read(15) (ctrl(i),i=1,32)
+      read(15) (ctrl(i),i=1,52)
       read(15) (qaold(i),i=1,2)
       read(15) output
       read(15) ttime
+!
+!     restart parameters
+!
+      read(15) (irstrt(i),i=1,2)
 !
       close(15)
 !
