@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !     
-!     construction of the B matrix      
+!     construction of the b matrix      
 !     
       subroutine resultnet(itg,ieg,ntg,
      &     bc,nload,sideload,nelemload,xloadact,lakon,ntmat_,
@@ -38,7 +38,7 @@
 !     
       integer mi(*),itg(*),ieg(*),ntg,nteq,nflow,nload,
      &     ielmat(mi(3),*),iflag,ider,iaxial,nalt,nalf,
-     &     nelemload(2,*),nope,nopes,mint2d,i,j,k,l,nrhcon(*),
+     &     nelemload(2,*),nope,nopes,mint2d,i,j,k,m,nrhcon(*),
      &     node,imat,ntmat_,id,ifaceq(8,6),ifacet(6,4),numf,
      &     ifacew(8,5),node1,node2,nshcon(*),nelem,ig,index,konl(20),
      &     ipkon(*),kon(*),idof,ineighe(*),idir,ncocon(2,*),
@@ -55,14 +55,15 @@
      &     field,prop(*),tg1,tg2,dtime,ttime,time,g(3),eta,
      &     xforcact(*),areaj,xflow,tvar(2),f,df(8),camt(*),camf(*),
      &     camp(*),tl2(8),cama(*),vamt,vamf,vamp,vama,term,
-     &     rhcon(0:1,ntmat_,*),xbodyact(7,*),sinktemp,kappa,a,T,Tt,Pt,
-     &     dtheta,ts1,ts2,xs2(3,7),xk1,xk2,xdenom1,xdenom2,expon,pt1,
-     &     pt2,dt1,dt2,xcst,xnum1,xnum2,Qred_crit,xflow_crit,
+     &     rhcon(0:1,ntmat_,*),xbodyact(7,*),sinktemp,kappa,A,T,Tt,pt,
+     &     dtheta,ts1,ts2,xs2(3,7),pt1,pt2,Qred_crit,xflow_crit,
      &     xflow0,xflow1,reltime,coefmpc(*),qat,qaf,ramt,ramf,ramp,
-     &     xflow2,R1,R2,Rout,Rin,Uout,Uin,heat,cocon(0:6,ntmat_,*),
-     &     Cp_cor,U,Ct,vold(0:mi(2),*),xloadold(2,*),omega,bnac,
-     &     xflow360
+     &     xflow2,heat,cocon(0:6,ntmat_,*),
+     &     Cp_cor,U,Ct,vold(0:mi(2),*),xloadold(2,*),bnac,
+     &     xflow360,Tt1,Tt2,T1,T2,heatnod,heatfac,A2,d,l,s
 !     
+!
+!
       include "gauss.f"
 !     
       data ifaceq /4,3,2,1,11,10,9,12,
@@ -87,6 +88,8 @@
 !     
       tvar(1)=time
       tvar(2)=ttime+time
+!
+      heatnod=0.d0
 !     
 !     calculating the maximum correction to the solution in the
 !     present network iteration
@@ -114,25 +117,30 @@
          do j=0,3
             if(nactdog(j,node).eq.0) cycle
             idof=nactdog(j,node)
-            bnac=(v(j,node)+bc(idof))-v(j,node)
+!
+            if(dabs(v(j,node)).gt.1.d-30) then
+               if(dabs(bc(idof)/v(j,node)).lt.1.d-13) bc(idof)=0.d0
+            endif
+            bnac=bc(idof)
+!
             if(j.eq.0) then
-               if(dabs(bnac).gt.camt(1)) then
-                  camt(1)=dabs(bc(idof))
+               if(dabs(bnac).gt.dabs(camt(1))) then
+                  camt(1)=bnac
                   camt(2)=node+0.5d0
                endif
             elseif(j.eq.1) then
-               if(dabs(bnac).gt.camf(1)) then
-                  camf(1)=dabs(bc(idof))
+               if(dabs(bnac).gt.dabs(camf(1))) then
+                  camf(1)=bnac
                   camf(2)=node+0.5d0
                endif
             elseif(j.eq.2) then
-               if(dabs(bnac).gt.camp(1)) then
-                  camp(1)=dabs(bc(idof))
+               if(dabs(bnac).gt.dabs(camp(1))) then
+                  camp(1)=bnac
                   camp(2)=node+0.5d0
                endif
             else
-               if(dabs(bnac).gt.cama(1)) then
-                  cama(1)=dabs(bc(idof))
+               if(dabs(bnac).gt.dabs(cama(1))) then
+                  cama(1)=bnac
                   cama(2)=node+0.5d0
                endif
             endif
@@ -161,6 +169,10 @@
 !
          enddo
       enddo
+c      write(*,*) 'resultnet'
+c      write(*,*) 'mass flow in node 3: ',v(1,3)
+c      write(*,*) 'pressure in node 4: ',v(2,4)
+c      write(*,*) 'mass flow in node 5: ',v(1,5)
 !
 !     update geometry changes
 !
@@ -173,7 +185,7 @@
             v(3,node)=v(3,node)+bc(nactdog(3,node))*dtheta
             if(v(3,node).gt.0.99999d0) then
                v(3,node)=0.99999d0
-            elseif(v(3,node).lt.0.12501) then
+            elseif(v(3,node).lt.0.12501d0) then
                v(3,node)=0.12501d0
             endif
             if(dabs(v(3,node)).gt.vama) vama=dabs(v(3,node))
@@ -187,18 +199,18 @@
             if(nactdog(3,node).eq.0) cycle
             index=ielprop(ieg(i))
 !           Interval factor unknown
-            if(prop(index+1).eq.2) then
+            if(nint(prop(index+1)).eq.2) then
 !              Using a smaller step gets better convergence(factor 0.5)
                v(3,node)=v(3,node)+bc(nactdog(3,node))*
      &            dtheta
-               if(v(3,node).lt.0.1)then
-                  v(3,node) = 0.1
+               if(v(3,node).lt.0.1d0)then
+                  v(3,node) = 0.1d0
                endif
 !           Hole diameter factor unknown               
-            elseif(prop(index+1).eq.3) then
+            elseif(nint(prop(index+1)).eq.3) then
                v(3,node)=v(3,node)+bc(nactdog(3,node))*dtheta
-               if(v(3,node).lt.0.1)then
-                  v(3,node) = 0.1
+               if(v(3,node).lt.0.1d0)then
+                  v(3,node) = 0.1d0
                endif
             endif
             if(dabs(v(3,node)).gt.vama) vama=dabs(v(3,node))
@@ -287,7 +299,7 @@
      &                 ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
      &                 cp,r,rho,physcon,g,co,dvi,numf,vold,set,shcon,
      &                 nshcon,rhcon,nrhcon,ntmat_,mi,ider,ttime,time,
-     &                 iaxial)
+     &                 iaxial,iplausi)
                   kflag=2
 !     
                endif
@@ -306,7 +318,7 @@ c      enddo
       do i=1,ntg
          node=itg(i)
          if(v(2,node).lt.0) then
-            write(*,*) 'wrong pressure; node ',node
+            write(*,*) 'wrong pressure; node ',node,v(2,node)
             iin=0
             return
          endif
@@ -317,6 +329,7 @@ c      enddo
       do i=1,ntg
          node=itg(i)
          if(v(0,node).lt.0) then
+            write(*,*) 'wrong temperature; node ',node,v(0,node)
             iin=0
             return
          endif
@@ -333,7 +346,7 @@ c      enddo
      &        (lakon(nelem)(4:5).eq.'RTA')) then
             xflow=v(1,kon(ipkon(nelem)+2))
             if(xflow.lt.0d0)then
-               Write(*,*)'*WARNING in resultgas.f'
+               Write(*,*)'*WARNING in resultnet.f'
                write(*,*)'Element',nelem,'of TYPE ABSOLUTE TO RELATIVE'
                write(*,*)'The flow direction is no more conform '
                write(*,*)'to element definition'
@@ -345,7 +358,7 @@ c      enddo
                nodem=kon(ipkon(nelem)+2)
                xflow=v(1,nodem)
                if (xflow.lt.0) then
-                  Write(*,*)'*WARNING in resultgas.f'
+                  Write(*,*)'*WARNING in resultnet.f'
                   write(*,*)'Element',nelem,'of TYPE RESTRICTOR'
                   write(*,*)'The flow direction is no more conform '
                   write(*,*)'to element definition'
@@ -368,7 +381,7 @@ c      enddo
                xflow2=v(1,nodem2)
 !     
                if((xflow0.lt.0).or.(xflow1.lt.0).or.(xflow2.lt.0)) then
-                  Write(*,*)'*WARNING in resultgas.f'
+                  Write(*,*)'*WARNING in resultnet.f'
                   write(*,*)'Element',nelem,'of TYPE BRANCH'
                   write(*,*)'The flow direction is no more conform '
                   write(*,*)'to element definition'
@@ -378,8 +391,10 @@ c      enddo
          endif
       enddo
 !     
-!     determining the static temperature
-!     case 1: chamber; static=total temperature
+!     determining the static temperature and checking for
+!     critical conditions (only for non-chambers, i.e. for
+!     nodes purely belonging to gas pipes or restrictors except
+!     restrictor wall orifice)
 !     
       do i=1,ntg
          node=itg(i)
@@ -391,7 +406,7 @@ c      enddo
             nodem=kon(ipkon(nelem)+2)
             T=v(3,node)
             Tt=v(0,node)
-            Pt=v(2,node)
+            pt=v(2,node)
             xflow=v(1,nodem)
 !     
             icase=0
@@ -401,26 +416,47 @@ c      enddo
      &           shcon,nshcon,cp,r,dvi,rhcon,nrhcon,rho)
 !     
             index=ielprop(nelem)
+            indexe=ipkon(nelem)
+!
             kappa=(cp/(cp-R))
 !     
-            if((lakon(nelem)(2:5).eq.'GAPF')
-     &           .or.(lakon(nelem)(2:5).eq.'GAPI'))then
-               A=prop(index+1)
-               if((lakon(nelem)(2:6).eq.'GAPFA')
-     &              .or.(lakon(nelem)(2:5).eq.'GAPIA'))then
+            if(lakon(nelem)(2:5).eq.'GAPF') then
+!
+!              distinguish between standard pipes and flexible
+!              pipes
+! 
+               if((lakon(nelem)(6:8).eq.'AFR').or.
+     &            (lakon(nelem)(6:8).eq.'ARL').or.
+     &            (lakon(nelem)(6:8).eq.'ARG').or.
+     &            (lakon(nelem)(6:8).eq.'IFR').or.
+     &            (lakon(nelem)(6:8).eq.'IRL')) then
+                  call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                   ttime,time,ielprop,mi,A,A2,d,l,s)
+               else
+                  A=prop(index+1)
+               endif
+!
+               if(lakon(nelem)(2:6).eq.'GAPFA') then
                   icase=0
-               elseif((lakon(nelem)(2:6).eq.'GAPFI')
-     &                 .or.(lakon(nelem)(2:5).eq.'GAPII'))then
+               elseif(lakon(nelem)(2:6).eq.'GAPFI') then
                   icase=1
                endif  
-            elseif(lakon(nelem)(2:3).eq.'OR') then
-               A=prop(index+1)
+!
+            elseif(lakon(nelem)(2:5).eq.'GAPR') then
+!
+!              rotating adiabatic gas pipe with variable cross section:
+!              check whether section 1 or 2                  
+!
+               if(kon(ipkon(nelem)+1).eq.node) then
+                  A=prop(index+1)
+               else
+                  A=prop(index+2)
+               endif
                icase=0
 !     
             elseif(lakon(nelem)(2:3).eq.'RE') then
-               index2=ipkon(nelem)
-               node1=kon(index2+1)
-               node2=kon(index2+3)
+               node1=kon(indexe+1)
+               node2=kon(indexe+3)
 !     
                if(lakon(nelem)(4:5).eq.'EX') then
                   if(lakon(nint(prop(index+4)))(2:6).eq.'GAPFA') then
@@ -435,7 +471,7 @@ c      enddo
 !     defining the sections
 !     
                if(lakon(nelem)(4:5).eq.'BE') then
-                  a=prop(index+1)
+                  A=prop(index+1)
 !     
                elseif(lakon(nelem)(4:5).eq.'BR') then
                   if(lakon(nelem)(4:6).eq.'BRJ') then
@@ -454,46 +490,68 @@ c      enddo
 !     
                else
 !     
-                  if(node.eq.node1) then
-                     a=prop(index+1)
-                  elseif(node.eq.node2) then
-                     a=prop(index+2)
+                  if((lakon(nelem)(2:5).eq.'RBSF')) then
+                     call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                       ttime,time,ielprop,mi,A,A2,d,l,s)
+                     if(node.eq.node2) then
+                        A=A2
+                     endif
+                  else
+                     if(node.eq.node1) then
+                        A=prop(index+1)
+                     elseif(node.eq.node2) then
+                        A=prop(index+2)
+                     endif
                   endif
                endif
+            elseif(lakon(nelem)(2:3).eq.'UP') then
+!
+!              user elements whose names start with UP are assumed
+!              to be Pipe-like (static and total temperatures differ)
+!
+               call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                       ttime,time,ielprop,mi,A,A2,d,l,s)
+c               A=prop(index+1)
+               icase=0
             endif
 !     
-            if(xflow.lt.0d0) then
-               inv=-1
-            else
-               inv=1
-            endif
-!     
-            if(icase.eq.0) then
-               Qred_crit=dsqrt(kappa/R)*(1.+0.5*(kappa-1.))
-     &              **(-0.5d0*(kappa+1.)/(kappa-1.))
-            else
-               Qred_crit=dsqrt(1/R)*(1.+0.5*(kappa-1.)/kappa)
-     &              **(-0.5d0*(kappa+1.)/(kappa-1.))
-            endif
-            xflow_crit=inv*Qred_crit*Pt*A/dsqrt(Tt)             
-!     
-            xflow360=xflow*iaxial
-            call ts_calc(xflow360,Tt,Pt,kappa,r,a,T,icase)
+            xflow360=dabs(xflow*iaxial)
+            call ts_calc(xflow360,Tt,pt,kappa,r,A,T,icase)
 !     
             v(3,node)=T
+!
+            if(lakon(nelem)(2:3).eq.'RE') then
 !     
-            if(dabs(v(1,nodem)).ge.dabs(xflow_crit)) then
-               v(1,nodem)=xflow_crit
-               if(icase.eq.1) then
+!           check whether the mass flow does not exceed
+!           critical conditions (only for restrictor elements)
+!
+               if(xflow.lt.0d0) then
+                  inv=-1
+               else
+                  inv=1
+               endif
 !     
-                  if(nactdog(0,node2).ne.0) then
-                     index2=ipkon(nelem)
-                     node1=kon(index2+1)
-                     node2=kon(index2+3)
-                     v(3,node2)=v(3,node1)
-                     v(0,node2)=v(3,node2)
-     &                    *(1+0.5d0*(kappa-1)/kappa)
-                     
+               if(icase.eq.0) then
+                  Qred_crit=dsqrt(kappa/R)*(1.d0+0.5d0*(kappa-1.d0))
+     &                 **(-0.5d0*(kappa+1.d0)/(kappa-1.d0))
+               else
+                  Qred_crit=dsqrt(1/R)*(1.d0+0.5d0*(kappa-1.d0)/kappa)
+     &                 **(-0.5d0*(kappa+1.d0)/(kappa-1.d0))
+               endif
+               xflow_crit=Qred_crit*pt*A/dsqrt(Tt)   
+!     
+               if(xflow360.ge.xflow_crit) then
+                  v(1,nodem)=inv*xflow_crit/iaxial
+                  if(icase.eq.1) then
+!     
+                     if(nactdog(0,node2).ne.0) then
+                        node1=kon(indexe+1)
+                        node2=kon(indexe+3)
+                        v(3,node2)=v(3,node1)
+                        v(0,node2)=v(3,node2)
+     &                       *(1+0.5d0*(kappa-1)/kappa)
+                        
+                     endif
                   endif
                endif
             endif
@@ -516,16 +574,22 @@ c      enddo
          if((node1.ne.0).and.(node2.ne.0)) then
 !
 !           the mass flow rates must always be oriented in the direction
-!           of the decreasing pressure gradient except for VORTEX, MOEHRING,
-!           RCAVO and RCAVI
+!           of the decreasing pressure gradient except for ACCTUBE, 
+!           CROSS SPLIT, MOEHRING, ROTATING CAVITY, RADIAL OUTFLOW and 
+!           INLET, RIMSEAL, S-PUMP and VORTEX
 !
             if((lakon(nelem)(2:8).ne.'ACCTUBE').and.
+     &         (lakon(nelem)(2:5).ne.'ATR').and.
+     &         (lakon(nelem)(2:5).ne.'RTA').and.   
      &         (lakon(nelem)(2:5).ne.'CROS').and.
+     &         (lakon(nelem)(2:3).ne.'LI').and.
+     &         (lakon(nelem)(2:3).ne.'LP').and.     
      &         (lakon(nelem)(2:4).ne.'MRG').and.
      &         (lakon(nelem)(2:4).ne.'RCV').and.
      &         (lakon(nelem)(2:3).ne.'RO').and.
      &         (lakon(nelem)(2:5).ne.'RIMS').and.
      &         (lakon(nelem)(2:6).ne.'SPUMP').and.
+     &         (lakon(nelem)(2:5).ne.'GAPR').and.
      &         (lakon(nelem)(2:3).ne.'VO')) then
                if(nactdog(1,nodem).ne.0) then
                   if(((v(1,nodem).gt.1.d-12).and.
@@ -623,42 +687,49 @@ c      enddo
 !     
 !     Definitions of the constant for isothermal flow elements
 !     
-         if((lakon(nelem)(2:6).eq.'GAPFI')
-     &        .or.(lakon(nelem)(2:6).eq.'GAPII')) then
+         if(lakon(nelem)(2:6).eq.'GAPFI') then
             if((node1.ne.0).and.(node2.ne.0)) then
-               A=prop(ielprop(nelem)+1)
+!
+               if((lakon(nelem)(7:8).eq.'FR').or.
+     &            (lakon(nelem)(7:8).eq.'RL')) then
+                  call calcgeomelemnet(vold,co,prop,lakon,nelem,
+     &                       ttime,time,ielprop,mi,A,A2,d,l,s)
+               else
+                  A=prop(ielprop(nelem)+1)
+               endif
+!
                pt1=v(2,node1)
                pt2=v(2,node2)
 !     
                xflow360=xflow*iaxial
+               icase=1
 !
                if(pt1.ge.pt2)then
-                  if(dabs(tg2/ts2-(1+0.5*(kappa-1)/kappa)).lt.1E-5) then
+                  if(dabs(tg2/ts2-(1.d0+0.5d0*(kappa-1)/kappa)).lt.1d-5)
+     &                      then
                      pt2=dabs(xflow360)*dsqrt(Tg2*R)/A
-     &                    *(1+0.5*(kappa-1)/kappa)
-     &                    **(0.5*(kappa+1)/(kappa-1))
+     &                    *(1.d0+0.5d0*(kappa-1.d0)/kappa)
+     &                    **(0.5d0*(kappa+1.d0)/(kappa-1.d0))
 !                  
                   endif
-                  tg1=v(0,node1)
-                  ts1=v(3,node1)
-                  call ts_calc(xflow360,Tg1,Pt1,kappa,r,a,Ts1,icase)
-                  call ts_calc(xflow360,Tg2,Pt2,kappa,r,a,Ts2,icase)
-                  v(3,node1)=ts1
-                  v(3,node2)=ts2
+                  Tt1=v(0,node1)-physcon(1)
+                  Tt2=v(0,node2)-physcon(1)
+                  T1=v(3,node1)
+                  call ts_calc(xflow360,Tt1,pt1,kappa,r,A,T1,icase)
+                  call ts_calc(xflow360,Tt2,pt2,kappa,r,A,T2,icase)
+                  v(3,node1)=T1
+                  v(3,node2)=T2
                else
                   pt1=v(2,node2)
                   pt2=v(2,node1)
-c                 next line has consequences in gaspipe.f
-c                  if(v(3,nodem).ge.(pt2/pt1))then
-c                     pt2=v(3,nodem)*pt1
                   if(v(2,nodem).ge.(pt2/pt1))then
                      pt2=v(2,nodem)*pt1
                   endif
 !
-                  tg1=v(0,node2)
-                  call ts_calc(xflow360,Tg1,Pt1,kappa,r,a,Ts1,icase)
-                  tg2=v(0,node1)
-                  call ts_calc(xflow360,Tg2,Pt2,kappa,r,a,Ts2,icase)
+                  Tt1=v(0,node2)-physcon(1)
+                  call ts_calc(xflow360,Tt1,pt1,kappa,r,A,T1,icase)
+                  Tt2=v(0,node1)-physcon(1)
+                  call ts_calc(xflow360,Tt2,pt2,kappa,r,A,T2,icase)
                endif
 !     
             endif
@@ -679,11 +750,10 @@ c                     pt2=v(3,nodem)*pt1
                      nalt=nalt+1
                   endif
 !     
-               elseif((lakon(nelem)(2:6).eq.'GAPFI')
-     &                 .or.(lakon(nelem)(2:6).eq.'GAPII')) then
+               elseif(lakon(nelem)(2:6).eq.'GAPFI') then
                   if((nacteq(3,node1).eq.node2)) then
 !     
-                     bc(ieq)=(ts2-ts1)
+                     bc(ieq)=(T2-T1)
 !     
                   endif
                endif
@@ -714,11 +784,10 @@ c                     pt2=v(3,nodem)*pt1
                      nalt=nalt+1
                   endif
 !     
-               elseif((lakon(nelem)(2:6).eq.'GAPFI')
-     &                .or. (lakon(nelem)(2:6).eq.'GAPII')) then
+               elseif(lakon(nelem)(2:6).eq.'GAPFI') then
                   if(nacteq(3,node2).eq.node1) then
 !     
-                     bc(ieq)=(ts2-ts1)
+                     bc(ieq)=(T2-T1)
 !
                   endif
                endif
@@ -736,7 +805,7 @@ c                     pt2=v(3,nodem)*pt1
                endif
             else
                if (nacteq(1,node2).ne.0) then
-                  if(nelem.ne.prop(ielprop(nelem)+14)) then
+                  if(nelem.ne.nint(prop(ielprop(nelem)+14))) then
                      ieq=nacteq(1,node2)
                      bc(ieq)=bc(ieq)+xflow-v(0,nodem)
                      qaf=qaf+dabs(xflow)
@@ -758,7 +827,8 @@ c                     pt2=v(3,nodem)*pt1
 !     
 !     for liquids: determine the gravity vector
 !     
-            if(lakon(nelem)(2:3).eq.'LI') then
+            if((lakon(nelem)(2:3).eq.'LI').or.
+     &         (lakon(nelem)(2:3).eq.'LP')) then
                do j=1,3
                   g(j)=0.d0
                enddo
@@ -782,7 +852,8 @@ c                     pt2=v(3,nodem)*pt1
      &           nactdog,identity,
      &           ielprop,prop,kflag,v,xflow,f,nodef,idirf,df,
      &           cp,r,rho,physcon,g,co,dvi,numf,vold,set,shcon,
-     &           nshcon,rhcon,nrhcon,ntmat_,mi,ider,ttime,time,iaxial)
+     &           nshcon,rhcon,nrhcon,ntmat_,mi,ider,ttime,time,
+     &           iaxial,iplausi)
             bc(ieq)=-f
          endif
       enddo
@@ -899,32 +970,32 @@ c                     pt2=v(3,nodem)*pt1
 !     integration to obtain the area and the mean
 !     temperature
 !     
-            do l=1,mint2d
+            do m=1,mint2d
                if((lakonl(4:5).eq.'8R').or.
      &              ((lakonl(4:4).eq.'6').and.(nopes.eq.4))) then
-                  xi=gauss2d1(1,l)
-                  et=gauss2d1(2,l)
-                  weight=weight2d1(l)
+                  xi=gauss2d1(1,m)
+                  et=gauss2d1(2,m)
+                  weight=weight2d1(m)
                elseif((lakonl(4:4).eq.'8').or.
      &                 (lakonl(4:6).eq.'20R').or.
      &                 ((lakonl(4:5).eq.'15').and.(nopes.eq.8))) then
-                  xi=gauss2d2(1,l)
-                  et=gauss2d2(2,l)
-                  weight=weight2d2(l)
+                  xi=gauss2d2(1,m)
+                  et=gauss2d2(2,m)
+                  weight=weight2d2(m)
                elseif(lakonl(4:4).eq.'2') then
-                  xi=gauss2d3(1,l)
-                  et=gauss2d3(2,l)
-                  weight=weight2d3(l)
+                  xi=gauss2d3(1,m)
+                  et=gauss2d3(2,m)
+                  weight=weight2d3(m)
                elseif((lakonl(4:5).eq.'10').or.
      &                 ((lakonl(4:5).eq.'15').and.(nopes.eq.6))) then
-                  xi=gauss2d5(1,l)
-                  et=gauss2d5(2,l)
-                  weight=weight2d5(l)
+                  xi=gauss2d5(1,m)
+                  et=gauss2d5(2,m)
+                  weight=weight2d5(m)
                elseif((lakonl(4:4).eq.'4').or.
      &                 ((lakonl(4:4).eq.'6').and.(nopes.eq.3))) then
-                  xi=gauss2d4(1,l)
-                  et=gauss2d4(2,l)
-                  weight=weight2d4(l)
+                  xi=gauss2d4(1,m)
+                  et=gauss2d4(2,m)
+                  weight=weight2d4(m)
                endif
 !     
                if(nopes.eq.8) then
@@ -959,20 +1030,19 @@ c                     pt2=v(3,nodem)*pt1
                   read(sideload(i)(2:2),'(i1)') jltyp
                   jltyp=jltyp+10
                   call film(h,sinktemp,temp,istep,
-     &                 iinc,tvar,nelem,l,coords,jltyp,field,nfield,
+     &                 iinc,tvar,nelem,m,coords,jltyp,field,nfield,
      &                 sideload(i),node,areaj,v,mi,ipkon,kon,lakon,
      &                 iponoel,inoel,ielprop,prop,ielmat,shcon,nshcon,
-     &                 rhcon,nrhcon,ntmat_,cocon,ncocon)
-                  if(nmethod.eq.1) h(1)=xloadold(1,i)+
-     &                 (h(1)-xloadold(1,i))*reltime
+     &                 rhcon,nrhcon,ntmat_,cocon,ncocon,
+     &                 ipobody,xbodyact,ibody,heatnod,heatfac)
                endif
                if(lakonl(5:7).eq.'0RA') then
-                  term=2.d0*(temp-sinktemp)*h(1)*dxsj2*weight
+                  term=2.d0*((temp-sinktemp)*h(1)+heatnod)*dxsj2*weight
                   bc(ieq)=bc(ieq)+term
                   qat=qat+dabs(term)
                   nalt=nalt+1
                else
-                  term=(temp-sinktemp)*h(1)*dxsj2*weight
+                  term=((temp-sinktemp)*h(1)+heatnod)*dxsj2*weight
                   bc(ieq)=bc(ieq)+term
                   qat=qat+dabs(term)
                   nalt=nalt+1
@@ -981,7 +1051,7 @@ c                     pt2=v(3,nodem)*pt1
          endif
       enddo
 !     
-!     prescribed heat generation: contribution the energy equations        
+!     prescribed heat generation: contribution to the energy equations        
 !     
       do i=1,ntg
          node=itg(i)
@@ -1007,129 +1077,39 @@ c                     pt2=v(3,nodem)*pt1
 !     
       do i=1,nflow
          nelem=ieg(i)
-         if(lakon(nelem)(2:3).ne.'VO') cycle
 !     
 !     free vortex and no temperature change
 !     
-         if((lakon(nelem)(2:5).eq.'VOFR').and.
-     &        (prop(ielprop(nelem)+8).eq.0)) cycle
+         if(lakon(nelem)(2:3).eq.'VO') then
+            if(lakon(nelem)(4:5).eq.'FR') then
+               if((nint(prop(ielprop(nelem)+8)).eq.0).or.
+     &            (nint(prop(ielprop(nelem)+8)).eq.1)) cycle
+            elseif(lakon(nelem)(4:5).eq.'FO') then
+               if(nint(prop(ielprop(nelem)+6)).eq.0) cycle
+            endif
 !     
-!     free vortex and temperature change in the absolute system
+            call calcheatnet(nelem,lakon,ipkon,kon,v,ielprop,prop,
+     &           ielmat,ntmat_,shcon,nshcon,rhcon,nrhcon,ipobody,ibody,
+     &           xbodyact,mi,nacteq,bc,qat,nalt)
 !     
-         if((lakon(nelem)(2:5).eq.'VOFR').and.
-     &        (prop(ielprop(nelem)+8).eq.1)) cycle
+         elseif(lakon(nelem)(2:5).eq.'GAPR') then
 !     
-!     forced vortex and no temperature change
+            call calcheatnet(nelem,lakon,ipkon,kon,v,ielprop,prop,
+     &           ielmat,ntmat_,shcon,nshcon,rhcon,nrhcon,ipobody,ibody,
+     &           xbodyact,mi,nacteq,bc,qat,nalt)
 !     
-         if((lakon(nelem)(2:5).eq.'VOFO').and.
-     &        (prop(ielprop(nelem)+6).eq.0)) cycle
-!     
-         nodem=kon(ipkon(nelem)+2)
-         xflow=v(1,nodem)
-         if(xflow.gt.0d0) then
-            node1=kon(ipkon(nelem)+1)
-            node2=kon(ipkon(nelem)+3)
+         elseif(lakon(nelem)(2:2).eq.'U') then
+            call calcheatnet(nelem,lakon,ipkon,kon,v,ielprop,prop,
+     &           ielmat,ntmat_,shcon,nshcon,rhcon,nrhcon,ipobody,ibody,
+     &           xbodyact,mi,nacteq,bc,qat,nalt)
          else
-            node1=kon(ipkon(nelem)+1)
-            node2=kon(ipkon(nelem)+3)
-         endif
-!     
-         if(xflow.gt.0d0) then
-            R1=prop(ielprop(nelem)+2)
-            R2=prop(ielprop(nelem)+1)
-            if(R1.gt.R2) then
-               Rout=R2
-               Rin=R1
-            else
-               Rout=R2
-               Rin=R1
-            endif
-         else
-            R1=prop(ielprop(nelem)+2)
-            R2=prop(ielprop(nelem)+1)
-            if(R1.gt.R2) then
-               Rout=R1
-               Rin=R2
-            else
-               Rout=R1
-               Rin=R2
-            endif
-         endif
-!     
-!     computing temperature corrected Cp=Cp(T) coefficient
-!     
-         Tg1=v(0,node1)
-         Tg2=v(0,node2)
-         if((lakon(nelem)(2:3).ne.'LP').and.
-     &      (lakon(nelem)(2:3).ne.'LI')) then
-            gastemp=(tg1+tg2)/2.d0
-         else
-            if(xflow.gt.0) then
-               gastemp=tg1
-            else
-               gastemp=tg2
-            endif
-         endif
-!
-         imat=ielmat(1,nelem)
-         call materialdata_tg(imat,ntmat_,gastemp,
-     &        shcon,nshcon,cp,r,dvi,rhcon,nrhcon,rho)
-!
-         call cp_corrected(cp,Tg1,Tg2,cp_cor)
-!     
-         Uout=prop(ielprop(nelem)+5)*Rout
-         Uin=prop(ielprop(nelem)+5)*Rin
-!     
-!     free and forced vortices with temperature 
-!     change in the relative system of coordinates 
-!     
-         if((lakon(nelem)(2:5).eq.'VOFR') .and.
-     &        (prop(ielprop(nelem)+8).eq.(-1))) then
-!     
-            Uout=prop(ielprop(nelem)+7)*Rout
-            Uin=prop(ielprop(nelem)+7)*Rin
-!     
-            heat=0.5d0*Cp/Cp_cor*(Uout**2-Uin**2)*xflow
-!     
-         elseif (((lakon(nelem)(2:5).eq.'VOFO')
-     &           .and.(prop(ielprop(nelem)+6).eq.(-1)))) then
-!     
-            Uout=prop(ielprop(nelem)+5)*Rout
-            Uin=prop(ielprop(nelem)+5)*Rin
-!     
-            heat=0.5d0*Cp/Cp_cor*(Uout**2-Uin**2)*xflow
-!     
-!     forced vortices with temperature change in the absolute system
-!     
-         elseif((lakon(nelem)(2:5).eq.'VOFO')
-     &           .and.((prop(ielprop(nelem)+6).eq.1))) then
-!     
-            heat=Cp/Cp_cor*(Uout**2-Uin**2)*xflow
-!     
-         endif
-!     
-!     including the resulting additional heat flux in the energy equation
-!     
-         if(xflow.gt.0d0)then
-            ieq=nacteq(0,node2)
-            if(ieq.ne.0) then
-               bc(ieq)=bc(ieq)+heat
-               qat=qat+dabs(heat)
-               nalt=nalt+1
-            endif
-         else
-            ieq=nacteq(0,node1)
-            if(ieq.ne.0) then
-               bc(ieq)=bc(ieq)+heat
-               qat=qat+dabs(heat)
-               nalt=nalt+1
-            endif
+            cycle
          endif
       enddo
 !     
 !     transfer element ABSOLUTE TO RELATIVE / RELATIVE TO ABSOLUTE
 !     
-      do i= 1, nflow
+      do i=1,nflow
          nelem=ieg(i)
 !     
          if((lakon(nelem)(2:4).eq.'ATR').or.
@@ -1170,8 +1150,11 @@ c                     pt2=v(3,nodem)*pt1
             U=prop(index+1)
             ct=prop(index+2)
 !     
-            if(ct.eq.0) then
-               nelemswirl=prop(index+3)
+!           if a swirl element was given by the user, this takes
+!           precendence to a value of ct
+!
+            if(nint(prop(index+3)).ne.0) then
+               nelemswirl=nint(prop(index+3))
                index2=ielprop(nelemswirl)
 !     
 !     previous element is a preswirl nozzle
@@ -1218,69 +1201,34 @@ c                     pt2=v(3,nodem)*pt1
          endif
       enddo 
 !
-!     in the case of generalized pipes if rotation occurs
-!     the outlet node temperature will change
-!
-      do i=1,nflow
-         nelem=ieg(i)
-!      
-         if(lakon(nelem)(2:5).eq.'GAPI') then
-            index=ielprop(nelem)
-            if((prop(index+8).ne.0).and.
-     &           (prop(index+9).ne.0).and.
-     &           (prop(index+8).ne.0)) then
-!     
-               nodem=kon(ipkon(nelem)+2)
-               xflow=v(1,nodem)
-               if(xflow.gt.0d0) then
-                  node1=kon(ipkon(nelem)+1)
-                  node2=kon(ipkon(nelem)+3)
-               else
-                  node1=kon(ipkon(nelem)+1)
-                  node2=kon(ipkon(nelem)+3)
-               endif
-               omega=prop(index+10)
-c               write(*,*) 'icase',icase
-               rin=prop(index+8)
-               rout=prop(index+9)
-               heat=0.5*omega**2*(rout**2-rin**2)*xflow
-!     
-!     influence on the temperature of node 2
-!     
-               if(xflow.gt.0d0)then
-                  ieq=nacteq(0,node2)
-                  if(ieq.ne.0) then
-                     bc(ieq)=bc(ieq)+heat
-                     qat=qat+dabs(heat)
-                     nalt=nalt+1
-                  endif
-               else
-                  ieq=nacteq(0,node1)
-                  if(ieq.ne.0) then
-                     bc(ieq)=bc(ieq)+heat
-                     qat=qat+dabs(heat)
-                     nalt=nalt+1
-                  endif
-               endif
-            endif
-         endif
-      enddo 
-!
 !     additional multiple point constraints
 !
       j=nteq+1
       do i=nmpc,1,-1
          if(labmpc(i)(1:7).ne.'NETWORK') cycle
          j=j-1
-         index=ipompc(i)
+         if(labmpc(i)(8:8).eq.' ') then
 !
-         do
-            node=nodempc(1,index)
-            idir=nodempc(2,index)
-            bc(j)=bc(j)-v(idir,node)*coefmpc(index)
-            index=nodempc(3,index)
-            if(index.eq.0) exit
-         enddo
+!           linear equation
+!
+            index=ipompc(i)
+!     
+            do
+               node=nodempc(1,index)
+               idir=nodempc(2,index)
+               bc(j)=bc(j)-v(idir,node)*coefmpc(index)
+               index=nodempc(3,index)
+               if(index.eq.0) exit
+            enddo
+         else
+!
+!           user-defined network equation
+!
+            call networkmpc_rhs(i,ipompc,nodempc,coefmpc,labmpc,
+     &          v,bc,j,mi,ipkon,kon,lakon,iponoel,
+     &          inoel,ielprop,prop,ielmat,
+     &          shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon)
+         endif
       enddo
 !
 !     determining the typical energy flow
@@ -1300,19 +1248,19 @@ c               write(*,*) 'icase',icase
       do i=1,ntg
          node=itg(i)
          if(nacteq(0,node).ne.0) then
-            ramt=max(ramt,bc(nacteq(0,node)))
+            ramt=max(ramt,dabs(bc(nacteq(0,node))))
          endif
          if(nacteq(1,node).ne.0) then
-            ramf=max(ramf,bc(nacteq(1,node)))
+            ramf=max(ramf,dabs(bc(nacteq(1,node))))
          endif
          if(nacteq(2,node).ne.0) then
-            ramp=max(ramp,bc(nacteq(2,node)))
+            ramp=max(ramp,dabs(bc(nacteq(2,node))))
          endif
       enddo
 !
-c      write(30,*) 'bc in resultgas'
-c      do i=1,9
-c         write(30,'(1x,e11.4)') bc(i)
+c      write(*,*) 'bc in resultnet'
+c      do i=1,3
+c         write(*,'(1x,e11.4)') bc(i)
 c      enddo
 !
       return
