@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -23,9 +23,10 @@
      &  veold,dtime,time,ttime,plicon,nplicon,plkcon,nplkcon,
      &  xstateini,xstiff,xstate,npmat_,matname,mi,ielas,icmd,
      &  ncmat_,nstate_,stiini,vini,ener,eei,enerini,istep,iinc,
-     &  springarea,reltime,calcul_fn,calcul_qa,calcul_cauchy,iener,
+     &  springarea,reltime,calcul_fn,calcul_qa,calcul_cauchy,nener,
      &  ikin,nal,ne0,thicke,emeini,pslavsurf,
-     &  pmastsurf,mortar,clearini,nea,neb,ielprop,prop,kscale)
+     &  pmastsurf,mortar,clearini,nea,neb,ielprop,prop,kscale,
+     &  list,ilist,smscale,mscalmethod,enerscal)
 !
 !     calculates stresses and the material tangent at the integration
 !     points and the internal forces at the nodes
@@ -40,12 +41,13 @@
       integer kon(*),konl(26),nea,neb,mi(*),mint2d,nopes,
      &  nelcon(2,*),nrhcon(*),nalcon(2,*),ielmat(mi(3),*),
      &  ielorien(mi(3),*),ntmat_,ipkon(*),ne0,iflag,null,kscale,
-     &  istep,iinc,mt,ne,mattyp,ithermal(2),iprestr,i,j,k,m1,m2,jj,
-     &  i1,m3,m4,kk,iener,indexe,nope,norien,iperturb(*),iout,
+     &  istep,iinc,mt,ne,mattyp,ithermal(*),iprestr,i,j,k,m1,m2,jj,
+     &  i1,m3,m4,kk,nener,indexe,nope,norien,iperturb(*),iout,
      &  nal,icmd,ihyper,nmethod,kode,imat,mint3d,iorien,ielas,
      &  istiff,ncmat_,nstate_,ikin,ilayer,nlayer,ki,kl,ielprop(*),
      &  nplicon(0:ntmat_,*),nplkcon(0:ntmat_,*),npmat_,calcul_fn,
-     &  calcul_cauchy,calcul_qa,nopered,mortar,jfaces,iloc,igauss
+     &  calcul_cauchy,calcul_qa,nopered,mortar,jfaces,igauss,
+     &  istrainfree,nlgeom_undo,list,ilist(*),m,j1,mscalmethod
 !
       real*8 co(3,*),v(0:mi(2),*),shp(4,26),stiini(6,mi(1),*),
      &  stx(6,mi(1),*),xl(3,26),vl(0:mi(2),26),stre(6),prop(*),
@@ -54,18 +56,21 @@
      &  alzero(*),orab(7,*),elas(21),rho,fn(0:mi(2),*),
      &  fnl(3,10),skl(3,3),beta(6),q(0:mi(2),26),xl2(3,8),
      &  vkl(0:3,3),t0(*),t1(*),prestr(6,mi(1),*),eme(6,mi(1),*),
-     &  ckl(3,3),vold(0:mi(2),*),eloc(9),veold(0:mi(2),*),
+     &  ckl(3,3),vold(0:mi(2),*),eloc(6),veold(0:mi(2),*),
      &  springarea(2,*),elconloc(21),eth(6),xkl(3,3),voldl(0:mi(2),26),
      &  xikl(3,3),ener(mi(1),*),emec(6),eei(6,mi(1),*),enerini(mi(1),*),
      &  emec0(6),vel(1:3,26),veoldl(0:mi(2),26),xsj2(3),shp2(7,8),
-     &  e,un,al,um,am1,xi,et,ze,tt,exx,eyy,ezz,exy,exz,eyz,
-     &  xsj,qa(3),vj,t0l,t1l,dtime,weight,pgauss(3),vij,time,ttime,
+     &  e,un,al,um,am1,xi,et,ze,tt,
+     &  xsj,qa(*),vj,t0l,t1l,dtime,weight,pgauss(3),vij,time,ttime,
      &  plicon(0:2*npmat_,ntmat_,*),plkcon(0:2*npmat_,ntmat_,*),
      &  xstiff(27,mi(1),*),xstate(nstate_,mi(1),*),plconloc(802),
      &  vokl(3,3),xstateini(nstate_,mi(1),*),vikl(3,3),
      &  gs(8,4),a,reltime,tlayer(4),dlayer(4),xlayer(mi(3),4),
      &  thicke(mi(3),*),emeini(6,mi(1),*),clearini(3,9,*),
-     &  pslavsurf(3,*),pmastsurf(6,*)
+     &  pslavsurf(3,*),pmastsurf(6,*),smscale(*),sum1,sum2,
+     &  scal,enerscal,elineng(6)
+!
+!
 !
       include "gauss.f"
 !
@@ -74,24 +79,56 @@
 !
       mt=mi(2)+1
       nal=0
+      qa(3)=-1.d0
+      qa(4)=0.d0
+      enerscal=0.d0
 !
-      do i=nea,neb
+      do m=nea,neb
+!
+         if(list.eq.1) then
+            i=ilist(m)
+         else
+            i=m
+         endif
+!
+!        check for strainless reactivated elements
+!
+         if(ielmat(1,i).lt.0) then
+            istrainfree=1
+            ielmat(1,i)=-ielmat(1,i)
+         else
+            istrainfree=0
+         endif
 !
          lakonl=lakon(i)
 !
-         if(ipkon(i).lt.0) cycle
+!        only structural elements (no fluid elements)
 !
-!        no 3D-fluid elements
+         if((ipkon(i).lt.0).or.(lakonl(1:1).eq.'F')) cycle
 !
-         if(lakonl(1:1).eq.'F') cycle
          if(lakonl(1:7).eq.'DCOUP3D') cycle
+!
+!        user elements
+!
+         if(lakonl(1:1).eq.'U') then
+            call resultsmech_u(co,kon,ipkon,lakon,ne,v,
+     &        stx,elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,
+     &        ielmat,ielorien,norien,orab,ntmat_,t0,t1,ithermal,prestr,
+     &        iprestr,eme,iperturb,fn,iout,qa,vold,nmethod,
+     &        veold,dtime,time,ttime,plicon,nplicon,plkcon,nplkcon,
+     &        xstateini,xstiff,xstate,npmat_,matname,mi,ielas,icmd,
+     &        ncmat_,nstate_,stiini,vini,ener,eei,enerini,istep,iinc,
+     &        reltime,calcul_fn,calcul_qa,calcul_cauchy,nener,
+     &        ikin,nal,ne0,thicke,emeini,i,ielprop,prop)
+            cycle
+         endif
 !
          if(lakonl(7:8).ne.'LC') then
 !
             imat=ielmat(1,i)
             amat=matname(imat)
             if(norien.gt.0) then
-               iorien=ielorien(1,i)
+               iorien=max(0,ielorien(1,i))
             else
                iorien=0
             endif
@@ -101,13 +138,15 @@
             else
                ihyper=0
             endif
-         else
-!     
-!     composite materials
+         elseif(lakonl(4:5).eq.'20') then
 !     
+!           composite materials    
+!           S8R - composite element
+!
             mint2d=4
             nopes=8
-!     determining the number of layers
+!
+!           determining the number of layers
 !     
             nlayer=0
             do k=1,mi(3)
@@ -142,8 +181,49 @@
                dlayer(k)=0.d0
             enddo
 !     
-         endif
+!
+!     S6 - composite element
+!
+         elseif(lakonl(4:5).eq.'15') then
+            mint2d=3
+            nopes=6
+!     determining the number of layers
 !     
+            nlayer=0
+            do k=1,mi(3)
+               if(ielmat(k,i).ne.0) then
+                  nlayer=nlayer+1
+               endif
+            enddo
+!     
+!     determining the layer thickness and global thickness
+!     at the shell integration points
+!     
+            iflag=1
+            indexe=ipkon(i)
+            do kk=1,mint2d
+               xi=gauss3d10(1,kk)
+               et=gauss3d10(2,kk)
+               call shape6tri(xi,et,xl2,xsj2,xs2,shp2,iflag)
+               tlayer(kk)=0.d0
+               do k=1,nlayer
+                  thickness=0.d0
+                  do j=1,nopes
+                     thickness=thickness+thicke(k,indexe+j)*shp2(4,j)
+                  enddo
+                  tlayer(kk)=tlayer(kk)+thickness
+                  xlayer(k,kk)=thickness
+               enddo
+            enddo
+            iflag=3
+!    
+            ilayer=0
+            do k=1,3
+               dlayer(k)=0.d0
+            enddo
+!     
+         endif
+!
          indexe=ipkon(i)
 c     Bernhardi start
          if(lakonl(1:5).eq.'C3D8I') then
@@ -215,7 +295,11 @@ c     Bernhardi end
          elseif(lakonl(4:4).eq.'4') then
             mint3d=1
          elseif(lakonl(4:5).eq.'15') then
-            mint3d=9
+            if(lakonl(7:8).eq.'LC') then
+               mint3d=6*nlayer
+            else
+               mint3d=9
+            endif
          elseif(lakonl(4:4).eq.'6') then
             mint3d=2
          elseif(lakonl(1:1).eq.'E') then
@@ -244,12 +328,14 @@ c     Bernhardi end
 !
 !        calculating the forces for the contact elements
 !
+c         write(*,*) 'resultsmech ',i,lakonl,mint3d
          if(mint3d.eq.0) then
 !
 !           "normal" spring and dashpot elements
 !
             kode=nelcon(1,imat)
-            if(lakonl(7:7).eq.'A') then
+            if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'1').or.
+     &            (lakonl(7:7).eq.'2')) then
                t0l=0.d0
                t1l=0.d0
                if(ithermal(1).eq.1) then
@@ -264,28 +350,30 @@ c     Bernhardi end
 !           spring elements (including contact springs)
 !     
             if(lakonl(2:2).eq.'S') then
-               if((lakonl(7:7).eq.'A').or.((mortar.eq.0).and.
+               if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'1').or.
+     &            (lakonl(7:7).eq.'2').or.((mortar.eq.0).and.
      &          ((nmethod.ne.1).or.(iperturb(1).ge.2).or.(iout.ne.-1))))
      &               then
                   call springforc_n2f(xl,konl,vl,imat,elcon,nelcon,elas,
      &              fnl,ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,
-     &              plicon,nplicon,npmat_,ener(1,i),iener,
+     &              plicon,nplicon,npmat_,ener(1,i),nener,
      &              stx(1,1,i),mi,springarea(1,konl(nope+1)),nmethod,
      &              ne0,nstate_,xstateini,xstate,reltime,
-     &              ielas,ener(1,i+ne))
+     &              ielas,ener(1,i+ne),ielorien,orab,norien,i,
+     &              smscale,mscalmethod)
                elseif((mortar.eq.1).and.
      &           ((nmethod.ne.1).or.(iperturb(1).ge.2).or.(iout.ne.-1)))
      &               then
-                  iloc=kon(indexe+nope+1)
                   jfaces=kon(indexe+nope+2)
                   igauss=kon(indexe+nope+1)
                   call springforc_f2f(xl,vl,imat,elcon,nelcon,elas,
      &              fnl,ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,
-     &              plicon,nplicon,npmat_,ener(1,i),iener,
-     &              stx(1,1,i),mi,springarea(1,iloc),nmethod,
+     &              plicon,nplicon,npmat_,ener(1,i),nener,
+     &              stx(1,1,i),mi,springarea(1,igauss),nmethod,
      &              ne0,nstate_,xstateini,xstate,reltime,
-     &              ielas,iloc,jfaces,igauss,pslavsurf,pmastsurf,
-     &              clearini,ener(1,i+ne),kscale)
+     &              ielas,jfaces,igauss,pslavsurf,pmastsurf,
+     &              clearini,ener(1,i+ne),kscale,konl,iout,i,
+     &              smscale,mscalmethod)
                endif
 !             
 !              next lines are not executed in linstatic.c before the
@@ -301,25 +389,6 @@ c     Bernhardi end
                      enddo
                   enddo
                endif
-!
-!              dashpot elements (including contact dashpots)
-!
-            elseif((nmethod.eq.4).or.(nmethod.eq.5).or.
-     &             ((abs(nmethod).eq.1).and.(iperturb(1).ge.2))) then
-c               do j=1,nope
-c                  konl(j)=kon(indexe+j)
-c                  do k=1,3
-c                     vel(k,j)=veold(k,konl(j))
-c                  enddo
-c               enddo
-c               call dashforc(xl,konl,vl,imat,elcon,nelcon,
-c     &              elas,fnl,ncmat_,ntmat_,nope,lakonl,t0l,t1l,kode,
-c     &              elconloc,plicon,nplicon,npmat_,vel,time,nmethod,mi)
-c               do j=1,nope
-c                  do k=1,3
-c                     fn(k,konl(j))=fn(k,konl(j))+fnl(k,j)
-c                  enddo
-c               enddo
             endif
          elseif(ikin.eq.1) then
             do j=1,nope
@@ -382,7 +451,7 @@ c               enddo
                   imat=ielmat(ilayer,i)
                   amat=matname(imat)
                   if(norien.gt.0) then
-                     iorien=ielorien(ilayer,i)
+                     iorien=max(0,ielorien(ilayer,i))
                   else
                      iorien=0
                   endif
@@ -409,10 +478,51 @@ c               enddo
                ze=gauss3d4(3,jj)
                weight=weight3d4(jj)
             elseif(lakonl(4:5).eq.'15') then
-               xi=gauss3d8(1,jj)
-               et=gauss3d8(2,jj)
-               ze=gauss3d8(3,jj)
-               weight=weight3d8(jj)
+               if(lakonl(7:8).ne.'LC') then
+                  xi=gauss3d8(1,jj)
+                  et=gauss3d8(2,jj)
+                  ze=gauss3d8(3,jj)
+                  weight=weight3d8(jj)
+               else
+                  kl=mod(jj,6)
+                  if(kl.eq.0) kl=6
+!
+                  xi=gauss3d10(1,kl)
+                  et=gauss3d10(2,kl)
+                  ze=gauss3d10(3,kl)
+                  weight=weight3d10(kl)
+!
+                  ki=mod(jj,3)
+                  if(ki.eq.0) ki=3
+!
+                  if(kl.eq.1) then
+                     ilayer=ilayer+1
+                     if(ilayer.gt.1) then
+                        do k=1,3
+                           dlayer(k)=dlayer(k)+xlayer(ilayer-1,k)
+                        enddo
+                     endif
+                  endif
+                  ze=2.d0*(dlayer(ki)+(ze+1.d0)/2.d0*xlayer(ilayer,ki))/
+     &                 tlayer(ki)-1.d0
+                  weight=weight*xlayer(ilayer,ki)/tlayer(ki)
+!
+!                 material and orientation
+!
+                  imat=ielmat(ilayer,i)
+                  amat=matname(imat)
+                  if(norien.gt.0) then
+                     iorien=max(0,ielorien(ilayer,i))
+                  else
+                     iorien=0
+                  endif
+!     
+                  if(nelcon(1,imat).lt.0) then
+                     ihyper=1
+                  else
+                     ihyper=0
+                  endif
+               endif
             elseif(lakonl(4:4).eq.'6') then
                xi=gauss3d7(1,jj)
                et=gauss3d7(2,jj)
@@ -494,51 +604,17 @@ c                  write(*,*) 'vnoeie',i,konl(m1),(vkl(m2,k),k=1,3)
 !
 !           calculating the strain
 !
-!           attention! exy,exz and eyz are engineering strains!
-!
-            exx=vkl(1,1)
-            eyy=vkl(2,2)
-            ezz=vkl(3,3)
-            exy=vkl(1,2)+vkl(2,1)
-            exz=vkl(1,3)+vkl(3,1)
-            eyz=vkl(2,3)+vkl(3,2)
-!
-            if(iperturb(2).eq.1) then
-!     
-!                 Lagrangian strain
-!     
-               exx=exx+(vkl(1,1)**2+vkl(2,1)**2+vkl(3,1)**2)/2.d0
-               eyy=eyy+(vkl(1,2)**2+vkl(2,2)**2+vkl(3,2)**2)/2.d0
-               ezz=ezz+(vkl(1,3)**2+vkl(2,3)**2+vkl(3,3)**2)/2.d0
-               exy=exy+vkl(1,1)*vkl(1,2)+vkl(2,1)*vkl(2,2)+
-     &              vkl(3,1)*vkl(3,2)
-               exz=exz+vkl(1,1)*vkl(1,3)+vkl(2,1)*vkl(2,3)+
-     &              vkl(3,1)*vkl(3,3)
-               eyz=eyz+vkl(1,2)*vkl(1,3)+vkl(2,2)*vkl(2,3)+
-     &              vkl(3,2)*vkl(3,3)
-!
-!           for frequency analysis or buckling with preload the
-!           strains are calculated with respect to the deformed
-!           configuration
-!
-            elseif(iperturb(1).eq.1) then
-               exx=exx+vokl(1,1)*vkl(1,1)+vokl(2,1)*vkl(2,1)+
-     &              vokl(3,1)*vkl(3,1)
-               eyy=eyy+vokl(1,2)*vkl(1,2)+vokl(2,2)*vkl(2,2)+
-     &              vokl(3,2)*vkl(3,2)
-               ezz=ezz+vokl(1,3)*vkl(1,3)+vokl(2,3)*vkl(2,3)+
-     &              vokl(3,3)*vkl(3,3)
-               exy=exy+vokl(1,1)*vkl(1,2)+vokl(1,2)*vkl(1,1)+
-     &              vokl(2,1)*vkl(2,2)+vokl(2,2)*vkl(2,1)+
-     &              vokl(3,1)*vkl(3,2)+vokl(3,2)*vkl(3,1)
-               exz=exz+vokl(1,1)*vkl(1,3)+vokl(1,3)*vkl(1,1)+
-     &              vokl(2,1)*vkl(2,3)+vokl(2,3)*vkl(2,1)+
-     &              vokl(3,1)*vkl(3,3)+vokl(3,3)*vkl(3,1)
-               eyz=eyz+vokl(1,2)*vkl(1,3)+vokl(1,3)*vkl(1,2)+
-     &              vokl(2,2)*vkl(2,3)+vokl(2,3)*vkl(2,2)+
-     &              vokl(3,2)*vkl(3,3)+vokl(3,3)*vkl(3,2)
-            endif
-!
+            call calctotstrain(vkl,vokl,eloc,elineng,iperturb)
+!            
+c!           attention! exy,exz and eyz are engineering strains!
+c!
+c            exx=vkl(1,1)
+c            eyy=vkl(2,2)
+c            ezz=vkl(3,3)
+c            exy=vkl(1,2)+vkl(2,1)
+c            exz=vkl(1,3)+vkl(3,1)
+c            eyz=vkl(2,3)+vkl(3,2)
+c!
 c            if(iperturb(2).eq.1) then
 c!     
 c!                 Lagrangian strain
@@ -552,35 +628,55 @@ c               exz=exz+vkl(1,1)*vkl(1,3)+vkl(2,1)*vkl(2,3)+
 c     &              vkl(3,1)*vkl(3,3)
 c               eyz=eyz+vkl(1,2)*vkl(1,3)+vkl(2,2)*vkl(2,3)+
 c     &              vkl(3,2)*vkl(3,3)
-c!     
+c!
+c!           for frequency analysis or buckling with preload the
+c!           strains are calculated with respect to the deformed
+c!           configuration
+c!
+c            elseif(iperturb(1).eq.1) then
+c               exx=exx+vokl(1,1)*vkl(1,1)+vokl(2,1)*vkl(2,1)+
+c     &              vokl(3,1)*vkl(3,1)
+c               eyy=eyy+vokl(1,2)*vkl(1,2)+vokl(2,2)*vkl(2,2)+
+c     &              vokl(3,2)*vkl(3,2)
+c               ezz=ezz+vokl(1,3)*vkl(1,3)+vokl(2,3)*vkl(2,3)+
+c     &              vokl(3,3)*vkl(3,3)
+c               exy=exy+vokl(1,1)*vkl(1,2)+vokl(1,2)*vkl(1,1)+
+c     &              vokl(2,1)*vkl(2,2)+vokl(2,2)*vkl(2,1)+
+c     &              vokl(3,1)*vkl(3,2)+vokl(3,2)*vkl(3,1)
+c               exz=exz+vokl(1,1)*vkl(1,3)+vokl(1,3)*vkl(1,1)+
+c     &              vokl(2,1)*vkl(2,3)+vokl(2,3)*vkl(2,1)+
+c     &              vokl(3,1)*vkl(3,3)+vokl(3,3)*vkl(3,1)
+c               eyz=eyz+vokl(1,2)*vkl(1,3)+vokl(1,3)*vkl(1,2)+
+c     &              vokl(2,2)*vkl(2,3)+vokl(2,3)*vkl(2,2)+
+c     &              vokl(3,2)*vkl(3,3)+vokl(3,3)*vkl(3,2)
+c            endif
+c!
+c!              storing the local strains
+c!
+c            if(iperturb(1).ne.-1) then
+c               eloc(1)=exx
+c               eloc(2)=eyy
+c               eloc(3)=ezz
+c               eloc(4)=exy/2.d0
+c               eloc(5)=exz/2.d0
+c               eloc(6)=eyz/2.d0
+c            else
+c!
+c!              linear iteration within a nonlinear increment:
+c!
+c               eloc(1)=vokl(1,1)+
+c     &           (vokl(1,1)**2+vokl(2,1)**2+vokl(3,1)**2)/2.d0
+c               eloc(2)=vokl(2,2)+
+c     &           (vokl(1,2)**2+vokl(2,2)**2+vokl(3,2)**2)/2.d0
+c               eloc(3)=vokl(3,3)+
+c     &           (vokl(1,3)**2+vokl(2,3)**2+vokl(3,3)**2)/2.d0
+c               eloc(4)=(vokl(1,2)+vokl(2,1)+vokl(1,1)*vokl(1,2)+
+c     &              vokl(2,1)*vokl(2,2)+vokl(3,1)*vokl(3,2))/2.d0
+c               eloc(5)=(vokl(1,3)+vokl(3,1)+vokl(1,1)*vokl(1,3)+
+c     &              vokl(2,1)*vokl(2,3)+vokl(3,1)*vokl(3,3))/2.d0
+c               eloc(6)=(vokl(2,3)+vokl(3,2)+vokl(1,2)*vokl(1,3)+
+c     &              vokl(2,2)*vokl(2,3)+vokl(3,2)*vokl(3,3))/2.d0
 c            endif
-!
-!              storing the local strains
-!
-            if(iperturb(1).ne.-1) then
-               eloc(1)=exx
-               eloc(2)=eyy
-               eloc(3)=ezz
-               eloc(4)=exy/2.d0
-               eloc(5)=exz/2.d0
-               eloc(6)=eyz/2.d0
-            else
-!
-!              linear iteration within a nonlinear increment:
-!
-               eloc(1)=vokl(1,1)+
-     &           (vokl(1,1)**2+vokl(2,1)**2+vokl(3,1)**2)/2.d0
-               eloc(2)=vokl(2,2)+
-     &           (vokl(1,2)**2+vokl(2,2)**2+vokl(3,2)**2)/2.d0
-               eloc(3)=vokl(3,3)+
-     &           (vokl(1,3)**2+vokl(2,3)**2+vokl(3,3)**2)/2.d0
-               eloc(4)=(vokl(1,2)+vokl(2,1)+vokl(1,1)*vokl(1,2)+
-     &              vokl(2,1)*vokl(2,2)+vokl(3,1)*vokl(3,2))/2.d0
-               eloc(5)=(vokl(1,3)+vokl(3,1)+vokl(1,1)*vokl(1,3)+
-     &              vokl(2,1)*vokl(2,3)+vokl(3,1)*vokl(3,3))/2.d0
-               eloc(6)=(vokl(2,3)+vokl(3,2)+vokl(1,2)*vokl(1,3)+
-     &              vokl(2,2)*vokl(2,3)+vokl(3,2)*vokl(3,3))/2.d0
-            endif
 !
 !                 calculating the deformation gradient (needed to
 !                 convert the element stiffness matrix from spatial
@@ -661,9 +757,9 @@ c     Bernhardi end
 !              calculating the deformation gradient of the old
 !              fields
 !
-               xikl(1,1)=vikl(1,1)+1
-               xikl(2,2)=vikl(2,2)+1.
-               xikl(3,3)=vikl(3,3)+1.
+               xikl(1,1)=vikl(1,1)+1.d0
+               xikl(2,2)=vikl(2,2)+1.d0
+               xikl(3,3)=vikl(3,3)+1.d0
                xikl(1,2)=vikl(1,2)
                xikl(1,3)=vikl(1,3)
                xikl(2,3)=vikl(2,3)
@@ -716,7 +812,11 @@ c     Bernhardi end
                      enddo
                   elseif(lakonl(4:6).eq.'20 ') then
                      nopered=20
-                     call lintemp(t0,t1,konl,nopered,jj,t0l,t1l)
+                     call lintemp(t0,konl,nopered,jj,t0l)
+                     call lintemp(t1,konl,nopered,jj,t1l)
+                  elseif(lakonl(4:6).eq.'10T') then
+                     call linscal10(t0,konl,t0l,null,shp)
+                     call linscal10(t1,konl,t1l,null,shp)
                   else
                      do i1=1,nope
                         t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -732,7 +832,11 @@ c     Bernhardi end
                      enddo
                   elseif(lakonl(4:6).eq.'20 ') then
                      nopered=20
-                     call lintemp_th(t0,vold,konl,nopered,jj,t0l,t1l,mi)
+                     call lintemp_th0(t0,konl,nopered,jj,t0l,mi)
+                     call lintemp_th1(vold,konl,nopered,jj,t1l,mi)
+                  elseif(lakonl(4:6).eq.'10T') then
+                     call linscal10(t0,konl,t0l,null,shp)
+                     call linscal10(vold,konl,t1l,mi(2),shp)
                   else
                      do i1=1,nope
                         t0l=t0l+shp(4,i1)*t0(konl(i1))
@@ -766,20 +870,19 @@ c     Bernhardi end
      &           nalcon,imat,amat,iorien,pgauss,orab,ntmat_,
      &           elas,rho,i,ithermal,alzero,mattyp,t0l,t1l,ihyper,
      &           istiff,elconloc,eth,kode,plicon,nplicon,
-     &           plkcon,nplkcon,npmat_,plconloc,mi(1),dtime,i,jj,
+     &           plkcon,nplkcon,npmat_,plconloc,mi(1),dtime,jj,
      &           xstiff,ncmat_)
 !
 !           determining the mechanical strain
 !
             if(ithermal(1).ne.0) then
-               do m1=1,6
-                  emec(m1)=eloc(m1)-eth(m1)
-c                 emec0(m1)=emeini(m1,jj,i)
-               enddo
+               call calcmechstrain(vkl,vokl,emec,eth,iperturb)
+c               do m1=1,6
+c                  emec(m1)=eloc(m1)-eth(m1)
+c               enddo
             else
                do m1=1,6
                   emec(m1)=eloc(m1)
-c                 emec0(m1)=emeini(m1,jj,i)
                enddo
             endif
             if(kode.le.-100) then
@@ -788,31 +891,41 @@ c                 emec0(m1)=emeini(m1,jj,i)
                enddo
             endif
 !
-!           subtracting the plastic initial strains
+!           subtracting the initial strains
 !
             if(iprestr.eq.2) then
-               do m1=1,6
-                  emec(m1)=emec(m1)-prestr(m1,jj,i)
-               enddo
+               if(istrainfree.eq.0) then
+                  do m1=1,6
+                     emec(m1)=emec(m1)-prestr(m1,jj,i)
+                  enddo
+               else
+                  do m1=1,6
+                     prestr(m1,jj,i)=emec(m1)
+                     emeini(m1,jj,i)=emec(m1)
+                     eme(m1,jj,i)=emec(m1)
+                     emec(m1)=0.d0
+                  enddo
+               endif
             endif
 !
 !           calculating the local stiffness and stress
 !
+            nlgeom_undo=0
             call mechmodel(elconloc,elas,emec,kode,emec0,ithermal,
      &           icmd,beta,stre,xkl,ckl,vj,xikl,vij,
      &           plconloc,xstate,xstateini,ielas,
      &           amat,t1l,dtime,time,ttime,i,jj,nstate_,mi(1),
      &           iorien,pgauss,orab,eloc,mattyp,qa(3),istep,iinc,
-     &           ipkon,nmethod,iperturb)
+     &           ipkon,nmethod,iperturb,qa(4),nlgeom_undo)
 !
             if(((nmethod.ne.4).or.(iperturb(1).ne.0)).and.
-     &         (nmethod.ne.5)) then
+     &         (nmethod.ne.5).and.(icmd.ne.3)) then
                do m1=1,21
                   xstiff(m1,jj,i)=elas(m1)
                enddo
             endif
 !
-            if(iperturb(1).eq.-1) then
+            if((iperturb(1).eq.-1).and.(nlgeom_undo.eq.0)) then
 !
 !                    if the forced displacements were changed at
 !                    the start of a nonlinear step, the nodal
@@ -823,12 +936,12 @@ c                 emec0(m1)=emeini(m1,jj,i)
 !                    in a quasi-static way (only applies to
 !                    quasi-static analyses (*STATIC))
 !
-               eloc(1)=exx-vokl(1,1)
-               eloc(2)=eyy-vokl(2,2)
-               eloc(3)=ezz-vokl(3,3)
-               eloc(4)=exy-(vokl(1,2)+vokl(2,1))
-               eloc(5)=exz-(vokl(1,3)+vokl(3,1))
-               eloc(6)=eyz-(vokl(2,3)+vokl(3,2))
+               eloc(1)=elineng(1)-vokl(1,1)
+               eloc(2)=elineng(2)-vokl(2,2)
+               eloc(3)=elineng(3)-vokl(3,3)
+               eloc(4)=elineng(4)-(vokl(1,2)+vokl(2,1))
+               eloc(5)=elineng(5)-(vokl(1,3)+vokl(3,1))
+               eloc(6)=elineng(6)-(vokl(2,3)+vokl(3,2))
 !
                if(mattyp.eq.1) then
                   e=elas(1)
@@ -881,57 +994,55 @@ c                 emec0(m1)=emeini(m1,jj,i)
 !           is requested), since it is input to the umat routine
 !
             if((iout.gt.0).or.(iout.eq.-2).or.(kode.le.-100).or.
-     &         ((nmethod.eq.4).and.(iperturb(1).gt.1).and.
-     &          (ithermal(1).le.1))) then
-               if(ithermal(1).eq.0) then
-                  do m1=1,6
-                     eth(m1)=0.d0
-                  enddo
-               endif
-               if(iener.eq.1) then
-                  ener(jj,i)=enerini(jj,i)+
-     &                 ((eloc(1)-eth(1)-emeini(1,jj,i))*
-     &                  (stre(1)+stiini(1,jj,i))+
-     &                  (eloc(2)-eth(2)-emeini(2,jj,i))*
-     &                  (stre(2)+stiini(2,jj,i))+
-     &                  (eloc(3)-eth(3)-emeini(3,jj,i))*
-     &                  (stre(3)+stiini(3,jj,i)))/2.d0+
-     &         (eloc(4)-eth(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
-     &         (eloc(5)-eth(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
-     &         (eloc(6)-eth(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
-c                  write(*,*) 'resultsmech ',jj,i,enerini(jj,i)
-c                  write(*,*) (eloc(k),k=1,6)
-c                  write(*,*) (eth(k),k=1,6)
-c                  write(*,*) (emeini(k,jj,i),k=1,6)
-c                  write(*,*) (stre(k),k=1,6)
-c                  write(*,*) (stiini(k,jj,i),k=1,6)
-c                  write(*,*) 'resultsmech2 ',jj,i,ener(jj,i)
+     &         ((nmethod.eq.4).and.
+     &          ((iperturb(1).gt.1).and.(nlgeom_undo.eq.0)).and.
+     &           (ithermal(1).le.1))) then
+c               
+c               if(ithermal(1).eq.0) then
+c                  do m1=1,6
+c                     eth(m1)=0.d0
+c                  enddo
+c               endif
+c               if(nener.eq.1) then
 c                  ener(jj,i)=enerini(jj,i)+
-c     &                 ((eloc(1)-eth(1)-eme(1,jj,i))*
+c     &                 ((eloc(1)-eth(1)-emeini(1,jj,i))*
 c     &                  (stre(1)+stiini(1,jj,i))+
-c     &                  (eloc(2)-eth(2)-eme(2,jj,i))*
+c     &                  (eloc(2)-eth(2)-emeini(2,jj,i))*
 c     &                  (stre(2)+stiini(2,jj,i))+
-c     &                  (eloc(3)-eth(3)-eme(3,jj,i))*
+c     &                  (eloc(3)-eth(3)-emeini(3,jj,i))*
 c     &                  (stre(3)+stiini(3,jj,i)))/2.d0+
-c     &            (eloc(4)-eth(4)-eme(4,jj,i))*(stre(4)+stiini(4,jj,i))+
-c     &            (eloc(5)-eth(5)-eme(5,jj,i))*(stre(5)+stiini(5,jj,i))+
-c     &            (eloc(6)-eth(6)-eme(6,jj,i))*(stre(6)+stiini(6,jj,i))
-               endif
-               eme(1,jj,i)=eloc(1)-eth(1)
-               eme(2,jj,i)=eloc(2)-eth(2)
-               eme(3,jj,i)=eloc(3)-eth(3)
-               eme(4,jj,i)=eloc(4)-eth(4)
-               eme(5,jj,i)=eloc(5)-eth(5)
-               eme(6,jj,i)=eloc(6)-eth(6)
-            endif
-!
-            if((iout.gt.0).or.(iout.eq.-2).or.(kode.le.-100)) then
+c     &         (eloc(4)-eth(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
+c     &         (eloc(5)-eth(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
+c     &         (eloc(6)-eth(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
+c               endif
 c               eme(1,jj,i)=eloc(1)-eth(1)
 c               eme(2,jj,i)=eloc(2)-eth(2)
 c               eme(3,jj,i)=eloc(3)-eth(3)
 c               eme(4,jj,i)=eloc(4)-eth(4)
 c               eme(5,jj,i)=eloc(5)-eth(5)
 c               eme(6,jj,i)=eloc(6)-eth(6)
+!               
+               if(nener.eq.1) then
+                  ener(jj,i)=enerini(jj,i)+
+     &                 ((emec(1)-emeini(1,jj,i))*
+     &                  (stre(1)+stiini(1,jj,i))+
+     &                  (emec(2)-emeini(2,jj,i))*
+     &                  (stre(2)+stiini(2,jj,i))+
+     &                  (emec(3)-emeini(3,jj,i))*
+     &                  (stre(3)+stiini(3,jj,i)))/2.d0+
+     &         (emec(4)-emeini(4,jj,i))*(stre(4)+stiini(4,jj,i))+
+     &         (emec(5)-emeini(5,jj,i))*(stre(5)+stiini(5,jj,i))+
+     &         (emec(6)-emeini(6,jj,i))*(stre(6)+stiini(6,jj,i))
+               endif
+               eme(1,jj,i)=emec(1)
+               eme(2,jj,i)=emec(2)
+               eme(3,jj,i)=emec(3)
+               eme(4,jj,i)=emec(4)
+               eme(5,jj,i)=emec(5)
+               eme(6,jj,i)=emec(6)
+            endif
+!
+            if((iout.gt.0).or.(iout.eq.-2).or.(kode.le.-100)) then
 !
                eei(1,jj,i)=eloc(1)
                eei(2,jj,i)=eloc(2)
@@ -944,7 +1055,7 @@ c               eme(6,jj,i)=eloc(6)-eth(6)
 !     updating the kinetic energy
 !
             if(ikin.eq.1) then
-               
+!               
                call materialdata_rho(rhcon,nrhcon,imat,rho,t1l,
      &              ntmat_,ithermal)
                do m1=1,3
@@ -1008,7 +1119,7 @@ c               eme(6,jj,i)=eloc(6)-eth(6)
                enddo
 c     Bernhardi start
                if(lakonl(1:5).eq.'C3D8R') then
-                  call hgforce (fn,elas,a,gs,vl,mi,konl)
+                  call hgforce(fn,elas,a,gs,vl,mi,konl)
                endif
 c     Bernhardi end
             endif
@@ -1020,7 +1131,6 @@ c     Bernhardi end
 !              changing the displacement gradients into
 !              deformation gradients
 !
-c               if(kode.ne.-50) then
                if((kode.ne.-50).and.(kode.gt.-100)) then
 c     Bernhardi start
                   xkl(1,1)=vkl(1,1)+1.0d0
@@ -1076,6 +1186,30 @@ c     Bernhardi end
             enddo
             nal=nal+3*nope
          endif
+         
+!        Calculation of additional kinetic energy if selevtive mass scaling 
+!        is used: E_kin_add= v^T *lamda *v *0.5
+!
+         if((ikin.eq.1).and.
+     &      ((mscalmethod.eq.1).or.(mscalmethod.eq.3)))then
+            do m1=1,3
+               sum2=0.d0
+               do i1=1,nope
+                  sum1=0.d0
+                  do j1=1,nope
+                    if(i1.eq.j1)then
+                      scal=smscale(i)*(nope-1)
+                    else
+                      scal=-1*smscale(i)
+                    endif
+                    sum1=sum1+(veoldl(m1,j1))*scal
+                  enddo
+                  sum2=sum2+sum1*veoldl(m1,i1)
+                enddo
+                enerscal=enerscal+sum2
+             enddo
+         endif
+!         
       enddo
 !
       return
