@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -25,7 +25,8 @@
      &  nshcon,cocon,ncocon,ntmat_,sideload,icfd,inomat,pslavsurf,
      &  islavact,cdn,mortar,islavnode,nslavnode,ntie,islavsurf,time,
      &  ielprop,prop,veold,ne0,nmpc,ipompc,nodempc,labmpc,energyini,
-     &  energy)
+     &  energy,orname,xload,itiefac,pmastsurf,springarea,tieset,ipobody,
+     &  ibody,xbody,nbody)
 !
 !     - stores the results in the .dat file, if requested
 !       - nodal quantities at the nodes
@@ -43,18 +44,20 @@
       character*6 prlab(*)
       character*8 lakon(*)
       character*20 sideload(*),labmpc(*)
-      character*81 set(*),prset(*)
+      character*80 orname(*)
+      character*81 set(*),prset(*),tieset(3,*)
       character*87 filab(*)
 !
       integer kon(*),inum(*),iperm(20),mi(*),ielorien(mi(3),*),
-     &  ipkon(*),nactdof(0:mi(2),*),nodeboun(*),icompressible,
+     &  ipkon(*),nactdof(0:mi(2),*),nodeboun(*),compressible,
      &  nelemload(2,*),ndirboun(*),ielmat(mi(3),*),nrhcon(*),
-     &  inotr(2,*),iorienloc,iflag,nload,mt,nk,ne,ithermal(2),i,
+     &  inotr(2,*),iorienloc,iflag,nload,mt,nk,ne,ithermal(*),i,
      &  norien,iperturb(*),iout,nboun,nmethod,node,nshcon(*),
      &  nfield,ndim,nstate_,nset,istartset(*),iendset(*),ialset(*),
      &  nprint,ntrans,ikin,ncocon(2,*),ntmat_,icfd,inomat(*),mortar,
      &  islavact(*),islavnode(*),nslavnode(*),ntie,islavsurf(2,*),
-     &  ielprop(*),ne0,index,nmpc,ipompc(*),nodempc(3,*),nactdoh
+     &  ielprop(*),ne0,index,nmpc,ipompc(*),nodempc(3,*),nactdoh,
+     &  iextrapolate,itiefac(2,*),ipobody(2,*),ibody(3,*),nbody 
 !
       real*8 co(3,*),v(0:mi(2),*),stx(6,mi(1),*),stn(6,*),cdn(6,*),
      &  qfx(3,mi(1),*),qfn(3,*),orab(7,*),fn(0:mi(2),*),pslavsurf(3,*),
@@ -62,12 +65,16 @@
      &  ener(mi(1),*),enern(*),eei(6,mi(1),*),rhcon(0:1,ntmat_,*),
      &  ttime,xstate(nstate_,mi(1),*),trab(7,*),xstaten(nstate_,*),
      &  eme(6,mi(1),*),emn(6,*),shcon(0:3,ntmat_,*),cocon(0:6,ntmat_,*),
-     &  prop(*),veold(0:mi(2),*),energy(*),energyini(*)
+     &  prop(*),veold(0:mi(2),*),energy(*),energyini(*),xload(2,*),
+     &  pmastsurf,springarea(2,*),xbody(7,*)
+!
+!
 !
       data iflag /3/
       data iperm /5,6,7,8,1,2,3,4,13,14,15,16,9,10,11,12,17,18,19,20/
 !
       mt=mi(2)+1
+      iextrapolate=0
 !
 !     no print requests
 !
@@ -89,11 +96,11 @@
 !        - for thermal and thermomechanical calculations (ithermal(1)>1)
 !        - for electromagnetic calculations (mi(2)=5)
 !
-         if((nmethod.eq.4).and.(iperturb(1).gt.1).and.
-     &      (ithermal(1).le.1).and.(mi(2).ne.5)) then
-            call calcenergy(ipkon,lakon,kon,co,ener,mi,ne,thicke,
-     &           ielmat,energyini,energy,ielprop,prop)
-         endif
+c         if((nmethod.eq.4).and.(iperturb(1).gt.1).and.
+c     &      (ithermal(1).le.1).and.(mi(2).ne.5)) then
+c            call calcenergy(ipkon,lakon,kon,co,ener,mi,ne,thicke,
+c     &           ielmat,energyini,energy,ielprop,prop)
+c         endif
 !
          return
       endif
@@ -104,13 +111,43 @@
      &  prlab,prset,v,t1,fn,ipkon,lakon,stx,eei,xstate,ener,
      &  mi(1),nstate_,ithermal,co,kon,qfx,ttime,trab,inotr,ntrans,
      &  orab,ielorien,norien,nk,ne,inum,filab,vold,ikin,ielmat,thicke,
-     &  eme,islavsurf,mortar,time,ielprop,prop,veold)
+     &  eme,islavsurf,mortar,time,ielprop,prop,veold,orname,
+     &  nelemload,nload,sideload,xload,rhcon,nrhcon,ntmat_,ipobody,
+     &  ibody,xbody,nbody)
 !
-      icompressible=0
+!     for facial information (*section print): if forces and/or
+!     moments in sections are requested, the stresses have to be
+!     extrapolated from the integration points to the nodes first
+!
+      do i=1,nprint
+         if(prlab(i)(1:3).eq.'SOF') then
+            nfield=6
+            ndim=6
+            if((norien.gt.0).and.(filab(3)(6:6).eq.'L')) then
+               iorienloc=1
+            else
+               iorienloc=0
+            endif
+            cflag=filab(3)(5:5)
+            force=.false.
+!     
+            call extrapolate(stx,stn,ipkon,inum,kon,lakon,nfield,nk,
+     &           ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
+     &           vold,force,ielmat,thicke,ielprop,prop)
+            iextrapolate=1
+            exit
+         endif
+      enddo
+!
+      compressible=0
       call printoutface(co,rhcon,nrhcon,ntmat_,vold,shcon,nshcon,
-     &  cocon,ncocon,icompressible,istartset,iendset,ipkon,lakon,kon,
+     &  cocon,ncocon,compressible,istartset,iendset,ipkon,lakon,kon,
      &  ialset,prset,ttime,nset,set,nprint,prlab,ielmat,mi,
-     &  ithermal,nactdoh,icfd)
+     &     ithermal,nactdoh,icfd,time,stn)
+!
+      call printoutcontact(co,vold,lakon,ne0,ne,pslavsurf,stx,
+     &  prset,ttime,nprint,prlab,mi,ipkon,kon,springarea,
+     &  time,tieset,itiefac,ntie,pmastsurf)
 !
 !     interpolation in the original nodes of 1d and 2d elements
 !     this operation has to be performed in any case since
@@ -124,7 +161,7 @@
          cflag=filab(1)(5:5)
          force=.false.
          call map3dto1d2d(v,ipkon,inum,kon,lakon,nfield,nk,
-     &        ne,cflag,co,vold,force,mi)
+     &        ne,cflag,co,vold,force,mi,ielprop,prop)
       endif
 !
       if((filab(2)(1:4).eq.'NT  ').and.(ithermal(1).le.1)) then
@@ -133,7 +170,7 @@
             cflag=filab(2)(5:5)
             force=.false.
             call map3dto1d2d(t1,ipkon,inum,kon,lakon,nfield,nk,
-     &           ne,cflag,co,vold,force,mi)
+     &           ne,cflag,co,vold,force,mi,ielprop,prop)
          endif
       endif
 !
@@ -155,7 +192,7 @@
                cflag=' '
                force=.true.
                call map3dto1d2d(fn,ipkon,inum,kon,lakon,nfield,nk,
-     &              ne,cflag,co,vold,force,mi)
+     &              ne,cflag,co,vold,force,mi,ielprop,prop)
             endif
          endif
       endif
@@ -219,6 +256,7 @@
          call extrapolate(stx,stn,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
 !
       endif
 !
@@ -237,6 +275,7 @@
          call extrapolate(eei,een,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
 !     determining the mechanical strains in the nodes for output in 
@@ -255,6 +294,7 @@
          call extrapolate(eme,emn,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
 !     determining the plastic equivalent strain in the nodes 
@@ -269,9 +309,10 @@
          call extrapolate(xstate,epn,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
-!     determining the total energy in the nodes 
+!     determining the internal energy in the nodes 
 !     for output in frd format
 !
       if(filab(7)(1:4).eq.'ENER') then
@@ -283,6 +324,7 @@
          call extrapolate(ener,enern,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
 !     determining the internal state variables in the nodes 
@@ -302,6 +344,7 @@
          call extrapolate(xstate,xstaten,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
 !     determining the heat flux in the nodes for output in frd format
@@ -320,16 +363,14 @@
          call extrapolate(qfx,qfn,ipkon,inum,kon,lakon,nfield,nk,
      &        ne,mi(1),ndim,orab,ielorien,co,iorienloc,cflag,
      &        vold,force,ielmat,thicke,ielprop,prop)
+         iextrapolate=1
       endif
 !
 !     if no element quantities requested in the nodes: calculate
 !     inum if nodal quantities are requested: used in subroutine frd
 !     to determine which nodes are active in the model 
 !
-      if((filab(3)(1:4).ne.'S   ').and.(filab(4)(1:4).ne.'E   ').and.
-     &   (filab(6)(1:4).ne.'PEEQ').and.(filab(7)(1:4).ne.'ENER').and.
-     &   (filab(8)(1:4).ne.'SDV ').and.(filab(9)(1:4).ne.'HFL ').and.
-     &   (filab(42)(1:3).ne.'ECD').and.(filab(32)(1:4).ne.'ME  ').and.
+      if((iextrapolate.eq.0).and.
      &   ((nmethod.ne.4).or.(iperturb(1).ge.2))) then
 !
          nfield=0
@@ -337,10 +378,10 @@
          iorienloc=0
          cflag=filab(1)(5:5)
          call createinum(ipkon,inum,kon,lakon,nk,ne,cflag,nelemload,
-     &       nload,nodeboun,nboun,ndirboun,ithermal,co,vold,mi,ielmat)
+     &       nload,nodeboun,nboun,ndirboun,ithermal,co,vold,mi,ielmat,
+     &       ielprop,prop)
       endif
 !
-c      if(ithermal(1).gt.1) then
       if(ithermal(2).gt.1) then
 !
 !        next section is executed if at least one step is thermal
@@ -361,7 +402,7 @@ c      if(ithermal(1).gt.1) then
             if((sideload(i)(3:4).ne.'FC').and.
      &         (sideload(i)(3:4).ne.'NP')) cycle
             node=nelemload(2,i)
-            if(icfd.eq.1) then
+            if(icfd.ne.0) then
                if(node.gt.0) then
                   if(inomat(node).ne.0) cycle
                endif
@@ -379,7 +420,7 @@ c      if(ithermal(1).gt.1) then
          do i=1,nload
             if((sideload(i)(3:4).ne.'CR')) cycle
             node=nelemload(2,i)
-            if(icfd.eq.1) then
+            if(icfd.ne.0) then
                if(node.gt.0) then
                   if(inomat(node).ne.0) cycle
                endif
@@ -411,7 +452,7 @@ c      if(ithermal(1).gt.1) then
          do i=1,nboun
             node=nodeboun(i)
             if(inum(node).ne.0) cycle
-            if(icfd.eq.1) then
+            if(icfd.ne.0) then
                if(inomat(node).ne.0) cycle
             endif
             if((cflag.ne.' ').and.(ndirboun(i).eq.3)) cycle
