@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -26,7 +26,7 @@
      &  calcul_fn,calcul_qa,nal,nea,neb,ithermal,nelemload,nload,
      &  nmethod,reltime,sideload,xload,xloadold,pslavsurf,
      &  pmastsurf,mortar,clearini,plicon,nplicon,ielprop,prop,
-     &  iponoel,inoel)
+     &  iponoel,inoel,network,ipobody,xbody,ibody)
 !
 !     calculates the heat flux and the material tangent at the integration
 !     points and the internal concentrated flux at the nodes
@@ -44,9 +44,9 @@
      &  i,j,k,m1,kk,i1,m3,indexe,nope,norien,iperturb(*),iout,
      &  nal,nmpc,kode,imat,mint3d,iorien,istiff,ncmat_,nstate_,
      &  nplkcon(0:ntmat_,*),npmat_,calcul_fn,calcul_qa,nea,neb,
-     &  nelemload(2,*),nload,ithermal(2),nmethod,nopered,iloc,
+     &  nelemload(2,*),nload,ithermal(*),nmethod,nopered,
      &  jfaces,node,nplicon(0:ntmat_,*),null,ielprop(*),
-     &  iponoel(*),inoel(2,*)
+     &  iponoel(*),inoel(2,*),network,ipobody(2,*),ibody(3,*)
 !
       real*8 co(3,*),v(0:mi(2),*),shp(4,26),reltime,
      &  xl(3,26),vl(0:mi(2),26),elcon(0:ncmat_,ntmat_,*),
@@ -54,12 +54,13 @@
      &  rho,fn(0:mi(2),*),tnl(19),timeend(2),q(0:mi(2),26),
      &  vkl(0:3,3),t0(*),vold(0:mi(2),*),coefmpc(*),
      &  springarea(2,*),elconloc(21),cocon(0:6,ntmat_,*),
-     &  shcon(0:3,ntmat_,*),sph,c1,xi,et,ze,xsj,qa(3),t0l,t1l,dtime,
+     &  shcon(0:3,ntmat_,*),sph,c1,xi,et,ze,xsj,qa(*),t0l,t1l,dtime,
      &  weight,pgauss(3),coconloc(6),qflux(3),time,ttime,
      &  t1lold,plkcon(0:2*npmat_,ntmat_,*),xstiff(27,mi(1),*),
      &  xstate(nstate_,mi(1),*),xstateini(nstate_,mi(1),*),
      &  xload(2,*),xloadold(2,*),clearini(3,9,*),pslavsurf(3,*),
-     &  pmastsurf(6,*),plicon(0:2*npmat_,ntmat_,*),prop(*)
+     &  pmastsurf(6,*),plicon(0:2*npmat_,ntmat_,*),prop(*),
+     &  xbody(7,*)
 !
       include "gauss.f"
 !
@@ -75,17 +76,24 @@
 !
       do i=nea,neb
 !
-         if(ipkon(i).lt.0) cycle
+!        only structural elements (no fluid elements)
 !
-!        no 3D-fluid elements
+         if((ipkon(i).lt.0).or.(lakon(i)(1:1).eq.'F')) cycle
 !
-         if(lakon(i)(1:1).eq.'F') cycle
          if(lakon(i)(1:7).eq.'DCOUP3D') cycle
+!
+!        strainless reactivated elements are labeled by a negative
+!        value of ielmat. If the step is mechanical or thermo-mechanical
+!        resultsmech is called before and the negative value is
+!        reversed. If the step is purely thermal the negative sign
+!        persists and has to be reverted here.
+!
+         ielmat(1,i)=abs(ielmat(1,i))
 !
          imat=ielmat(1,i)
          amat=matname(imat)
          if(norien.gt.0) then
-            iorien=ielorien(1,i)
+            iorien=max(0,ielorien(1,i))
          else
             iorien=0
          endif
@@ -135,7 +143,8 @@ c!
 c!           local contact spring number
 c!
 c            if(lakon(i)(7:7).eq.'C') konl(nope+1)=kon(indexe+nope+1)
-         elseif(lakon(i)(1:2).eq.'D ') then
+         elseif((lakon(i)(1:2).eq.'D ').or.
+     &          ((lakon(i)(1:1).eq.'D').and.(network.eq.1))) then
 !
 !           no entry or exit elements
 !
@@ -223,13 +232,12 @@ c            if(lakon(i)(7:7).eq.'C') konl(nope+1)=kon(indexe+nope+1)
      &                    springarea(1,konl(nope+1)),timeend,matname,
      &                    konl(nope),i,istep,iinc,iperturb)
                      elseif(mortar.eq.1) then
-                        iloc=kon(indexe+nope+1)
                         jfaces=kon(indexe+nope+2)
                         igauss=kon(indexe+nope+1)
                         node=0
                         call springforc_f2f_th(xl,vl,imat,elcon,nelcon,
      &                    tnl,ncmat_,ntmat_,nope,lakonl,kode,elconloc,
-     &                    plicon,nplicon,npmat_,mi,springarea(1,iloc),
+     &                    plicon,nplicon,npmat_,mi,springarea(1,igauss),
      &                    nmethod,reltime,jfaces,igauss,
      &                    pslavsurf,pmastsurf,clearini,timeend,istep,
      &                    iinc,plkcon,nplkcon,node,i,matname)
@@ -241,13 +249,14 @@ c            if(lakon(i)(7:7).eq.'C') konl(nope+1)=kon(indexe+nope+1)
 !
                   call advecforc(nope,vl,ithermal,xl,nelemload,
      &                 i,nload,lakon,xload,istep,time,ttime,
-     &                 dtime,sideload,vold,mi,xloadold,reltime,nmethod,
+     &                 dtime,sideload,v,mi,xloadold,reltime,nmethod,
      &                 tnl,iinc,iponoel,inoel,ielprop,prop,ielmat,shcon,
      &                 nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,
-     &                 ncocon)
+     &                 ncocon,ipobody,xbody,ibody)
                endif
 !
-            elseif(lakonl(1:2).eq.'D ') then
+            elseif((lakonl(1:2).eq.'D ').or.
+     &             ((lakonl(1:1).eq.'D').and.(network.eq.1))) then
 !
 !              generic networkelement
 !
@@ -362,8 +371,11 @@ c            if(lakon(i)(7:7).eq.'C') konl(nope+1)=kon(indexe+nope+1)
                enddo
             elseif(lakon(i)(4:6).eq.'20 ') then
                nopered=20
-               call lintemp_th(t0,vold,konl,nopered,kk,t0l,t1lold,mi)
-               call lintemp_th(t0,v,konl,nopered,kk,t0l,t1l,mi)
+               call lintemp_th1(vold,konl,nopered,kk,t1lold,mi)
+               call lintemp_th1(v,konl,nopered,kk,t1l,mi)
+            elseif(lakon(i)(4:6).eq.'10T') then
+               call linscal10(vold,konl,t1lold,mi(2),shp)
+               call linscal10(v,konl,t1l,mi(2),shp)
             else
                do i1=1,nope
                   t1lold=t1lold+shp(4,i1)*vold(0,konl(i1))
