@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !
       subroutine rigidmpc(ipompc,nodempc,coefmpc,irefnode,irotnode,
      &  labmpc,nmpc,nmpc_,mpcfree,ikmpc,ilmpc,nk,nk_,nodeboun,ndirboun,
-     &  ikboun,ilboun,nboun,nboun_,node,typeboun,co)
+     &  ikboun,ilboun,nboun,nboun_,node,typeboun,co,jmin,jmax)
 !
 !     generates three rigid body MPC's for node "node" about reference
 !     (translational) node irefnode and rotational node irotnode 
@@ -29,23 +29,24 @@
       character*20 labmpc(*)
 !
       integer ipompc(*),nodempc(3,*),nmpc,nmpc_,mpcfree,nk,nk_,
-     &  ikmpc(*),
+     &  ikmpc(*),jmin,jmax,
      &  ilmpc(*),node,id,mpcfreeold,j,idof,l,nodeboun(*),
      &  ndirboun(*),ikboun(*),ilboun(*),nboun,nboun_,irefnode,
      &  irotnode
 !
       real*8 coefmpc(*),co(3,*),e(3,3,3)
 !
-      data e /0.,0.,0.,0.,0.,-1.,0.,1.,0.,
-     &        0.,0.,1.,0.,0.,0.,-1.,0.,0.,
-     &        0.,-1.,0.,1.,0.,0.,0.,0.,0./
+      data e /0.d0,0.d0,0.d0,0.d0,0.d0,-1.d0,0.d0,1.d0,0.d0,
+     &        0.d0,0.d0,1.d0,0.d0,0.d0,0.d0,-1.d0,0.d0,0.d0,
+     &        0.d0,-1.d0,0.d0,1.d0,0.d0,0.d0,0.d0,0.d0,0.d0/
 !
       nk=nk+1
       if(nk.gt.nk_) then
          write(*,*) '*ERROR in rigidmpc: increase nk_'
          call exit(201)
       endif
-      do j=1,3
+c      do j=1,3
+      do j=jmin,jmax
          idof=8*(node-1)+j
          call nident(ikmpc,idof,nmpc,id)
          if(id.gt.0) then
