@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -31,7 +31,7 @@
       integer ogden,hyperfoam,taylor
 !
       integer nelconst,kode,kk(84),i,j,k,l,m,nt,icmd,istart,iend,
-     &  nc,n,ithermal,ii,jj,mm,neig
+     &  nc,n,ithermal(*),ii,jj,mm,neig
 !
       real*8 elconloc(*),elas(*),emec(*),didc(3,3,3),
      &  d2idc2(3,3,3,3,3),dibdc(3,3,3),d2ibdc2(3,3,3,3,3),dudc(3,3),
@@ -412,7 +412,7 @@ c      if(icmd.eq.1) then
                         d2ldc2(k,l,m,n,i)=(dcos(tt+(i-1)*2.d0*pi/3.d0)*
      &                   (-(2.d0*v1*didc(k,l,1)-3.d0*didc(k,l,2))*
      &                    (2.d0*v1*didc(m,n,1)-3.d0*didc(m,n,2))/
-     &                    (6.d0*(v1*v1-3.d0*v2)**1.5)+
+     &                    (6.d0*(v1*v1-3.d0*v2)**1.5d0)+
      &                    (2.d0*didc(k,l,1)*didc(m,n,1)+2.d0*v1*
      &                     d2idc2(k,l,m,n,1)-3.d0*d2idc2(k,l,m,n,2))/
      &                     (3.d0*dsqrt(v1*v1-3.d0*v2)))
