@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
       subroutine scavenge_pump(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &        nactdog,identity,ielprop,prop,iflag,v,xflow,f,
      &        nodef,idirf,df,cp,r,physcon,dvi,numf,set,ntmat_,mi,
-     &        ttime,time,iaxial)
+     &        ttime,time,iaxial,iplausi)
 !     
 !     scavenge pump element
 !
@@ -32,10 +32,12 @@
 !     
       integer nelem,nactdog(0:3,*),numf,node1,node2,nodem,
      &     ielprop(*),nodef(5),idirf(5),index,iflag,
-     &     ipkon(*),kon(*),mi(*),ntmat_,iaxial
+     &     ipkon(*),kon(*),mi(*),ntmat_,iaxial,iplausi
 !     
       real*8 prop(*),v(0:mi(2),*),xflow,f,df(5),kappa,cp,physcon(*)
      &     ,dvi,R,ttime,time
+!
+!
 !  
       if (iflag.eq.0) then
          identity=.true.
@@ -49,6 +51,7 @@
          endif
 !     
       elseif (iflag.eq.1) then      
+         if(v(1,nodem).ne.0.d0) return
 !
       elseif (iflag.eq.2) then
 !
