@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -36,9 +36,7 @@
 !
       real*8 xi,et,ze,xsj,omg,omh,omr,opg,oph,opr
 !
-      intent(in) xi,et,ze,xl,iflag
 !
-      intent(out) shp,xsj
 !
 !     shape functions and their glocal derivatives
 !
