@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -36,9 +36,7 @@
       real*8 z1,z2,z3,z4,z5,z6,z7,z8
       real*8 gb(8,4),gs(8,4),s0,a
 !
-      intent(in) xl
 !
-      intent(out) shp,gs,a,xsj
 !
       gb = reshape((
      &      / 1.0d0, 1.0d0,-1.0d0,-1.0d0,-1.0d0,-1.0d0, 1.0d0, 1.0d0,
@@ -208,7 +206,7 @@ c     calculate hourglass control stiffness factor a
 c     (to be used in hgstiffness() and hgforce())
 c
 c     in ABAQUS, a 0.005 is used as default value.
-      a=0.0005*a/vol
+      a=0.0005d0*a/vol
 c
       return
       end
