@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
       subroutine shellsections(inpc,textpart,set,istartset,iendset,
      &  ialset,nset,ielmat,matname,nmat,ielorien,orname,norien,
      &  thicke,kon,ipkon,offset,irstrt,istep,istat,n,iline,ipol,
-     &  inl,ipoinp,inp,lakon,iaxial,ipoinpc,mi,icomposite,nelcon)
+     &  inl,ipoinp,inp,lakon,iaxial,ipoinpc,mi,icomposite,nelcon,ier)
 !
 !     reading the input deck: *SHELL SECTION
 !
@@ -34,18 +34,19 @@
       character*132 textpart(16)
 !
       integer mi(*),istartset(*),iendset(*),ialset(*),ielmat(mi(3),*),
-     &  ielorien(mi(3),*),kon(*),ipkon(*),indexe,irstrt,nset,nmat,
-     &  norien,nlayer,iset,icomposite,nelcon(2,*),
+     &  ielorien(mi(3),*),kon(*),ipkon(*),indexe,irstrt(*),nset,nmat,
+     &  norien,nlayer,iset,icomposite,nelcon(2,*),ier,numnod,
      &  istep,istat,n,key,i,j,k,l,imaterial,iorientation,ipos,
      &  iline,ipol,inl,ipoinp(2,*),inp(3,*),iaxial,ipoinpc(0:*)
 !
       real*8 thicke(mi(3),*),thickness,offset(2,*),offset1
 !
-      if((istep.gt.0).and.(irstrt.ge.0)) then
+      if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) 
      &        '*ERROR reading *SHELL SECTION: *SHELL SECTION should'
          write(*,*) '  be placed before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       nodalthickness=.false.
@@ -68,8 +69,11 @@
             nodalthickness=.true.
          elseif(textpart(i)(1:7).eq.'OFFSET=') then
             read(textpart(i)(8:27),'(f20.0)',iostat=istat) offset1
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*SHELL SECTION%",ier)
+               return
+            endif
          elseif(textpart(i)(1:9).eq.'COMPOSITE') then
             composite=.true.
          else
@@ -92,26 +96,27 @@
             write(*,*) 
      &        '*ERROR reading *SHELL SECTION: nonexistent material'
             call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
-            call exit(201)
+     &           "*SHELL SECTION%",ier)
+            return
          endif
          imaterial=i
       elseif(material(1:1).ne.' ') then
          write(*,*) '*ERROR reading *SHELL SECTION: COMPOSITE and'
          write(*,*) '       MATERIAL are mutually exclusive parameters'
-         call exit(201)
+         ier=1
+         return
       endif
 !
 !     check for the existence of the orientation, if any
 !
       if(orientation(1:1).eq.' ') then
          iorientation=0
-      elseif(nelcon(1,i).eq.2) then
-         write(*,*) '*INFO reading *SOLID SECTION: an orientation'
-         write(*,*) '      is for isotropic materials irrelevant'
-         call inputinfo(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         iorientation=0
+c      elseif(nelcon(1,i).eq.2) then
+c         write(*,*) '*INFO reading *SHELL SECTION: an orientation'
+c         write(*,*) '      is for isotropic materials irrelevant'
+c         call inputinfo(inpc,ipoinpc,iline,
+c     &"*SHELL SECTION%")
+c         iorientation=0
       else
          do i=1,norien
             if(orname(i).eq.orientation) exit
@@ -120,8 +125,8 @@
             write(*,*)
      &         '*ERROR reading *SHELL SECTION: nonexistent orientation'
             call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
-            call exit(201)
+     &           "*SHELL SECTION%",ier)
+            return
          endif
          iorientation=i
       endif
@@ -136,8 +141,8 @@
          write(*,*) '*ERROR reading *SHELL SECTION: element set ',elset
          write(*,*) '       has not yet been defined. '
          call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
-         call exit(201)
+     &        "*SHELL SECTION%",ier)
+         return
       endif
       iset=i
 !
@@ -153,9 +158,24 @@
                write(*,*) 
      &    '*ERROR reading *SHELL SECTION: shell thickness is lacking'
                call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
+     &              "*SHELL SECTION%",ier)
+               return
+            endif
+            if(thickness.le.0.d0) then
+               write(*,*) 
+     &    '*ERROR reading *SHELL SECTION: shell thickness is zero'
+               write(*,*) '       or negative'
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*SHELL SECTION%",ier)
             endif
             if(iaxial.eq.180) thickness=thickness/iaxial
+         else
+!
+!           for those elements for which nodal thickness is activated
+!           the thickness is set to -1.d0
+!
+            thickness=-1.d0
+         endif
             do j=istartset(iset),iendset(iset)
                if(ialset(j).gt.0) then
                   if(lakon(ialset(j))(1:1).ne.'S') then
@@ -165,10 +185,12 @@
      &                 '       only be used for shell elements.'
                      write(*,*) '       Element ',ialset(j),
      &                 ' is not a shell element.'
-                     call exit(201)
+                     ier=1
+                     return
                   endif
                   indexe=ipkon(ialset(j))
-                  do l=1,8
+                  read(lakon(ialset(j))(2:2),'(i1)') numnod
+                  do l=1,numnod
                      thicke(1,indexe+l)=thickness
                   enddo
                   ielmat(1,ialset(j))=imaterial
@@ -186,10 +208,12 @@
      &                    '       only be used for shell elements.'
                         write(*,*) '       Element ',k,
      &                    ' is not a shell element.'
-                        call exit(201)
+                        ier=1
+                        return
                      endif
                      indexe=ipkon(k)
-                     do l=1,8
+                     read(lakon(k)(2:2),'(i1)') numnod
+                     do l=1,numnod
                         thicke(1,indexe+l)=thickness
                      enddo
                      ielmat(1,k)=imaterial
@@ -200,14 +224,15 @@
             enddo
             call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &           ipoinp,inp,ipoinpc)
-         endif
+c         endif
 !     
       else
          if(nodalthickness) then
             write(*,*) '*ERROR reading shellsections: for composite'
             write(*,*) '       materials is the parameter NODAL'
             write(*,*) '       THICKNESS not allowed'
-            call exit(201)
+            ier=1
+            return
          endif
 !
 c         icomposite=1
@@ -218,7 +243,8 @@ c         icomposite=1
                write(*,*) 
      &     '*ERROR reading *SHELL SECTION: shell thickness is lacking'
                call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
+     &              "*SHELL SECTION%",ier)
+               return
             endif
             if(iaxial.eq.180) thickness=thickness/iaxial
 !     
@@ -229,7 +255,8 @@ c         icomposite=1
                write(*,*) 
      &              '*ERROR reading *SHELL SECTION: no material defined'
                call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
+     &              "*SHELL SECTION%",ier)
+               return
             endif
 !     
 !     check for the existence of the material
@@ -241,8 +268,8 @@ c         icomposite=1
                write(*,*) 
      &      '*ERROR reading *SHELL SECTION: nonexistent material'
                call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
-               call exit(201)
+     &              "*SHELL SECTION%",ier)
+               return
             endif
             imaterial=i
 !     
@@ -264,8 +291,8 @@ c            else
      &        '*ERROR reading *SHELL SECTION: nonexistent orientation'
                   write(*,*) '  '
                   call inputerror(inpc,ipoinpc,iline,
-     &"*SHELL SECTION%")
-                  call exit(201)
+     &                 "*SHELL SECTION%",ier)
+                  return
                endif
                iorientation=i
             endif
@@ -274,19 +301,22 @@ c            else
 !     
             do j=istartset(iset),iendset(iset)
                if(ialset(j).gt.0) then
-                  if(lakon(ialset(j))(1:3).ne.'S8R') then
+                  if((lakon(ialset(j))(1:3).ne.'S8R').and.
+     &               (lakon(ialset(j))(1:2).ne.'S6')) then
                      write(*,*) 
      &           '*ERROR reading *SHELL SECTION: *SHELL SECTION'
                      write(*,*) 
      &                 '       with the option COMPOSITE can'
                      write(*,*) 
-     &                 '       only be used for S8R shell elements.'
+     &               '       only be used for S8R or S6 shell elements.'
                      write(*,*) '       Element ',ialset(j),
-     &                 ' is not a S8R shell element.'
-                     call exit(201)
+     &                 ' is not a S8R nor a S6 shell element.'
+                     ier=1
+                     return
                   endif
                   indexe=ipkon(ialset(j))
-                  do l=1,8
+                  read(lakon(ialset(j))(2:2),'(i1)') numnod
+                  do l=1,numnod
                      thicke(nlayer,indexe+l)=thickness
                   enddo
                   ielmat(nlayer,ialset(j))=imaterial
@@ -298,19 +328,22 @@ c            else
                   do
                      k=k-ialset(j)
                      if(k.ge.ialset(j-1)) exit
-                     if(lakon(k)(1:3).ne.'S8R') then
+                     if((lakon(k)(1:3).ne.'S8R').and.
+     &                  (lakon(k)(1:2).ne.'S6')) then
                         write(*,*) 
      &              '*ERROR reading *SHELL SECTION: *SHELL SECTION'
                         write(*,*) 
      &                       '    with the option COMPOSITE can'
                         write(*,*) 
-     &                    '       only be used for shell elements.'
+     &               '       only be used for S8R or S6 shell elements.'
                         write(*,*) '       Element ',k,
-     &                    ' is not a S8R shell element.'
-                        call exit(201)
+     &                    ' is not a S8R nor a S6 shell element.'
+                        ier=1
+                        return
                      endif
                      indexe=ipkon(k)
-                     do l=1,8
+                     read(lakon(k)(2:2),'(i1)') numnod
+                     do l=1,numnod
                         thicke(nlayer,indexe+l)=thickness
                      enddo
                      ielmat(nlayer,k)=imaterial
