@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -27,23 +27,25 @@
      &  neigh15(2,18),neigh20(2,24)
 !
       real*8 dist,distmin,co(3,*)
+!
+!
 !
       neigh2=reshape((/1,2/),(/2,1/))
       neigh4=reshape((/1,2,2,3,3,1,1,4,2,4,3,4/),(/2,6/))
       neigh6=reshape((/1,2,2,3,3,1,4,5,5,6,6,4,1,4,2,5,3,6/),(/2,9/))
       neigh8=reshape((/1,2,2,3,3,4,4,1,5,6,6,7,7,8,8,5,
      &                 1,5,2,6,3,7,4,8/),(/2,12/))
-      neigh10=reshape((/1,5,5,2,2,6,6,3,3,7,7,1,
-     &                  1,8,8,4,2,9,9,4,3,10,10,4/),(/2,12/))
-      neigh15=reshape((/1,7,7,2,2,8,8,3,3,9,9,1,
-     &                  4,10,10,5,5,11,11,6,6,12,12,4,
-     &                  1,13,13,4,2,14,14,5,3,15,15,6/),(/2,18/))
-      neigh20=
-     &     reshape((/1,9,9,2,2,10,10,3,3,11,11,4,4,12,12,1,
-     &               5,13,13,6,6,14,14,7,7,15,15,8,8,16,16,5,
-     &               1,17,17,5,2,18,18,6,3,19,19,7,4,20,20,8/),(/2,24/))
+c      neigh10=reshape((/1,5,5,2,2,6,6,3,3,7,7,1,
+c     &                  1,8,8,4,2,9,9,4,3,10,10,4/),(/2,12/))
+c      neigh15=reshape((/1,7,7,2,2,8,8,3,3,9,9,1,
+c     &                  4,10,10,5,5,11,11,6,6,12,12,4,
+c     &                  1,13,13,4,2,14,14,5,3,15,15,6/),(/2,18/))
+c      neigh20=
+c     &     reshape((/1,9,9,2,2,10,10,3,3,11,11,4,4,12,12,1,
+c     &               5,13,13,6,6,14,14,7,7,15,15,8,8,16,16,5,
+c     &               1,17,17,5,2,18,18,6,3,19,19,7,4,20,20,8/),(/2,24/))
 
-!     determining the distance between nodes 
+!     determining the smallest distance between nodes 
 !     
       distmin=1.d30
 !
@@ -52,14 +54,21 @@
          if(lakon(i)(1:1).eq.'F') cycle
          indexe=ipkon(i)
          if(lakon(i)(4:4).eq.'2') then
-            n=24
+            if((lakon(i)(7:7).eq.'A').or.
+     &           (lakon(i)(7:7).eq.'L').or.
+     &           (lakon(i)(7:7).eq.'S').or.
+     &           (lakon(i)(7:7).eq.'E')) then
+               n=4
+            else
+               n=12
+            endif
             do j=1,n
-               j1=kon(indexe+neigh20(1,j))
-               j2=kon(indexe+neigh20(2,j))
+               j1=kon(indexe+neigh8(1,j))
+               j2=kon(indexe+neigh8(2,j))
                dist=(co(1,j1)-co(1,j2))**2+
      &              (co(2,j1)-co(2,j2))**2+
      &              (co(3,j1)-co(3,j2))**2
-               distmin=min(dist,distmin)
+               distmin=min(dist/4.d0,distmin)
             enddo
          elseif(lakon(i)(1:8).eq.'ESPRNGA1') then
             n=1
@@ -72,7 +81,14 @@
                distmin=min(dist,distmin)
             enddo
          elseif(lakon(i)(4:4).eq.'8') then
-            n=12
+            if((lakon(i)(7:7).eq.'A').or.
+     &           (lakon(i)(7:7).eq.'L').or.
+     &           (lakon(i)(7:7).eq.'S').or.
+     &           (lakon(i)(7:7).eq.'E')) then
+               n=4
+            else
+               n=12
+            endif
             do j=1,n
                j1=kon(indexe+neigh8(1,j))
                j2=kon(indexe+neigh8(2,j))
@@ -92,17 +108,24 @@
                distmin=min(dist,distmin)
             enddo
          elseif(lakon(i)(4:5).eq.'10') then
-            n=12
+            n=6
             do j=1,n
-               j1=kon(indexe+neigh10(1,j))
-               j2=kon(indexe+neigh10(2,j))
+               j1=kon(indexe+neigh4(1,j))
+               j2=kon(indexe+neigh4(2,j))
                dist=(co(1,j1)-co(1,j2))**2+
      &              (co(2,j1)-co(2,j2))**2+
      &              (co(3,j1)-co(3,j2))**2
-               distmin=min(dist,distmin)
+               distmin=min(dist/4.d0,distmin)
             enddo
          elseif(lakon(i)(4:4).eq.'6') then
-            n=9
+            if((lakon(i)(7:7).eq.'A').or.
+     &           (lakon(i)(7:7).eq.'L').or.
+     &           (lakon(i)(7:7).eq.'S').or.
+     &           (lakon(i)(7:7).eq.'E')) then
+               n=3
+            else
+               n=9
+            endif
             do j=1,n
                j1=kon(indexe+neigh6(1,j))
                j2=kon(indexe+neigh6(2,j))
@@ -112,30 +135,30 @@
                distmin=min(dist,distmin)
             enddo
          elseif(lakon(i)(4:5).eq.'15') then
-            n=18
+            if((lakon(i)(7:7).eq.'A').or.
+     &           (lakon(i)(7:7).eq.'L').or.
+     &           (lakon(i)(7:7).eq.'S').or.
+     &           (lakon(i)(7:7).eq.'E')) then
+               n=3
+            else
+               n=9
+            endif
             do j=1,n
-               j1=kon(indexe+neigh15(1,j))
-               j2=kon(indexe+neigh15(2,j))
+               j1=kon(indexe+neigh6(1,j))
+               j2=kon(indexe+neigh6(2,j))
                dist=(co(1,j1)-co(1,j2))**2+
      &              (co(2,j1)-co(2,j2))**2+
      &              (co(3,j1)-co(3,j2))**2
-               distmin=min(dist,distmin)
+               distmin=min(dist/4.d0,distmin)
             enddo
          else
             cycle
          endif
       enddo
+!
+c      distmin=dsqrt(distmin)*1.0e-04
+      distmin=dsqrt(distmin)*1.0e-06
+      write(*,*) 'smalldist ',distmin
 !     
-!     0.1% of the smallest distance used for the variation
-!     
-      distmin=(distmin**0.5)*0.001
-!     
-!     Write results in file
-!     
-!     open(16,file='results_distance.out',status='unknown')
-!     write(16,'(1x,i6,1x,i6,1x,f5.4)') mindistnod(1,1),
-!     & mindistnod(2,1),distmin
-!     close(16)  
-      
       return
       end
