@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,11 +20,13 @@
      &  ialset,nset,ielmat,matname,nmat,ielorien,orname,norien,
      &  lakon,thicke,kon,ipkon,irstrt,istep,istat,n,iline,ipol,inl,
      &  ipoinp,inp,cs,mcs,iaxial,ipoinpc,mi,co,ixfree,xnor,iponor,
-     &  nelcon)
+     &  ier,orab)
 !
 !     reading the input deck: *SOLID SECTION
 !
       implicit none
+!
+      logical nodalthickness
 !
       character*1 inpc(*)
       character*8 lakon(*)
@@ -33,21 +35,25 @@
       character*132 textpart(16)
 !
       integer mi(*),istartset(*),iendset(*),ialset(*),ielmat(mi(3),*),
-     &  ielorien(mi(3),*),kon(*),ipkon(*),indexe,irstrt,nset,nmat,
+     &  ielorien(mi(3),*),kon(*),ipkon(*),indexe,irstrt(*),nset,nmat,
      &  norien,ielem,node1,node2,m,indexx,ixfree,iponor(2,*),
      &  istep,istat,n,key,i,j,k,l,imaterial,iorientation,ipos,
      &  iline,ipol,inl,ipoinp(2,*),inp(3,*),mcs,iaxial,ipoinpc(0:*),
-     &  nelcon(2,*)
+     &  ier,numnod
 !
       real*8 thicke(mi(3),*),thickness,pi,cs(17,*),xn(3),co(3,*),p(3),
-     &  dd,xnor(*)
+     &     dd,xnor(*),orab(7,*)
 !
-      if((istep.gt.0).and.(irstrt.ge.0)) then
+!
+!
+      if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) '*ERROR reading *SOLID SECTION: *SOLID SECTION'
          write(*,*)'       should be placed before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
+      nodalthickness=.false.
       pi=4.d0*datan(1.d0)
 !
       orientation='
@@ -66,6 +72,8 @@
             elset(81:81)=' '
             ipos=index(elset,' ')
             elset(ipos:ipos)='E'
+         elseif(textpart(i)(1:14).eq.'NODALTHICKNESS') then
+            nodalthickness=.true.
          else
             write(*,*) 
      &      '*WARNING reading *SOLID SECTION: parameter not recognized:'
@@ -76,7 +84,7 @@
          endif
       enddo
 !
-!     check for the existence of the set,the material and orientation
+!     check for the existence of the material
 !
       do i=1,nmat
          if(matname(i).eq.material) exit
@@ -95,19 +103,15 @@
      &      '*ERROR reading *SOLID SECTION: nonexistent material'
          write(*,*) '  '
          call inputerror(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         call exit(201)
+     &        "*SOLID SECTION%",ier)
+         return
       endif
       imaterial=i
+!
+!     check for the existence of the orientation
 !
       if(orientation.eq.'                    ') then
          iorientation=0
-      elseif(nelcon(1,i).eq.2) then
-         write(*,*) '*INFO reading *SOLID SECTION: an orientation'
-         write(*,*) '      is for isotropic materials irrelevant'
-         call inputinfo(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         iorientation=0
       else
          do i=1,norien
             if(orname(i).eq.orientation) exit
@@ -117,19 +121,21 @@
      &       '*ERROR reading *SOLID SECTION: nonexistent orientation'
             write(*,*) '  '
             call inputerror(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-            call exit(201)
+     &           "*SOLID SECTION%",ier)
+            return
          endif
          iorientation=i
       endif
+!
+!     check for the existence of the set
 !
       if(ipos.eq.0) then
          write(*,*) '*ERROR reading *SOLID SECTION: no element set ',
      &        elset
          write(*,*) '       was been defined. '
          call inputerror(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         call exit(201)
+     &        "*SOLID SECTION%",ier)
+         return
       endif
       do i=1,nset
          if(set(i).eq.elset) exit
@@ -139,8 +145,8 @@
          write(*,*) '*ERROR reading *SOLID SECTION: element set ',elset
          write(*,*) '  has not yet been defined. '
          call inputerror(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
-         call exit(201)
+     &        "*SOLID SECTION%",ier)
+         return
       endif
 !
 !     assigning the elements of the set the appropriate material
@@ -148,17 +154,35 @@
 !
       do j=istartset(i),iendset(i)
          if(ialset(j).gt.0) then
-            if((lakon(ialset(j))(1:1).eq.'B').or.
-     &         (lakon(ialset(j))(1:1).eq.'S')) then
+            k=ialset(j)
+            if((lakon(k)(1:1).eq.'B').or.
+     &         (lakon(k)(1:1).eq.'S')) then
                write(*,*) 
      &          '*ERROR reading *SOLID SECTION: *SOLID SECTION can'
                write(*,*) '       not be used for beam or shell elements
      &'
-               write(*,*) '       Faulty element: ',ialset(j)
-               call exit(201)
+               write(*,*) '       Faulty element: ',k
+               ier=1
+               return
+            endif
+            ielmat(1,k)=imaterial
+            if(ielorien(1,k).lt.0) then
+!
+!              an orientation based on a distribution has the
+!              same name as the distribution. However, to a
+!              distribution which is not used in any orientation
+!              definition no local coordinate system has been
+!              assigned (i.e. orab(7,..)=0.d0)
+!
+               if((orname(-ielorien(1,k)).eq.orientation).and.
+     &              (orab(7,-ielorien(1,k)).ne.0.d0)) then
+                  ielorien(1,k)=-ielorien(1,k)
+               else
+                  ielorien(1,k)=iorientation
+               endif
+            else
+               ielorien(1,k)=iorientation
             endif
-            ielmat(1,ialset(j))=imaterial
-            ielorien(1,ialset(j))=iorientation
          else
             k=ialset(j-2)
             do
@@ -171,31 +195,55 @@
                   write(*,*) '       not be used for beam or shell eleme
      &nts'
                   write(*,*) '       Faulty element: ',k
-                  call exit(201)
+                  ier=1
+                  return
                endif
                ielmat(1,k)=imaterial
-               ielorien(1,k)=iorientation
+               if(ielorien(1,k).lt.0) then
+!
+!                 an orientation based on a distribution has the
+!                 same name as the distribution. However, to a
+!                 distribution which is not used in any orientation
+!                 definition no local coordinate system has been
+!                 assigned (i.e. orab(7,..)=0.d0)
+!     
+                  if((orname(-ielorien(1,k)).eq.orientation).and.
+     &                 (orab(7,-ielorien(1,k)).ne.0.d0)) then
+                     ielorien(1,k)=-ielorien(1,k)
+                  else
+                     ielorien(1,k)=iorientation
+                  endif
+               else
+                  ielorien(1,k)=iorientation
+               endif
             enddo
          endif
       enddo
 !
       call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &     ipoinp,inp,ipoinpc)
+      if(istat.lt.0) return
 !
-!     assigning a thickness to plane stress elements and an angle to
+!     assigning a thickness to plane stress/strain elements and an angle to
 !     axisymmetric elements
 !
-cccc      if((key.eq.0).or.(lakon(ialset(istartset(i)))(1:2).eq.'CA')) then
-         if(key.eq.0) then
-            read(textpart(1)(1:20),'(f20.0)',iostat=istat) thickness
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SOLID SECTION%")
+      if(key.eq.0) then
+!
+!        second line with thickness given
+!        
+         read(textpart(1)(1:20),'(f20.0)',iostat=istat) thickness
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*SOLID SECTION%",ier)
+            return
+         endif
 !
 !        for axial symmetric structures:
 !           thickness for axial symmetric elements: 2 degrees
 !           thickness for plane stress elements: reduced by 180
 !           thickness for plane strain elements: reduced by 180
 !
+         if(.not.nodalthickness) then
             if(iaxial.eq.180) then
                if(lakon(ialset(istartset(i)))(1:2).eq.'CA') then
                   thickness=datan(1.d0)*8.d0/iaxial
@@ -205,9 +253,13 @@ cccc      if((key.eq.0).or.(lakon(ialset(istartset(i)))(1:2).eq.'CA')) then
                   thickness=thickness/iaxial
                endif
             endif
-cccc         else
-cccc            thickness=datan(1.d0)*8.d0/iaxial
-cccc         endif
+         else
+!
+!           for those elements for which nodal thickness is activated
+!           the thickness is set to -1.d0
+!
+            thickness=-1.d0
+         endif
 !
 !        assigning the thickness to each node of the corresponding
 !        elements (thickness specified)
@@ -215,12 +267,13 @@ cccc         endif
          do j=istartset(i),iendset(i)
             if(ialset(j).gt.0) then
                if((lakon(ialset(j))(1:2).eq.'CP').or.
-     &            (lakon(ialset(j))(1:2).eq.'CA')) then
+     &              (lakon(ialset(j))(1:2).eq.'CA')) then
 !
-!                 plane stress/strain  or axisymmetric elements
+!                 plane stress/strain or axisymmetric elements
 !
                   indexe=ipkon(ialset(j))
-                  do l=1,8
+                  read(lakon(ialset(j))(4:4),'(i1)') numnod
+                  do l=1,numnod
                      thicke(1,indexe+l)=thickness
                   enddo
                elseif(lakon(ialset(j))(1:1).eq.'T') then
@@ -262,7 +315,8 @@ cccc         endif
                      write(*,*) 
      &                    '*ERROR reading *SOLID SECTION: normal'
                      write(*,*) '       in direction 1 has zero size'
-                     call exit(201)
+                     ier=1
+                     return
                   endif
                   do l=1,3
                      p(l)=p(l)/dd
@@ -284,9 +338,10 @@ cccc         endif
                   k=k-ialset(j)
                   if(k.ge.ialset(j-1)) exit
                   if((lakon(k)(1:2).eq.'CP').or.
-     &               (lakon(k)(1:2).eq.'CA')) then
+     &                 (lakon(k)(1:2).eq.'CA')) then
                      indexe=ipkon(k)
-                     do l=1,8
+                     read(lakon(k)(4:4),'(i1)') numnod
+                     do l=1,numnod
                         thicke(1,indexe+l)=thickness
                      enddo
                   elseif(lakon(k)(1:1).eq.'T') then
@@ -327,7 +382,8 @@ cccc         endif
                         write(*,*) 
      &                       '*ERROR reading *SOLID SECTION: normal'
                         write(*,*) '       in direction 1 has zero size'
-                        call exit(201)
+                        ier=1
+                        return
                      endif
                      do l=1,3
                         p(l)=p(l)/dd
@@ -346,24 +402,35 @@ cccc         endif
                enddo
             endif
          enddo
-         else
+      else
 !
+!        no second line (no thickness given)
+!         
 !        assigning the thickness to each node of the corresponding
-!        elements (thickness not specified: only axisymmetric elements)
+!        elements (thickness not specified: only axisymmetric elements
+!        or plane stress/strain elements with nodal thickness)
 !
-            thickness=datan(1.d0)*8.d0/iaxial
+         thickness=datan(1.d0)*8.d0/iaxial
          do j=istartset(i),iendset(i)
             if(ialset(j).gt.0) then
                if(lakon(ialset(j))(1:2).eq.'CA') then
-c                  write(*,*) 'solidsections ',
-c     &               ialset(j),lakon(ialset(j)),thickness
 !
 !                 axisymmetric elements
 !
                   indexe=ipkon(ialset(j))
-                  do l=1,8
+                  read(lakon(ialset(j))(4:4),'(i1)') numnod
+                  do l=1,numnod
                      thicke(1,indexe+l)=thickness
                   enddo
+               elseif((lakon(ialset(j))(1:3).eq.'CPS').or.
+     &                 (lakon(ialset(j))(1:3).eq.'CPE')) then
+                  if(nodalthickness) then
+                     indexe=ipkon(ialset(j))
+                     read(lakon(ialset(j))(4:4),'(i1)') numnod
+                     do l=1,numnod
+                        thicke(1,indexe+l)=-1.d0
+                     enddo
+                  endif
                endif
             else
                k=ialset(j-2)
@@ -375,35 +442,89 @@ c     &               ialset(j),lakon(ialset(j)),thickness
 !                 axisymmetric elements
 !
                      indexe=ipkon(k)
-                     do l=1,8
+                     read(lakon(k)(4:4),'(i1)') numnod
+                     do l=1,numnod
                         thicke(1,indexe+l)=thickness
                      enddo
+                  elseif((lakon(k)(1:3).eq.'CPS').or.
+     &                    (lakon(k)(1:3).eq.'CPE')) then
+                     if(nodalthickness) then
+                        indexe=ipkon(k)
+                        read(lakon(k)(4:4),'(i1)') numnod
+                        do l=1,numnod
+                           thicke(1,indexe+l)=-1.d0
+                        enddo
+                     endif
                   endif
                enddo
             endif
          enddo
-         endif
-!
+      endif
+!     
 !        defining cyclic symmetric conditions for axisymmetric
 !        elements (needed for cavity radiation)
 !
-         do j=istartset(i),iendset(i)
-            if(ialset(j).gt.0) then
-               if(lakon(ialset(j))(1:2).eq.'CA') then
+      do j=istartset(i),iendset(i)
+         if(ialset(j).gt.0) then
+            if(lakon(ialset(j))(1:2).eq.'CA') then
+               if(mcs.gt.1) then
+                  write(*,*) '*ERROR reading *SOLID SECTION: '
+                  write(*,*) '       axisymmetric elements cannot be
+     &combined with cyclic symmetry'
+                  ier=1
+                  return
+               elseif(mcs.eq.1) then
+                  if(int(cs(1,1)).ne.int(2.d0*pi/thickness+0.5d0)) 
+     &                 then
+                     write(*,*) '*ERROR reading *SOLID SECTION: '
+                     write(*,*) '       it is not allowed to define t
+     &wo different'
+                     write(*,*) '       angles for an axisymmetric st
+     &ructure'
+                     ier=1
+                     return
+                  else
+                     exit
+                  endif
+               endif
+               mcs=1
+               cs(1,1)=2.d0*pi/thickness+0.5d0
+               cs(2,1)=-0.5d0
+               cs(3,1)=-0.5d0
+               cs(5,1)=1.5d0
+               do k=6,9
+                  cs(k,1)=0.d0
+               enddo
+               cs(10,1)=1.d0
+               cs(11,1)=0.d0
+               cs(12,1)=-1.d0
+               cs(14,1)=0.5d0
+               cs(15,1)=dcos(thickness)
+               cs(16,1)=dsin(thickness)
+               exit
+            endif
+         else
+            k=ialset(j-2)
+            do
+               k=k-ialset(j)
+               if(k.ge.ialset(j-1)) exit
+               if(lakon(k)(1:2).eq.'CA') then
                   if(mcs.gt.1) then
                      write(*,*) '*ERROR reading *SOLID SECTION: '
-                     write(*,*) '       axisymmetric elements cannot be
-     &combined with cyclic symmetry'
-                     call exit(201)
+                     write(*,*) '       axisymmetric elements cannot 
+     &be combined with cyclic symmetry'
+                     ier=1
+                     return
                   elseif(mcs.eq.1) then
                      if(int(cs(1,1)).ne.int(2.d0*pi/thickness+0.5d0)) 
-     &                 then
+     &                    then
                         write(*,*) '*ERROR reading *SOLID SECTION: '
-                        write(*,*) '       it is not allowed to define t
-     &wo different'
-                        write(*,*) '       angles for an axisymmetric st
-     &ructure'
-                        call exit(201)
+                        write(*,*) '       it is not allowed to defin
+     &e two different'
+                        write(*,*) '       angles for an axisymmetric
+     &structure'
+                        ier=1
+                        return
                      else
                         exit
                      endif
@@ -419,62 +540,20 @@ c     &               ialset(j),lakon(ialset(j)),thickness
                   cs(10,1)=1.d0
                   cs(11,1)=0.d0
                   cs(12,1)=-1.d0
-                  cs(14,1)=0.5
+                  cs(14,1)=0.5d0
                   cs(15,1)=dcos(thickness)
                   cs(16,1)=dsin(thickness)
                   exit
                endif
-            else
-               k=ialset(j-2)
-               do
-                  k=k-ialset(j)
-                  if(k.ge.ialset(j-1)) exit
-c                  if(lakon(ialset(j))(1:2).eq.'CA') then
-                  if(lakon(k)(1:2).eq.'CA') then
-                     if(mcs.gt.1) then
-                        write(*,*) '*ERROR reading *SOLID SECTION: '
-                        write(*,*) '       axisymmetric elements cannot 
-     &be combined with cyclic symmetry'
-                        call exit(201)
-                     elseif(mcs.eq.1) then
-                        if(int(cs(1,1)).ne.int(2.d0*pi/thickness+0.5d0)) 
-     &                       then
-                           write(*,*) '*ERROR reading *SOLID SECTION: '
-                           write(*,*) '       it is not allowed to defin
-     &e two different'
-                           write(*,*) '       angles for an axisymmetric
-     & structure'
-                           call exit(201)
-                        else
-                           exit
-                        endif
-                     endif
-                     mcs=1
-                     cs(1,1)=2.d0*pi/thickness+0.5d0
-                     cs(2,1)=-0.5d0
-                     cs(3,1)=-0.5d0
-                     cs(5,1)=1.5d0
-                     do k=6,9
-                        cs(k,1)=0.d0
-                     enddo
-                     cs(10,1)=1.d0
-                     cs(11,1)=0.d0
-                     cs(12,1)=-1.d0
-                     cs(14,1)=0.5
-                     cs(15,1)=dcos(thickness)
-                     cs(16,1)=dsin(thickness)
-                     exit
-                  endif
-              enddo
-            endif
-         enddo
-!
-         if(key.eq.0) then
-            call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
-     &           ipoinp,inp,ipoinpc)
+            enddo
          endif
-cccc      endif
-!
+      enddo
+!     
+      if(key.eq.0) then
+         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &        ipoinp,inp,ipoinpc)
+      endif
+!     
       return
       end
 
