@@ -1,6 +1,6 @@
 
 /*     CalculiX - A 3-dimensional finite element program                   */
-/*              Copyright (C) 1998-2015 Guido Dhondt                          */
+/*              Copyright (C) 1998-2020 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -466,7 +466,7 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 
 	if(symmetryflagi4==0){
 	    printf(" Factoring the system of equations using the symmetric spooles solver\n");
-	}else if(symmetryflagi4==2){
+	}else if((symmetryflagi4==2)&&(*inputformat!=4)){
 	    printf(" Factoring the system of equations using the unsymmetric spooles solver\n");
 	}
 
@@ -499,6 +499,8 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 			if(fabs(ad[i*(int)*nzs+j])>1.e-20) nent++;
 		    }
 		}
+	    }else if(*inputformat==4){
+	      nent = *nzs;
 	    }
 	    
 	    InpMtx_init(mtxA, INPMTX_BY_ROWS, SPOOLES_REAL, nent, size);
@@ -510,8 +512,10 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 	          and au (first lower triangle, then upper triangle; lower
 	          and upper triangle have nonzero's at symmetric positions)
 	       2: full matrix in field ad
-	       3: sparse upper triangular matrix in ad (diagonal)
-	          and au (upper triangle)  */
+	       3: sparse full matrix in ad (diagonal)
+	          and au (off-diagonal positions)
+               4: sparse full matrix in au, row after row
+                  (CFD applications, call from compfluid.c)  */
 	    
 	    if(*inputformat==0){
 		ipoint = 0;
@@ -589,6 +593,9 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 			InpMtx_inputRealEntry(mtxA, col, col, ad[col]);
 			for (ipo = ipoint; ipo < ipoint + icol[col]; ipo++) {
 			    int row = irow[ipo] - 1;
+			    /* next change on 27.06.2020 */
+			    /*InpMtx_inputRealEntry(mtxA, col, row,
+						    au[ipo]);*/
 			    InpMtx_inputRealEntry(mtxA, row, col,
 						  au[ipo]);
 			}
@@ -600,12 +607,26 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 			InpMtx_inputRealEntry(mtxA, col, col, ad[col]-*sigma*adb[col]);
 			for (ipo = ipoint; ipo < ipoint + icol[col]; ipo++) {
 			    int row = irow[ipo] - 1;
+			    /* next change on 27.06.2020 */
+			    /* InpMtx_inputRealEntry(mtxA, col, row, 
+			       au[ipo]-*sigma*aub[ipo]);*/
 			    InpMtx_inputRealEntry(mtxA, row, col, 
 						  au[ipo]-*sigma*aub[ipo]);
 			}
 			ipoint = ipoint + icol[col];
 		    }
 		}
+	    }else if(*inputformat==4){
+	        int row;
+	        for(row = 0; row < size; row++){
+		    for(ipo = irow[row]; ipo < irow[row+1]; ipo++){
+		        col = icol[ipo]-1;
+			//			printf("spooles %d %d %d %f\n",ipo,row,col,au[ipo]);
+		        InpMtx_inputRealEntry(mtxA, col, row, 
+					au[ipo]);
+		     }
+	        }
+		  
 	    }		  
 	    
 	    InpMtx_changeStorageMode(mtxA, INPMTX_BY_VECTORS);
@@ -693,7 +714,7 @@ void spooles_factor(double *ad, double *au,  double *adb, double *aub,
 		factor(&pfi, mtxA, size, msgFile,&symmetryflagi4);
 	}
 #else
-	printf(" Using 1 cpu for spooles.\n\n");
+	//	printf(" Using 1 cpu for spooles.\n\n");
 	factor(&pfi, mtxA, size, msgFile,&symmetryflagi4);
 #endif
 }
@@ -819,6 +840,8 @@ void spooles_factor_rad(double *ad, double *au,  double *adb, double *aub,
 			if(fabs(ad[i*(int)*nzs+j])>1.e-20) nent++;
 		    }
 		}
+	    }else if(*inputformat==4){
+	        nent = *nzs;
 	    }
 	    
 	    InpMtx_init(mtxA, INPMTX_BY_ROWS, SPOOLES_REAL, nent, size);
@@ -921,6 +944,16 @@ void spooles_factor_rad(double *ad, double *au,  double *adb, double *aub,
 			ipoint = ipoint + icol[col];
 		    }
 		}
+	    }else if(*inputformat==4){
+	        int row;
+	        for(row = 0; row < size; row++){
+		    for(ipo = irow[row]; ipo < irow[row+1]; ipo++){
+		        col = icol[ipo] - 1;
+		        InpMtx_inputRealEntry(mtxA, col, row, 
+					au[ipo]);
+		     }
+	        }
+		  
 	    }		  
 	    
 	    InpMtx_changeStorageMode(mtxA, INPMTX_BY_VECTORS);
@@ -1086,6 +1119,7 @@ void spooles_cleanup_rad()
 	fclose(msgFilf);
 }
 
+
 /** 
  * spooles: Main interface between Calculix and spooles:
  *
@@ -1101,8 +1135,16 @@ void spooles(double *ad, double *au, double *adb, double *aub, double *sigma,
              ITG *symmetryflag, ITG *inputformat, ITG *nzs3)
 {
 
-  if(*neq==0) return;
+    if(*neq==0){
+	return;
+    }else if(*neq==1){
+	b[0]/=ad[0];
+	return;
+    }
 
+    /*    FORTRAN(spooles_write,(ad,au,adb,aub,sigma,b,icol,irow,neq,nzs,
+	  symmetryflag,inputformat,nzs3));*/
+    
   spooles_factor(ad,au,adb,aub,sigma,icol,irow,neq,nzs,symmetryflag,
 		 inputformat,nzs3);
   
