@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,9 +18,10 @@
 !
       subroutine springforc_n2f(xl,konl,vl,imat,elcon,nelcon,
      &  elas,fnl,ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,
-     &  plicon,nplicon,npmat_,senergy,iener,cstr,mi,
+     &  plicon,nplicon,npmat_,senergy,nener,cstr,mi,
      &  springarea,nmethod,ne0,nstate_,xstateini,
-     &  xstate,reltime,ielas,venergy)
+     &  xstate,reltime,ielas,venergy,ielorien,orab,norien,
+     &  nelem,smscale,mscalmethod)
 !
 !     calculates the force of the spring (node-to-face penalty)
 !
@@ -29,23 +30,26 @@
       character*8 lakonl
 !
       integer konl(9),i,j,imat,ncmat_,ntmat_,nope,nterms,iflag,mi(*),
-     &  kode,niso,id,nplicon(0:ntmat_,*),npmat_,nelcon(2,*),iener,
-     &  nmethod,ne0,nstate_,ielas
+     &  kode,niso,id,nplicon(0:ntmat_,*),npmat_,nelcon(2,*),nener,
+     &  nmethod,ne0,nstate_,ielas,norien,nelem,ielorien(mi(3),*),
+     &  iorien,idof,idof1,idof2,mscalmethod
 !
       real*8 xl(3,10),elas(21),ratio(9),t1l,al(3),vl(0:mi(2),10),
      &  pl(3,10),xn(3),dm,alpha,beta,fnl(3,10),tp(3),te(3),ftrial(3),
-     &  dist,t(3),dftrial,overclosure,venergy,
+     &  dist,t(3),dftrial,overclosure,venergy,orab(7,*),a(3,3),
      &  elcon(0:ncmat_,ntmat_,*),pproj(3),xsj2(3),xs2(3,7),clear,
-     &  shp2(7,9),xi,et,elconloc(21),plconloc(802),xk,fk,dd,val,
+     &  shp2(7,9),xi,et,elconloc(*),plconloc(802),xk,fk,dd,val,
      &  xiso(200),yiso(200),dd0,plicon(0:2*npmat_,ntmat_,*),
      &  um,eps,pi,senergy,cstr(6),dg,dfshear,dfnl,
-     &  springarea(2),overlap,pres,
+     &  springarea(2),overlap,pres,xn1(3),xn2(3),
      &  xstate(nstate_,mi(1),*),xstateini(nstate_,mi(1),*),t1(3),t2(3),
-     &  dt1,dte,alnew(3),reltime
+     &  dt1,dte,alnew(3),reltime,smscale(*)
+!
+!
 !
       iflag=2
 !
-      if(iener.eq.1) venergy=0.d0
+      if(nener.eq.1) venergy=0.d0
 !
 !     actual positions of the nodes belonging to the contact spring
 !     (otherwise no contact force)
@@ -90,56 +94,11 @@
             enddo
             val=dd-dd0
 !     
-!     interpolating the material data
+!           calculating the spring force and the spring energy
 !     
-            call materialdata_sp(elcon,nelcon,imat,ntmat_,i,t1l,
-     &           elconloc,kode,plicon,nplicon,npmat_,plconloc,ncmat_)
-!     
-!     calculating the spring force and the spring constant
-!     
-            if(kode.eq.2)then
-               xk=elconloc(1)
-               fk=xk*val
-               if(iener.eq.1) then
-                  senergy=fk*val/2.d0
-               endif
-            else
-               niso=int(plconloc(801))
-               do i=1,niso
-                  xiso(i)=plconloc(2*i-1)
-                  yiso(i)=plconloc(2*i)
-               enddo
-               call ident(xiso,val,niso,id)
-               if(id.eq.0) then
-                  xk=0.d0
-                  fk=yiso(1)
-                  if(iener.eq.1) then
-                     senergy=fk*val
-                  endif
-               elseif(id.eq.niso) then
-                  xk=0.d0
-                  fk=yiso(niso)
-                  if(iener.eq.1) then
-                     senergy=yiso(1)*xiso(1)
-                     do i=2,niso
-                        senergy=senergy+(xiso(i)-xiso(i-1))*
-     &                       (yiso(i)+yiso(i-1))/2.d0
-                     enddo
-                     senergy=senergy+(val-xiso(niso))*yiso(niso)
-                  endif
-               else
-                  xk=(yiso(id+1)-yiso(id))/(xiso(id+1)-xiso(id))
-                  fk=yiso(id)+xk*(val-xiso(id))
-                  if(iener.eq.1) then
-                     senergy=yiso(1)*xiso(1)
-                     do i=2, id
-                        senergy=senergy+(xiso(i)-xiso(i-1))*
-     &                       (yiso(i)+yiso(i-1))/2.d0
-                     enddo
-                     senergy=senergy+(val-xiso(id))*(fk+yiso(id))/2.d0
-                  endif
-               endif
-            endif
+            call calcspringforc(imat,elcon,nelcon,ncmat_,ntmat_,t1l,
+     &           kode,plicon,nplicon,npmat_,senergy,nener,fk,val)
+!            
          else
 !     
 !           GAP-element
@@ -166,33 +125,116 @@
             fnl(i,2)=fk*xn(i)
          enddo
          return
+      elseif(lakonl(7:7).eq.'1') then
+!
+!        spring1-element
+!        determine the direction of action of the spring
+!
+         idof=nint(elcon(3,1,imat))
+!
+         if(norien.gt.0) then
+            iorien=max(0,ielorien(1,nelem))
+         else
+            iorien=0
+         endif
+!
+         if(iorien.eq.0) then
+            do i=1,3
+               xn(i)=0.d0
+            enddo
+            xn(idof)=1.d0
+         else
+            call transformatrix(orab(1,iorien),xl(1,1),a)
+            do i=1,3
+               xn(i)=a(i,idof)
+            enddo
+         endif
+!
+!        change in spring length
+!
+         val=vl(1,1)*xn(1)+vl(2,1)*xn(2)+vl(3,1)*xn(3)
+!     
+!        calculating the spring force and the spring energy
+!     
+         call calcspringforc(imat,elcon,nelcon,ncmat_,ntmat_,t1l,
+     &        kode,plicon,nplicon,npmat_,senergy,nener,fk,val)
+!
+         do i=1,3
+            fnl(i,1)=fk*xn(i)
+         enddo
+         return
+      elseif(lakonl(7:7).eq.'2') then
+!
+!        spring2-element
+!        determine the direction of action of the spring
+!
+         idof1=nint(elcon(3,1,imat))
+         idof2=nint(elcon(4,1,imat))
+!
+         if(norien.gt.0) then
+            iorien=max(0,ielorien(1,nelem))
+         else
+            iorien=0
+         endif
+!
+         if(iorien.eq.0) then
+            do i=1,3
+               xn1(i)=0.d0
+               xn2(i)=0.d0
+            enddo
+            xn1(idof1)=1.d0
+            xn2(idof2)=1.d0
+         else
+            call transformatrix(orab(1,iorien),xl(1,1),a)
+            do i=1,3
+               xn1(i)=a(i,idof1)
+            enddo
+            call transformatrix(orab(1,iorien),xl(1,2),a)
+            do i=1,3
+               xn2(i)=a(i,idof2)
+            enddo
+         endif
+!
+!        change in spring length
+!
+         val=(vl(1,1)*xn1(1)+vl(2,1)*xn1(2)+vl(3,1)*xn1(3))
+     &      -(vl(1,2)*xn2(1)+vl(2,2)*xn2(2)+vl(3,2)*xn2(3))
+!     
+!        calculating the spring force and the spring energy
+!     
+         call calcspringforc(imat,elcon,nelcon,ncmat_,ntmat_,t1l,
+     &        kode,plicon,nplicon,npmat_,senergy,nener,fk,val)
+!
+         do i=1,3
+            fnl(i,1)=fk*xn1(i)
+            fnl(i,2)=-fk*xn2(i)
+         enddo
+         return
       endif
 !
       nterms=nope-1
 !
-!     vector vr connects the dependent node with its projection
-!     on the independent face
+!     vector al connects the dependent node with its projection
+!     on the independent face = vec_r (User's
+!     manual -> theory -> boundary conditions -> node-to-face
+!     penalty contact)
 !
       do i=1,3
          pproj(i)=pl(i,nope)
       enddo
-      call attach(pl,pproj,nterms,ratio,dist,xi,et)
+      call attach_2d(pl,pproj,nterms,ratio,dist,xi,et)
       do i=1,3
          al(i)=pl(i,nope)-pproj(i)
       enddo
 !
 !     determining the jacobian vector on the surface 
 !
-      if(nterms.eq.9) then
-         call shape9q(xi,et,pl,xsj2,xs2,shp2,iflag)
-      elseif(nterms.eq.8) then
+      if(nterms.eq.8) then
          call shape8q(xi,et,pl,xsj2,xs2,shp2,iflag)
       elseif(nterms.eq.4) then
          call shape4q(xi,et,pl,xsj2,xs2,shp2,iflag)
       elseif(nterms.eq.6) then
          call shape6tri(xi,et,pl,xsj2,xs2,shp2,iflag)
-      elseif(nterms.eq.7) then
-         call shape7tri(xi,et,pl,xsj2,xs2,shp2,iflag)
       else
          call shape3tri(xi,et,pl,xsj2,xs2,shp2,iflag)
       endif
@@ -213,16 +255,7 @@
       if(nmethod.eq.1) then
          clear=clear-springarea(2)*(1.d0-reltime)
       endif
-      if(clear.le.0.d0) cstr(1)=clear
-!     
-!     MPADD start 
-!     (keep also positive clearance in the frd file)
-c      if(nmethod.eq.4) then
-c         cstr(1)=clear
-c      else
-c         if(clear.le.0.d0) cstr(1)=clear
-c      endif
-!     MPADD end
+      cstr(1)=clear
 !
 !     representative area: usually the slave surface stored in
 !     springarea; however, if no area was assigned because the
@@ -253,7 +286,7 @@ c      endif
             endif
             elas(1)=dexp(-beta*clear+dlog(alpha))
          endif
-         if(iener.eq.1) then
+         if(nener.eq.1) then
             senergy=elas(1)/beta
          endif
       elseif(int(elcon(3,1,imat)).eq.2) then
@@ -268,16 +301,24 @@ c      endif
 ! 
             if(clear.le.0.d0)then
                pi=4.d0*datan(1.d0)
-               eps=elcon(1,1,imat)*pi/elcon(2,1,imat)
-               elas(1)=(-springarea(1)*elcon(2,1,imat)*clear*
+               xk=elcon(2,1,imat)
+!
+!              spring scaling for explicit dynamics
+!
+               if((mscalmethod.eq.2).or.(mscalmethod.eq.3)) then
+                  xk=xk*smscale(nelem)
+               endif
+!               
+               eps=elcon(1,1,imat)*pi/xk
+               elas(1)=(-springarea(1)*xk*clear*
      &              (0.5d0+datan(-clear/eps)/pi))
-               if(iener.eq.1)
-     &             senergy=springarea(1)*elcon(2,1,imat)*(clear**2/4.d0+ 
+               if(nener.eq.1)
+     &             senergy=springarea(1)*xk*(clear**2/4.d0+ 
      &             (0.5d0*datan(-clear/eps)*clear**2+
      &             0.5d0*(eps*clear+datan(-clear/eps)*eps**2))/pi)
             else
                elas(1)=0.d0
-               if(iener.eq.1) senergy=0.d0
+               if(nener.eq.1) senergy=0.d0
             endif
 !     
          else
@@ -285,7 +326,7 @@ c      endif
             eps=elcon(1,1,imat)*pi/elcon(2,1,imat)
             elas(1)=(-springarea(1)*elcon(2,1,imat)*clear*
      &           (0.5d0+datan(-clear/eps)/pi)) 
-            if(iener.eq.1) then
+            if(nener.eq.1) then
                senergy=-elas(1)*clear/2.d0
             endif
          endif
@@ -307,12 +348,12 @@ c      endif
          call ident(xiso,overlap,niso,id)
          if(id.eq.0) then
             pres=yiso(1)
-            if(iener.eq.1) then
+            if(nener.eq.1) then
                senergy=yiso(1)*overlap
             endif
          elseif(id.eq.niso) then
             pres=yiso(niso)
-            if(iener.eq.1) then
+            if(nener.eq.1) then
                senergy=yiso(1)*xiso(1)
                do i=2,niso
                   senergy=senergy+(xiso(i)-xiso(i-1))*
@@ -323,7 +364,7 @@ c      endif
          else
             xk=(yiso(id+1)-yiso(id))/(xiso(id+1)-xiso(id))
             pres=yiso(id)+xk*(overlap-xiso(id))
-            if(iener.eq.1) then
+            if(nener.eq.1) then
                senergy=yiso(1)*xiso(1)
                do i=2, id
                   senergy=senergy+(xiso(i)-xiso(i-1))*
@@ -333,7 +374,7 @@ c      endif
             endif
          endif
          elas(1)=springarea(1)*pres
-         if(iener.eq.1) senergy=springarea(1)*senergy
+         if(nener.eq.1) senergy=springarea(1)*senergy
       else
          write(*,*) '*ERROR in springforc: no overclosure model'
          write(*,*) '       selected. This is mandatory in a penalty'
@@ -378,6 +419,12 @@ c      endif
 !     slip curve
 !     
             xk=elcon(7,1,imat)*springarea(1)
+!
+!           spring scaling for explicit dynamics
+!
+            if((mscalmethod.eq.2).or.(mscalmethod.eq.3)) then
+               xk=xk*smscale(nelem)
+            endif
 !     
 !     calculating the relative displacement between the slave node
 !     and its projection on the master surface
@@ -390,17 +437,17 @@ c      endif
             enddo
 !     
 !     calculating the difference in relative displacement since
-!     the start of the increment = lamda^*
+!     the start of the increment = vec_s
 !     
             do i=1,3
                al(i)=alnew(i)-xstateini(3+i,1,ne0+konl(nope+1))
             enddo
 !     
-!     ||lambda^*||
+!     s=||vec_s||
 !     
             val=al(1)*xn(1)+al(2)*xn(2)+al(3)*xn(3)
 !     
-!     update the relative tangential displacement
+!     update the relative tangential displacement vec_t
 !     
             do i=1,3
                t(i)=xstateini(6+i,1,ne0+konl(nope+1))+al(i)-val*xn(i)
@@ -436,7 +483,6 @@ c      endif
                ftrial(i)=xk*te(i)
             enddo
             dftrial=xk*dte
-c            dftrial=dsqrt(ftrial(1)**2+ftrial(2)**2+ftrial(3)**2)
 !     
 !     check whether stick or slip
 !     
@@ -457,7 +503,7 @@ c     write(*,*)'STICK'
 !
 !              shear elastic energy
 !
-               if(iener.eq.1) senergy=senergy+dftrial*dte
+               if(nener.eq.1) senergy=senergy+dftrial*dte
             else
 !     
 !     slip
@@ -478,7 +524,7 @@ c     write(*,*)'SLIP'
 !
 !              shear elastic and viscous energy
 !
-               if(iener.eq.1) then
+               if(nener.eq.1) then
                   senergy=senergy+dfshear*dfshear/xk
                   venergy=dg*dfshear
                endif
