@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,9 +19,12 @@
       subroutine springstiff_n2f(xl,elas,konl,voldl,s,imat,elcon,nelcon,
      &  ncmat_,ntmat_,nope,lakonl,t1l,kode,elconloc,plicon,
      &  nplicon,npmat_,iperturb,springarea,nmethod,mi,ne0,
-     &  nstate_,xstateini,xstate,reltime,nasym)
+     &  nstate_,xstateini,xstate,reltime,nasym,ielorien,orab,
+     &  norien,nelem)
 !
 !     calculates the stiffness of a spring (node-to-face penalty)
+!     (User's manual -> theory -> boundary conditions -> 
+!      node-to-face penalty contact)
 !
       implicit none
 !
@@ -29,7 +32,8 @@
 !
       integer konl(20),i,j,imat,ncmat_,ntmat_,k,l,nope,nterms,iflag,
      &  i1,kode,niso,id,nplicon(0:ntmat_,*),npmat_,nelcon(2,*),
-     &  iperturb(*),nmethod,mi(*),ne0,nstate_,nasym
+     &  iperturb(*),nmethod,mi(*),ne0,nstate_,nasym,ielorien(mi(3),*),
+     &  norien,nelem,idof,idof1,idof2,iorien
 !
       real*8 xl(3,10),elas(21),ratio(9),pproj(3),val,shp2(7,9),
      &  al(3),s(60,60),voldl(0:mi(2),10),pl(3,10),xn(3),dm,
@@ -43,14 +47,10 @@
      &  xstate(nstate_,mi(1),*),xstateini(nstate_,mi(1),*),
      &  um,eps,pi,dftdt(3,3),tp(3),te(3),ftrial(3),clear,
      &  dftrial,dfnl,dfshear,dg,dte,alnew(3),dfn(3,10),reltime,
-     &  overlap,pres,dpresdoverlap,overclosure
+     &  overlap,pres,dpresdoverlap,overclosure,orab(7,*),
+     &  xn1(3),xn2(3),a(3,3)
 !
-      intent(in) xl,konl,voldl,imat,elcon,nelcon,
-     &  ncmat_,ntmat_,nope,lakonl,t1l,kode,plicon,
-     &  nplicon,npmat_,iperturb,nmethod,mi,ne0,
-     &  nstate_,xstateini,reltime,nasym
 !
-      intent(inout) s,xstate,elas,springarea,elconloc
 !
       iflag=4
 !
@@ -158,6 +158,133 @@
             enddo
          enddo
          return
+      elseif(lakonl(7:7).eq.'1') then
+!
+!        spring1-element
+!        determine the direction of action of the spring
+!
+         idof=nint(elcon(3,1,imat))
+!
+         if(norien.gt.0) then
+            iorien=max(0,ielorien(1,nelem))
+         else
+            iorien=0
+         endif
+!
+         if(iorien.eq.0) then
+            do i=1,3
+               xn(i)=0.d0
+            enddo
+            xn(idof)=1.d0
+         else
+            call transformatrix(orab(1,iorien),xl(1,1),a)
+            do i=1,3
+               xn(i)=a(i,idof)
+            enddo
+         endif
+!     
+!     interpolating the material data
+!     
+         call materialdata_sp(elcon,nelcon,imat,ntmat_,i,t1l,
+     &           elconloc,kode,plicon,nplicon,npmat_,plconloc,ncmat_)
+!     
+!     calculating the spring constant
+!     
+         if(kode.eq.2)then
+            xk=elconloc(1)
+         else
+            niso=int(plconloc(801))
+            do i=1,niso
+               xiso(i)=plconloc(2*i-1)
+               yiso(i)=plconloc(2*i)
+            enddo
+            call ident(xiso,val,niso,id)
+            if(id.eq.0) then
+               xk=0.d0
+            elseif(id.eq.niso) then
+               xk=0.d0
+            else
+               xk=(yiso(id+1)-yiso(id))/(xiso(id+1)-xiso(id))
+            endif
+         endif
+!
+!        assembling the stiffness matrix
+!
+         do i=1,3
+            do j=1,3
+               s(i,j)=xk*xn(i)*xn(j)
+            enddo
+         enddo
+         return
+      elseif(lakonl(7:7).eq.'2') then
+!
+!        spring2-element
+!        determine the direction of action of the spring
+!
+         idof1=nint(elcon(3,1,imat))
+         idof2=nint(elcon(4,1,imat))
+!
+         if(norien.gt.0) then
+            iorien=max(0,ielorien(1,nelem))
+         else
+            iorien=0
+         endif
+!
+         if(iorien.eq.0) then
+            do i=1,3
+               xn1(i)=0.d0
+               xn2(i)=0.d0
+            enddo
+            xn1(idof1)=1.d0
+            xn2(idof2)=1.d0
+         else
+            call transformatrix(orab(1,iorien),xl(1,1),a)
+            do i=1,3
+               xn1(i)=a(i,idof1)
+            enddo
+            call transformatrix(orab(1,iorien),xl(1,2),a)
+            do i=1,3
+               xn2(i)=a(i,idof2)
+            enddo
+         endif
+!     
+!     interpolating the material data
+!     
+         call materialdata_sp(elcon,nelcon,imat,ntmat_,i,t1l,
+     &           elconloc,kode,plicon,nplicon,npmat_,plconloc,ncmat_)
+!     
+!        calculating the spring constant
+!     
+         if(kode.eq.2)then
+            xk=elconloc(1)
+         else
+            niso=int(plconloc(801))
+            do i=1,niso
+               xiso(i)=plconloc(2*i-1)
+               yiso(i)=plconloc(2*i)
+            enddo
+            call ident(xiso,val,niso,id)
+            if(id.eq.0) then
+               xk=0.d0
+            elseif(id.eq.niso) then
+               xk=0.d0
+            else
+               xk=(yiso(id+1)-yiso(id))/(xiso(id+1)-xiso(id))
+            endif
+         endif
+!
+!        assembling the stiffness matrix
+!
+         do i=1,3
+            do j=1,3
+               s(i,j)=xk*xn1(i)*xn1(j)
+               s(i,j+3)=-xk*xn1(i)*xn2(j)
+               s(i+3,j)=-xk*xn2(i)*xn1(j)
+               s(i+3,j+3)=xk*xn2(i)*xn2(j)
+            enddo
+         enddo
+         return
+!
       endif
 !
 !     contact springs
@@ -165,34 +292,32 @@
       nterms=nope-1
 !
 !     vector al connects the actual position of the slave node 
-!     with its projection on the master face
+!     with its projection on the master face = vec_r (User's
+!     manual -> theory -> boundary conditions -> node-to-face
+!     penalty contact)
 !
       do i=1,3
          pproj(i)=pl(i,nope)
       enddo
-      call attach(pl,pproj,nterms,ratio,dist,xi,et)
+      call attach_2d(pl,pproj,nterms,ratio,dist,xi,et)
       do i=1,3
          al(i)=pl(i,nope)-pproj(i)
       enddo
 !
 !     determining the jacobian vector on the surface 
 !
-      if(nterms.eq.9) then
-         call shape9q(xi,et,pl,xm,xs2,shp2,iflag)
-      elseif(nterms.eq.8) then
+      if(nterms.eq.8) then
          call shape8q(xi,et,pl,xm,xs2,shp2,iflag)
       elseif(nterms.eq.4) then
          call shape4q(xi,et,pl,xm,xs2,shp2,iflag)
       elseif(nterms.eq.6) then
          call shape6tri(xi,et,pl,xm,xs2,shp2,iflag)
-      elseif(nterms.eq.7) then
-         call shape7tri(xi,et,pl,xm,xs2,shp2,iflag)
       else
          call shape3tri(xi,et,pl,xm,xs2,shp2,iflag)
       endif
 !
-!     dxi(i,j) is the derivative of xi w.r.t. pl(i,j),
-!     det(i,j) is the derivative of eta w.r.t. pl(i,j)
+!     d xi / d vec_u_j
+!     d eta / d vec_u_j
 !
 !     dxi and det are determined from the orthogonality
 !     condition
@@ -225,8 +350,7 @@
          enddo
       enddo
 !
-!     dal(i,j,k) is the derivative of al(i) w.r.t pl(j,k)
-!     ( d al / d u_k)
+!     d vec_r / d vec_u_k
 !
       do i=1,nope
          do j=1,3
@@ -455,17 +579,17 @@
             enddo
 !     
 !     calculating the difference in relative displacement since
-!     the start of the increment = lamda^*
+!     the start of the increment = vec_s
 !     
             do i=1,3
                al(i)=alnew(i)-xstateini(3+i,1,ne0+konl(nope+1))
             enddo
 !     
-!     ||lambda^*||
+!     s=||vec_s||
 !     
             val=al(1)*xn(1)+al(2)*xn(2)+al(3)*xn(3)
 !     
-!     update the relative tangential displacement
+!     update the relative tangential displacement vec_t
 !     
             do i=1,3
                t(i)=xstateini(6+i,1,ne0+konl(nope+1))+al(i)-val*xn(i)
@@ -479,7 +603,7 @@
                xstate(6+i,1,ne0+konl(nope+1))=t(i)
             enddo
 !     
-!     d al/d u_k -> d al^*/d u_k
+!     d vec_s/ d vec_u_k
 !     notice: xi & et are const.
 !     
             do k=1,nope
@@ -500,7 +624,7 @@
                dal(j,j,nope)=1.d0
             enddo
 !     
-!     (d al/d u_k).||m|| -> (d al^*/d u_k).||m||
+!     d s/ dvec_u_k.||m||
 !     
             do k=1,nope
                do i=1,3
@@ -511,7 +635,7 @@
                enddo
             enddo
 !     
-!     d t/d u_k
+!     d vec_t/d vec_u_k
 !     
             do k=1,nope
                do j=1,3
