@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,8 +20,8 @@
      &  istat,n,tinc,tper,tmin,tmax,idrct,iline,ipol,inl,ipoinp,inp,
      &  ithermal,cs,ics,tieset,istartset,
      &  iendset,ialset,ipompc,nodempc,coefmpc,nmpc,nmpc_,ikmpc,
-     &  ilmpc,mpcfree,mcs,set,nset,labmpc,ipoinpc,iexpl,cfd,ttime,
-     &  iaxial,nelcon,nmat,tincf)
+     &  ilmpc,mpcfree,mcs,set,nset,labmpc,ipoinpc,iexpl,nef,ttime,
+     &  iaxial,nelcon,nmat,tincf,ier)
 !
 !     reading the input deck: *STATIC
 !
@@ -31,6 +31,7 @@
 !             4: sgi solver
 !             5: TAUCS
 !             7: pardiso
+!             8: pastix
 !
 !      iexpl==0:  structure:implicit, fluid:incompressible
 !
@@ -43,11 +44,11 @@
       character*81 set(*),tieset(3,*)
       character*132 textpart(16)
 !
-      integer nmethod,iperturb,isolver,istep,istat,n,key,i,idrct,
-     &  iline,ipol,inl,ipoinp(2,*),inp(3,*),ithermal,ics(*),iexpl,
+      integer nmethod,iperturb(*),isolver,istep,istat,n,key,i,idrct,
+     &  iline,ipol,inl,ipoinp(2,*),inp(3,*),ithermal(*),ics(*),iexpl,
      &  istartset(*),iendset(*),ialset(*),ipompc(*),nodempc(3,*),
      &  nmpc,nmpc_,ikmpc(*),ilmpc(*),mpcfree,nset,mcs,ipoinpc(0:*),
-     &  cfd,iaxial,nelcon(2,*),nmat
+     &  nef,iaxial,nelcon(2,*),nmat,ier
 !
       real*8 tinc,tper,tmin,tmax,cs(17,*),coefmpc(*),ttime,tincf
 !
@@ -56,18 +57,20 @@
       tmax=0.d0
       timereset=.false.
 !
-      if((iperturb.eq.1).and.(istep.ge.1)) then
-         write(*,*) '*ERROR reading *STATIC: perturbation analysis is'
-         write(*,*) '       not provided in a *STATIC step. Perform'
-         write(*,*) '       a genuine nonlinear geometric calculation'
-         write(*,*) '       instead (parameter NLGEOM)'
-         call exit(201)
-      endif
+c      if((iperturb.eq.1).and.(istep.ge.1)) then
+c         write(*,*) '*ERROR reading *STATIC: perturbation analysis is'
+c         write(*,*) '       not provided in a *STATIC step. Perform'
+c         write(*,*) '       a genuine nonlinear geometric calculation'
+c         write(*,*) '       instead (parameter NLGEOM)'
+c         ier=1
+c         return
+c      endif
 !
       if(istep.lt.1) then
          write(*,*) '*ERROR reading *STATIC: *STATIC can only be used'
          write(*,*) '       within a STEP'
-         call exit(201)
+         ier=1
+         return
       endif
 c!
 c!     no creep allowed in a *STATIC step
@@ -78,8 +81,8 @@ c      enddo
 !
 !     no heat transfer analysis
 !
-      if(ithermal.gt.1) then
-         ithermal=1
+      if(ithermal(1).gt.1) then
+         ithermal(1)=1
       endif
 !
 !     default solver
@@ -97,6 +100,8 @@ c      enddo
          solver(1:5)='TAUCS'
       elseif(isolver.eq.7) then
          solver(1:7)='PARDISO'
+      elseif(isolver.eq.8) then
+         solver(1:6)='PASTIX'
       endif
 !
       do i=2,n
@@ -131,6 +136,8 @@ c      enddo
          isolver=5
       elseif(solver(1:7).eq.'PARDISO') then
          isolver=7
+      elseif(solver(1:6).eq.'PASTIX') then
+         isolver=8
       else
          write(*,*) '*WARNING reading *STATIC: unknown solver;'
          write(*,*) '         the default solver is used'
@@ -159,13 +166,13 @@ c      enddo
      &        ipol,inl,ipoinp,inp,nmethod,key,ipoinpc)
          nmethod=1
          do i=1,mcs
-            cs(2,i)=-0.5
-            cs(3,i)=-0.5
+            cs(2,i)=-0.5d0
+            cs(3,i)=-0.5d0
          enddo
       endif
 !
       if((istat.lt.0).or.(key.eq.1)) then
-         if((iperturb.ge.2).or.(cfd.eq.1)) then
+         if((iperturb(1).ge.2).or.(nef.gt.0)) then
             write(*,*) '*WARNING reading *STATIC: a nonlinear analysis i
      &s requested'
             write(*,*) '         but no time increment nor step is speci
@@ -186,54 +193,83 @@ c            tincf=1.d-2
       endif
 !
       read(textpart(1)(1:20),'(f20.0)',iostat=istat) tinc
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*STATIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*STATIC%",ier)
+         return
+      endif
       read(textpart(2)(1:20),'(f20.0)',iostat=istat) tper
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*STATIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*STATIC%",ier)
+         return
+      endif
       read(textpart(3)(1:20),'(f20.0)',iostat=istat) tmin
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*STATIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*STATIC%",ier)
+         return
+      endif
       read(textpart(4)(1:20),'(f20.0)',iostat=istat) tmax
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*STATIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*STATIC%",ier)
+         return
+      endif
       read(textpart(5)(1:20),'(f20.0)',iostat=istat) tincf
-      if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*STATIC%")
+      if(istat.gt.0) then
+         call inputerror(inpc,ipoinpc,iline,
+     &        "*STATIC%",ier)
+         return
+      endif
 !
       if(tper.lt.0.d0) then
          write(*,*) '*ERROR reading *STATIC: step size is negative'
-         call exit(201)
+         ier=1
+         return
       elseif(tper.le.0.d0) then
          tper=1.d0
       endif
       if(tinc.lt.0.d0) then
          write(*,*) '*ERROR reading *STATIC: initial increment size is n
      &egative'
-         call exit(201)
+         ier=1
+         return
       elseif(tinc.le.0.d0) then
          tinc=tper
       endif
       if(tinc.gt.tper) then
          write(*,*) '*ERROR reading *STATIC: initial increment size exce
      &eds step size'
-         call exit(201)
+         ier=1
+         return
       endif
-c      if((cfd.eq.1).and.(tincf.le.0.d0)) then
-c         write(*,*) '*WARNING reading *STATIC: initial CFD increment siz
-c     &e is zero or negative; the default of 0.01 is taken'
-c         tincf=1.d-2
-c      endif
 !      
       if(idrct.ne.1) then
-c         if(dabs(tmin).lt.1.d-10) then
-c            tmin=min(tinc,1.d-5*tper)
          if(dabs(tmin).lt.1.d-6*tper) then
+            write(*,*) '*WARNING reading *STATIC:'
+            write(*,*) '         the minimum increment ',tmin
+            write(*,*) '         is smaller then 1.e-6 times the '
+            write(*,*) '         step time;'
+            write(*,*) '         the minimum increment is changed'
+            write(*,*) '         to ',min(tinc,1.d-6*tper)
+            write(*,*) '         which is the minimum of the initial'
+            write(*,*) 
+     &         '         increment time and 1.e-6 times the step time'
             tmin=min(tinc,1.d-6*tper)
          endif
          if(dabs(tmax).lt.1.d-10) then
             tmax=1.d+30
          endif
+         if(tinc.gt.dabs(tmax)) then
+            write(*,*) '*WARNING reading *STATIC:'
+            write(*,*) '         the initial increment ',tinc
+            write(*,*) '         exceeds the maximum increment ',
+     &          tmax
+            write(*,*) '         the initial increment is reduced'
+            write(*,*) '         to the maximum value'
+            tinc=dabs(tmax)
+         endif
       endif
 !
       if(timereset)ttime=ttime-tper
