@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2007 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -53,10 +53,7 @@
      &     springmm,springms,elcon(0:ncmat_,ntmat_,*),springfac,
      &     xmacont,areaslav,xk,temax
 !     
-      intent(in) ne,ne0,lakon,kon,ipkon,mi,ielmat,elcon,mortar,
-     &     adb,nactdof,springarea,ncmat_,ntmat_,stx
       
-      intent(out) temax
 !     
 !     Initialization
 !
@@ -80,7 +77,7 @@
 !           nope = number of master nodes + 1 (1 slave node)
 !  
             nope=ichar(lakonl(8:8))-47
-            springfac=1.0
+            springfac=1.d0
 !     
             do indexn=1,nope                   
                xmacont=0.0d0
