@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
       subroutine surfacebehaviors(inpc,textpart,elcon,nelcon,
      &  imat,ntmat_,ncmat_,irstrt,istep,istat,n,iline,ipol,inl,ipoinp,
      &  inp,ipoinpc,npmat_,plicon,nplicon,nstate_,
-     &  ichangesurfacebehavior)
+     &  ichangesurfacebehavior,ier)
 !
 !     reading the input deck: *SURFACE BEHAVIOR
 !
@@ -27,6 +27,9 @@
 !     nint(elcon(3,1,imat))=2: linear
 !     nint(elcon(3,1,imat))=3: tabular
 !     nint(elcon(3,1,imat))=4: tied
+!
+!     hard behavior is reduced to linear behavior with the
+!     default values
 ! 
       implicit none
 !
@@ -34,26 +37,28 @@
       character*132 textpart(16)
 !
       integer nelcon(2,*),imat,ntmat_,istep,istat,ipoinpc(0:*),
-     &  n,key,i,ncmat_,irstrt,iline,ipol,inl,ipoinp(2,*),inp(3,*),
-     &  ntmat,npmat,npmat_,nplicon(0:ntmat_,*),nstate_,
+     &  n,key,i,ncmat_,irstrt(*),iline,ipol,inl,ipoinp(2,*),inp(3,*),
+     &  ntmat,npmat,npmat_,nplicon(0:ntmat_,*),nstate_,ier,
      &  ichangesurfacebehavior
 !
       real*8 elcon(0:ncmat_,ntmat_,*),plicon(0:2*npmat_,ntmat_,*),
      &  temperature
 !
-      if((istep.gt.0).and.(irstrt.ge.0).and.
+      if((istep.gt.0).and.(irstrt(1).ge.0).and.
      &   (ichangesurfacebehavior.eq.0)) then
          write(*,*) '*ERROR reading *SURFACE BEHAVIOR:'
          write(*,*) '       *SURFACE BEHAVIOR should be placed'
          write(*,*) '       before all step definitions'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       if(imat.eq.0) then
          write(*,*) '*ERROR reading *SURFACE BEHAVIOR:'
          write(*,*) '       *SURFACE BEHAVIOR should be preceded'
          write(*,*) '       by a *SURFACE INTERACTION card'
-         call exit(201)
+         ier=1
+         return
       endif
       pressureoverclosure=' '
 !
@@ -69,6 +74,13 @@
          elseif(textpart(i)(1:35).eq.'PRESSURE-OVERCLOSURE=TIED') 
      &      then
             pressureoverclosure='D'
+         elseif(textpart(i)(1:25).eq.'PRESSURE-OVERCLOSURE=HARD') then
+            write(*,*) '*WARNING reading *SURFACE BEHAVIOR: hard'
+            write(*,*) '         behavior is by default reduced to'
+            write(*,*) '         linear behavior'
+            call inputwarning(inpc,ipoinpc,iline,
+     &"*SURFACE BEHAVIOR%")
+            pressureoverclosure='L'
          else
             write(*,*) 
      &   '*WARNING reading *SURFACE BEHAVIOR: parameter not recognized:'
@@ -82,7 +94,8 @@
          write(*,*) '*ERROR reading *SURFACE BEHAVIOR:'
          write(*,*) '       no PRESSURE-OVERCLOSURE defined on the'
          write(*,*) '       *SURFACE BEHAVIOR card'
-         call exit(201)
+         ier=1
+         return
       endif
 !
       if(nelcon(1,imat).ne.-51) nelcon(1,imat)=max(nelcon(1,imat),2)
@@ -97,7 +110,8 @@
             write(*,*) '       line is lacking for exponential'
             write(*,*) '       behavior'
             call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+     &           "*SURFACE BEHAVIOR%",ier)
+            return
          endif
 !     
          elcon(3,1,imat)=1.5d0
@@ -107,8 +121,11 @@
          do i=1,2
             read(textpart(i)(1:20),'(f20.0)',iostat=istat)
      &           elcon(i,1,imat)
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*SURFACE BEHAVIOR%",ier)
+               return
+            endif
          enddo
 !     
 !     checking the values
@@ -116,12 +133,14 @@
          if(elcon(1,1,imat).le.0.d0) then
             write(*,*) '*ERROR reading *SURFACE BEHAVIOR: c_0 must'
             write(*,*) '       exceed zero'
-            call exit(201)
+            ier=1
+            return
          endif
          if(elcon(2,1,imat).lt.0.d0) then
             write(*,*) '*ERROR reading *SURFACE BEHAVIOR: p_0 must'
             write(*,*) '       not be smaller than zero'
-            call exit(201)
+            ier=1
+            return
          endif
 !     
 !     transforming the parameters c_0 into
@@ -142,11 +161,39 @@
          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &        ipoinp,inp,ipoinpc)
          if((istat.lt.0).or.(key.eq.1)) then
-            write(*,*) '*ERROR reading *SURFACE BEHAVIOR: data'
-            write(*,*) '       line is lacking for linear'
-            write(*,*) '       behavior'
-            call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+c            write(*,*) '*ERROR reading *SURFACE BEHAVIOR: data'
+c            write(*,*) '       line is lacking for linear'
+c            write(*,*) '       behavior'
+c            call inputerror(inpc,ipoinpc,iline,
+c     &            "*SURFACE BEHAVIOR%",ier)
+c             return
+            write(*,*) '*WARNING reading *SURFACE BEHAVIOR: K must'
+            write(*,*) '         be strictly positive'
+            write(*,*) 
+     &       '         the following default will be used:',
+     &                  50.d0*elcon(1,1,1)
+            write(*,*) 
+     &       '         the user is advised to analyze the results' 
+            write(*,*) '         carefully' 
+            write(*,*)
+            elcon(2,1,imat)=50.d0*elcon(1,1,1)
+            write(*,*) '*INFO reading *SURFACE BEHAVIOR: the tension'
+            write(*,*) '      value at large clearances'
+            write(*,*) 
+     &      '      (only relevant for node-to-surface contact)'
+            write(*,*) '      must be strictly positive'
+            write(*,*) 
+     &       '      the following default will be used:',
+     &                  elcon(1,1,1)/70000.d0
+            write(*,*) 
+     &       '      the user is advised to analyze the results' 
+            write(*,*) 
+     &       '      carefully' 
+            write(*,*)
+            elcon(1,1,imat)=elcon(1,1,1)/70000.d0
+            elcon(3,1,imat)=2.5d0
+            elcon(4,1,imat)=1.d-3
+            return
          endif
 !     
          if(pressureoverclosure.eq.'L') then
@@ -162,23 +209,53 @@
 !     
          read(textpart(1)(1:20),'(f20.0)',iostat=istat)
      &        elcon(2,1,imat)
-         if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*SURFACE BEHAVIOR%",ier)
+            return
+         endif
 !     
          if(elcon(2,1,imat).le.0.d0) then
-            write(*,*) '*ERROR reading *SURFACE BEHAVIOR: K must'
-            write(*,*) '       be strictly positive'
-            call exit(201)
+            write(*,*) '*WARNING reading *SURFACE BEHAVIOR: K must'
+            write(*,*) '         be strictly positive'
+            write(*,*) 
+     &       '         the following default will be used:',
+     &                  50.d0*elcon(1,1,1)
+            write(*,*) 
+     &       '         the user is advised to analyze the results' 
+            write(*,*) 
+     &       '         carefully and, if possible, to come up with' 
+            write(*,*) 
+     &       '         a experimentally based K' 
+            call inputwarning(inpc,ipoinpc,iline,
+     &"*SURFACE BEHAVIOR%")
+            elcon(2,1,imat)=50.d0*elcon(1,1,1)
          endif
 !     
 !     tension at large clearances
 !     
          read(textpart(2)(1:20),'(f20.0)',iostat=istat)
      &        elcon(1,1,imat)
-         if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
-!
-c         elcon(1,1,imat)=-elcon(1,1,imat)
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*SURFACE BEHAVIOR%",ier)
+            return
+         endif
+         if(elcon(1,1,imat).le.0.d0) then
+            write(*,*) '*INFO reading *SURFACE BEHAVIOR: the tension'
+            write(*,*) '      value at large clearances'
+            write(*,*) 
+     &      '      (only relevant for node-to-surface contact)'
+            write(*,*) '      must be strictly positive'
+            write(*,*) 
+     &       '      the following default will be used:',
+     &                  elcon(1,1,1)/70000.d0
+            write(*,*) 
+     &       '      the user is advised to analyze the results' 
+            write(*,*) 
+     &       '      carefully' 
+            elcon(1,1,imat)=elcon(1,1,1)/70000.d0
+        endif
 !     
 !     value of c0coef. If the clearance is inferior to 
 !     c0coef*sqrt(slave_area) a contact spring element
@@ -186,8 +263,11 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
 !     
          read(textpart(3)(1:20),'(f20.0)',iostat=istat)
      &        elcon(4,1,imat)
-         if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+         if(istat.gt.0) then
+            call inputerror(inpc,ipoinpc,iline,
+     &           "*SURFACE BEHAVIOR%",ier)
+            return
+         endif
 !     
 !     default value
 !     
@@ -214,8 +294,11 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
      &           ipoinp,inp,ipoinpc)
             if((istat.lt.0).or.(key.eq.1)) exit
             read(textpart(3)(1:20),'(f20.0)',iostat=istat)temperature
-            if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+            if(istat.gt.0) then
+               call inputerror(inpc,ipoinpc,iline,
+     &              "*SURFACE BEHAVIOR%",ier)
+               return
+            endif
 !     
 !     first temperature
 !     
@@ -225,7 +308,8 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
                if(ntmat.gt.ntmat_) then
                   write(*,*) '*ERROR reading *SURFACE BEHAVIOR:'
                   write(*,*) '       increase ntmat_'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                nplicon(0,imat)=ntmat
                plicon(0,ntmat,imat)=temperature
@@ -238,7 +322,8 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
                if(ntmat.gt.ntmat_) then
                   write(*,*) '*ERROR reading *SURFACE BEHAVIOR:' 
                   write(*,*) '       increase ntmat_'
-                  call exit(201)
+                  ier=1
+                  return
                endif
                nplicon(0,imat)=ntmat
                plicon(0,ntmat,imat)=temperature
@@ -246,14 +331,18 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
             do i=1,2
                read(textpart(i)(1:20),'(f20.0)',iostat=istat) 
      &              plicon(2*npmat+i,ntmat,imat)
-               if(istat.gt.0) call inputerror(inpc,ipoinpc,iline,
-     &"*SURFACE BEHAVIOR%")
+               if(istat.gt.0) then
+                  call inputerror(inpc,ipoinpc,iline,
+     &                 "*SURFACE BEHAVIOR%",ier)
+                  return
+               endif
             enddo
             npmat=npmat+1
             if(npmat.gt.npmat_) then
                write(*,*) 
      &              '*ERROR reading *SURFACE BEHAVIOR: increase npmat_'
-               call exit(201)
+               ier=1
+               return
             endif
             nplicon(ntmat,imat)=npmat
          enddo
@@ -262,7 +351,8 @@ c         elcon(1,1,imat)=-elcon(1,1,imat)
             write(*,*) 
      &           '*ERROR reading *SURFACE BEHAVIOR: *SURFACE BEHAVIOR'
             write(*,*) '       card without data'
-            call exit(201)
+            ier=1
+            return
          endif
 !
 !        check whether the difference between the overclosure data
