@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,9 +18,9 @@
 !
       subroutine umat_main(amat,iel,iint,kode,elconloc,emec,emec0,
      &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
-     &        icmd,ielas,mi,
-     &        nstate_,xstateini,xstate,stre,stiff,iorien,pgauss,
-     &        orab,pnewdt,istep,iinc,ipkon,nmethod)
+     &        icmd,ielas,mi,nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,pnewdt,istep,iinc,ipkon,nmethod,
+     &        iperturb,depvisc,eloc,nlgeom_undo)
 !
 !     calculates stiffness and stresses for a user defined material
 !     law
@@ -29,30 +29,31 @@
 !
       character*80 amat,amatloc
 !
-      integer ithermal,icmd,kode,ielas,iel,iint,nstate_,mi(*),iorien,
-     &  istep,iinc,ipkon(*),nmethod
+      integer ithermal(*),icmd,kode,ielas,iel,iint,nstate_,mi(*),iorien,
+     &  istep,iinc,ipkon(*),nmethod,iperturb(*),nlgeom_undo
 !
-      real*8 elconloc(21),stiff(21),emec(6),emec0(6),beta(6),stre(6),
+      real*8 elconloc(*),stiff(21),emec(6),emec0(6),beta(6),stre(6),
      &  vj,t1l,dtime,xkl(3,3),xikl(3,3),vij,pgauss(3),orab(7,*),
-     &  time,ttime,pnewdt
+     &  time,ttime,pnewdt,depvisc,eloc(6)
 !
       real*8 xstate(nstate_,mi(1),*),xstateini(nstate_,mi(1),*)
 !
-      if(amat(1:1).eq.'@') then
+      
+      if(amat(1:8).eq.'ABAQUSNL') then
 
-         call umat_abaqusnl(amat,iel,iint,kode,elconloc,emec,
-     &        emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
-     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
-     &        iorien,pgauss,orab,istep,iinc,pnewdt)
-         
-      elseif(amat(1:8).eq.'ABAQUSNL') then
-!
          amatloc(1:72)=amat(9:80)
          amatloc(73:80)='        '
          call umat_abaqusnl(amatloc,iel,iint,kode,elconloc,emec,
      &        emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
      &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
-     &        iorien,pgauss,orab,istep,iinc,pnewdt)
+     &        iorien,pgauss,orab,istep,iinc,pnewdt,nmethod,iperturb)
+         
+      elseif(amat(1:9).eq.'@ABAQUSNL') then
+!
+         call umat_abaqusnl(amat,iel,iint,kode,elconloc,emec,
+     &        emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
+     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,istep,iinc,pnewdt,nmethod,iperturb)
 !
       elseif(amat(1:6).eq.'ABAQUS') then
 !
@@ -61,8 +62,15 @@
          call umat_abaqus(amatloc,iel,iint,kode,elconloc,emec,
      &        emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
      &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
-     &        iorien,pgauss,orab,istep,iinc,pnewdt)
+     &        iorien,pgauss,orab,istep,iinc,pnewdt,nmethod,iperturb)
 !
+      elseif(amat(1:7).eq.'@ABAQUS') then
+
+         call umat_abaqus(amat,iel,iint,kode,elconloc,emec,
+     &        emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
+     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,istep,iinc,pnewdt,nmethod,iperturb)
+!        
       elseif(amat(1:10).eq.'ANISO_PLAS') then
 !
          amatloc(1:70)=amat(11:80)
@@ -81,7 +89,7 @@
      &        iel,iint,kode,elconloc,emec,emec0,
      &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
      &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
-     &        iorien,pgauss,orab,nmethod,pnewdt)
+     &        iorien,pgauss,orab,nmethod,pnewdt,depvisc)
 !
       elseif(amat(1:10).eq.'CIARLET_EL') then
 !
@@ -112,6 +120,16 @@
      &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
      &        icmd,ielas,mi(1),
      &        nstate_,xstateini,xstate,stre,stiff,iorien,pgauss,orab)
+!
+      elseif(amat(1:12).eq.'LIN_EL_COROT') then
+!
+         amatloc(1:68)=amat(13:80)
+         amatloc(69:80)='            '
+         call umat_lin_el_corot(amatloc,iel,iint,kode,
+     &        elconloc,emec,emec0,
+     &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
+     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,eloc,nlgeom_undo)
 !
       elseif(amat(1:10).eq.'LIN_ISO_EL') then
 !
@@ -172,6 +190,33 @@
      &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
      &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
      &        iorien,pgauss,orab,pnewdt,ipkon)
+!
+      elseif(amat(1:18).eq.'UNDO_NLGEOM_LIN_EL') then
+!
+         amatloc(1:62)=amat(19:80)
+         amatloc(63:80)='                  '
+         call umat_undo_nlgeom_lin_el(amatloc,iel,iint,kode,
+     &        elconloc,emec,emec0,
+     &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
+     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,eloc,nlgeom_undo)
+!
+      elseif(amat(1:22).eq.'UNDO_NLGEOM_LIN_ISO_EL') then
+!
+         amatloc(1:58)=amat(23:80)
+         amatloc(59:80)='                      '
+         call umat_undo_nlgeom_lin_iso_el(amatloc,iel,iint,kode,
+     &        elconloc,emec,emec0,
+     &        beta,xikl,vij,xkl,vj,ithermal,t1l,dtime,time,ttime,
+     &        icmd,ielas,mi(1),nstate_,xstateini,xstate,stre,stiff,
+     &        iorien,pgauss,orab,eloc,nlgeom_undo)
+!
+      elseif(amat(1:1).eq.'@') then
+!
+         call call_external_umat_user(amat,iel,iint,kode,elconloc,
+     &        emec,emec0,beta,xikl,vij,xkl,vj,ithermal,t1l,
+     &        dtime,time,ttime,icmd,ielas,mi(1),nstate_,xstateini,
+     &        xstate,stre,stiff,iorien,pgauss,orab,pnewdt,ipkon)
       else
          write(*,*) '*ERROR in umat: no user material subroutine'
          write(*,*) '       defined for material ',amat
