@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 !     calculates stiffness and stresses for a user defined material
 !     law
 !
-!     icmd=3: calcutates stress at mechanical strain
+!     icmd=3: calculates stress at mechanical strain
 !     else: calculates stress at mechanical strain and the stiffness
 !           matrix
 !
@@ -119,7 +119,7 @@
 !
       character*80 amat
 !
-      integer ithermal,icmd,kode,ielas,iel,iint,nstate_,mi(*),iorien
+      integer ithermal(*),icmd,kode,ielas,iel,iint,nstate_,mi(*),iorien
 !
       integer index(18),i,j,k,l,ipiv(18),info,ichange,neq,lda,ldb,
      &  nrhs,iplas,icounter
@@ -131,7 +131,7 @@
      &  t(42),gl(18,18),gr(18,18),ee(6),c1111,c1122,c1212,dd,
      &  skl(3,3),xmtran(3,3),ddsdde(6,6),xx(6,18)
 !
-      real*8 elconloc(21),stiff(21),emec(6),emec0(6),beta(6),stre(6),
+      real*8 elconloc(*),stiff(21),emec(6),emec0(6),beta(6),stre(6),
      &  vj,t1l,dtime,xkl(3,3),xokl(3,3),voj,pgauss(3),orab(7,*),
      &  elas(21),time,ttime
 !
@@ -162,131 +162,131 @@
 !     18. n=0,0,1   l=1,-1,0
 !
       xm=reshape((
-     &   /0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.2041241452319E+00,-0.2041241452319E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.0000000000000E+00,-0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.4082482904639E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00,-0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.4082482904639E+00, 0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &   -0.2041241452319E+00, 0.0000000000000E+00, 0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.2041241452319E+00, 0.2041241452319E+00,
-     &    0.0000000000000E+00,-0.4082482904639E+00, 0.4082482904639E+00,
-     &    0.2041241452319E+00,-0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &   -0.2041241452319E+00, 0.0000000000000E+00,-0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.2041241452319E+00,-0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.4082482904639E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.0000000000000E+00, 0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.2041241452319E+00, 0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.3535533905933E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00,-0.3535533905933E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.0000000000000E+00, 0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.0000000000000E+00,-0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.3535533905933E+00, 0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.3535533905933E+00,-0.3535533905933E+00/
+     &   /0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.2041241452319d+00,-0.2041241452319d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.0000000000000d+00,-0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.4082482904639d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00,-0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.4082482904639d+00, 0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &   -0.2041241452319d+00, 0.0000000000000d+00, 0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.2041241452319d+00, 0.2041241452319d+00,
+     &    0.0000000000000d+00,-0.4082482904639d+00, 0.4082482904639d+00,
+     &    0.2041241452319d+00,-0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &   -0.2041241452319d+00, 0.0000000000000d+00,-0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.2041241452319d+00,-0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.4082482904639d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.0000000000000d+00, 0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.2041241452319d+00, 0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.3535533905933d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00,-0.3535533905933d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.0000000000000d+00, 0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.0000000000000d+00,-0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.3535533905933d+00, 0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.3535533905933d+00,-0.3535533905933d+00/
      &    ),(/6,18/))
 !
       xx=reshape((
-     &   /0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.2041241452319E+00,-0.2041241452319E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.0000000000000E+00,-0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.4082482904639E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00,-0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.4082482904639E+00, 0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &   -0.2041241452319E+00, 0.0000000000000E+00, 0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.2041241452319E+00, 0.2041241452319E+00,
-     &    0.0000000000000E+00,-0.4082482904639E+00, 0.4082482904639E+00,
-     &    0.2041241452319E+00,-0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &   -0.2041241452319E+00, 0.0000000000000E+00,-0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.2041241452319E+00,-0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.4082482904639E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.2041241452319E+00, 0.0000000000000E+00,
-     &    0.4082482904639E+00, 0.0000000000000E+00,-0.4082482904639E+00,
-     &    0.2041241452319E+00, 0.0000000000000E+00, 0.2041241452319E+00,
-     &    0.4082482904639E+00,-0.4082482904639E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00,-0.2041241452319E+00, 0.2041241452319E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.3535533905933E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00,-0.3535533905933E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.0000000000000E+00, 0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.3535533905933E+00, 0.0000000000000E+00,-0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.3535533905933E+00, 0.3535533905933E+00,
-     &    0.0000000000000E+00, 0.0000000000000E+00, 0.0000000000000E+00,
-     &    0.0000000000000E+00, 0.3535533905933E+00,-0.3535533905933E+00/
+     &   /0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.2041241452319d+00,-0.2041241452319d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.0000000000000d+00,-0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.4082482904639d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00,-0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.4082482904639d+00, 0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &   -0.2041241452319d+00, 0.0000000000000d+00, 0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.2041241452319d+00, 0.2041241452319d+00,
+     &    0.0000000000000d+00,-0.4082482904639d+00, 0.4082482904639d+00,
+     &    0.2041241452319d+00,-0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &   -0.2041241452319d+00, 0.0000000000000d+00,-0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.2041241452319d+00,-0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.4082482904639d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.2041241452319d+00, 0.0000000000000d+00,
+     &    0.4082482904639d+00, 0.0000000000000d+00,-0.4082482904639d+00,
+     &    0.2041241452319d+00, 0.0000000000000d+00, 0.2041241452319d+00,
+     &    0.4082482904639d+00,-0.4082482904639d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00,-0.2041241452319d+00, 0.2041241452319d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.3535533905933d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00,-0.3535533905933d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.0000000000000d+00, 0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.3535533905933d+00, 0.0000000000000d+00,-0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.3535533905933d+00, 0.3535533905933d+00,
+     &    0.0000000000000d+00, 0.0000000000000d+00, 0.0000000000000d+00,
+     &    0.0000000000000d+00, 0.3535533905933d+00,-0.3535533905933d+00/
      &    ),(/6,18/))
 !
       h=reshape((
-     &   /0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00, 0.1E+01,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &    0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,
-     &   -0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00, 0.1E+01,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.1E+01,-0.1E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00,-0.1E+00, 0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,
-     &    0.1E+01,-0.1E+00,-0.1E+00,-0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00,
-     &   -0.1E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,
-     &   -0.1E+00,-0.1E+00,-0.1E+00, 0.1E+01,-0.1E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00, 0.0E+00,
-     &    0.0E+00, 0.0E+00, 0.0E+00,-0.1E+00,-0.1E+00,-0.1E+00,-0.1E+00,
-     &   -0.1E+00, 0.1E+01/),(/18,18/))
+     &   /0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00, 0.1d+01,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &    0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,
+     &   -0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00, 0.1d+01,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.1d+01,-0.1d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00,-0.1d+00, 0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,
+     &    0.1d+01,-0.1d+00,-0.1d+00,-0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00,
+     &   -0.1d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,
+     &   -0.1d+00,-0.1d+00,-0.1d+00, 0.1d+01,-0.1d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00, 0.0d+00,
+     &    0.0d+00, 0.0d+00, 0.0d+00,-0.1d+00,-0.1d+00,-0.1d+00,-0.1d+00,
+     &   -0.1d+00, 0.1d+01/),(/18,18/))
 !
 !     elastic constants
 !
