@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2007 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -175,6 +175,13 @@
                      else
                         nopem=8
                      endif
+                  else
+                     write(*,*) '*ERROR in updatecontpen'
+                     write(*,*) '       unknown element type'
+                     write(*,*) '       in contact master set'
+                     write(*,*) '       element number :',nelemm
+                     write(*,*) '       element label:',lakon(nelemm)
+                     call exit(201)
                   endif
 !     
 !     actual position of the nodes belonging to the
