@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -30,13 +30,13 @@
 !
       integer ipompc(*),nodempc(3,*),nmpc,nmpc_,mpcfree,nk,nk_,
      &  ikmpc(*),idir,indexmax,indexold(3),indexnew(3),nodeold,nodemax,
-     &  ilmpc(*),node,id,mpcfreeold,idof,l,nodeboun(*),iperturb(2),
+     &  ilmpc(*),node,id,mpcfreeold,idof,l,nodeboun(*),iperturb(*),
      &  ndirboun(*),ikboun(*),ilboun(*),nboun,nboun_,nnodes,nodevector,
      &  index,index1,node1,i,j,nkn,idirold,idirmax,noderef,idirref,
      &  nendnode
 !
       real*8 coefmpc(*),co(3,*),aa(3),dd,cgx(3),pi(3),c1,c4,c9,
-     &  c10,amax,xcoef,transcoef(3),xboun(*)
+     &  c10,amax,xcoef,transcoef(3),xboun(*),stdev
 !
       save nodevector
 !
@@ -59,6 +59,7 @@
      &                   '         on the dependent side of another'
                      write(*,*) '         MPC. ',label
                      write(*,*) '         constraint cannot be applied'
+                     write(*,*)
                      return
                   endif
                endif
@@ -196,6 +197,23 @@
                cgx(i)=cgx(i)/nkn
             enddo
 !
+!           calculating a standard deviation; this quantity will
+!           serve as a limit for checking the closeness of individual
+!           nodes to the center of gravity
+!     
+            stdev=0.d0
+!
+            index=ipompc(nmpc)
+            do
+               node=nodempc(1,index)
+               if(node.eq.nodevector) exit
+               do j=1,3
+                  stdev=stdev+(co(j,node)-cgx(j))**2
+               enddo
+               index=nodempc(3,nodempc(3,nodempc(3,index)))
+            enddo
+            stdev=stdev/nkn
+!
 !           calculating the derivatives
 !
 !           loop over all nodes belonging to the mean rotation MPC
@@ -219,7 +237,8 @@
                enddo
 !
                c1=pi(1)*pi(1)+pi(2)*pi(2)+pi(3)*pi(3)
-               if(c1.lt.1.d-20) then
+c               if(c1.lt.1.d-20) then
+               if(c1.lt.stdev*1.d-10) then
                   if(label(8:9).ne.'BS') then
                      write(*,*) '*WARNING in usermpc: node ',node
                      write(*,*) '         is very close to the '
@@ -229,6 +248,7 @@
                      write(*,*) '         mean rotation MPC.'
                      write(*,*) '         This node is not taken'
                      write(*,*) '         into account in the MPC'
+                     write(*,*)
                   endif
                   index=nodempc(3,nodempc(3,nodempc(3,index)))
                   cycle
@@ -280,48 +300,6 @@
                index=nodempc(3,index)
                if(nodempc(3,index).eq.0) exit
             enddo
-c!
-c!     looking for the maximum tangent to decide which DOF should be
-c!     taken to be the dependent one
-c!
-c            index=ipompc(nmpc)
-c            if(dabs(coefmpc(index)).lt.1.d-5) then
-c!
-c!              changing the DOF of the dependent degree of freedom
-c!
-c               amax=dabs(coefmpc(index))
-c               imax=1
-c               a(1)=coefmpc(index)
-c               do i=2,3
-c                  index=nodempc(3,index)
-c                  a(i)=coefmpc(index)
-c                  if(dabs(a(i)).gt.amax) then
-c                     amax=dabs(a(i))
-c                     imax=i
-c                  endif
-c               enddo
-c!
-c               index=ipompc(nmpc)
-c               nodempc(2,index)=imax
-c               a1=a(1)
-c               coefmpc(index)=a(imax)
-c               do i=2,3
-c                  index=nodempc(3,index)
-c                  if(i.eq.imax) then
-c                     nodempc(2,index)=1
-c                     coefmpc(index)=a1
-c                  else
-c                     nodempc(2,index)=i
-c                  endif
-c               enddo
-c!
-c!              updating ikmpc and ilmpc
-c!               
-c               index=ipompc(nmpc)
-c               idofrem=8*(nodempc(1,index)-1)+1
-c               idofins=8*(nodempc(1,index)-1)+imax
-c               call changedepterm(ikmpc,ilmpc,nmpc,nmpc,idofrem,idofins)
-c            endif
 !
 !           look for biggest coefficient of all but the last
 !           regular node. The general form of the MPC is:
@@ -418,11 +396,13 @@ c            endif
                   write(*,*) '         generated for the mean'
                   write(*,*) '         rotation in node ',noderef
                   write(*,*) '         and direction ',idirref+3
+                  write(*,*)
                else
                   write(*,*) '*WARNING in usermpc: no MPC is '
                   write(*,*) '         generated for the mean'
                   write(*,*) '         rotation definition starting '
                   write(*,*) '         with node ',noderef
+                  write(*,*)
                endif
 !
 !              removing the MPC
@@ -538,12 +518,12 @@ c            endif
             write(*,*) '      effects are turned on'
             write(*,*)
             if(iperturb(1).eq.0) iperturb(1)=2
-         elseif(labmpc(nmpc)(1:3).eq.'GAP') then
-            iperturb(2)=1
-            write(*,*) '*INFO in usermpc: nonlinear geometric'
-            write(*,*) '      effects are turned on'
-            write(*,*)
-            if(iperturb(1).eq.0) iperturb(1)=2
+c         elseif(labmpc(nmpc)(1:3).eq.'GAP') then
+c            iperturb(2)=1
+c            write(*,*) '*INFO in usermpc: nonlinear geometric'
+c            write(*,*) '      effects are turned on'
+c            write(*,*)
+c            if(iperturb(1).eq.0) iperturb(1)=2
          elseif(labmpc(nmpc)(1:4).eq.'USER') then
             iperturb(2)=1
             write(*,*) '*INFO in usermpc: nonlinear geometric'
