@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2015 Guido Dhondt
+!     Copyright (C) 1998-2020 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !     
       subroutine vortex(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &     nactdog,identity,ielprop,prop,iflag,v,xflow,f,
-     &     nodef,idirf,df,cp,R,numf,set,mi,ttime,time,iaxial)
+     &     nodef,idirf,df,cp,R,numf,set,mi,ttime,time,iaxial,iplausi)
 !     
 !     vortex element
 !
@@ -31,22 +31,17 @@
       character*81 set(*)
 !     
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,iaxial,
-     &     inv,ipkon(*),kon(*),t_chang,nelemswirl,mi(*)
+     &     ielprop(*),nodef(*),idirf(*),index,iflag,iaxial,
+     &     inv,ipkon(*),kon(*),t_chang,nelemswirl,mi(*),iplausi
 !
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),kappa,r,cp,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),kappa,r,cp,
      &     p1,p2,T1,T2,km1,pi,ttime,time,r2d,r1d,eta,U1,
      &     c1u,c2u,cinput,r1,r2,omega,K1,ciu,expon,
      &     Ui,Kr,cte1,cte2,qred_crit,A,xflow_oil
 !
-      intent(in) nodem,nelem,lakon,kon,ipkon,
-     &     nactdog,ielprop,iflag,v,
-     &     cp,R,set,mi,ttime,time,iaxial
 !
-      intent(out) identity,node1,node2,xflow,numf,nodef,idirf,prop,
-     &  f,df
 !     
-      if (iflag.eq.0) then
+      if(iflag.eq.0) then
          identity=.true.
 !     
          if(nactdog(2,node1).ne.0)then
@@ -57,38 +52,15 @@
             identity=.false.
          endif
 !     
-      elseif (iflag.eq.1)then
+      elseif(iflag.eq.1)then
+         if(v(1,nodem).ne.0.d0) then
+            xflow=v(1,nodem)
+            return
+         endif
 !
-c         kappa=(cp/(cp-R))
-c         pi=4.d0*datan(1.d0)
-c         index=ielprop(nelem)
-c         qred_crit=dsqrt(kappa/R)*
-c     &        (1+0.5d0*(kappa-1))**(-0.5*(kappa+1)/(kappa-1))
-c!
-c!     Because there is no explicit expression relating massflow
-c!     with to pressure loss for vortices
-c!     For  FREE as well as for FORCED VORTICES
-c!     initial mass flow is set to Qred_crit/2 = 0.02021518917
-c!     with consideration to flow direction
-c!
-c         node1=kon(ipkon(nelem)+1)
-c         node2=kon(ipkon(nelem)+3)
-c         p1=v(2,node1)
-c         p2=v(2,node2)
-c         T1=v(0,node1)
-c         T2=v(0,node2)
-c!
-c!     abstract cross section
-c         A=10E-6
-c!
-c         if(p1.gt.p2) then
-c            xflow=0.5/dsqrt(T1)*A*P1*qred_crit
-c         else
-c            xflow=-0.5/dsqrt(T1)*A*P1*qred_crit
-c         endif
          xflow=0.d0
 !        
-      elseif (iflag.eq.2)then
+      elseif(iflag.eq.2)then
 !     
          numf=4
          index=ielprop(nelem) 
@@ -255,7 +227,7 @@ c         endif
                cte1=c1u**2/(2*Cp*T1)
                cte2=1-(R1/R2)**2
                
-               f=P2/P1-1d0-eta*((1+cte1*cte2)**expon-1d0)
+               f=p2/p1-1d0-eta*((1+cte1*cte2)**expon-1d0)
 !     
                df(1)=-p2/p1**2
 !     
@@ -271,7 +243,7 @@ c         endif
                cte1=c2u**2/(2*Cp*T2)
                cte2=1-(R2/R1)**2
 !     
-               f=P1/P2-1d0-eta*((1+cte1*cte2)**expon-1d0)
+               f=p1/p2-1d0-eta*((1+cte1*cte2)**expon-1d0)
 !     
                df(1)=1/p2
 !     
@@ -528,7 +500,7 @@ c         endif
                cte1=c1u**2/(2*Cp*T1)
                cte2=1-(R1/R2)**2
                
-               f=P2/P1-1d0-eta*((1+cte1*cte2)**expon-1d0)
+               f=p2/p1-1d0-eta*((1+cte1*cte2)**expon-1d0)
 !     
                df(1)=-p2/p1**2
 !     
@@ -544,7 +516,7 @@ c         endif
                cte1=c2u**2/(2*Cp*T2)
                cte2=1-(R2/R1)**2
 !     
-               f=P1/P2-1d0-eta*((1+cte1*cte2)**expon-1d0)
+               f=p1/p2-1d0-eta*((1+cte1*cte2)**expon-1d0)
 !     
                df(1)=1/p2
 !     
@@ -603,21 +575,21 @@ c         endif
 !
          if(inv.eq.1) then
             write(1,56)'       Inlet node ',node1,' :     Tt1 = ',T1,
-     &           '  , Ts1 = ',T1,'  , Pt1 = ',P1
+     &           '  , Ts1 = ',T1,'  , Pt1 = ',p1
             write(1,*)'             Element ',nelem,lakon(nelem)
             write(1,57)'             C1u = ',C1u,
      &'  , C2u = ',C2u
-            write(1,56)'       Outlet node ',node2,' :    Tt2 = ',T2,
-     &           '  , Ts2 = ',T2,'  , Pt2 = ',P2
+            write(1,56)'      Outlet node ',node2,' :    Tt2 = ',T2,
+     &           '  , Ts2 = ',T2,'  , Pt2 = ',p2
 !     
          else if(inv.eq.-1) then
             write(1,56)'       Inlet node ',node2,':     Tt1 = ',T1,
-     &           '  , Ts1 = ',T1,'  , Pt1 = ',P1
+     &           '  , Ts1 = ',T1,'  , Pt1 = ',p1
             write(1,*)'             Element ',nelem,lakon(nelem)
             write(1,57)'             C1u = ',C1u,
      &'  , C2u = ',C2u
-            write(1,56)'       Outlet node ',node1,'     Tt2 = ',
-     &           T2,'  , Ts2 = ',T2,'  , Pt2 = ',P2
+            write(1,56)'      Outlet node ',node1,'     Tt2 = ',
+     &           T2,'  , Ts2 = ',T2,'  , Pt2 = ',p2
          endif
       endif
 !
