@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2015 Guido Dhondt
+!              Copyright (C) 1998-2020 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -27,9 +27,7 @@
 !
       real*8 a2u,al,um,au,p1,p2,p3
 !
-      intent(in) vo,al,um
 !
-      intent(inout) v
 !
       a2u=al+2.d0*um
       au=al+um
